/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

//translations
#include <locale.h>
#include <libintl.h>

#include <gtk/gtk.h>
#include <swfdec-gtk/swfdec-gtk.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <zip.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "editeur.h"
#include "filesfuncs.h"
#include "svgfuncs.h"
#include "timeline.h"
#include "dlgdivers.h"
#include "biblio.h"
#include "clipboard.h"
#include "cursor.h"

extern GtkWidget *pListViewLayers;
extern GooCanvasItem *rootEditorItems;
extern GList *pSelectedItemsList; /* liste des items, mais VALIDE, sans le fond et l'origine */
extern gint LayerCount;
/* noms de fichiers */
gchar *chemin;/* nom (temporaire) complet du fichier en cours d'utilisation */
gchar chemin_complet[512]; /* stocke de manière durable le chemin complet jusqu'au fichier */
gchar *chemin_export;
gboolean fModified=FALSE;
gboolean fFileNamed=FALSE;
/* player SWF */
 SwfdecPlayer *player_preview=NULL;
 GtkWidget *widget_swf;
/* autoarchivage biblio */
static gboolean auto_archive_biblio=FALSE;
static GSList *recent_files_list=NULL;
static gchar *prefered_folder=NULL;
static gpointer *pMenusRecentFiles;

/***********************************
 * management of recent files
 * *********************************/
GSList *cb_update_recent_files_list(GSList *list, gchar *path)
{
	GSList *node;
	gint i;
	
	list= g_slist_append(list,path);
	if (g_slist_length(list)>6)
		    {
				printf("liste trop longue ! \n");
		        
				// then we swap the text content of pointers 
				for(i=0;i<6;i++)
				   {
					   node=g_slist_nth(list,i+1);
					   gtk_menu_item_set_label(pMenusRecentFiles[i],
				         (const gchar *)g_strdup_printf("%s",
					     (gchar *)(node->data)
						 ));					
				   }
				list=g_slist_remove(recent_files_list,g_slist_nth_data(recent_files_list,0));  
			}
	else
	  {		
        i=	g_slist_length(list);		
        gtk_widget_set_sensitive (pMenusRecentFiles[i-1],TRUE);
        gtk_menu_item_set_label(pMenusRecentFiles[i-1],
				      (const gchar *)g_strdup_printf("%s",
					     (gchar *)g_strdelimit((gchar *)path,"_",' ')
						 ));					
	  }
	return list;   
}

GSList *cb_get_recent_files_list(void)
{
	return recent_files_list;
}
/* store here the pointers to the 6 submenus for recent files */
void cb_init_menus_recent_files(gpointer *p)
{
  gint i;
  
  pMenusRecentFiles=g_new(gpointer,6);	
  
  pMenusRecentFiles	[0]=p[0];
  pMenusRecentFiles	[1]=p[1];
  pMenusRecentFiles	[2]=p[2];
  pMenusRecentFiles	[3]=p[3];
  pMenusRecentFiles	[4]=p[4];
  pMenusRecentFiles	[5]=p[5];
}
/* we open the existing file, and then set the recent files menus */
void cb_reload_recent_files_list(void)
{
  gchar *path_test;
  gboolean existe;
  gint i,NbLus;
  FILE *f;
  gchar buffer[2048];
  GSList *node;
  
  recent_files_list=NULL;
  for(i=0;i<=5;i++)
		  {
			  gtk_widget_set_sensitive (pMenusRecentFiles[i],FALSE);
		  }
		  
  path_test=g_strdup_printf("%s/.ellarecentfiles",getenv("HOME"));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (existe)
    {		     
		f=fopen(path_test,"r");
		i=0; 
		 while ((fgets(buffer,sizeof(buffer),f))&&(i<6))
		  {
			  if ( strlen((gchar *)buffer)>1 )
			     {
			       gtk_widget_set_sensitive (pMenusRecentFiles[i],TRUE);
			       recent_files_list=cb_update_recent_files_list(recent_files_list,
				        g_strndup((const gchar *)buffer,
						strlen((gchar*)buffer)-1)
						);
			     //  gtk_menu_item_set_label(pMenusRecentFiles[i],
				   //   (const gchar *)g_strdup_printf("_%d:%s",i+1,
					 //    (gchar *)g_strdelimit((gchar *)buffer,"_",' ')
						// ));
			       i++;
			     }
		  }
		fclose(f);
	 }			  
}

gint cb_save_recent_files_list(GSList *list)
{
  gchar *path_list;
  FILE *f;
  GSList *node;
  gint i;
  	
  if (list!=NULL)
	{
	  path_list=g_strdup_printf("%s/.ellarecentfiles",getenv("HOME"));
	  f=fopen(path_list,"w");
	  for(i=0;node=g_slist_nth(recent_files_list,i);i++)
	    {
			fprintf(f,"%s\n",(gchar *)(node->data));
		}
	  fclose(f);
	  g_free(path_list);	
	  return 0;
	}	
  else
    return -1;	
}
/************************************
 * management of prefered folder
 * *********************************/
void cb_prefered_folder_init(void)
{
	prefered_folder=g_strdup_printf("%s", getenv("HOME"));
}
void prefered_folder_reset(void)
{
	if (prefered_folder!=NULL)
	   g_free(prefered_folder);
}
void prefered_folder_set(gchar *path)
{
	if (path!=NULL)
	  {
		  prefered_folder_reset();
		  prefered_folder=g_strdup_printf("%s",path);
	  }
}
gchar *cb_get_prefered_folder(void)
{
	return prefered_folder;
}


/********************************
 * appui sur la touche STOP du
 * lecteur
 * ******************************/
void player_stop(void)
{
	kill_swf_player();
}
void
on_buttonPlayerStop_clicked (GtkButton       *button,  gpointer         *user_data)
{
 GtkWidget *nbook;
 GtkWidget *win;
 GtkWidget *label;
 gulong duree;
 
 nbook=user_data[0];
 win=user_data[1];
 label=user_data[2];	
 player_stop();
}									   
void
on_buttonPlayerPause_clicked (GtkButton       *button,  gpointer         *user_data)
{
 GtkWidget *nbook;
 GtkWidget *win;
 GtkWidget *label;
 gulong duree;
 
 nbook=user_data[0];
 win=user_data[1];
 label=user_data[2];	
 swf_player_pause();
}	
/*********************************
 * fonction pour lire dans le 
 * lecteur l'animation en cours
 * ******************************/
gulong player_build_preview(GtkWidget *win)
{
 gdouble w,h;
 GdkColor cp;
 gchar buf[512];
 gchar buf2[512];
 FILE *fich_swf;
 gint erreur;
 gint max_frames,freq;		
 gulong max_msecs;							
				
 cursor_busy(win);										
 /* récupérer les données utiles */
 w=editor_get_page_width();
 h=editor_get_page_height();
 cp=editor_get_page_color();
 /* now we computing the total duration in msecs */
 max_frames= timeline_get_max_absolute_frames();
 freq=timeline_get_movie_freq();
 max_msecs=(max_frames/freq)*1000;
/* compilation */
 sprintf(buf,"%s/.ella/temp.sc",getenv("HOME"));
 sprintf(buf2,"%s/.ella/player.swf",getenv("HOME"));
 fich_swf=ouvrir_fichier_swf_export(buf, win);
 exporter_en_tete_swf(fich_swf,
                      buf2, 
                      (guint)w, (guint)h, 
                      5, 
                      timeline_get_movie_freq(),
                      cp,0,0);
 /* on va parcourir  pListViewLayers */

 exporter_calques_vers_swf(fich_swf,
                       pListViewLayers,
					   timeline_get_layers_count(),
					   MAX_FRAMES,
					   editor_get_images_stack());

 fermer_fichier_swf_export(fich_swf);
 /* exécuter le compilateur */
 sprintf(buf,"swfc %s/.ella/temp.sc",getenv("HOME"));
 erreur = system(buf);
 cursor_normal(win);
 /* à ce stade, la preview est dans le dossier caché */
 return max_msecs;
}/*end play animation */

void
    on_buttonPlayerPlay_clicked(GtkButton  *button, gpointer         *user_data)
{
 GtkWidget *nbook;
 GtkWidget *win;
 GtkWidget *label;
 GtkWidget *play_key;
 gulong duree;
 
 nbook=user_data[0];
 win=user_data[1];
 label=user_data[2];
 play_key=user_data[3];
 	
 duree=player_build_preview(win);	
 preview_animation(win, duree, label,play_key,timeline_get_root_ruler(),timeline_get_zooming(),timeline_get_movie_freq());
}								
/************************************
 * open an Ella File : parameter,
 * a correct path to the file
 * *********************************/
gint open_file(GtkWidget *window1,gchar *chemin)
{
  FILE *fichier_ella, *fichier;
  gboolean sortie=FALSE;
  char buf[LONG_STRING+1];
  gchar *instr=NULL;
  gchar *param=NULL;
  gchar *nom_calque;
  gchar *nom_item;
  gdouble valeur=0;
  gint val_int,val_current_sound_index;
  gint r,v,b,a=65535;
  gint wd,ht,ma;
  gboolean visible,etat;
  guint max_fr,count;
  GooCanvasItem *rootKF;

  gchararray texte,fonte;
  gdouble x,y,xf,yf,rx,ry,w,h,lw,rota;
  guint fill_color,stroke_color;
  gint fr,fg,fb,sr,sg,sb,fa,sa; 

  char *positionEntree = NULL;
  struct zip *archive;
  struct zip_file *fpz=NULL;
  struct zip_stat sbzip;
  gchar *path_temp_archive,*path_temp_datas;
  gint i,err,nto;
  gchar *p;
  GdkPixbuf *pb;
  GdkPixbufFormat *fmt;
  GError *error=NULL;
  gchar *path_to_bitmap, *path_to_sound, *path_polypath=NULL;
  gint width, height;
  GSList *list=NULL;
  GSList *pile_sons=NULL;
  gint current_kf;
 
	/* effacer tout le contenu */
    efface_curseur_regle();
    efface_curseur_timeline();
    timeline_raz_vars();
    timeline_clear_layers();
    editor_reset_item_counter();
	raz_pile_images(pile_images);
	raz_pile_sound(get_sound_stack_pointer());
    pile_images=NULL;
	/* décompresser l'archive vers le dossier temporaire */
	err=0;
    archive=zip_open(chemin,ZIP_CHECKCONS,&err);
    if(err != 0 || !archive)
    {
        zip_error_to_str(buf, sizeof buf, err, errno);
        fprintf(stderr,"«%s» %d in function «%s» ,err=%d\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,err,buf);
        exit;/* fonction sans retour de valeur !!! */
    }    	
    /* déterminer le nombre de fichiers contenus */
    nto = zip_get_num_files(archive);
	//printf("nbre fichiers:%d\n",nto);
	for (i=0;i<nto;i++)
      {
       zip_stat_index(archive, i, 0, &sbzip);       
       /*pour savoir la taille du fichier et ainsi pouvoir le lire en entier*/
       if(zip_stat(archive, sbzip.name, 0, &sbzip) == -1)
        {
          strcpy(buf,zip_strerror(archive));
          fprintf(stderr,"«%s» %d in function «%s»\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,buf);
          zip_fclose(fpz);
          zip_close(archive);
          exit;
        }
      fpz=zip_fopen(archive,sbzip.name,ZIP_FL_UNCHANGED);
      if(!fpz)
       {
        strcpy(buf,zip_strerror(archive));
        fprintf(stderr,"«%s» %d in function «%s»\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,buf);
        zip_close(archive);
        exit;
       }
       p=g_malloc(sbzip.size*sizeof(gchar)+1);
       assert(p != NULL);
       /*lit le fichier*/
       if(zip_fread(fpz,p,sbzip.size) != sbzip.size)
        {
          fprintf(stderr,"«%s» %d in function «%s»\n«read error ...»\n",__FILE__,__LINE__,__FUNCTION__);
          g_free(p);
          zip_fclose(fpz);
          zip_close(archive);
          exit;
        }
        /* écriture en mode binaire */
		path_temp_datas=g_strdup_printf("%s/.ella/%s",getenv("HOME"),sbzip.name);
        fichier = fopen(path_temp_datas,"w");
        fwrite(p,sizeof(gchar),sbzip.size,fichier);
        fclose(fichier);
        zip_fclose(fpz);
        free(p);
		g_free(path_temp_datas);
      }/* for i */   
    /* lecture terminée, fermeture de l'archive */
    zip_close(archive);  
	

	/* toutes les données sont dans le dossier caché */	
	
    /* ouvrir le fichier de description */
	path_temp_datas=g_strdup_printf("%s/.ella/%s",getenv("HOME"),"descriptor.lst");	
    fichier_ella=ouvrir_fichier_ella_charger(path_temp_datas);
    /* lecture séquentielle ->EOF */
	 pile_sons=get_sound_stack_pointer();
     while (feof(fichier_ella)==0)
      {
      	fgets(buf,LONG_STRING,fichier_ella);		
      	instr=g_strndup (buf,8);
      	if (strlen(buf)>8)
      	    param=g_substr (buf, 8, strlen(buf)-2);
      	/* aiguiller selon l'instruction */  
      	if (g_strcmp0(instr,"[PGEBGR]")==0)
            r= atoi(param);
        if (g_strcmp0(instr,"[PGEBGG]")==0)
            v= atoi(param);
        if (g_strcmp0(instr,"[PGEBGB]")==0)
            b= atoi(param);
        if (g_strcmp0(instr,"[PGEWID]")==0)
		   {
             editor_set_page_width((gdouble)atoi(param));
			 wd=atoi(param);
           }
        if (g_strcmp0(instr,"[PGEHEI]")==0)
           {
             editor_set_page_height((gdouble)atoi(param));
			 ht=atoi(param);
           }
        if (g_strcmp0(instr,"[MOVFRQ]")==0)
             // régler la fréquence du film
             timeline_set_movie_freq(atoi(param),TRUE);
         if (g_strcmp0(instr,"[LYRVIS]")==0)
           {
            val_int= atoi(param);
            if (val_int==0)
              visible=FALSE;
            else
              visible=TRUE;
            // passer la valeur au layer
            timeline_layer_set_visibility(LayerCount,visible);
           }
        if (g_strcmp0(instr,"[LYRNME]")==0)
		  {
            // param contient le nom ; passer la valeur au Layer			
             timeline_layer_set_name(LayerCount,g_strcompress(param));
		  }
        if (g_strcmp0(instr,"[LYRNEW]")==0)
            // initialiser un nouveau layer
            timeline_new_layer();
        if (g_strcmp0(instr,"[LYRSND]")==0)
            // initialiser un nouveau layer
            timeline_new_sound_layer();			
        if (g_strcmp0(instr,"[LYRMXF]")==0)
		   {
            // param contient le nombre de trames actives du layer
            max_fr=atoi(param);
            // modifier le contenu du layer
            timeline_layer_set_actives_frames(LayerCount, max_fr);
           } 
        if (g_strcmp0(instr,"[KEYBLK]")==0)
           {
           	 // image clé vide
           	 val_int= atoi(param);
           	 rootKF=timeline_layer_set_index_keyframes(LayerCount, val_int, ELLA_FRAME_KEY_EMPTY);
           	 if (val_int!=0)
           	     g_object_set(rootKF,"visibility",GOO_CANVAS_ITEM_INVISIBLE,NULL); 
		   }
		if (g_strcmp0(instr,"[KEYSND]")==0)
           {
           	 // sound keyframe, can't support other items than sounds
           	 val_current_sound_index= atoi(param);
           	 rootKF=timeline_layer_set_index_keyframes(LayerCount, val_current_sound_index, ELLA_FRAME_KEY_USED_FOR_SOUND);
           	 if (val_int!=0)
           	     g_object_set(rootKF,"visibility",GOO_CANVAS_ITEM_INVISIBLE,NULL); 
		   }   
		if (g_strcmp0(instr,"[KEYUSD]")==0)
           {
           	 // image clé pleine
           	 val_int= atoi(param);
           	 rootKF=timeline_layer_set_index_keyframes(LayerCount, val_int, ELLA_FRAME_KEY_USED);
           	 if (val_int!=0)
           	    g_object_set(rootKF,"visibility",GOO_CANVAS_ITEM_INVISIBLE,NULL); 
		   }
		if (g_strcmp0(instr,"[KEYMVT]")==0)/* mis en place le 30/12/2009 */
           {
           	 // image clé pleine avec mouvement
           	 val_int= atoi(param);
			 current_kf=val_int;
           	 rootKF=timeline_layer_set_index_keyframes(LayerCount, val_int, ELLA_FRAME_KEY_USED+ELLA_FRAME_MOVEMENT);
			 /* récupérer le pointeur sur la keyframe_list */
			 list=timeline_get_keyframes_list();/* car pointe déjà sur cette valeur ???? */
			 /* mettre à zéro le contenu des transitions */
			 etat=set_keyframe_datas(list, current_kf,"inter",FALSE,0);
			 etat=set_keyframe_datas(list, current_kf,"zoomf",FALSE,0);
			 etat=set_keyframe_datas(list, current_kf,"rotatf",FALSE,0);								
			 etat=set_keyframe_datas(list, current_kf,"alphaf",FALSE,0);	
			 etat=set_keyframe_datas(list, current_kf,"lumif",FALSE,0);	
			 etat=set_keyframe_datas(list, current_kf,"morpf",FALSE,0);	
			 etat=set_keyframe_datas(list, current_kf,"rotatv",FALSE,0);
			 etat=set_keyframe_datas(list, current_kf,"lumiv",FALSE,100);
			 etat=set_keyframe_datas(list, current_kf,"alphav",FALSE,100);
			 etat=set_keyframe_datas(list, current_kf,"zoomv",FALSE,100);			 						
			 etat=set_keyframe_datas(list, current_kf,"morpv",FALSE,0);	
           	 if (val_int!=0)
           	    g_object_set(rootKF,"visibility",GOO_CANVAS_ITEM_INVISIBLE,NULL); 
		   }
		if (g_strcmp0(instr,"[MOTIOF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf, "inter",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"inter",FALSE,0); 	
		   }   
		if (g_strcmp0(instr,"[ZOOMIF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf,"zoomf",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"zoomf",FALSE,0); 	
		   }	
		if (g_strcmp0(instr,"[LUMINF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf, "lumif",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"lumif",FALSE,0); 	
		   }		   	   
		if (g_strcmp0(instr,"[ROTATF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf, "rotatf",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"rotatf",FALSE,0); 	
		   }	
		if (g_strcmp0(instr,"[ALPHAF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf, "alphaf",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"alphaf",FALSE,0); 	
		   }		   	   
		if (g_strcmp0(instr,"[MORPHF]")==0)
           {	
			  val_int= atoi(param);
			  if (val_int==1)
			     etat=set_keyframe_datas(list, current_kf, "morpf",TRUE,0);
			  else  
			     etat=set_keyframe_datas(list, current_kf,"morpf",FALSE,0); 	
		   }   
		if (g_strcmp0(instr,"[ITMSND]")==0)
           {   
			   /* a new sound item */
			   texte=g_strcompress( g_strdup_printf("%s",param));
			   list=timeline_get_keyframes_list();
			   path_temp_archive=g_strdup_printf("%s/.ella/%s",getenv("HOME"),texte);
			   path_to_sound=g_strdup_printf("/tmp/%s",texte);
			  // printf("contenu pile son:%s\n",find_sound_item_on_stack(pile_sons,texte));
			   if (find_sound_item_on_stack(pile_sons,texte)==NULL)
			     {
				   /* we're updating the objects (items) library */	 
				  // printf("ajout de :%s-%s\n",texte, path_to_sound);
			       add_tree_list_biblio(pBiblio,3,texte,path_to_sound,TRUE);
				   pile_sons=add_pile_sound(pile_sons, texte,FALSE);				 
			     }
			   set_liste_keyframes_for_sound(list, val_current_sound_index,texte);
			   /* now we must push on stack the file reference and copy the file to /tmp directory */			  
			   copier_fichier(path_temp_archive,path_to_sound);			   			
			   g_free(path_temp_archive);
			   g_free(path_to_sound);
			   g_free(texte);			   
		   }   
		if (g_strcmp0(instr,"[ITMNEW]")==0)
           {       	 
           	  nom_item=g_strdup_printf("%s",param);
			  if (strncmp("B",nom_item,1)==0)		
               {
			     // printf("bitmap \n");
				  /* pixbuf déjà chargé ? */		   
			      pb=find_item_on_images_stack(pile_images,nom_item);
			      if (pb==NULL)
			        {
					  /* créer le pixbuf */
					  path_to_bitmap=g_strdup_printf("%s/.ella/%s%s",getenv("HOME"),nom_item,".png");
					  /* vérifier contenu et validité du fichier */
	                  fmt=gdk_pixbuf_get_file_info(path_to_bitmap,&width,&height);
					  if (fmt==NULL)
						{
							printf("erreur lecture ! \n");
						}
					  else
						{
							printf("width %d height %d \n",width, height);
						}
					  /* contraindre les dimensions !!! */
					  pb=gdk_pixbuf_new_from_file_at_scale(path_to_bitmap,
                                                         width,height,
                                                         TRUE,&error);
					  pile_images=(GSList *)add_pile_images(pile_images,nom_item, pb);
					  g_free(path_to_bitmap);
				    }/* pixbuf non initialisé */
		        }/* traitement bitmap */			  			  
		   }
		if (g_strcmp0(instr,"[ITMROT]")==0)
           	 rota=atof(param);		   
		if (g_strcmp0(instr,"[ITMLIW]")==0)
           	 lw=atof(param);
		if (g_strcmp0(instr,"[ITMXPS]")==0)
           	 x=atof(param);
		if (g_strcmp0(instr,"[ITMCTX]")==0)
           	 x=atof(param);
		if (g_strcmp0(instr,"[ITMXPE]")==0)
           	 xf=atof(param);
		if (g_strcmp0(instr,"[ITMRDX]")==0)
           	 rx=atof(param);
		if (g_strcmp0(instr,"[ITMYPS]")==0)
           	 y=atof(param);
		if (g_strcmp0(instr,"[ITMYPE]")==0)
           	 yf=atof(param);
		if (g_strcmp0(instr,"[ITMRDY]")==0)
           	 ry=atof(param);
		if (g_strcmp0(instr,"[ITMCTY]")==0)
           	 y=atof(param);
		if (g_strcmp0(instr,"[ITMWID]")==0)
           	 w=atof(param);
		if (g_strcmp0(instr,"[ITMHEI]")==0)
           	 h=atof(param);
		if (g_strcmp0(instr,"[ITMFCR]")==0)
           	 fr=atoi(param);
		if (g_strcmp0(instr,"[ITMFCG]")==0)
           	 fg=atoi(param);
		if (g_strcmp0(instr,"[ITMFCB]")==0)
           	 fb=atoi(param);
		if (g_strcmp0(instr,"[ITMFAL]")==0)
           	 fa=atoi(param);
		if (g_strcmp0(instr,"[ITMSCR]")==0)
           	 sr=atoi(param);
		if (g_strcmp0(instr,"[ITMSCG]")==0)
           	 sg=atoi(param);
		if (g_strcmp0(instr,"[ITMSCB]")==0)
           	 sb=atoi(param);
		if (g_strcmp0(instr,"[ITMSAL]")==0)
           	 sa=atoi(param);
		if (g_strcmp0(instr,"[ITMTXT]")==0)
			texte=g_strcompress( g_strdup_printf("%s",param));
		if (g_strcmp0(instr,"[ITMFNT]")==0)
           	 fonte=g_strdup_printf("%s",param);
		if (g_strcmp0(instr,"[ITMPTH]")==0)
			path_polypath=g_strcompress( g_strdup_printf("%s",param));	 			 
		if (g_strcmp0(instr,"[ITMEND]")==0)
           {
			  fill_color=convert_color_to_rgba ((gdouble)(fr/255.0),
			            (gdouble) (fg/255.0),(gdouble) (fb/255.0),
						(gdouble)(fa/100.0));
			  stroke_color=convert_color_to_rgba ((gdouble)(sr/255.0),
			             (gdouble) (sg/255.0),(gdouble) (sb/255.0),
						 (gdouble)(sa/100.0));			 
           	  add_item(rootKF,nom_item,x,y,w,h,rx,ry,xf,yf,lw,fill_color,stroke_color,
			            texte, fonte,rota,path_polypath);
		      if (path_polypath!=NULL)
			      g_free(path_polypath);				
           	  /* mettre à jour intelligemment le compteur d'items */
           	  editor_add_new_item(nom_item);           	 
		   }
      	g_free(instr);
      	g_free(param);
      	//sortie=TRUE;
	  }/*wend */
    /* fermer le fichier */
    fermer_fichier_ella(fichier_ella);
    raz_pile_sound(get_sound_stack_pointer());
	/* modifier le titre de la fenêtre principale */
	sprintf(buf,"Ella-%s",g_path_get_basename(chemin));
	sprintf(chemin_complet,"%s",chemin);
    gtk_window_set_title (GTK_WINDOW (window1), buf);
	g_free(chemin);
    /* passer la nouvelle couleur de fond */
    editor_set_page_color(r,v,b);
	/* redimensionner la page */
	editor_page_resize((gdouble)wd,(gdouble)ht,(gdouble) dlg_get_page_margins());
    /* afficher le contenu */
    //printf("layercount %d \n",LayerCount);
    for (count=1;count<=LayerCount;count++)
       {
       	 timeline_draw_layer_bg(count);
	   }/* next count */
    /* afficher les curseurs */
    timeline_put_cursors();
    fFileNamed=TRUE;
	fModified=FALSE;
	return 0;  	
}		
/***********************************
 * special callbacks for 
 * recent files menus
 * *******************************/
void
on_recent1_activate                    (GtkMenuItem     *menuitem,
                                       GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,0);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1);
	 }	 
}


void
on_recent2_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,1);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1); 
	 }	
}


void
on_recent3_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,2);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1); 
	 }
}


void
on_recent4_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,3);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1); 
	 }
}


void
on_recent5_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,4);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1); 
	 }
}


void
on_recent6_activate                    (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  GSList *node;
  gchar *path_test;
  gboolean existe;
  gint erreur;
  	
  node=g_slist_nth(recent_files_list,5);

 /* the 'recent file exists ? */
  path_test=g_strdup_printf("%s",(gchar *)(node->data));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {  
		dlg_file_error();
	 }
  else
     {
		cursor_busy(window1); 
		erreur=open_file(window1,path_test); 
		cursor_normal(window1); 
	 }
}
/*********************************
 * fonctions servant à différents
 * appels, soit menus,
 * soit boutons
 * bug résolu le 2/1/2010 :
 * pour une raison inconnue, il
 * a fallu ajouter une pseudo ligne
 * en fin de fichier ([FILEND])
 * *************************************/
gint ouvrir_fichier_ella(GtkWidget *window1)
{
  GtkWidget *dialog_ouvrir; 
  guint reponse;
  	
	/* dlg choix du fichier */
    dialog_ouvrir=create_DialogOpenFile(gettext( "Open ..."),"*.ella",NULL,NULL);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_ouvrir), cb_get_prefered_folder());
    reponse=gtk_dialog_run(GTK_DIALOG(dialog_ouvrir));
    /* récupérer le nom et le tester	*/
    if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dialog_ouvrir);
           return 0;
      }
    /* donc réponse OK */
    chemin = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_ouvrir));
    gtk_widget_destroy(dialog_ouvrir);
	/* save the prefered folder */
	prefered_folder_set(g_path_get_dirname(chemin));
	cursor_busy(window1);
	reponse=open_file(window1,chemin);
	cursor_normal(window1);
	return 0;
}
/*************************************
 * enregistrer et enregistrer/sous
 * dans la même fonction
 * **********************************/
void enregistrer(gchar *chemin)
{
 FILE *fichier_ella;
 gdouble w,h;
 GdkColor cp;
 gchar *path_temp,*path_temp_archive,*chemin_ella,*chemin_biblio_ella;
 struct zip *archive;
 struct zip_source *zs;
 struct zip_file *fpz=NULL;
 struct zip_stat sb;
 gint err=0;
 gchar buf[512];
 gchar *command;
 gint i, nIndex;
 GSList *temp, *pile;
 gint j,erreur;
 GdkPixbuf *pb=NULL;
 gchar *titre,*image_file,*sound_file,*extension,*chemin_sans_extension;
 
 /* récupérer les données utiles */
  w=editor_get_page_width();
  h=editor_get_page_height();
  cp=editor_get_page_color();
  /* création des chemins d'accès complets avec les bonnes extensions */
  /* we test if 'chemin' as already and '.ella' extension */
  extension= g_strrstr(chemin, ".ella");
  if (extension==NULL)
	     {
           chemin_ella=g_strdup_printf("%s%s",chemin,".ella");
		   chemin_biblio_ella=g_strdup_printf("%s%s",chemin,".bibella");
	     }
  else
     { 
	       chemin_sans_extension=file_str_sub_extension (chemin, 0,extension-chemin-1);
	       chemin_ella=g_strdup_printf("%s",chemin);
		   chemin_biblio_ella=g_strdup_printf("%s%s",chemin_sans_extension,".bibella");
	 }
 /* sauvegarde vers le dossier caché, puis transfert vers la vraie cible */
  path_temp=g_strdup_printf("%s/.ella/descriptor.lst",getenv("HOME"));
  path_temp_archive=g_strdup_printf("%s/.ella/ella.zip",getenv("HOME"));
 /* effacer données existantes, car sinon pas de réecriture par dessus !!! */
  /* erase all *.zip files in the hidden directory */
  command = g_strdup_printf("rm %s/.ella/*.*",getenv("HOME"));
  err = system(command);
  g_free(command); 
 /* d'abord le descripteur */
  fichier_ella=ouvrir_fichier_ella_sauver(path_temp);
  sauver_en_tete_ella(fichier_ella, 
                      (guint)w, (guint)h, 
                      5, 
                      timeline_get_movie_freq(),
                      cp);
  sauver_donnees_ella(fichier_ella, pListViewLayers, LayerCount, MAX_FRAMES);
  fermer_fichier_ella(fichier_ella);
  /* zipper */
  err=0;
  archive=zip_open(path_temp_archive,ZIP_CREATE,&err);/* avec ce mode, le fichier est écrasé ! */
  /* par contre, pour éviter de récréer les paths absolus, il suffit de juste mettre le nom, sans chemin, 
    dans zip_add()  */
  if(err != 0 || !archive)
    {
        zip_error_to_str(buf, sizeof buf, err, errno);
        fprintf(stderr,"«%s» %d in function «%s» ,err=%d\n«%s»\n",__FILE__,__LINE__,__FUNCTION__,err,buf);
        exit(EXIT_FAILURE);
    }   
   zs = zip_source_file(archive,path_temp,0,0); /* lit par défaut tout le fichier */
   if (zs==NULL)
    {
     printf("erreur source \n");
     return;
    } 
  /* ajouter à l'archive le fichier texte de description */ 
  err = zip_add(archive,"descriptor.lst",zs);
  /* recommencer pour transférer tous les fichiers bitmaps */
  j=0;
  pile=editor_get_images_stack();
  /* début boucle */
  temp = g_slist_nth (pile, j);				  
  while (temp!=NULL)
	  { 		
			pb=((stack_images *)(temp->data))->pixb;   
			titre=g_strdup_printf("%s",((stack_images *)(temp->data))->nom); 
			/* exporter le pixbuf vers le dossier caché */  
		    image_file=g_strdup_printf("%s/.ella/%s%s",getenv("HOME"),titre,".png");	    
		    erreur = gdk_pixbuf_save (pb, image_file, "png", NULL,
                                    "quality", "90", NULL);  
			 /* crée une sorte de lien symbolique vers le VRAI emplacement */						
			 zs = zip_source_file(archive,image_file,0,0); /* lit par défaut tout le fichier */
             if (zs==NULL)
              {
                 printf("erreur source \n");
				 return;
			  } 
			/* ne met dans l'archive que le nom_fichier du lien symbolique ci-dessus */  
			err = zip_add(archive,g_path_get_basename(image_file),zs);						 
			g_free(titre);
			g_free(image_file);  	  
		    j++;
		    temp = g_slist_nth (pile, j);
	  }/* wend */
  /* now, we're adding to archive all sound files, which are store in /tmp directory */
  pile=get_sound_stack_pointer();
  j=0;
  temp = g_slist_nth (pile, j);
  while (temp!=NULL)
     {
		 titre=g_strdup_printf("%s",((stack_sound *)(temp->data))->snd_file); 
		 sound_file=g_strdup_printf("/tmp/%s",titre);
		 /* virtual link to zip archive */
		 zs = zip_source_file(archive,sound_file,0,0); /* lit par défaut tout le fichier */
		 if (zs==NULL)
              {
                 printf("erreur source \n");
				 return;
			  } 
		 err = zip_add(archive,g_path_get_basename(sound_file),zs);	  
		 g_free(titre);
		 g_free(sound_file);
		 j++;
		 temp = g_slist_nth (pile, j);
	 }	  
  /* utilisation terminée, fermeture de l'archive */
  zip_close(archive); 
  /* transférer vers la cible réelle */
  copier_fichier(path_temp_archive,chemin_ella);
  /* faut-il archiver la biblioautomatiquement ? */
  if (auto_archive_biblio)
     {
         biblio_save_biblio(chemin_biblio_ella,GTK_TREE_VIEW(pListViewBiblio));
	 }
  else
     {
       g_free(chemin_biblio_ella);/* car freed dans la fonction biblio_save !! */
     }
  fModified=FALSE;  
  fFileNamed=TRUE;
  sprintf(chemin_complet,"%s",chemin);
  g_free(path_temp);
  g_free(path_temp_archive);
  g_free(chemin_ella);
}
/*************************************************** 
 * enregistrer_sous pouvant être appelé de partout 
 * ************************************************/
gboolean enregistrer_ella(GtkWidget *window1)
{
 FILE *fichier_ella;
 gdouble w,h;
 GdkColor cp;
 gint reponse;
 GtkWidget *dialog_save;
 gchar buf[512];
 gchar *nom,*chemin_sans_extension;
 gchar *extension;
 
 /* dialogue sauvegarde */
 dialog_save = create_file_save_dialog ();   
 gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_save), cb_get_prefered_folder());
 gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog_save), gettext( "noname.ella"));
 /* penser aux filtres */
 gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog_save),TRUE);/* contrôle overwrite */
 reponse=gtk_dialog_run(GTK_DIALOG(dialog_save)); 
 /* traitement de la réponse */
   if (reponse==GTK_RESPONSE_OK) 
    {
	  /* attention, faire un test pour ajouter l'extension ! */	
	  chemin = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_save));
	  /* update recent files list */
	  recent_files_list=cb_update_recent_files_list(recent_files_list, g_strdup_printf("%s",chemin));
	  /* save the prefered folder */
	  prefered_folder_set(g_path_get_dirname(chemin));
	  extension= g_strrstr(chemin, ".ella");
	  if (extension==NULL)
	     {
	  		printf("chemin directement sans extension :%s;\n",chemin);
			chemin_sans_extension=g_strdup_printf("%s",chemin);
		 }
	  else
	     {
			 /* on retire l'extension */
			 chemin_sans_extension=file_str_sub_extension (chemin, 0,extension-chemin-1);
			 printf("chemin sans extension après nettoyage=%s<<\n",chemin_sans_extension);
		 }
	  gtk_widget_destroy(dialog_save);
	  cursor_busy(window1);
      enregistrer(chemin_sans_extension);
	  cursor_normal(window1);
	  /* modifier le titre de la fenêtre principale */
	  nom=g_path_get_basename(chemin);
	  sprintf(buf,"Ella-%s",nom);
	  sprintf(chemin_complet,"%s",chemin_sans_extension);	  
      gtk_window_set_title (GTK_WINDOW (window1), buf);
	  g_free(nom);
	  g_free(chemin);
	  g_free(chemin_sans_extension);
	  fModified=FALSE;
	  fFileNamed=TRUE;
	  return TRUE;
   }/* if OK */
  else
   {
	   gtk_widget_destroy(dialog_save);
	   return FALSE;
   }  
}
/**********************************
 * nouveau projet ?
 * *******************************/
void nouveau(GtkWidget *window1)
{
  gboolean retour;
  GtkWidget *dlg;
  gint reponse;
  GdkColor pPgColor;
  
 /* fichier modifié, mais pas enregistré ? */
 if (((fModified)&&(!fFileNamed))||(editor_get_modified()))
   {
	  /* dialogue */
	  dlg=create_dialogAskBeforeNew ();
	  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
	  if (reponse==GTK_RESPONSE_OK)
		  {
			gtk_widget_destroy(dlg);
	        retour=enregistrer_ella(window1); 
	        if (!retour)
	        return;
		  }
     if (reponse==GTK_RESPONSE_CANCEL)
	    {
			gtk_widget_destroy(dlg);
			return;
		}    
     if (reponse==GTK_RESPONSE_REJECT)
	    {
			gtk_widget_destroy(dlg);
		}             
   }
 
 /* effacer tout le contenu */
 efface_curseur_regle();
 efface_curseur_timeline();
 timeline_raz_vars();
 timeline_clear_layers();
 editor_reset_item_counter();

 editor_set_modified(FALSE);

 /* mettre en blanc la fond d'écran */
 editor_wipe_page();
 pPgColor=editor_get_page_color();
 editor_set_page_color(pPgColor.red,pPgColor.green, pPgColor.blue);
 fModified=FALSE;
 fFileNamed=FALSE;	
 timeline_re_init(rootEditorItems);
 // déjà fait au dessus !raz_pile_images(pile_images);
 pile_images=NULL;
 gtk_window_set_title (GTK_WINDOW (window1), gettext( "Ella-noname.ella"));
}
void
 on_nouveau1_activate                   (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
 {
   nouveau(window1); 
 }


void on_ouvrir1_activate(GtkMenuItem  *menuitem, GtkWidget *window1)
{
  gint erreur;
  
	erreur=ouvrir_fichier_ella(window1);
}


void
on_enregistrer1_activate               (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
 gboolean retour;
 /* tester si le fichier a un nom */
 if (fFileNamed)
   {
	   cursor_busy(window1);
	   enregistrer(chemin_complet);
	   cursor_normal(window1);
   }
 else
   {
	   retour=enregistrer_ella(window1);
   }
}


void
on_enregistrer_sous1_activate          (GtkMenuItem     *menuitem,
                                        GtkWidget *window1)
{
  gboolean retour;
  
  retour=enregistrer_ella(window1);
}


void
on_importer1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/********************************************
 *  importer image dans la bibliothèque 
 * *****************************************/
void
on_une_image_bitmap1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext( "Import a bitmap image"),1);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   path_to_file_to_import = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   /* transférer fichier vers répertoire temporaire */
   short_name=(gchar *)g_basename(path_to_file_to_import);
   path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
   copier_fichier(path_to_file_to_import,path_target);
   /* reste à mettre à jour la future list_store ! */
   add_tree_list_biblio(pBiblio,1,short_name,path_target,FALSE);
   //g_free(path_to_file_to_import); ne pas faire free
   //g_free(path_to_temp_dir);nepas faire free
   g_free(path_target);
   //g_free(short_name); ne pas faire free
}
/********************************/
/**** importer une image svg ****/
/********************************/
void
on_une_image_vectorielle1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext("Import a SVG file" ),2);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   path_to_file_to_import = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   /* transférer fichier vers répertoire temporaire */
   short_name=(gchar *)g_basename(path_to_file_to_import);
   path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
   copier_fichier(path_to_file_to_import,path_target);
   /* reste à mettre à jour la future list_store ! */
   add_tree_list_biblio(pBiblio,2,short_name,path_target,FALSE);
   //g_free(path_to_file_to_import); ne pas faire free
   //g_free(path_to_temp_dir);nepas faire free
   g_free(path_target);
   //g_free(short_name); ne pas faire free
}
/***********************************
 * import a 'symbol', in fact a
 * SVG file which will be 
 * recoded
 * ********************************/
void
on_import_symbol_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext("Import a Symbol..." ),4);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   path_to_file_to_import = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   /* transférer fichier vers répertoire temporaire */
   short_name=(gchar *)g_basename(path_to_file_to_import);
   path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
   copier_fichier(path_to_file_to_import,path_target);
   /* reste à mettre à jour la future list_store ! */
   add_tree_list_biblio(pBiblio,4,short_name,path_target,FALSE); 
   g_free(path_target);
}										
/*******************************
 * importer un son 
 * ****************************/
void on_importer_son_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse;
  GtkWidget *dlg;
  gchar *path_to_file_to_import;
  gchar *path_to_temp_dir;
  gchar *short_name;
  gchar *path_target;
  
  /* dlg choix du fichier */
  dlg=dlg_create_dialogImporter (gettext( "Import a sound or an image"),3);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg), getenv("HOME"));
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
   /* donc réponse OK */
   path_to_file_to_import = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dlg));
   gtk_widget_destroy(dlg);
   path_to_temp_dir= (gchar *)g_get_tmp_dir();
   /* transférer fichier vers répertoire temporaire */
   short_name=(gchar *)g_basename(path_to_file_to_import);
   path_target=g_strdup_printf("%s/%s",path_to_temp_dir,short_name);
   copier_fichier(path_to_file_to_import,path_target);
   /* reste à mettre à jour la future list_store ! */
   add_tree_list_biblio(pBiblio,3,short_name,path_target,FALSE);
   //g_free(path_to_file_to_import); ne pas faire free
   //g_free(path_to_temp_dir);nepas faire free
   g_free(path_target);
   //g_free(short_name); ne pas faire free	
}										

void
on_exporter1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
/*********************************
 * exporter une animation au
 * format flash
 * ******************************/
void on_animation_flash1_activate (GtkMenuItem     *menuitem,
                                   GtkWidget *win)
{
 gdouble w,h;
 GdkColor cp;
 gchar temp_file[512];
 gchar buf[512];
 gchar buf2[512];
 gchar *command;
 FILE *fich_swf;
 gint reponse,erreur;
 GtkWidget *dialog_export;
 gchar *extension,*chemin_sans_extension;
 
 /* dialogue pour obtenir le nom du fichier */
 dialog_export = create_DialogExportFile (gettext( "Export to Flash-SWF ..."),"*.swf",NULL,NULL);  
 gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_export), cb_get_prefered_folder());
 reponse=gtk_dialog_run(GTK_DIALOG(dialog_export));
 /* récupérer le nom et le tester	*/
 if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dialog_export);
           return;
      }
 /* donc réponse OK */
 cursor_busy(win);	
 chemin_export = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_export));
 gtk_widget_destroy(dialog_export);										
 /* correct the path, by removing extension if exists */
 extension= g_strrstr(chemin_export, ".swf");
 if (extension==NULL)
	     {
			chemin_sans_extension=g_strdup_printf("%s",chemin_export);
		 }
	  else
	     {
			 /* on retire l'extension */
			 chemin_sans_extension=file_str_sub_extension (chemin_export, 0,extension-chemin_export-1);
		 } 										
 /* récupérer les données utiles */
 w=editor_get_page_width();
 h=editor_get_page_height();
 cp=editor_get_page_color();

/* compilation */
 sprintf(buf,"%s/.ella/temp.sc",getenv("HOME"));
 sprintf(buf2,"%s/.ella/temp.swf",getenv("HOME"));
 fich_swf=ouvrir_fichier_swf_export(buf,win);
 exporter_en_tete_swf(fich_swf,
                      buf2, 
                      (guint)w, (guint)h, 
                      5, 
                      timeline_get_movie_freq(),
                      cp,0,0);
 /* on va parcourir  pListViewLayers */
 /*  gint LayerCount=0; /* nombre total de calques en cours, comptage humain */
 exporter_calques_vers_swf(fich_swf,
                       pListViewLayers,
					   timeline_get_layers_count(),
					   MAX_FRAMES,
					   editor_get_images_stack());

 fermer_fichier_swf_export(fich_swf);
 /* exécuter le compilateur */
 sprintf(buf,"swfc %s/.ella/temp.sc",getenv("HOME"));
 erreur = system(buf);
 if(erreur == 32512)
 {
        GtkWidget * dialog = gtk_message_dialog_new (win,
                GTK_DIALOG_DESTROY_WITH_PARENT,
                GTK_MESSAGE_ERROR,
                GTK_BUTTONS_CLOSE,
                "swfc not found, you need to install it to export in Flash.");
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
 }
 copier_fichier(buf2,g_strdup_printf("%s%s",chemin_sans_extension,".swf"));
 cursor_normal(win);
}
/*****************************************
 * export the animation as a SVG file
 * **************************************/

void
on_animation_svg1_activate             (GtkMenuItem     *menuitem,
                                        GtkWidget *win)
{
 gint reponse,erreur;
 GtkWidget *dialog_export;	
 gchar *nom_svg;
 gdouble w,h;
 GdkColor cp;
 FILE *fich_svg,*fich_js;
 gchar *extension,*chemin_sans_extension;
  
 /* dialogue pour obtenir le nom du fichier */
 dialog_export = create_DialogExportFile (gettext( "Export to SVG..."),"*.svg",NULL,NULL);  
 gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_export), cb_get_prefered_folder());
 reponse=gtk_dialog_run(GTK_DIALOG(dialog_export));
 /* récupérer le nom et le tester	*/
 if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dialog_export);
           return;
      }
    /* donc réponse OK */		
 cursor_busy(win);	
 chemin_export = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_export));
 gtk_widget_destroy(dialog_export);																				
 /* correct the path, by removing extension if exists */
 extension= g_strrstr(chemin_export, ".svg");
 if (extension==NULL)
	     {
	  		chemin_sans_extension=g_strdup_printf("%s",chemin_export);
		 }
	  else
	     {	 /* on retire l'extension */
			 chemin_sans_extension=file_str_sub_extension (chemin_export, 0,extension-chemin_export-1);
		 } 	
 
 /* récupérer les données utiles */	
 w=editor_get_page_width();
 h=editor_get_page_height();
 cp=editor_get_page_color();
  
 nom_svg=g_strdup_printf("%s%s",chemin_sans_extension,".svg");
 fich_svg=svg_open_file_to_export(nom_svg);
 svg_put_header_to_file(fich_svg, w, h, editor_get_page_margins(), editor_get_page_margins());
 svg_put_background(fich_svg, w, h, cp);
 svg_put_movie(fich_svg,pListViewLayers,
					   timeline_get_layers_count(),
					   MAX_FRAMES,
					   timeline_get_movie_freq(),
					   editor_get_images_stack());
 svg_close_file_to_export(fich_svg);
 g_free(nom_svg);
 cursor_normal(win);
}
/***************************************
 * exportation en JavaScript
 * ************************************/
void
on_comme_animation_javascript1_activate
                                        (GtkMenuItem     *menuitem,
                                        GtkWidget *win)
{
 gint reponse,erreur;
 GtkWidget *dialog_export;	
 gchar *nom_de_base,*nom_html,*nom_js;
 gdouble w,h;
 GdkColor cp;
 FILE *fich_html,*fich_js;
 gchar *extension,*chemin_sans_extension;
  
 /* dialogue pour obtenir le nom du fichier */
 dialog_export = create_DialogExportFile (gettext( "Export to JavaScript..."),"*.js",NULL,NULL);  
 gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_export), cb_get_prefered_folder());
 reponse=gtk_dialog_run(GTK_DIALOG(dialog_export));
 /* récupérer le nom et le tester	*/
 if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dialog_export);
           return;
      }
    /* donc réponse OK */	
 cursor_busy(win);		
 chemin_export = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog_export));
 gtk_widget_destroy(dialog_export);																				
 /* récuperer le nom principal */
 /* correct the path, by removing extension if exists */
 extension= g_strrstr(chemin_export, ".html");
 if (extension==NULL)
	     {
	  		chemin_sans_extension=g_strdup_printf("%s",chemin_export);
		 }
	  else
	     {	 /* on retire l'extension */
			 chemin_sans_extension=file_str_sub_extension (chemin_export, 0,extension-chemin_export-1);
		 }

 nom_html=g_strdup_printf("%s%s",chemin_sans_extension,".html");
 printf("choisi html:%s\n",nom_html); 
 nom_js=g_strdup_printf("%s%s",chemin_sans_extension,".js");
  printf("choisi js:%s\n",nom_js); 
 /* récupérer les données utiles */	
 w=editor_get_page_width();
 h=editor_get_page_height();
 cp=editor_get_page_color();
 
 printf("----------------------------------------- EXPORT JS --------------\n"); 	
 /* écriture de l'html */
 fich_html=fopen(nom_html,"w");
 fprintf(fich_html,"<!DOCTYPE html>\n");
 fprintf(fich_html,"<html> \n");
 fprintf(fich_html,"  <head> \n");
 fprintf(fich_html,"    <meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\" /> \n");
 fprintf(fich_html,"    <title>%s</title>\n",nom_de_base);
 fprintf(fich_html,"  </head> \n");
 fprintf(fich_html,"   \n");
 fprintf(fich_html,"  <body>\n");
 fprintf(fich_html,"      <canvas id=\"%s\" width=\"%d\" height=\"%d\"></canvas>\n",nom_de_base,(gint)w,(gint)h);
 fprintf(fich_html,"      <div id=\"alert_message\"></div>\n");
 fprintf(fich_html,"    <script type=\"text/javascript\" src=\"%s.js\" onload=\"main_loop()\">Sorry, you must have the javascript enabled to see this animation.</script> \n",nom_de_base);
 fprintf(fich_html,"  </body>\n");
 fprintf(fich_html,"</html>\n");
 fclose(fich_html); 
 /* écriture du JS */
 fich_js=fopen(nom_js,"w");
 fprintf(fich_js,"textOnFail = \"Sorry, your web browser isn't able to read this animation. Please try to download <a href='http://mozilla.org/firefox/'>Firefox</a>, a free, open source and standart-compliant browser.\";\n");
 fprintf(fich_js,"canvas = document.getElementById('%s');\n",nom_de_base);
 fprintf(fich_js,"if (canvas.getContext){\n");
 fprintf(fich_js,"    ctx = canvas.getContext('2d');\n");
 fprintf(fich_js,"}else{\n");
 fprintf(fich_js,"    document.getElementById('alert_message').innerHTML = textOnFail;\n");
 fprintf(fich_js,"}\n  \n");
 fprintf(fich_js,"function main_loop (){\n");

 fprintf(fich_js,"        ctx.fillStyle= \"rgb(%d,%d,%d)\";\n",(gint)((cp.red/65535.0)*255.0),
                                                           (gint)((cp.green/65535.0)*255.0),
														   (gint)((cp.blue/65535.0)*255.0));
 fprintf(fich_js,"        ctx.fillRect(0,0,%d,%d);\n",(gint)w,(gint)h);
 fprintf(fich_js,"    frame++;\n");
 fprintf(fich_js,"    setTimeOut('main_loop', %d/%d);\n",MAX_FRAMES,timeline_get_movie_freq());
 fprintf(fich_js,"}\n");
 fprintf(fich_js,"  \n");
 fclose(fich_js); 
 /* nettoyage mémoire */
 g_free(nom_de_base);
 g_free(nom_html);
 g_free(nom_js);	
 cursor_normal(win);					
}										

/********************************
 * options par défaut
 * ****************************/
void
on_options1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 gint reponse,z;
 GtkWidget *dlg;
 GdkColor pcolor;
 
 /* récupérer les valeurs par défaut, pas les valeurs courantes */
 z=timeline_get_zooming();
 switch (z)
   {
	   case 100:{z=0;break;}
	   case 75:{z=1;break;}
	   case 50:{z=2;break;}
   }
 dlg=create_dialogPrefs ((gint)editor_get_default_page_width(),
                         (gint)editor_get_default_page_height(),
						 (gint)editor_get_default_margins(),
						 timeline_get_default_movie_freq(),
						 512,
						 editor_get_default_page_color(),
						 auto_archive_biblio,
						 z);
 reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
 gtk_widget_destroy(GTK_WIDGET(dlg));
 if (reponse==GTK_RESPONSE_OK)
    {
		/* passer les variables par défaut ; quelle lourdeur pour éviter
		 * des variables globales !*/
	    editor_set_default_page_width(dlg_get_default_width());
		editor_set_default_page_height(dlg_get_default_height());
		editor_set_default_margins(dlg_get_default_margins());
		editor_set_default_page_color(dlg_get_default_page_color().red,
		                              dlg_get_default_page_color().green,
									  dlg_get_default_page_color().blue);
		timeline_set_default_movie_freq(dlg_get_default_fps());
		auto_archive_biblio=dlg_get_biblio_autosave();
		/* reste à faire le piétage ! */
	}
}
/****************************************
 * quitter Ella
****************************************/
void
on_quitter1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{
 gint x,y,w,h;
 gint *xp,*yp,*wp,*hp;
 GtkWidget *window;
 
 
 window=user_data[0];
 /* get the current window geometry */
 gtk_window_get_position(GTK_WINDOW(window),&x,&y);
 gtk_window_get_size(GTK_WINDOW(window),&w,&h);
 /* store the values */
 xp=user_data[1];
 yp=user_data[2]; 
 wp=user_data[3];
 hp=user_data[4];
 *xp=x;
 *yp=y;
 *wp=w;
 *hp=h;
 if (dialog_ask_before_quit()==GTK_RESPONSE_OK)	
     gtk_main_quit();
}
/**********************************
 * clipboard
 * *******************************/
/* gestion de l'activabilité des sous-menus */
void on_edition_activate(GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{
	/* récupérer les variables multiples */
	GtkWidget  *menu_copier;
    GtkWidget  *menu_coller;
    GtkWidget  *menu_couper;
    GtkWidget  *menu_annuler;
	gint etat_clipboard, etat_annul;
	gboolean existe_selection_editeur=FALSE;
	gboolean existe_selection_timeline=FALSE;/* todo */
	
	menu_copier=user_data[0];
	menu_coller=user_data[1];
	menu_couper=user_data[2];
	menu_annuler=user_data[3];
	
	/* activer ou non les menus */
	etat_clipboard=clipboard_get_atom();
	existe_selection_editeur= editor_get_selection_state();
	if (etat_clipboard!=CLIP_EMPTY)
	    {
			gtk_widget_set_sensitive(menu_coller,TRUE);
			gtk_widget_set_sensitive(menu_copier,existe_selection_editeur);
			gtk_widget_set_sensitive(menu_couper,existe_selection_editeur);
			//gtk_widget_set_sensitive(menu_annuler,FALSE);
		}
    else
	    {
			gtk_widget_set_sensitive(menu_coller,FALSE);
			gtk_widget_set_sensitive(menu_copier,existe_selection_editeur);
			gtk_widget_set_sensitive(menu_couper,existe_selection_editeur);
			//gtk_widget_set_sensitive(menu_annuler,FALSE);
		}
}
/* copier */
void
on_copier1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_copy_items();
}

/* coller */
void
on_coller1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_paste_items();
}

/* couper */
void
on_couper1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_cut_items();
}

/****************************
 * Undo the last editor's
 * operation
 * *************************/
void
on_annuler1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   undo_operation();
}


void
on_calques1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/*** nouveau calque de dessin ***/
void
on_nouveau_calque1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  timeline_new_layer_from_menu();
}
/**********************************
 * new sound layer
 * ******************************/
void
on_nouveau_calque_sonore1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	timeline_new_sound_layer_from_menu();
}										
/* dupliquer un calque */
void
on_dupliquer_calque1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 timeline_duplicate_layer();
}

/**** renommer un calque par dialogue ****/
void
on_renommer_calque1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint reponse,layer;
  GtkWidget *dlg, *entree;
  gchar *nom_actuel,*nouveau_nom;
  	  
  entree= gtk_entry_new ();
  /* accéder à la timeline pour obtenir le nom actuel du calque courant */
  layer=timeline_get_current_layer()-1;
  nom_actuel=timeline_layer_get_name(layer+1);
  dlg=create_dialogRenameLayer (nom_actuel,entree);
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  /* récupérer le nom et le tester	*/
  if (reponse!=GTK_RESPONSE_OK)
      {
           gtk_widget_destroy(dlg);
           return;
      }
  /* lire la valeur */
  nouveau_nom=(gchar *)gtk_entry_get_text(GTK_ENTRY(entree));
  /* tranmettre le nouveau nom */   
  timeline_layer_set_name(layer+1,nouveau_nom); 
  gtk_widget_destroy(dlg);
}

/* monter le calque */
void
on_remonter_le_calque_sur_la_pile1_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 timeline_layer_up();
}

/* descendre le calque */
void
on_descendre_le_calque_sur_la_pile1_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 timeline_layer_down();
}
/***** effacer calque ****/
void
on_supprimer_calque1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 timeline_delete_layer_from_menu();
}

/******************************
 * dupliquer la sélection ;
 * consiste à copier/coller à 
 * la même place
 * ***************************/
void
on_dupliquer1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   editor_duplicate();
}

/*** monter un objet au sommet de la pile ***/
void
on_monter_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   editor_raise(pSelectedItemsList);
}

/*** descendre un objet en bas de la pile ***/
void
on_descendre_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_lower(pSelectedItemsList);
}

/* rotations arbitraires */

void
on_rotation1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
void
on_90sens_horaire1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   editor_rotate(90);	
}
void
on_90_anti_horaire1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   editor_rotate(-90);	
}
void
on_180_sens_horaire1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	   editor_rotate(180);
}	
void
on_270_sens_horaire1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	   editor_rotate(270);
}	   											
/* sélectionner tous les objets */
void
on_tout_selectionner1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
     editor_select_all();
}
/* annuler la sélection dans l'éditeur */
void
on_deselectionner_tout1_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    editor_unselect();
}
/* suppression rapide d'un objet */
void
on_supprimer1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_delete_items();
}

/********* fin fonctions édition objets *************/
/* fonctions liées aux trames : même astuce : le pré'appel du menu
 * va permettre, par passage de variables multiples, d'interdire
 * certaines fonctions */
void
on_trames1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{
	guint cell;
    /* récupérer les variables multiples */
	GtkWidget  *menu_add_frames_to_cursor;
    GtkWidget  *menu_add_frames_from_cursor;
	GtkWidget *menu_remove_frames_to_cursor;
	GtkWidget *menu_remove_frames_from_cursor;
	GtkWidget *menu_new_keyframe;
	GtkWidget *menu_new_blank_keyframe;
	GtkWidget *menu_new_tween;
	GtkWidget *menu_remove_keyframe;
	GtkWidget *menu_set_keyframe_to_blank;
	GtkWidget *menu_remove_interpola;
	
    
	menu_add_frames_to_cursor=user_data[0];
	menu_add_frames_from_cursor=user_data[1];
	menu_remove_frames_to_cursor=user_data[2];
	menu_remove_frames_from_cursor=user_data[3];
	menu_new_keyframe=user_data[4];
	menu_new_blank_keyframe=user_data[5];
	menu_new_tween=user_data[6];
	menu_remove_keyframe=user_data[7];
	menu_set_keyframe_to_blank=user_data[8];
	menu_remove_interpola=user_data[9];
	
	cell=timeline_frame_cell_get();
	/* activer ou non les menus */
	gtk_widget_set_sensitive(menu_new_tween,FALSE);
	gtk_widget_set_sensitive(menu_remove_keyframe,FALSE);
	gtk_widget_set_sensitive(menu_set_keyframe_to_blank,FALSE);
	gtk_widget_set_sensitive(menu_remove_interpola,FALSE);
	/* si curseur sur image-clé */
	if (cell>=ELLA_FRAME_KEY_USED)
	   {
		  gtk_widget_set_sensitive(menu_new_tween,TRUE); 
		  gtk_widget_set_sensitive(menu_remove_keyframe,TRUE);
		  gtk_widget_set_sensitive(menu_set_keyframe_to_blank,TRUE);
	   }
	if (cell>=ELLA_FRAME_MOVEMENT)
	   {
		  gtk_widget_set_sensitive(menu_remove_interpola,TRUE); 
	   }
	if (cell>ELLA_FRAME_NON_USED)
	   {
		  gtk_widget_set_sensitive(menu_add_frames_to_cursor,TRUE); 
		  gtk_widget_set_sensitive(menu_add_frames_from_cursor,TRUE);
		  gtk_widget_set_sensitive(menu_remove_frames_to_cursor,TRUE);
		  gtk_widget_set_sensitive(menu_remove_frames_from_cursor,TRUE);
		  gtk_widget_set_sensitive(menu_new_keyframe,TRUE);
		  gtk_widget_set_sensitive(menu_new_blank_keyframe,TRUE);
	   }
	else
	   {
		  gtk_widget_set_sensitive(menu_add_frames_to_cursor,TRUE); 
		  gtk_widget_set_sensitive(menu_add_frames_from_cursor,FALSE);
		  gtk_widget_set_sensitive(menu_remove_frames_to_cursor,FALSE);
		  gtk_widget_set_sensitive(menu_remove_frames_from_cursor,FALSE);
		  gtk_widget_set_sensitive(menu_new_keyframe,FALSE);
		  gtk_widget_set_sensitive(menu_new_blank_keyframe,FALSE);
	   }	
}
/********************************
 *  insérer jusqu'à la position 
 * du curseur 
 * *****************************/
void
on_inserer_trames_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   timeline_add_keyframes_to_cursor();
}
/****************************************************
/* ajouter des images APRES la position du curseur 
 * 
 * ATTENTION : interdire cette fonction si l'image
 * courante n'est pas encore initialisée
 * *************************************************/
void
on_ajouter_apres_la_position_courante1_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse, n_images,valeur;

  dlg=create_dialogAddFramesAfterCursor (timeline_get_cursor_xpos()-1, MAX_FRAMES,&valeur);
  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
  /* récupérer le nombre d'images demandées */
  gtk_widget_destroy(dlg);
  if (reponse==GTK_RESPONSE_OK)
    {
       printf("valeur saisie =%d\n",valeur);
	   if (valeur<0)
	       valeur=0;		   
	   timeline_add_keyframes_from_cursor(valeur);
	}  
}
/***********************************************
/* supprimer jusqu'au curseur 
 * ********************************************/
void
on_supprimer2_activate(GtkMenuItem *menuitem,gpointer         user_data)
{
   timeline_remove_selected_frames(1,
                                   timeline_get_cursor_ypos(),
								   timeline_get_cursor_xpos(),
								   timeline_get_cursor_ypos());
}
/***********************************************
/* supprimer depuis le curseur 
 * ********************************************/
void
on_supprimer_apres_la_position_courante1_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  timeline_remove_selected_frames(timeline_get_cursor_xpos(),
                                  timeline_get_cursor_ypos(),
								  MAX_FRAMES,
								  timeline_get_cursor_ypos());
}
/***********************************************
/* supprimer les trames sélectionnées
 * ********************************************/
void
on_supprimer_les_trames_selectionnees1_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   timeline_remove_selection();
}

/************************************
 *  créer une nouvelle image-clé 
 * *********************************/
void
on_poser_une_image_cle1_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  timeline_new_keyframe();
}

/***********************************
 *  retirer une image-clé 
 * ********************************/
void
on_retirer_une_image_cle1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   timeline_remove_keyframe();
}
/***********************************
 *  poser une image-clé vide 
 * ********************************/
void
on_poser_une_image_cle_vide1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   timeline_new_empty_keyframe();
}

/**********************************
 *  créer une interpolation 
 * ********************************/
void
on_creer_une_interpolation_de_mouvement1_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   timeline_define_interpolation();
}
/***********************************
 * mettre à blanc une iamge-clé
************************************/
void
on_mettre_a_blanc_image_cle1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  timeline_set_keyframe_to_blank();
}
/***********************************
 * supprimer une interpolation
 * *******************************/
void on_remove_interpola_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	timeline_remove_interpolation();
}										
										
/***** taille du canevas *****/

void
on_taille1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse,w,h,m;
  
  dlg=create_dialogPageSize ((gint)editor_get_page_width(),
                             (gint)editor_get_page_height(),
							 (gint)editor_get_page_margins());
  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
  gtk_widget_destroy(dlg);
  if (reponse==GTK_RESPONSE_OK)
    {
		w=dlg_get_page_width();
		editor_set_page_width((gdouble)w);
		h=dlg_get_page_height();
		editor_set_page_height((gdouble)h);
		m=dlg_get_page_margins();
		printf("tailles w=%d h=%d \n",(gint)w,(gint)h);
		editor_page_resize((gdouble) w,(gdouble) h, (gdouble) m);
	}
}

/**** choix de la couleur de page ****/
void
on_couleur_de_la_page1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editor_set_page_color_menu();
}

/*************************************
 * change the movie footage
 * **********************************/
void
on_format_animation1_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse;
 
  dlg=create_dialogFootage ();
  reponse=gtk_dialog_run(GTK_DIALOG(dlg));
  gtk_widget_destroy(dlg);   
}


void
on_visualisation_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
/****************************
 * jouer une animation
 * *************************/

void
on_jouer_animation1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         *user_data)
{	
 GtkWidget *nbook;
 GtkWidget *win;
 GtkWidget *label;
 GtkWidget *play_key;
 gulong duree;
 
 nbook=user_data[0];
 win=user_data[1];
 label=user_data[2];
 play_key=user_data[3];
  
 printf("nbre page du nb:%d\n",gtk_notebook_get_n_pages(GTK_NOTEBOOK(nbook)));
 printf("page courante:%d\n", gtk_notebook_get_current_page (GTK_NOTEBOOK(nbook)));
 if (gtk_notebook_get_current_page (GTK_NOTEBOOK(nbook))!=1)
   {
	    gtk_notebook_set_current_page (GTK_NOTEBOOK(nbook), 1);
   }
 /* lancer la lecture */
 duree=player_build_preview(win);	
 preview_animation(win, duree,label,play_key,timeline_get_root_ruler(),timeline_get_zooming(),timeline_get_movie_freq());
}
/*******************************
/* zooming avant sur timeline 
 * ****************************/
void
on_editeur_zoom_avant_activate         (GtkMenuItem     *menuitem,
                                        GtkWidget *combo)
{
 gint z;
  
  /* lire la valeur actuelle de la combobox */
  z=timeline_get_zooming();
  switch (z)
    {
     case 100:
	  { z=0;break;}
	 case 75:
	  { z=1;break;}
	 case 50:
	  { z=2;break;}
	 case 33:
	  { z=3;break;}
	}/* end switch */   
  z=z-1;
  if (z<0)
     z=0; /* cyclique */
  switch (z)
    {
     case 0:
	  { z=100;break;}
	 case 1:
	  { z=75;break;}
	 case 2:
	  { z=50;break;}
	 case 3:
	  { z=33;break;}
	}/* end switch */ 
 
 		/* tout effacer */
		timeline_wipe_before_zooming();
		/* tramsmettre la nouvelle valeur du zooming */
		timeline_set_zooming(z);
		/* réafficher avec la nouvelle valeur */
        timeline_redraw_after_zooming();		
	  
}
/* zooming arrière sur timeline */

void
on_editeur_zoom_arriere_activate       (GtkMenuItem     *menuitem,
                                         GtkWidget *combo)
{
 gint z;
  
  /* lire la valeur actuelle de la combobox */
  z=timeline_get_zooming();
  switch (z)
    {
     case 100:
	  { z=0;break;}
	 case 75:
	  { z=1;break;}
	 case 50:
	  { z=2;break;}
	 case 33:
	  { z=3;break;}
	}/* end switch */   
  z=z+1;
  if (z>2)
     z=2; /*non cyclique */
 
  switch (z)
    {
     case 0:
	  { z=100;break;}
	 case 1:
	  { z=75;break;}
	 case 2:
	  { z=50;break;}
	 case 3:
	  { z=33;break;}
	}/* end switch */ 
 
 		/* tout effacer */
		timeline_wipe_before_zooming();
		/* tramsmettre la nouvelle valeur du zooming */
		timeline_set_zooming(z);
		/* réafficher avec la nouvelle valeur */
        timeline_redraw_after_zooming();		
	 
}
/*******************************
 * dialogue 'à propos'         *
 * ****************************/

void
on_a_propos1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 GtkWidget *dlg;
 gint reponse;
 
 dlg=create_aboutElla ();
 reponse=gtk_dialog_run(GTK_DIALOG(dlg));
 gtk_widget_destroy(dlg); 
}

/********************************
 * invoque le navigateur par
 * defaut du système, sous
 * Debian : x-www-browser
 *******************************/
void
on_aide_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gint erreur;
  
  erreur=system("x-www-browser ella_help.html");
}


void
on_utilisation_de_la_souris1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}
/**********************************
 * demande quitter via case de 
 * fermeture
 * ********************************/
gboolean
on_window_ella_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         *user_data)
{
gint x,y,w,h;
 gint *xp,*yp,*wp,*hp;
 GtkWidget *window;
 
 
 window=user_data[0];
 /* get the current window geometry */
 gtk_window_get_position(GTK_WINDOW(window),&x,&y);
 gtk_window_get_size(GTK_WINDOW(window),&w,&h);

 /* store the values */
 xp=user_data[1];
 yp=user_data[2]; 
 wp=user_data[3];
 hp=user_data[4];
 *xp=x;
 *yp=y;
 *wp=w;
 *hp=h;	
	
  if (dialog_ask_before_quit()==GTK_RESPONSE_OK)	
     gtk_main_quit();
  return TRUE;
}

/*******************************
 * appels depuis les boutons de
 * l'IHM
 * ****************************/
void
on_buttonNew_clicked                   (GtkButton       *button,
                                       GtkWidget *window1)
{
   nouveau(window1); 
}
/*********************************
 * ouverture rapide par le
 * bouton
 * ******************************/
void on_buttonOpen_clicked (GtkButton       *button,
                                        GtkWidget *window1)
{
	gint erreur;
	
    erreur=ouvrir_fichier_ella(window1);
}
/********************************
 * sauvegarde rapide par clic
 * sur le bouton
 * *****************************/

void
on_buttonSave_clicked                  (GtkButton       *button,
                                        GtkWidget *window1)
{
 gboolean retour;
 
 /* tester si le fichier a un nom */
 if (fFileNamed)
   {
	   enregistrer(chemin_complet);
   }
 else
   {
	   retour=enregistrer_ella(window1);
   }
}
/*******************************
 * clipboard
 * ****************************/
/* copier */
void
on_buttonCopy_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  editor_copy_items();
}

/* coller */
void
on_buttonPaste_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  editor_paste_items();
}

/* couper */
void
on_buttonCut_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	editor_cut_items();
}

/********************************
 * call for undo by the 'undo'
 * button of the toolbar
 * *****************************/
void
on_buttonUndo_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
   undo_operation();
}
/************************************** 
 * callbacks onglet 'transitions' 
 * ***********************************/
/* demande une interpolation de mouvement */
void
on_checkbuttonInterpola_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat, reussi;
	gint col,lig;
	GSList *liste_kf;
	
	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"inter",
							etat,
							0);
	if (!reussi)
	   printf("problème passage datas\n");
	/* mettre à jour l'affichage */						
}
/************************
 * callback for morphing
 * *********************/
void
on_checkbuttonMorphing_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;


	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"morpf",
							etat,
							0);
	if (!reussi)
	   printf("problème passage datas\n");
}										
/*******************/									
void
on_checkbuttonRotation_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;


	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"rotatf",
							etat,
							0);
	if (!reussi)
	   printf("problème passage datas\n");
	/* mettre à jour l'affichage */	
			
}										
/********************/
void
on_checkbuttonZoom_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;


	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"zoomf",
							etat,
							100);	
}										
/**** valeur de la rotation changée ****/
void
on_spinbuttonRotation_changed          (GtkWidget *spf,
                                        gpointer         user_data)
{
	gdouble valeur;
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;

	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */	
	valeur=gtk_spin_button_get_value (GTK_SPIN_BUTTON(spf));
	/* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"rotatv",
							TRUE,
							valeur);
}										
/************************/
void
on_checkbuttonFading_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;


	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"lumif",
							etat,
							100);		
}										
/*********************/
void
on_spinbuttonFading_changed            (GtkWidget *spf,
                                        gpointer         user_data)
{
	gdouble valeur;
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;

	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */	
	valeur=gtk_spin_button_get_value (GTK_SPIN_BUTTON(spf));
	/* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"lumiv",
							TRUE,
							valeur);	
}										
/********************/
void
on_checkbuttonAlpha_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;


	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */
	etat=gtk_toggle_button_get_active(togglebutton);
    /* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"alphaf",
							etat,
							100);		
}										
/****************************/
void
on_spinbuttonAlpha_changed             (GtkWidget *spf,
                                        gpointer         user_data)
{
	gdouble valeur;
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;

	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */	
	valeur=gtk_spin_button_get_value (GTK_SPIN_BUTTON(spf));
	/* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"alphav",
							TRUE,
							valeur);	
}										
/**************************/
void
on_spinbuttonZoom_changed              (GtkWidget *spf,
                                        gpointer         user_data)
{
	gdouble valeur;
	gboolean etat,reussi;	
	gint col,lig;
	GSList *liste_kf;

	/* où sommes-nous sur la timeline ? Coordonnées humaines */
	col=timeline_get_current_frame();
	lig=timeline_get_current_layer();
	/* lire le pointeur actuel sur la keyframe_list */
    liste_kf=timeline_get_keyframes_list();	
	/* lire le contenu */	
	valeur=gtk_spin_button_get_value (GTK_SPIN_BUTTON(spf));
	/* mettre dans la mémoire */
	reussi=set_keyframe_datas(liste_kf, col-1,
							"zoomv",
							TRUE,
							valeur);	
}										
/***********************************
 * zooming de la table de montage
 * elle-même
 * ********************************/
void
on_comboTimelineZoom_changed           (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
  gint z,z_actuel;
  /* lire la valeur actuelle de la combobox */
  z=gtk_combo_box_get_active(GTK_COMBO_BOX(combobox));
  switch (z)
    {
     case 0:
	  { z=100;break;}
	 case 1:
	  { z=75;break;}
	 case 2:
	  { z=50;break;}
	 case 3:
	  { z=33;break;}
	}/* end switch */ 
  /* lire valeur actuelle, si identique ne rien faire */     
  z_actuel=timeline_get_zooming();
  if (z!=z_actuel)
    {
		/* tout effacer */
		timeline_wipe_before_zooming();
		/* tramsmettre la nouvelle valeur du zooming */
		timeline_set_zooming(z);
		/* réafficher avec la nouvelle valeur */
        timeline_redraw_after_zooming();		
	}
}
/**************************
 * renvoyer l'état du flag
 * sur l'autoarchivage de
 * la bibliothèque
 * ***********************/
gboolean get_autoarchive_biblio(void)
{
	return auto_archive_biblio;
}
/**************************
 * stocker l'état du flag
 * sur l'autoarchivage de
 * la bibliothèque
 * ***********************/
void set_autoarchive_biblio(gboolean etat)
{
	auto_archive_biblio=etat;
}
/*******************************
 * align selected item(s)
 * ****************************/
void
on_align_items1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dlg;
  gint reponse, h_align,v_align,z_align;
  gboolean resultat;

  dlg=create_dialogAlignement ();
  reponse=gtk_dialog_run(GTK_DIALOG(dlg)); 
  gtk_widget_destroy(dlg);
  if (reponse==GTK_RESPONSE_OK)
    {
       h_align=dlg_get_horizontal_align();
	   v_align=dlg_get_vertical_align();
	   z_align=dlg_get_zone_align();
	   /* OK : we get parameters, and send modifications to editor */	
	   resultat=editor_set_align(h_align, v_align, z_align);   
    }   
}										
