/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/

//translations
#include <locale.h>
#include <libintl.h>

/*******************************
 * Management of clipboard and
 * undo operations
 * ***************************/
#include <gtk/gtk.h>
#include <goocanvas.h>
#include "ellasyms.h"
#include "clipboard.h"
#include <string.h>

/* variables */
extern guint ItemCounter;
extern GooCanvasItem *rootKF; /* source des dessins de chaque image-clé */
clip *clipboard;
cancel *undo;	
static GtkWidget *p_undo_submenu;
static GtkWidget *p_undo_button;
/***********************************
 * fonction pour obtenir le 
 * pointeur sur le GdkPixbuf associé
 * à une image dont le titre est
 * connu ; idem celle dans filesfunc
 * a besoin de pile_images déclarée
 * dans ellasyms
 * ********************************/
GdkPixbuf *find_image_pixbuf(GSList *pile,gchar *titre)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   GdkPixbuf *pb=NULL;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return NULL;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
   if (titre==NULL)
	  return NULL;	
   /* début boucle */
	temp = g_slist_nth (pile, j);				  
	while ((temp!=NULL)&&(trouve==FALSE))
	  { 
		if (  strcmp(   ((stack_images *)(temp->data))->nom,titre)==0   )
          {          	
			trouve=TRUE;
			pb=((stack_images *)(temp->data))->pixb;
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */			  
   return pb;
}
/*******************************/
/* initialialiser le clipboard */
/******************************/
void clipboard_init(void)
{
	clipboard=NULL;
	clipboard=g_malloc(sizeof(clip));
	clipboard->atom=CLIP_EMPTY;
	clipboard->content=NULL;
}
/************************************
 * contenu (atom) du clipboard
 * *********************************/
gint clipboard_get_atom(void)
{
	gint at;
	
	at=CLIP_EMPTY;	
	if (clipboard!=NULL)
	   at=clipboard->atom;
	return at;
}
/************************************/
/* effacer le contenu du clipboard */
/**********************************/
void clipboard_reset(void)
{
	gint count,nIndex;
	GList *node;
	
	if (clipboard==NULL)
	    return;
	/* quel atom ? */
	if ((clip *)(clipboard->atom)==CLIP_EMPTY)
	    return;
	if ((clip *)(clipboard->content)==NULL)
	    return;		
	/* détruire la liste */	
	/* il faut aiguiller selon le contenu */
	if (clipboard->atom==CLIP_ITEMS)
	   {	  
		 /* ce sont des goocanvasitems de l'éditeur */
	     for (nIndex = 0; node = g_list_nth (clipboard->content, nIndex); nIndex++) 
	        {
			   /* effacer la zone de texte éventuelle */
			   if ( ((ClipItem *)(node->data))->path!=NULL )
			      g_free( ((ClipItem *)(node->data))->path  );
			   /* effacer les datas */
			   clipboard->content=g_list_remove(clipboard->content,node->data);   
          	   count++;
            }/* next */		       
		 clipboard->content=NULL;
		 clipboard->atom=CLIP_EMPTY;
	   }
}
/*************************************
 * copy items to clipboard
 * ***********************************/
void clipboard_copy_items(GList *list)
{
	gint i;
	GList *node;
	guint32 fcolor,scolor;
    gchararray title,texte,fonte;
  	gdouble lw,angle;
    gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
    gdouble xc,yc,rx,ry; /* pour les ellipses */
    GooCanvasPoints *l_pts;	
	gboolean transfo;
  	ClipItem *objet;
	GdkPixbuf *pb;/* pointeur sur le pixbuf de la bitmap */
	gchar *path,*diese;   
	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  /* lire l'objet contenu dans *list */
		  /* partie commune à tous les objets */
		   g_object_get((GooCanvasItem *)(node->data), "title",&title,
		        "fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,
                "line-width",&lw,"width",&width,"height",&height,NULL);
		  /* saisie angle de rotation */
		   transfo=goo_canvas_item_get_simple_transform((GooCanvasItem *)(node->data),&xt,&yt,&echelle,&angle);	
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->titre_item=NULL;
		  /* partie spécifique */		
		  if (strncmp(title,"R",1)==0)
           {
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL);  
			 objet->objet=ITEM_RECT;
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 clipboard->content=g_list_append(clipboard->content,objet);
		   }
		  if (strncmp(title,"P",1)==0)
           {
			 diese= g_strrstr(title, "#");
			 if (diese!=NULL)
			    { 
				  path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
			      g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"width",&width,"height",&height,NULL);  
			      objet->objet=ITEM_PATH;
			      objet->x=x;
			      objet->y=y;
			      objet->w=width;
			      objet->h=height;
			      objet->xr=1.0;
			      objet->yr=1.0; 	
			      objet->contour=scolor;
			      objet->remplis=fcolor;
			      objet->lw=lw;
			      objet->angle=angle;
				  objet->path=g_strdup_printf("%s",path);
			      clipboard->content=g_list_append(clipboard->content,objet);
				  g_free(path);
			    }
		   } 
		  if (strncmp(title,"U",1)==0)
           {
			 diese= g_strrstr(title, "#");
			 if (diese!=NULL)
			    {  
					printf("copy a poly \n");
				  path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
			      g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"width",&width,"height",&height,NULL);  
			      objet->objet=ITEM_PATH;
			      objet->x=x;
			      objet->y=y;
			      objet->w=width;
			      objet->h=height;
			      objet->xr=1.0;
			      objet->yr=1.0; 	
			      objet->contour=scolor;
			      objet->remplis=fcolor;
			      objet->lw=lw;
			      objet->angle=angle;
				  objet->path=g_strdup_printf("%s",path);
			      clipboard->content=g_list_append(clipboard->content,objet);
				  g_free(path);
			    }
		   } 		   
		  if (strncmp(title,"B",1)==0)
           {
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL);  
			 pb=find_image_pixbuf(pile_images,title); 	
			 objet->objet=ITEM_BITMAP;
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->pbi=pb;
			 objet->angle=angle;
			 clipboard->content=g_list_append(clipboard->content,objet);			 		   
		   }		   
		  if (strncmp(title,"C",1)==0)
           {
     	     g_object_get((GooCanvasItem *)(node->data),"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);			   
			 objet->objet=ITEM_ELLIPSE;
			 objet->xc=xc;
			 objet->yc=yc;
			 objet->rx=rx;
			 objet->ry=ry;
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 clipboard->content=g_list_append(clipboard->content,objet);			 
		   }		   
		  if (strncmp(title,"L",1)==0)
           {
             l_pts= goo_canvas_points_new(2);	
             g_object_get((GooCanvasItem *)(node->data),"points",&l_pts,NULL);			 
			 objet->objet=ITEM_LINE;
			 objet->x1=l_pts->coords[0];
			 objet->y1=l_pts->coords[1];
			 objet->x2=l_pts->coords[2];
			 objet->y2=l_pts->coords[3];
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 clipboard->content=g_list_append(clipboard->content,objet);			 
             goo_canvas_points_unref(l_pts);			 		   
		   }		   
		  if (strncmp(title,"T",1)==0)
           {
              g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"text",&texte,"font",&fonte,NULL);			   
			  objet->objet=ITEM_TXT;
			  objet->x=x;
			  objet->y=y;
			  objet->w=width;
			  objet->h=height;
			  objet->xr=1.0;
			  objet->yr=1.0; 	
			  objet->contour=scolor;
			  objet->remplis=fcolor;
			  objet->lw=lw;
			  objet->texte=texte;
			  objet->fonte=fonte;
			  objet->angle=angle;
			  clipboard->content=g_list_append(clipboard->content,objet);
		   }		   
	  }
}
/*********************************
 * copier une liste dans le 
 * clipboard
 * ******************************/
void clipboard_copy(gint atom,GList *list,gdouble coin_x,gdouble coin_y)
{
	
	if (atom<0)
	    return;
	if (list==NULL)
	    return;	
    /* écraser la contenu actuel */	
	clipboard_reset();	
	/* aiguiller selon le type d'atom */
	switch (atom)
	  {
	    case CLIP_ITEMS:
	     /* items de l'éditeur */
	     {	
		   clipboard_copy_items(list);	
		   clipboard->atom=CLIP_ITEMS; 
		   clipboard->coin_x=coin_x;
		   clipboard->coin_y=coin_y;
		   break;
	     }
	    default:
	       break;   
	   }/* end switch */
}
/*****************************************
 * paste items at the position x,y
 * root=items group of the current keyframe
 * we must use this function to fill
 * the 'undo buffer' in order to undo
 * a 'pasting'
 * ****************************************/
void clipboard_paste_items(GooCanvasItem *root,gdouble x,gdouble y)
{
	gint i,choix;
	GList *node;
	ClipItem *objet;
	GooCanvasItem *item;
	gchar *titre,*image_file;
	gdouble tx,ty;/* changement des axes lors du collage */
	gboolean erreur;
	GError *error=NULL;
    stack_images *temp_data;
	GdkPixbuf *pix;
	gdouble angle,w,h,xr,yr;
 		
	if (root==NULL)
	   return;
	/* le clipboard contient-il des items ? */
	if (clipboard==NULL)
	    return;
	/* quel atom ? */
	if ((gint)(clip *)(clipboard->atom)!=CLIP_ITEMS)
	    return;
	if ((clip *)(clipboard->content)==NULL)
	    return;	 
	tx=	clipboard->coin_x;
	ty= clipboard->coin_y;
	/* undo bufer */
	if (undo==NULL)
	    {printf("pb buffer undo \n");return;}
	else
	  {
		  
	  }
		
	/* passer tout la liste et la coller en place */
	for (i = 0; node = g_list_nth (clipboard->content,i); i++) 
	  {		
		  /* switcher selon le type d'item mémorisé */
		  choix=(gint)((ClipItem*)(node->data))->objet;
		  angle=((ClipItem*)(node->data))->angle;
		  switch (choix)
		    {
				case ITEM_RECT:
				  {
					item=goo_canvas_rect_new(root,
										x+(((ClipItem*)(node->data))->x-tx),
										y+(((ClipItem*)(node->data))->y-ty),
										((ClipItem*)(node->data))->w,((ClipItem*)(node->data))->h,
										"line-width", ((ClipItem*)(node->data))->lw,
										"radius-x", ((ClipItem*)(node->data))->xr,
										"radius-y", ((ClipItem*)(node->data))->yr,
										"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
										"fill-color-rgba", ((ClipItem*)(node->data))->remplis,
										NULL);	
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("R%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }
				case ITEM_PATH:
				  {			 
	                item= goo_canvas_path_new (root,((ClipItem*)(node->data))->path,
					         "line-width",((ClipItem*)(node->data))->lw, 
	                        "stroke-color-rgba", ((ClipItem*)(node->data))->contour,
							"fill-color-rgba",((ClipItem*)(node->data))->remplis,NULL);                   	         
			        /* then we size the object */
			        g_object_set(G_OBJECT(item),"x",x+(((ClipItem*)(node->data))->x-tx),
					                               "y",y+(((ClipItem*)(node->data))->y-ty),
												   "width",((ClipItem*)(node->data))->w,
												   "height",((ClipItem*)(node->data))->h,NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("P%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }
				case ITEM_POLYLINE:
				  {			 
	                item= goo_canvas_path_new (root,((ClipItem*)(node->data))->path,
					         "line-width",((ClipItem*)(node->data))->lw, 
	                        "stroke-color-rgba", ((ClipItem*)(node->data))->contour,
							"fill-color-rgba",((ClipItem*)(node->data))->remplis,NULL);                   	         
			        /* then we size the object */
			        g_object_set(G_OBJECT(item),"x",x+(((ClipItem*)(node->data))->x-tx),
					                               "y",y+(((ClipItem*)(node->data))->y-ty),
												   "width",((ClipItem*)(node->data))->w,
												   "height",((ClipItem*)(node->data))->h,NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("U%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }				  
				case ITEM_ELLIPSE:
				  {
					item=goo_canvas_ellipse_new(root,
										x+(((ClipItem*)(node->data))->xc-tx)-((ClipItem*)(node->data))->w/2,
										y+(((ClipItem*)(node->data))->yc-ty)-((ClipItem*)(node->data))->h/2,
										((ClipItem*)(node->data))->rx,
										((ClipItem*)(node->data))->ry,
										"line-width", ((ClipItem*)(node->data))->lw,
										"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
										"fill-color-rgba", ((ClipItem*)(node->data))->remplis,
										NULL);	
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("C%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 goo_canvas_item_rotate(item,angle,x,y);
						 }					  					  
					  break;
				  }
				case ITEM_LINE:
				  {
					  item=goo_canvas_polyline_new_line(root,
						x+(((ClipItem*)(node->data))->x1-tx),
						y+(((ClipItem*)(node->data))->y1-ty),
						x+(((ClipItem*)(node->data))->x2-tx),
						y+(((ClipItem*)(node->data))->y2-ty),						
						"line-width",((ClipItem*)(node->data))->lw, 
						"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
						NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("L%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }					  					
					  break;
				  }
				case ITEM_TXT:
				  {
					item= goo_canvas_text_new (root, ((ClipItem*)(node->data))->texte,
					     x+(((ClipItem*)(node->data))->x-tx),
				      	 y+(((ClipItem*)(node->data))->y-ty),
						 -1,
				         GTK_ANCHOR_CENTER,
				        "font", ((ClipItem*)(node->data))->fonte,
				        "fill-color-rgba", ((ClipItem*)(node->data))->remplis,
				        "stroke-color-rgba",((ClipItem*)(node->data))->contour,
				         NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  ItemCounter++;
					  titre=g_strdup_printf("T%d",ItemCounter);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 goo_canvas_item_rotate(item,angle,x,y);
						 }						  					 					  
					  break;
				  }
				case ITEM_BITMAP:
				  {
                      /* sauvegarder depuis le pixbuf une nouvelle image PNG */
					  ItemCounter++;
					  titre=g_strdup_printf("B%d",ItemCounter);	
					  pix=gdk_pixbuf_copy(((ClipItem*)(node->data))->pbi);	            	  
					  item=goo_canvas_image_new(root,pix,
					                   x+(((ClipItem*)(node->data))->x-tx),
				      	               y+(((ClipItem*)(node->data))->y-ty),NULL);
					  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* mettre dans la pile d'images */
					  temp_data = g_malloc(sizeof(stack_images));
					  temp_data->nom = g_strdup(titre); 
					  temp_data->pixb = pix;
					  pile_images = g_slist_append (pile_images,temp_data);	
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }						  				  				   
					  break;
				  }				  
				default:printf("hors items \n");
				      break;      
			}/* end switch*/
	    if (item!=NULL)
		   {
			    /* création de l'espace mémoire */
		        objet=g_malloc(sizeof(ClipItem));
		        objet->path=NULL;
		        objet->item=item;
		        undo->content=g_list_append(undo->content,objet);	
		   }		
	  }/* next i*/
}
/*****************************************
 * coller à la position donnée le contenu
 * du clipboard
 * **************************************/
void clipboard_paste(gint atom,GooCanvasItem *root,gdouble x,gdouble y)
{
	if (atom==CLIP_ITEMS)
	  clipboard_paste_items(root,x,y);
}
/******************************************/
/* initialialiser le système d'annulation */
/*****************************************/
void undo_init(void)
{
	undo=NULL;
	undo=g_malloc(sizeof(cancel));
	undo->item=NULL;
	undo->atom=UNDO_NOTHING;
	undo->content=NULL;	
	gtk_widget_set_sensitive (p_undo_submenu, FALSE);
}
/************************************/
/* reset the undo memory           */
/**********************************/
void undo_reset(void)
{
	gint count,nIndex;
	GList *node;

    gtk_widget_set_sensitive (p_undo_submenu, FALSE);
    gtk_widget_set_sensitive (p_undo_button, FALSE);		
	if (undo==NULL)
	    return;
	/* quel atom ? */
	if ((cancel *)(undo->atom)==UNDO_NOTHING)
	    return;
	if ((cancel *)(undo->content)==NULL)
	    return;				
 
	/* il faut aiguiller selon le contenu */
	/* reset undo data list if any */	
	if ((undo->atom==UNDO_CUT_ITEMS) || (undo->atom==UNDO_ERASE_ITEMS))
	   {	  
		 /* ce sont des goocanvasitems de l'éditeur */
	     for (nIndex = 0; node = g_list_nth (clipboard->content, nIndex); nIndex++) 
	        {
			   /* remove path definition string if exists */
			   if ( ((ClipItem *)(node->data))->path!=NULL )
			      g_free( ((ClipItem *)(node->data))->path  );
			   if ( ((ClipItem *)(node->data))->titre_item!=NULL )
			      g_free( ((ClipItem *)(node->data))->titre_item  );	  
			   /* effacer les datas */
			   undo->content=g_list_remove(undo->content,node->data);   
          	   count++;
            }/* next */		       
		 undo->content=NULL;
		 undo->atom=UNDO_NOTHING;
	   }
}
/******************************************
 * store a pointer on undo sub menu
 * ****************************************/
void undo_store_variables(GtkWidget *undo_menu, GtkWidget *button_undo)
{
	p_undo_submenu=undo_menu;
	p_undo_button=button_undo;
}
/*************************************
 * copy items to undo buffer
 * ***********************************/
void undo_copy_items(GList *list)
{
	gint i;
	GList *node;
	guint32 fcolor,scolor;
    gchararray title,texte,fonte;
  	gdouble lw,angle;
    gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
    gdouble xc,yc,rx,ry; /* pour les ellipses */
    GooCanvasPoints *l_pts;	
	gboolean transfo;
  	ClipItem *objet;
	GdkPixbuf *pb;/* pointeur sur le pixbuf de la bitmap */
	gchar *path,*diese;   
	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  /* lire l'objet contenu dans *list */
		  /* partie commune à tous les objets */
		   g_object_get((GooCanvasItem *)(node->data), "title",&title,
		        "fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,
                "line-width",&lw,"width",&width,"height",&height,NULL);
		  /* saisie angle de rotation */
		   transfo=goo_canvas_item_get_simple_transform((GooCanvasItem *)(node->data),&xt,&yt,&echelle,&angle);	
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->titre_item=g_strdup_printf("%s",title);
		  objet->item=(GooCanvasItem *)(node->data);
		  /* partie spécifique */		
		  if (strncmp(title,"R",1)==0)
           {
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL);  
			 objet->objet=ITEM_RECT;
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 undo->content=g_list_append(undo->content,objet);
		   }
		  if (strncmp(title,"P",1)==0)
           {
			 diese= g_strrstr(title, "#");
			 if (diese!=NULL)
			    { 
				  path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
			      g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"width",&width,"height",&height,NULL);  
			      objet->objet=ITEM_PATH;
			      objet->x=x;
			      objet->y=y;
			      objet->w=width;
			      objet->h=height;
			      objet->xr=1.0;
			      objet->yr=1.0; 	
			      objet->contour=scolor;
			      objet->remplis=fcolor;
			      objet->lw=lw;
			      objet->angle=angle;
				  objet->path=g_strdup_printf("%s",path);
			      undo->content=g_list_append(undo->content,objet);
				  g_free(path);
			    }
		   } 
		  if (strncmp(title,"U",1)==0)
           {
			 diese= g_strrstr(title, "#");
			 if (diese!=NULL)
			    {  
				  path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
			      g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"width",&width,"height",&height,NULL);  
			      objet->objet=ITEM_PATH;
			      objet->x=x;
			      objet->y=y;
			      objet->w=width;
			      objet->h=height;
			      objet->xr=1.0;
			      objet->yr=1.0; 	
			      objet->contour=scolor;
			      objet->remplis=fcolor;
			      objet->lw=lw;
			      objet->angle=angle;
				  objet->path=g_strdup_printf("%s",path);
			      undo->content=g_list_append(undo->content,objet);
				  g_free(path);
			    }
		   } 		   
		  if (strncmp(title,"B",1)==0)
           {
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL);  
			 pb=find_image_pixbuf(pile_images,title); 	
			 objet->objet=ITEM_BITMAP;
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->pbi=pb;
			 objet->angle=angle;
			 undo->content=g_list_append(undo->content,objet);			 		   
		   }		   
		  if (strncmp(title,"C",1)==0)
           {
     	     g_object_get((GooCanvasItem *)(node->data),"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);			   
			 objet->objet=ITEM_ELLIPSE;
			 objet->xc=xc;
			 objet->yc=yc;
			 objet->rx=rx;
			 objet->ry=ry;
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 undo->content=g_list_append(undo->content,objet);			 
		   }		   
		  if (strncmp(title,"L",1)==0)
           {
             l_pts= goo_canvas_points_new(2);	
             g_object_get((GooCanvasItem *)(node->data),"points",&l_pts,NULL);			 
			 objet->objet=ITEM_LINE;
			 objet->x1=l_pts->coords[0];
			 objet->y1=l_pts->coords[1];
			 objet->x2=l_pts->coords[2];
			 objet->y2=l_pts->coords[3];
			 objet->w=width;
			 objet->h=height;
			 objet->xr=1.0;
			 objet->yr=1.0; 	
			 objet->contour=scolor;
			 objet->remplis=fcolor;
			 objet->lw=lw;
			 objet->angle=angle;
			 undo->content=g_list_append(undo->content,objet);			 
             goo_canvas_points_unref(l_pts);			 		   
		   }		   
		  if (strncmp(title,"T",1)==0)
           {
              g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,"text",&texte,"font",&fonte,NULL);			   
			  objet->objet=ITEM_TXT;
			  objet->x=x;
			  objet->y=y;
			  objet->w=width;
			  objet->h=height;
			  objet->xr=1.0;
			  objet->yr=1.0; 	
			  objet->contour=scolor;
			  objet->remplis=fcolor;
			  objet->lw=lw;
			  objet->texte=texte;
			  objet->fonte=fonte;
			  objet->angle=angle;
			  undo->content=g_list_append(undo->content,objet);
		   }		   
	  }
}
/*********************************
 * copy a selected items list
 * to the undo buffer
 * ******************************/
void undo_copy(gint atom,GList *list,gdouble coin_x,gdouble coin_y)
{
	
	if (atom<0)
	    return;
	if (list==NULL)
	    return;	
    /* erase current undo buffer */	
	undo_reset();	

	switch (atom)
	  {
	    case UNDO_CUT_ITEMS: 
		case UNDO_ERASE_ITEMS:
	     {	
		   undo->atom=UNDO_CUT_ITEMS;	 
		   undo_copy_items(list);	
		   undo->coin_x=coin_x;
		   undo->coin_y=coin_y;
		   break;
	     }
	    default:
	       break;   
	   }/* end switch */
}
/*****************************************
 * paste items at the position x,y
 * root=items group of the current keyframe
 * items are picked from the undo buffer
 * ****************************************/
void undo_paste_items(GooCanvasItem *root)
{
	gint i,choix;
	GList *node;
	GooCanvasItem *item;
	gchar *titre,*image_file;
	gboolean erreur;
	GError *error=NULL;
    stack_images *temp_data;
	GdkPixbuf *pix;
	gdouble angle,w,h,xr,yr;
		
	if (root==NULL)
	   return;
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_CUT_ITEMS)
	    return;
	if ((cancel *)(undo->content)==NULL)
	    return;	 

	/* passer tout la liste et la coller en place */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 
	  {		
		  /* switcher selon le type d'item mémorisé */
		  choix=(gint)((ClipItem*)(node->data))->objet;
		  angle=((ClipItem*)(node->data))->angle;
		  switch (choix)
		    {
				case ITEM_RECT:
				  {
					item=goo_canvas_rect_new(root,
										((ClipItem*)(node->data))->x,
										((ClipItem*)(node->data))->y,
										((ClipItem*)(node->data))->w,((ClipItem*)(node->data))->h,
										"line-width", ((ClipItem*)(node->data))->lw,
										"radius-x", ((ClipItem*)(node->data))->xr,
										"radius-y", ((ClipItem*)(node->data))->yr,
										"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
										"fill-color-rgba", ((ClipItem*)(node->data))->remplis,
										NULL);	
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }
				case ITEM_PATH:
				  {			 
	                item= goo_canvas_path_new (root,((ClipItem*)(node->data))->path,
					         "line-width",((ClipItem*)(node->data))->lw, 
	                        "stroke-color-rgba", ((ClipItem*)(node->data))->contour,
							"fill-color-rgba",((ClipItem*)(node->data))->remplis,NULL);                   	         
			        /* then we size the object */
			        g_object_set(G_OBJECT(item),"x",((ClipItem*)(node->data))->x,
					                               "y",((ClipItem*)(node->data))->y,
												   "width",((ClipItem*)(node->data))->w,
												   "height",((ClipItem*)(node->data))->h,NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }
				case ITEM_POLYLINE:
				  {			 
	                item= goo_canvas_path_new (root,((ClipItem*)(node->data))->path,
					         "line-width",((ClipItem*)(node->data))->lw, 
	                        "stroke-color-rgba", ((ClipItem*)(node->data))->contour,
							"fill-color-rgba",((ClipItem*)(node->data))->remplis,NULL);                   	         
			        /* then we size the object */
			        g_object_set(G_OBJECT(item),"x",((ClipItem*)(node->data))->x,
					                               "y",((ClipItem*)(node->data))->y,
												   "width",((ClipItem*)(node->data))->w,
												   "height",((ClipItem*)(node->data))->h,NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* rotation si besoin */
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }							  
					  break;
				  }				  
				case ITEM_ELLIPSE:
				  {
					item=goo_canvas_ellipse_new(root,
										((ClipItem*)(node->data))->xc,
										((ClipItem*)(node->data))->yc,
										((ClipItem*)(node->data))->rx,
										((ClipItem*)(node->data))->ry,
										"line-width", ((ClipItem*)(node->data))->lw,
										"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
										"fill-color-rgba", ((ClipItem*)(node->data))->remplis,
										NULL);	
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 goo_canvas_item_rotate(item,angle,((ClipItem*)(node->data))->x,((ClipItem*)(node->data))->y);
						 }					  					  
					  break;
				  }
				case ITEM_LINE:
				  {
					  item=goo_canvas_polyline_new_line(root,
						((ClipItem*)(node->data))->x1,
						((ClipItem*)(node->data))->y1,
						((ClipItem*)(node->data))->x2,
						((ClipItem*)(node->data))->y2,						
						"line-width",((ClipItem*)(node->data))->lw, 
						"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
						NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }					  					
					  break;
				  }
				case ITEM_TXT:
				  {
					item= goo_canvas_text_new (root, ((ClipItem*)(node->data))->texte,
					     ((ClipItem*)(node->data))->x,
				      	 ((ClipItem*)(node->data))->y,
						 -1,
				         GTK_ANCHOR_CENTER,
				        "font", ((ClipItem*)(node->data))->fonte,
				        "fill-color-rgba", ((ClipItem*)(node->data))->remplis,
				        "stroke-color-rgba",((ClipItem*)(node->data))->contour,
				         NULL);
					  /* mettre un nom et incrémeter le compteur d'items */	
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
	                  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  if (angle!=0)
					     {
							 goo_canvas_item_rotate(item,angle,((ClipItem*)(node->data))->x,((ClipItem*)(node->data))->y);
						 }						  					 					  
					  break;
				  }
				case ITEM_BITMAP:
				  {
                      /* sauvegarder depuis le pixbuf une nouvelle image PNG */
					  titre=g_strdup_printf("%s",((ClipItem*)(node->data))->titre_item);
					  pix=gdk_pixbuf_copy(((ClipItem*)(node->data))->pbi);	            	  
					  item=goo_canvas_image_new(root,pix,
					                   ((ClipItem*)(node->data))->x,
				      	               ((ClipItem*)(node->data))->y,NULL);
					  g_object_set(G_OBJECT(item),"title",titre,NULL);	
					  /* mettre dans la pile d'images */
					  temp_data = g_malloc(sizeof(stack_images));
					  temp_data->nom = g_strdup(titre); 
					  temp_data->pixb = pix;
					  pile_images = g_slist_append (pile_images,temp_data);	
					  if (angle!=0)
					     {
							 g_object_get(item,"x",&xr,"y",&yr,"width",&w,"height",&h,NULL);
							 goo_canvas_item_rotate(item,angle,xr+(w/2),yr+(h/2));
						 }						  				  				   
					  break;
				  }				  
				default:printf("hors items \n");
				      break;      
			}/* end switch*/
	  }/* next i*/
}
/*****************************************
 * paste the content of the undo buffer
 * **************************************/
void undo_paste(gint atom,GooCanvasItem *root)
{
	if (atom==UNDO_CUT_ITEMS)
	 {
	   undo_paste_items(root);
     }
}
/***********************************
 * undo a moving : store datas in
 * order to undo
 * ********************************/
void undo_store_moving(GList *list)
{
	gint i;
	GList *node;
  	ClipItem *objet;
	gdouble angle;
    gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
    gdouble xc,yc,rx,ry; /* pour les ellipses */
	gboolean transfo;
	gchararray title;
    GooCanvasPoints *l_pts;	
		
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  /* lire l'objet contenu dans *list */
		   g_object_get((GooCanvasItem *)(node->data), "title",&title,"width",&width,"height",&height,NULL);
		  /* saisie angle de rotation */
		   transfo=goo_canvas_item_get_simple_transform((GooCanvasItem *)(node->data),&xt,&yt,&echelle,&angle);	
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->titre_item=g_strdup_printf("%s",title);
		  objet->item=(GooCanvasItem *)(node->data);
		  objet->angle=angle;
		  if (strncmp(title,"R",1)==0) 
           {
			 objet->objet=ITEM_RECT;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 undo->content=g_list_append(undo->content,objet);
		   }	
		  if  (strncmp(title,"P",1)==0) 
           {
			 objet->objet=ITEM_PATH;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 undo->content=g_list_append(undo->content,objet);
		   }		   
		  if  (strncmp(title,"U",1)==0) 
           {
			 objet->objet=ITEM_POLYLINE;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 undo->content=g_list_append(undo->content,objet);
		   }		
		  if  (strncmp(title,"B",1)==0)
           {
			 objet->objet=ITEM_BITMAP;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 undo->content=g_list_append(undo->content,objet);
		   }		      
		  if  (strncmp(title,"T",1)==0)
           {
			 objet->objet=ITEM_TXT;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 undo->content=g_list_append(undo->content,objet);
		   }		   		   
		  if (strncmp(title,"L",1)==0)
           {
             l_pts= goo_canvas_points_new(2);	
			  objet->objet=ITEM_LINE;
             g_object_get((GooCanvasItem *)(node->data),"points",&l_pts,NULL);			 
			 objet->x1=l_pts->coords[0];
			 objet->y1=l_pts->coords[1];
			 objet->x2=l_pts->coords[2];
			 objet->y2=l_pts->coords[3];
			 undo->content=g_list_append(undo->content,objet);			 
             goo_canvas_points_unref(l_pts);			 		   
		   }	
		  if (strncmp(title,"C",1)==0)
           {
     	     g_object_get((GooCanvasItem *)(node->data),"center_x",&xc,"center_y",&yc,NULL);	
			  objet->objet=ITEM_ELLIPSE;		   
			 objet->xc=xc;
			 objet->yc=yc;
			 objet->rx=rx;
			 objet->ry=ry;
			 undo->content=g_list_append(undo->content,objet);			 
		   }			   		   	  
      }/* next i*/				
}
/*********************************
 * undo a moving
 * ******************************/
void undo_move(void)
{
	gint i;
	GList *node;
    gdouble x,y,xt,yt,xc,yc,echelle,rota,width,height;
	gboolean transforme=FALSE;
	GooCanvasPoints *l_pts;
							
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if (((gint)(cancel *)(undo->atom)!=UNDO_MOVING)&&((gint)(cancel *)(undo->atom)!=UNDO_ALIGN_ITEMS))
	    return;	
	if ((cancel *)(undo->content)==NULL)
	    return;

	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {
		 transforme=goo_canvas_item_get_simple_transform(((ClipItem*)(node->data))->item,&xt,&yt,&echelle,&rota); 
		 g_object_get(((ClipItem*)(node->data))->item,"width",&width,"height",&height,NULL);
		 switch ((gint)((ClipItem*)(node->data))->objet)
		   {
			  case ITEM_BITMAP:
			  case ITEM_PATH:
			  case ITEM_POLYLINE:
			  case ITEM_RECT:
			  case ITEM_TXT:
			    {
					g_object_get(((ClipItem*)(node->data))->item,"x",&x,"y",&y,NULL);
					xt=x+(width/2);
                    yt=y+(height/2); 
					goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-rota,xt,yt);/* annuler rotation */
					g_object_set(((ClipItem*)(node->data))->item,"x", ((ClipItem*)(node->data))->x,
					        "y", ((ClipItem*)(node->data))->y, NULL);
					 /* remettre l'angle */
	                g_object_get(((ClipItem*)(node->data))->item,"x",&x,"y",&y,NULL);
	                xt=x+(width/2);
                    yt=y+(height/2); 
	                goo_canvas_item_rotate(((ClipItem*)(node->data))->item,rota,xt,yt);
					break;
				}
			  case ITEM_ELLIPSE:
			    {
                    g_object_get(((ClipItem*)(node->data))->item,"center_x",&xc,"center_y",&yc,NULL);
	                goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-rota,xc,yc);/* annuler rotation */
                    g_object_set(((ClipItem*)(node->data))->item,
					              "center_x",((ClipItem*)(node->data))->xc,
								  "center_y",((ClipItem*)(node->data))->yc,NULL);
	                g_object_get(((ClipItem*)(node->data))->item,"center_x",&xc,"center_y",&yc,NULL);
	                goo_canvas_item_rotate(((ClipItem*)(node->data))->item,rota,xc,yc);/* remettre rotation */					
					break;
				}
			  case ITEM_LINE:
			    {
					g_object_get(((ClipItem*)(node->data))->item,"x",&x,"y",&y,NULL);
					xt=x+(width/2);
                    yt=y+(height/2); 
					goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-rota,xt,yt);/* annuler rotation */
					l_pts= goo_canvas_points_new(2);/* lignes simples */
					l_pts->coords[0]=(gdouble)((ClipItem*)(node->data))->x1;
                    l_pts->coords[1]=(gdouble)((ClipItem*)(node->data))->y1;
                    l_pts->coords[2]=(gdouble)((ClipItem*)(node->data))->x2;
                    l_pts->coords[3]=(gdouble)((ClipItem*)(node->data))->y2;
                    g_object_set(((ClipItem*)(node->data))->item,"points",l_pts,NULL);       
                    goo_canvas_points_unref(l_pts); 
	                g_object_get(((ClipItem*)(node->data))->item,"x",&x,"y",&y,NULL);
	                xt=x+(width/2);
                    yt=y+(height/2); 
	                goo_canvas_item_rotate(((ClipItem*)(node->data))->item,rota,xt,yt);
					break;
				}
			  default:;			 
		   }/*end switch*/ 
	  }/*next i*/			
}
/*********************************
 * undo a sizing
 * ******************************/
void undo_size(void)
{
	gint i;
	GList *node;
    gdouble x,y,xt,yt,xc,yc,echelle,rota,width,height;
	gboolean transforme=FALSE;
	GooCanvasPoints *l_pts;
							
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_SIZING)
	    return;	
	if ((cancel *)(undo->content)==NULL)
	    return;
printf("passé \n");
	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {		
		 switch ((gint)((ClipItem*)(node->data))->objet)
		   {
			  case ITEM_BITMAP:
			  case ITEM_PATH:
			  case ITEM_POLYLINE:
			  case ITEM_RECT:
			  case ITEM_TXT:
			    {					
					g_object_set(((ClipItem*)(node->data))->item,"x", ((ClipItem*)(node->data))->x,
					        "y", ((ClipItem*)(node->data))->y,
							"width",((ClipItem*)(node->data))->w,
							"height",((ClipItem*)(node->data))->h,
							 NULL);					
					break;
				}
			  case ITEM_ELLIPSE:
			    {                   
                    g_object_set(((ClipItem*)(node->data))->item,
					              "center_x",((ClipItem*)(node->data))->xc,
								  "center_y",((ClipItem*)(node->data))->yc,
								  "radius_x",((ClipItem*)(node->data))->rx,
								  "radius_y",((ClipItem*)(node->data))->ry,
								  NULL);				
					break;
				}
			  case ITEM_LINE:
			    {					
					l_pts= goo_canvas_points_new(2);/* lignes simples */
					l_pts->coords[0]=(gdouble)((ClipItem*)(node->data))->x1;
                    l_pts->coords[1]=(gdouble)((ClipItem*)(node->data))->y1;
                    l_pts->coords[2]=(gdouble)((ClipItem*)(node->data))->x2;
                    l_pts->coords[3]=(gdouble)((ClipItem*)(node->data))->y2;
                    g_object_set(((ClipItem*)(node->data))->item,"points",l_pts,NULL);       
                    goo_canvas_points_unref(l_pts); 
	                break;
				}
			  default:;			 
		   }/*end switch*/ 
	  }/*next i*/			
}
/*********************************
 * undo a rotating
 * ******************************/
void undo_rotate(void)
{
	gint i;
	GList *node;
    gdouble x,y,xt,yt,xc,yc,echelle,rota,width,height;
	gboolean transforme=FALSE;
	GooCanvasPoints *l_pts;
							
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_ROTATING)
	    return;	
	if ((cancel *)(undo->content)==NULL)
	    return;

	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {
		 transforme=goo_canvas_item_get_simple_transform(((ClipItem*)(node->data))->item,&xt,&yt,&echelle,&rota); 
		 g_object_get(((ClipItem*)(node->data))->item,"width",&width,"height",&height,NULL);
		 switch ((gint)((ClipItem*)(node->data))->objet)
		   {
			  case ITEM_BITMAP:
			  case ITEM_PATH:
			  case ITEM_POLYLINE:
			  case ITEM_RECT:
			  case ITEM_TXT:
			    {					
					xt=((ClipItem*)(node->data))->x+(width/2);
                    yt=((ClipItem*)(node->data))->y+(height/2); 
					goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-((ClipItem*)(node->data))->angle,xt,yt);/* annuler rotation */					
					break;
				}
			  case ITEM_ELLIPSE:
			    {
                    xt=((ClipItem*)(node->data))->xc;
                    yt=((ClipItem*)(node->data))->yc; 
					goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-((ClipItem*)(node->data))->angle,xt,yt);/* annuler rotation */								
					break;
				}
			  case ITEM_LINE:
			    {
					g_object_get(((ClipItem*)(node->data))->item,"x",&x,"y",&y,NULL);
					xt=x+(width/2);
                    yt=y+(height/2); 
					goo_canvas_item_rotate(((ClipItem*)(node->data))->item,-((ClipItem*)(node->data))->angle,xt,yt);/* annuler rotation */					
					break;
				}
			  default:;			 
		   }/*end switch*/ 
	  }/*next i*/			
}
/****************************************
 * undo a font change : store datas in
 * order to undo
 * *************************************/
void undo_store_font_change(GList *list)
{
	gint i;
	GList *node;
  	ClipItem *objet;
    guint32 fcolor,scolor;
	gchararray font;
		
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  g_object_get((GooCanvasItem *)(node->data),"font",&font,NULL);
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->item=(GooCanvasItem *)(node->data);
		  objet->fonte=g_strdup_printf("%s",font);	
		  undo->content=g_list_append(undo->content,objet);
	  }/* next i*/
}
/****************************************
 * undo a font change 
 * *************************************/
void undo_font_change()
{
	gint i;
	GList *node;
  	ClipItem *objet;
	gchararray font;
	
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_FONT_CHANGE)
	    return;	
	if ((cancel *)(undo->content)==NULL)
	    return;

	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {
		  g_object_set(((ClipItem*)(node->data))->item,"font",((ClipItem*)(node->data))->fonte,NULL);
	  }/* next i*/
}
/****************************************
 * undo a color change : store datas in
 * order to undo
 * *************************************/
void undo_store_color_change(GList *list)
{
	gint i;
	GList *node;
  	ClipItem *objet;
    guint32 fcolor,scolor;
		
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		   g_object_get((GooCanvasItem *)(node->data),"fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,NULL);
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->item=(GooCanvasItem *)(node->data);
		  objet->contour=scolor;
		  objet->remplis=fcolor;	
		  undo->content=g_list_append(undo->content,objet);
	  }/* next i*/
}
/**********************************
 * undo a color change
 * *******************************/
void undo_color_change(void)
{
	gint i;
	GList *node;
							
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_COLOR_CHANGE)
	    return;	
	if ((cancel *)(undo->content)==NULL)
	    return;

	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {	
		 g_object_set(((ClipItem*)(node->data))->item,"stroke-color-rgba", ((ClipItem*)(node->data))->contour,
										"fill-color-rgba", ((ClipItem*)(node->data))->remplis, NULL); 
	  }/*next i*/
}
/***********************************
 * undo a sizing : store datas in
 * order to undo
 * ********************************/
void undo_store_sizing(GList *list)
{
	gint i;
	GList *node;
  	ClipItem *objet;
	gdouble angle;
    gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
    gdouble xc,yc,rx,ry; /* pour les ellipses */
	gboolean transfo;
	gchararray title;
    GooCanvasPoints *l_pts;	
		
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  /* lire l'objet contenu dans *list */
		   g_object_get((GooCanvasItem *)(node->data), "title",&title,"width",&width,"height",&height,NULL);
		  /* saisie angle de rotation */
		   transfo=goo_canvas_item_get_simple_transform((GooCanvasItem *)(node->data),&xt,&yt,&echelle,&angle);	
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->titre_item=g_strdup_printf("%s",title);
		  objet->item=(GooCanvasItem *)(node->data);
		  objet->angle=angle;
		  if (strncmp(title,"R",1)==0) 
           {
			 objet->objet=ITEM_RECT;   
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y; 
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }	
		  if  (strncmp(title,"P",1)==0) 
           {
			 objet->objet=ITEM_PATH;  	
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;		 
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		   
		  if  (strncmp(title,"U",1)==0) 
           {
			 objet->objet=ITEM_POLYLINE;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		
		  if  (strncmp(title,"B",1)==0)
           {
			 objet->objet=ITEM_BITMAP; 
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		      
		  if  (strncmp(title,"T",1)==0)
           {
			 objet->objet=ITEM_TXT; 
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		   		   
		  if (strncmp(title,"L",1)==0)
           {
             l_pts= goo_canvas_points_new(2);	
			  objet->objet=ITEM_LINE;
             g_object_get((GooCanvasItem *)(node->data),"points",&l_pts,NULL);			 
			 objet->x1=l_pts->coords[0];
			 objet->y1=l_pts->coords[1];
			 objet->x2=l_pts->coords[2];
			 objet->y2=l_pts->coords[3];
			 undo->content=g_list_append(undo->content,objet);			 
             goo_canvas_points_unref(l_pts);			 		   
		   }	
		  if (strncmp(title,"C",1)==0)
           {
     	     g_object_get((GooCanvasItem *)(node->data),"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);	
			  objet->objet=ITEM_ELLIPSE;		   
			 objet->xc=xc;
			 objet->yc=yc;
			 objet->rx=rx;
			 objet->ry=ry;
			 undo->content=g_list_append(undo->content,objet);			 
		   }			   		   	  
      }/* next i*/				
}
/***********************************
 * undo a sizing : store datas in
 * order to undo
 * ********************************/
void undo_store_rotating(GList *list,gdouble angle1)
{
	gint i;
	GList *node;
  	ClipItem *objet;
	gdouble angle;
    gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
    gdouble xc,yc,rx,ry; /* pour les ellipses */
	gboolean transfo;
	gchararray title;
    GooCanvasPoints *l_pts;	
	
	const gdouble to_degrees = 180/ PI_PERSO; 
				
	angle1=(gdouble)(angle1*to_degrees);		
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;	
	for (i = 0; node = g_list_nth (list,i); i++) 
	  {
		  /* lire l'objet contenu dans *list */
		   g_object_get((GooCanvasItem *)(node->data), "title",&title,"width",&width,"height",&height,NULL);
		  /* saisie angle de rotation */
		   transfo=goo_canvas_item_get_simple_transform((GooCanvasItem *)(node->data),&xt,&yt,&echelle,&angle);	
		  /* création de l'espace mémoire */
		  objet=g_malloc(sizeof(ClipItem));
		  objet->path=NULL;
		  objet->titre_item=g_strdup_printf("%s",title);
		  objet->item=(GooCanvasItem *)(node->data);
		  objet->angle=angle1;
		  if (strncmp(title,"R",1)==0) 
           {
			 objet->objet=ITEM_RECT;   
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y; 
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }	
		  if  (strncmp(title,"P",1)==0) 
           {
			 objet->objet=ITEM_PATH;  	
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;		 
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		   
		  if  (strncmp(title,"U",1)==0) 
           {
			 objet->objet=ITEM_POLYLINE;  
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		
		  if  (strncmp(title,"B",1)==0)
           {
			 objet->objet=ITEM_BITMAP; 
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		      
		  if  (strncmp(title,"T",1)==0)
           {
			 objet->objet=ITEM_TXT; 
			 g_object_get((GooCanvasItem *)(node->data),"x",&x,"y",&y,NULL); 
			 objet->x=x;
			 objet->y=y;
			 objet->w=width;
			 objet->h=height;
			 undo->content=g_list_append(undo->content,objet);
		   }		   		   
		  if (strncmp(title,"L",1)==0)
           {
             l_pts= goo_canvas_points_new(2);	
			  objet->objet=ITEM_LINE;
             g_object_get((GooCanvasItem *)(node->data),"points",&l_pts,NULL);			 
			 objet->x1=l_pts->coords[0];
			 objet->y1=l_pts->coords[1];
			 objet->x2=l_pts->coords[2];
			 objet->y2=l_pts->coords[3];
			 undo->content=g_list_append(undo->content,objet);			 
             goo_canvas_points_unref(l_pts);			 		   
		   }	
		  if (strncmp(title,"C",1)==0)
           {
     	     g_object_get((GooCanvasItem *)(node->data),"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);	
			  objet->objet=ITEM_ELLIPSE;		   
			 objet->xc=xc;
			 objet->yc=yc;
			 objet->rx=rx;
			 objet->ry=ry;
			 undo->content=g_list_append(undo->content,objet);			 
		   }			   		   	  
      }/* next i*/				
}

/**********************************************
 * cancel a 'paste' operation
 * ******************************************/
void undo_paste_operation(void)
{
	gint i;
	GList *node;
							
	/* there is items in the undo buffer ? */
	if (undo==NULL)
	    return;
	/* which atom ? */
	if ((gint)(cancel *)(undo->atom)!=UNDO_PASTE_ITEMS)
	    return;			
	if ((cancel *)(undo->content)==NULL)
	    return;
	/* now, we can cancel the moving of all selected and moved items */
	for (i = 0; node = g_list_nth (undo->content,i); i++) 	
	  {	
		  goo_canvas_item_remove (((ClipItem*)(node->data))->item);
	  }/* next i*/
}
/********************************************
 * store an operation in order to undo
 * ****************************************/
void undo_store_datas(gint operation, GooCanvasItem *objet, GList *list, gdouble cx,gdouble cy,gdouble angle_rotation)
{
  /* flush the undo bufer */
  undo_reset();
  /* unset the undo sub menu */
  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("Nothing to _undo"));	
  gtk_widget_set_sensitive (p_undo_submenu, FALSE);
  gtk_widget_set_sensitive (p_undo_button, FALSE);
  /* set the new content of the undo buffer */
  switch (operation)
    {
		case UNDO_NEW_DRAWING:
		  {	  /* we only store a pointer on a GooCanvasItem */
			  undo->atom=UNDO_NEW_DRAWING;
			  undo->item=objet;
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo last drawing"));	              
			  break;
		  }
		case UNDO_CUT_ITEMS: 
		case UNDO_ERASE_ITEMS:
		  {
			  undo_copy(operation,list,cx,cy);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo delete items"));
			  break;
		  } 
		case UNDO_MOVING:
		  {
			  undo->atom=UNDO_MOVING;
			  undo_store_moving(list);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo moving"));
			  break;
		  }   
		case UNDO_SIZING:
		  {
			  undo->atom=UNDO_SIZING;
			  undo_store_sizing(list);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo resizing"));
			  break;
		  }	
		case UNDO_ROTATING:
		  {
			  undo->atom=UNDO_ROTATING;
			  undo_store_rotating(list,angle_rotation);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo rotating"));
			  break;
		  }		  	 
		case UNDO_PASTE_ITEMS:
		  {
			  undo->atom=UNDO_PASTE_ITEMS;
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo paste items"));
			  break;
		  }	   
		case UNDO_COLOR_CHANGE:
		  {
			  undo->atom=UNDO_COLOR_CHANGE;
			  undo_store_color_change(list);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo color change"));
			  break;
		  }	
		case UNDO_FONT_CHANGE:
		  {
			  undo->atom=UNDO_FONT_CHANGE;
			  undo_store_font_change(list);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo font change"));
			  break;
		  }	
		case UNDO_ALIGN_ITEMS:
		  {
			  undo->atom=UNDO_ALIGN_ITEMS;
			  undo_store_moving(list);
			  gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("_Undo align items"));
			  break;
		  }  		  		  
		default:;  
	}/* end switch */
  gtk_widget_set_sensitive (p_undo_submenu, TRUE);
  gtk_widget_set_sensitive (p_undo_button, TRUE);
}
/****************************************************
 * respond to an 'undo' inquiry
 * *************************************************/
gboolean undo_operation(void)
{
	/* we can only undo IF the undo buffer isn't empty */
	if (undo==NULL)
	   return TRUE;
	switch (undo->atom)
	  {
		  case UNDO_NEW_DRAWING:
		    {
				goo_canvas_item_remove (undo->item);
				undo_reset();
				break;
			}
		  case UNDO_CUT_ITEMS: 
		  case UNDO_ERASE_ITEMS:	
		     {
				 undo_paste(UNDO_CUT_ITEMS,rootKF);
				 undo_reset();
				 break;
			 }
		  case UNDO_MOVING:
		     {
				undo_move();
				undo_reset(); 
				break;
			 }	 
		  case UNDO_SIZING:
		     {
				undo_size();
				undo_reset(); 
				break;
			 }	
		  case UNDO_ROTATING:
		     {
				undo_rotate();
				undo_reset(); 
				break;
			 }	
		  case UNDO_PASTE_ITEMS:
		     {  
				 undo_paste_operation();
				 undo_reset();
				 break;
			 }
		  case UNDO_COLOR_CHANGE:
		     {
				 undo_color_change();
				 undo_reset();
				 break;
			 }	 
		  case UNDO_FONT_CHANGE:
		     {
				 undo_font_change();
				 undo_reset();
				 break;
			 }
		   case UNDO_ALIGN_ITEMS:
		     {
				undo_move();
				undo_reset(); 
				break;
			 }		 		 	  			  			 
		  default:;	
	  }/* end switch*/	
	/* and we unset the 'undo' submenu */
	gtk_menu_item_set_label(GTK_MENU_ITEM (p_undo_submenu),(const gchar *)gettext("Nothing to _undo"));	
	gtk_widget_set_sensitive (p_undo_submenu, FALSE);
	gtk_widget_set_sensitive (p_undo_button, FALSE);
	return TRUE;
}
