/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/
//translations
#include <locale.h>
#include <libintl.h>

#include <stdlib.h>
#include <gtk/gtk.h>
#include "gdk/gdkkeysyms.h"
#include <string.h>
#include <math.h>
#include <goocanvas.h>
#include "clipboard.h"
#include "editeur.h"
#include "dlgediteur.h"
#include "ellasyms.h"
#include "cursor.h"

/* codes des objets :
 * B = image bitmap
 * F = fond de page
 * K =  calque
 * C = ellipse/cercle
 * L = ligne simple
 * R = rectangle
 * P = path (filled)
 * U = path (not filled)
 * /
/* compiler ainsi : gcc `pkg-config --cflags --libs gtk+-2.0 goocanvas` -o editeur editeur.c */
/* XPM */
/* XPM */
#include "./icones_rcs/tool_path.xpm"
#include "./icones_rcs/editor_img.xpm"



/* liste (pile) stockant les noms d'onjets et les pixbufs associés */
 //GSList *pile_images=NULL;	  
#include "interface_editor.h"

 /* éditeur */
 static GtkWidget *labelRotationAngle;
 static GtkWidget *labelMousePos;
 static GtkWidget *labelMouseTips;
 GtkWidget *pToolbarEditor;
 GtkToolItem *pEditorModeItem; /* obligatoire pour récupérer le pointeur pour forcer l'activation */
 static guint EditorMode=EDITOR_MODE_SELECT; /* 0= mode sélection */
 static fAutoPlacement=FALSE;/* flag for autoplacement of items */
 static fJustSelected=FALSE;/* in order to avoid auto call to change color of a multislection */
 static gdouble AutoPLacementScale=5.0;/* scale for the virtual scale used for autplacement of items */
 extern gboolean SelectedArea; /* indique si une sélection est en cours */
 static gdouble xev,yev;/* obligé car je ne me tire pas des pointeurs dans certains cas */
 static gdouble x1sel,y1sel,x2sel,y2sel,wsel,hsel; /* coordonnées utilisées de la sélection courante */
 static gdouble x1move,y1move; /* coordonnées pour la translation des items */
 static gdouble x1drw,y1drw,x2drw,y2drw,x_delta,y_delta; /* coordonnées utilisées pour les tracés */
 static gboolean fdrawing=FALSE;/* flag pour indiquer s'il faut tracer ou pas */
 static gboolean fDontRespondToClick=FALSE;
 static gboolean fresizing=FALSE; /* flag indiquant si l'on doit suivre la souris */
 static gboolean frotation=FALSE; /* flag indiquant si une rotation est en cours */
 static gboolean fDoubleClick=FALSE; /* flag in order to cancel a release event after a double-click */
 static gboolean fmoved=FALSE; /* flag indiquant si la souris a bougé */
 extern GList *pSelectedItemsList; /* liste des items, mais VALIDE, sans le fond et l'origine */
 static gboolean fClipboardFull=FALSE; /* flag indiquant si le presse-papiers contient un élément */
 static gdouble editor_canvas_max_width= 550; /* largeur maximum du canvas de l'éditeur vectoriel */
 static gdouble editor_canvas_max_height= 440; /* hauteur maximum du canvas de l'éditeur vectoriel */
 guint ItemCounter=0; /* compte le nombre d'objets et permet de les numéroter/classer */
 /* couleurs */
 static GdkColor PenColor ={0, 0, 0, 0};/* noir par défaut */
 static GdkColor FillColor ={0, 65000, 0, 0};/* rouge par défaut */;
 static GdkColor PageColor={0, 65535, 65535, 65535}; /* fond de page blanc par défaut */
 static gint Alpha=100;/* canal alpha*/
 /* styles de traits */
 static gdouble LineWidth = 1.0; /* 1 pixel par défaut */
 static gdouble XLineRound = 1.0;  /* arrondi horizontal */
 static gdouble YLineRound = 1.0; /* arrondi vertical */
 /* fontes */
 static PangoFontDescription *FntDesc=NULL;
 /* valeurs par défaut */
 static gdouble editor_default_width;
 static gdouble editor_default_height;
 static gdouble editor_default_margins;
 static GdkColor editor_defaut_page_color;
 /* flag indiquant si c'est un 1er appel aux entrées de l'onglet "propriétés" */
 static fPropertiesEntries=TRUE;/* pour éviter l'appel lors de l'initialisation des champs */
 static gdouble xmem,ymem,wmem,hmem;/* mémoriser les valeurs pour annuler un changement */
 static gint PropertiesChange=0;/* stocke le type de changement : 0=rien, 1=position, 2=dimensions */
 /* pointeur local sur les widgets 'style' de l'onglet propriétés */
 static gpointer *widgets_styles;
 gint r_rempli_mem,v_rempli_mem,b_rempli_mem,r_cont_mem,v_cont_mem,b_cont_mem,a_rempli_mem,a_cont_mem;
 gdouble ep_ligne_mem;

  static GtkWidget *pWinMemo;
 /* forwards */
 void selection_box(GooCanvasItem *ancetre, gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t);
 GList *get_items_selected(GooCanvasItem *item,
                           gdouble x1z, gdouble y1z, gdouble x2z, gdouble y2z);
 gdouble resize_compute(gdouble before, gdouble after);
 void item_move_to(GooCanvasItem *item, gdouble xtran, gdouble ytran);
 void item_resize(GooCanvasItem *item, gdouble xorg,gdouble yorg,gdouble h, gdouble v);
 gboolean cb_editor_button_press(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win);
 gdouble correct_x(gdouble x);		      
 gdouble correct_y(gdouble y);
 static gboolean fFirstCall=TRUE;
 static gboolean fFirstCallPropertiesStyles=TRUE;
 static gint NbElementsSelectedItemsList;
 static gint NbCallsToReject;
 /* pointers on color preview of notification area */
  static 	GtkWidget *preview_Pp;
  static 	GtkWidget *preview_Ap;
 void item_rotate(GooCanvasItem *item, gdouble angle);
 void editor_copy_items(void);
 void editor_paste_items(void);
 /* vars for path objects */
 gdouble xmem_path,ymem_path;
 gint NbElemPath=0;
 GSList *ListElemPath=NULL;

 /*Force the ratio when we resize an object with the mouse*/
 int force_ratio = 0;
 

/* utilitaires */
/******************************************
 * with a coordinate, computes a new
 * coordinate indexed on the autoplacement
 * scale
 * ****************************************/
gdouble get_autoplaced_coordinate(gdouble valeur, gdouble pas_grille)
{
	valeur=(gdouble) ((gint)valeur / (gint)pas_grille);/* division entière */
	valeur=(gdouble)valeur*pas_grille;/* arrondie au plus proche inférieur */
	return valeur;
}
/******************************************
 * callback for autoplacement check button
 * ****************************************/
void
on_checkbuttonEditorAttractionMagnetic_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean etat;
	
	etat=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(togglebutton));
	fAutoPlacement=etat;	
}										
/*************************************
 * stocke en local le pointeur sur
 * les widgets des styles
 * **********************************/
void editor_store_pointer_to_styles(gpointer *p)
{
	widgets_styles=p;
}
/**************************************
 * donne le nombre d'éléments d'une
 * GList
 * ***********************************/
gint get_list_items_number(GList *list)
{
	gint nbelem=0;
	gint count=0;

    if (list!=NULL)
	   {
		   for(count=0;g_list_nth(list,count);count++)
		      {
				  nbelem++;
			  }
	   }	
	// printf("nombre éléments de la liste :%d\n",nbelem);  
	return nbelem;
}
/***************************************
 * gestion des entries des propriétés
 * en entrée : pointeur sur
 *  pSelectedItemsList
 * et coordonnées des 2 coins du 
 * rectangle de sélection suivi des
 * dimensions de ce rectangle
 * ************************************/
void editor_set_entries(GList *list,gdouble x,gdouble y,gdouble w,gdouble h)
{
	gint n_elems;
	
	if (list==NULL)
	   return;
	/* combien d'éléments ? */
	n_elems=get_list_items_number(list);	
	/* quelquesoit le nbre élements, les dimensions sont à afficher */
	x=correct_x(x)-EDITOR_CANVAS_MARGINS;
	y=correct_y(y)-EDITOR_CANVAS_MARGINS;
    gtk_entry_set_text(GTK_ENTRY(entryXPosProperties),(gchar *)g_strdup_printf("%d",(gint)x));
    gtk_entry_set_text(GTK_ENTRY(entryYPosProperties),(gchar *)g_strdup_printf("%d",(gint)y));
    gtk_entry_set_text(GTK_ENTRY(entryWidthProperties),(gchar *)g_strdup_printf("%d",(gint)w));
    gtk_entry_set_text(GTK_ENTRY(entryHeightProperties),(gchar *)g_strdup_printf("%d",(gint)h));	
	/* préserver les dimensions si demande d'annulation */
	xmem=x;	ymem=y;
	wmem=w;	hmem=h;
	fPropertiesEntries=TRUE; /* pour éviter un double appel */
	gtk_widget_set_sensitive(buttonRazDimProperties,FALSE);
	PropertiesChange=PROP_CANCEL_NONE;
	if (n_elems>1)
	  {
		  /* si nbelem >1 alors moins de possibilités ; juste régler les dimensions extrêmes */
	  }
	else   
	    {
		}
}
/******************************************
 * 2 callbacks for expose events of
 * notification zone of editor
 * ***************************************/
void editor_react_notification_area_fillcolor_expose_event(GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         *user_data)
{
  cairo_t *cr1;
     
	  
  cr1 = gdk_cairo_create(widget->window);	
  cairo_set_source_rgb(cr1,(gdouble)(FillColor.red/65535.0),
                         (gdouble)(FillColor.green/65535.0),
						 (gdouble)(FillColor.blue/65535.0));
  cairo_rectangle(cr1,0,0,12,20);
  cairo_fill(cr1);
  cairo_destroy(cr1);
}					
void editor_react_notification_area_strokecolor_expose_event(GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         *user_data)
{
  cairo_t *cr1;
  	  
  cr1 = gdk_cairo_create(widget->window);	
  cairo_set_source_rgb(cr1,(gdouble)(PenColor.red/65535.0),
                         (gdouble)(PenColor.green/65535.0),
						 (gdouble)(PenColor.blue/65535.0));
  cairo_rectangle(cr1,0,0,12,20);
  cairo_fill(cr1);
  cairo_destroy(cr1);	
}						
/*********************************************
 * callbacks for the two preview areas of the
 * styles
 * *******************************************/
void editor_react_styles_area_fillcolor_expose_event(GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         *user_data)
{
	cairo_t *cr_rempli;
	guint32 tmp_color;
	gint r_rempli,v_rempli,b_rempli,a_rempli;
	GtkWidget *area_rempli;
	GtkWidget *spb_rr;
	GtkWidget *spb_br;
	GtkWidget *spb_vr;
	GtkWidget *spb_ar;

	/* passage des pointeurs depuis la structure */
	spb_rr=user_data[0];
	spb_vr=user_data[1];
	spb_br=user_data[2];
	spb_ar=user_data[3];	
 		
	area_rempli=widgets_styles[8];
	cr_rempli = gdk_cairo_create(area_rempli->window);

	r_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_rr));
	v_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_vr));
	b_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_br));
	a_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_ar));
	/* dessiner les previews */
    cairo_set_source_rgba(cr_rempli, (gdouble)(r_rempli/255.0),
                          (gdouble)(v_rempli/255.0),
                          (gdouble)(b_rempli/255.0),
						  (gdouble)(a_rempli/100.0));
    cairo_rectangle(cr_rempli,0,0,24,24);
    cairo_fill(cr_rempli);
	cairo_destroy(cr_rempli);					
}
void editor_react_styles_area_strokecolor_expose_event(GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         *user_data)
{
	cairo_t *cr_cont;
	guint32 tmp_color;
	gint r_cont,v_cont,b_cont,a_cont;	
	GtkWidget *spb_rc;
	GtkWidget *spb_bc;
	GtkWidget *spb_vc;
	GtkWidget *spb_ac;
	GtkWidget *area_cont;
	
	spb_rc=user_data[4];
	spb_vc=user_data[5];
	spb_bc=user_data[6];
	spb_ac=user_data[7];	
	area_cont=user_data[9];	
    cr_cont = gdk_cairo_create(area_cont->window);
	r_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_rc));
	v_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_vc));
	b_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_bc));
	a_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_ac));
    cairo_set_source_rgba(cr_cont, (gdouble)(r_cont/255.0),
                          (gdouble)(v_cont/255.0),
                          (gdouble)(b_cont/255.0),
						  (gdouble)(a_cont/100.0));
    cairo_rectangle(cr_cont,0,0,24,24);
    cairo_fill(cr_cont);	
	cairo_destroy(cr_cont);					
}
/***********************************************
 * gestion des éléments de l'onglet propriétés
 * consacré aux 'styles' ; en entrée, un pointeur
 * sur la GList contenant la sélection
 * ********************************************/
void editor_set_entries_styles(GList *list, gint nbre)
{
	cairo_t *cr_rempli,*cr_cont; 
	guint32 tmp_color;
	gdouble ep_trait;
	gint r_rempli,v_rempli,b_rempli,a_rempli;
	gint r_cont,v_cont,b_cont,a_cont;
	gint count;
	GtkWidget *spb_ep_trait;
	GtkWidget *spb_rr;
	GtkWidget *spb_br;
	GtkWidget *spb_vr;
	GtkWidget *spb_rc;
	GtkWidget *spb_bc;
	GtkWidget *spb_vc;
	GtkWidget *spb_ar;
	GtkWidget *spb_ac;
	GtkWidget *widget;
	GtkWidget *area_rempli,*area_cont;
	GtkWidget *button_annul;
	GList *node;
	gchararray title;
	
	if (list==NULL)
	    return;
	/* passage des pointeurs depuis la structure */
	spb_rr=widgets_styles[0];
	spb_vr=widgets_styles[1];
	spb_br=widgets_styles[2];
	spb_ar=widgets_styles[3];
	spb_rc=widgets_styles[4];
	spb_vc=widgets_styles[5];
	spb_bc=widgets_styles[6];
	spb_ac=widgets_styles[7];	
	area_rempli=widgets_styles[8];
	area_cont=widgets_styles[9];
	spb_ep_trait=widgets_styles[10];	
	button_annul=widgets_styles[11];	
	/* récupérer les valeurs actuelles, uniquement sur le 1er élément */
	node = g_list_nth (list, 0);
	g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);
	if (title==NULL)
	   return; 
	/* zones de preview */
	cr_rempli = gdk_cairo_create(area_rempli->window);
    cr_cont = gdk_cairo_create(area_cont->window);   
	/* valeurs par défaut */
	ep_trait=1.0;
	r_rempli=255;
	v_rempli=255;
	b_rempli=255;
	r_cont=0;
	v_cont=0;
	b_cont=0;
	a_rempli=100;
	a_cont=100;
	if ((strncmp("A",title,1)!=0)&&(strncmp("K",title,1)!=0)&&(strncmp("F",title,1)!=0))
			    {
					/* lire la valeur de couleur de remplissage */
					g_object_get((GooCanvasItem *)(node->data),"fill-color-rgba",&tmp_color,NULL);					                            
					r_rempli=(gint)(((tmp_color & 0xFF000000)>>24)/255.0*255);
                    v_rempli=(gint)(((tmp_color & 0x00FF0000)>>16)/255.0*255);
					b_rempli=(gint)(((tmp_color & 0x0000FF00)>>8)/255.0*255);					
					a_rempli=(gint)((tmp_color  & 0x000000000000FF)/255.0*100);
					g_object_get((GooCanvasItem *)(node->data),"stroke-color-rgba",&tmp_color,NULL);					                            
					r_cont=(gint)(((tmp_color & 0xFF000000)>>24)/255.0*255);
                    v_cont=(gint)(((tmp_color & 0x00FF0000)>>16)/255.0*255);
					b_cont=(gint)(((tmp_color & 0x0000FF00)>>8)/255.0*255);					
					a_cont=(gint)((tmp_color  & 0x000000000000FF)/255.0*100);
					g_object_get((GooCanvasItem *)(node->data),"line-width",&ep_trait,NULL);
					/* stockage valeurs par défaut */
					r_rempli_mem=r_rempli;
					v_rempli_mem=v_rempli;
					b_rempli_mem=b_rempli;
					r_cont_mem=r_cont;
					v_cont_mem=v_cont;
					b_cont_mem=b_cont;
					a_rempli_mem=a_rempli;
					a_cont_mem=a_cont;
					ep_ligne_mem=ep_trait;
				}
	/* dessiner les previews */
    cairo_set_source_rgba(cr_rempli, (gdouble)(r_rempli/255.0),
                          (gdouble)(v_rempli/255.0),
                          (gdouble)(b_rempli/255.0),
						  (gdouble)(a_rempli/100.0));
    cairo_rectangle(cr_rempli,0,0,24,24);
    cairo_fill(cr_rempli);
    cairo_set_source_rgba(cr_cont, (gdouble)(r_cont/255.0),
                          (gdouble)(v_cont/255.0),
                          (gdouble)(b_cont/255.0),
						  (gdouble)(a_cont/100.0));
    cairo_rectangle(cr_cont,0,0,24,24);
    cairo_fill(cr_cont);	
	cairo_destroy(cr_rempli);
	cairo_destroy(cr_cont);		
	NbCallsToReject=nbre;		
	/* il faut stocker dans les widgets les valeurs actuelles ; cela provoque un événement */
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ep_trait),ep_trait);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_rr),(gdouble)r_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_vr),(gdouble)v_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_br),(gdouble)b_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_rc),(gdouble)r_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_vc),(gdouble)v_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_bc),(gdouble)b_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ac),(gdouble)a_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ar),(gdouble)a_rempli);
	gtk_widget_set_sensitive(button_annul,FALSE);
}
/***********************************************
 * réactions aux changements dans les éléments
 * (gtkentries) de la partie 'style' de l'onglet
 * 'propriétés' ; appelé par la fonction 
 * callback de chaque élément ;
 * le paramètre est une "structure de pointeurs"
 * *********************************************/
void editor_react_properties_styles_entries(GtkEditable     *editable,
											gpointer *user_data)
{   
	cairo_t *cr_rempli,*cr_cont; 
	gdouble ep_trait;
	gint r_rempli,v_rempli,b_rempli,a_rempli;
	gint r_cont,v_cont,b_cont,a_cont;
	gint count;
	GtkWidget *spb_ep_trait;
	GtkWidget *spb_rr;
	GtkWidget *spb_br;
	GtkWidget *spb_vr;
	GtkWidget *spb_rc;
	GtkWidget *spb_bc;
	GtkWidget *spb_vc;
	GtkWidget *spb_ar;
	GtkWidget *spb_ac;
	GtkWidget *widget;
	GtkWidget *area_rempli,*area_cont;
	GtkWidget *button_annul;
	GList *node;
	gchararray title;
	
	if (NbCallsToReject>=0)
	   {
		    //printf("rejet entries ! \n");
		    NbCallsToReject--;
		   return;
	   }
	// printf("je réponds au cb styles \n");  
	/* passage des pointeurs depuis la structure */
	spb_rr=user_data[0];
	spb_vr=user_data[1];
	spb_br=user_data[2];
	spb_ar=user_data[3];
	spb_rc=user_data[4];
	spb_vc=user_data[5];
	spb_bc=user_data[6];
	spb_ac=user_data[7];	
	area_rempli=user_data[8];
	area_cont=user_data[9];
	spb_ep_trait=user_data[10];
    button_annul=user_data[11];
	
	cr_rempli = gdk_cairo_create(area_rempli->window);
    cr_cont = gdk_cairo_create(area_cont->window);
	
	/* lire les valeurs */
	r_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_rr));
	v_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_vr));
	b_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_br));
	r_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_rc));
	v_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_vc));
	b_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_bc));
	a_cont=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_ac));
	a_rempli=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spb_ar));
	ep_trait=gtk_spin_button_get_value(GTK_SPIN_BUTTON(spb_ep_trait));
    /* dessiner les previews */
    cairo_set_source_rgba(cr_rempli, (gdouble)(r_rempli/255.0),
                          (gdouble)(v_rempli/255.0),
                          (gdouble)(b_rempli/255.0),
						  (gdouble)(a_rempli/100.0));
    cairo_rectangle(cr_rempli,0,0,24,24);
    cairo_fill(cr_rempli);
    cairo_set_source_rgba(cr_cont, (gdouble)(r_cont/255.0),
                          (gdouble)(v_cont/255.0),
                          (gdouble)(b_cont/255.0),
						  (gdouble)(a_cont/100.0));
    cairo_rectangle(cr_cont,0,0,24,24);
    cairo_fill(cr_cont);	
	cairo_destroy(cr_rempli);
	cairo_destroy(cr_cont);		
	/* transmettre les valeurs aux objets */
	if (pSelectedItemsList)
	  {
	   if (!fJustSelected)
	     {
			gtk_widget_set_sensitive(button_annul,TRUE);
			goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
			for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
				{          	                          
					/* passer les valeurs changées aux objets sélectionnés */
					/* faire un tesst sur l'item */
					g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);
					if ((strncmp("A",title,1)!=0)&&(strncmp("K",title,1)!=0)&&(strncmp("F",title,1)!=0))
						{	                                 
							g_object_set((GooCanvasItem *)(node->data),
								"stroke-color-rgba",
								convert_color_to_rgba((gdouble)(r_cont/255.0),
					                         (gdouble)(v_cont/255.0), 
											 (gdouble)(b_cont/255.0),
											 (gdouble)(a_cont/100.0)),
												NULL);
							g_object_set((GooCanvasItem *)(node->data),
									"fill-color-rgba",
									convert_color_to_rgba((gdouble)(r_rempli/255.0),
					                         (gdouble)(v_rempli/255.0), 
											 (gdouble)(b_rempli/255.0),
											 (gdouble)(a_rempli/100.0)),
									NULL);	  
							g_object_set((GooCanvasItem *)(node->data), "line-width",ep_trait,NULL);	   
						}/* if pas ITEM A, c'est à dire bordure */			
				} /* for */  
			selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);   
		}/* endif fJustSelected=FALSE */
	   else
	     fJustSelected=FALSE;/* for next call */
	  }/* endif pSelecetdItemsList */
}
/*****************************************
 * annuler les changements de styles
 * ***************************************/
void editor_undo_entries_styles(GList *list)
{
	guint32 tmp_color;
	cairo_t *cr_rempli,*cr_cont; 
	gdouble ep_trait;
	gint r_rempli,v_rempli,b_rempli,a_rempli;
	gint r_cont,v_cont,b_cont,a_cont;
	gint count;
	GtkWidget *spb_ep_trait;
	GtkWidget *spb_rr;
	GtkWidget *spb_br;
	GtkWidget *spb_vr;
	GtkWidget *spb_rc;
	GtkWidget *spb_bc;
	GtkWidget *spb_vc;
	GtkWidget *spb_ar;
	GtkWidget *spb_ac;
	GtkWidget *widget;
	GtkWidget *area_rempli,*area_cont;
	GtkWidget *button_annul;
	
	if (list==NULL)
	    return;
	/* passage des pointeurs depuis la structure */
	spb_rr=widgets_styles[0];
	spb_vr=widgets_styles[1];
	spb_br=widgets_styles[2];
	spb_ar=widgets_styles[3];
	spb_rc=widgets_styles[4];
	spb_vc=widgets_styles[5];
	spb_bc=widgets_styles[6];
	spb_ac=widgets_styles[7];	
	area_rempli=widgets_styles[8];
	area_cont=widgets_styles[9];
	spb_ep_trait=widgets_styles[10];	
	button_annul=widgets_styles[11];	
	/* zones de preview */
	cr_rempli = gdk_cairo_create(area_rempli->window);
    cr_cont = gdk_cairo_create(area_cont->window);   
	/* valeurs par défaut */
	ep_trait=ep_ligne_mem;
	r_rempli=r_rempli_mem;
	v_rempli=v_rempli_mem;
	b_rempli=b_rempli_mem;
	r_cont=r_cont_mem;
	v_cont=v_cont_mem;
	b_cont=b_cont_mem;
	a_rempli=a_rempli_mem;
	a_cont=a_cont_mem;
	
	/* dessiner les previews */
    cairo_set_source_rgba(cr_rempli, (gdouble)(r_rempli/255.0),
                          (gdouble)(v_rempli/255.0),
                          (gdouble)(b_rempli/255.0),
						  (gdouble)(a_rempli/100.0));
    cairo_rectangle(cr_rempli,5,5,64,32);
    cairo_fill(cr_rempli);
    cairo_set_source_rgba(cr_cont, (gdouble)(r_cont/255.0),
                          (gdouble)(v_cont/255.0),
                          (gdouble)(b_cont/255.0),
						  (gdouble)(a_cont/100.0));
    cairo_rectangle(cr_cont,5,5,64,32);
    cairo_fill(cr_cont);	
	cairo_destroy(cr_rempli);
	cairo_destroy(cr_cont);					
	/* il faut stocker dans les widgets les valeurs actuelles ; cela provoque un événement */
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ep_trait),ep_trait);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_rr),(gdouble)r_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_vr),(gdouble)v_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_br),(gdouble)b_rempli);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_rc),(gdouble)r_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_vc),(gdouble)v_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_bc),(gdouble)b_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ac),(gdouble)a_cont);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spb_ar),(gdouble)a_rempli);
	gtk_widget_set_sensitive(button_annul,FALSE);
}
/******************************************
 * annuler les changements dans l'onglet
 * propriétés, section 'position' & taille
 * ***************************************/
void editor_cancel_position_size_change(void)
{
   gint count;
   GList *node;
   gdouble x,y,w,h,th,tv;
   gchar *valeur;
   	
	switch (PropertiesChange)
	  {
		case PROP_CANCEL_NONE:
		   break;  
		case PROP_CANCEL_POS:
		  {
			x=xmem+EDITOR_CANVAS_MARGINS;	
			y=ymem+EDITOR_CANVAS_MARGINS;
            if ((pSelectedItemsList) && (x>=0))
	          {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
                        {          	                          
                          item_move_to((GooCanvasItem *)(node->data),x-x1sel,y-y1sel);
                        } /* for */ 	
					 x1sel=x1sel+(x-x1sel);
					 x2sel=x1sel+wsel;
					 y1sel=y1sel+(y-y1sel);
					 y2sel=y1sel+hsel;
					 selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);
					 editor_set_entries(pSelectedItemsList,x1sel,y1sel,wsel,hsel);
					 //editor_set_entries_styles(pSelectedItemsList);
			  }/* endif */				  
			PropertiesChange=PROP_CANCEL_NONE;  
			gtk_widget_set_sensitive(buttonRazDimProperties,FALSE);
		    break;	  
		  }
		case PROP_CANCEL_SIZE:
		  {
			/* calculer la nouvelle taille */
  	  	    th=resize_compute(wsel,wmem);
  	  	    tv=resize_compute(hsel,hmem); 
            if (pSelectedItemsList)
	          {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
                        {          	                          
                          item_resize((GooCanvasItem *)(node->data),x1sel,y1sel,th,tv);
                        } /* for */ 	
					 x2sel=x1sel+wmem;
					 y2sel=y1sel+hmem;
					 wsel=wmem;
					 hsel=hmem;
					 selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);
					 editor_set_entries(pSelectedItemsList,x1sel,y1sel,wsel,hsel);
			  }/* endif */				  
			PropertiesChange=PROP_CANCEL_NONE;  
			gtk_widget_set_sensitive(buttonRazDimProperties,FALSE);			
			break;  
		  }
		default:printf("inattendu ! \n");    
	  }/* end switch */  	  
}
/*****************************************
 * réaction à un appel de 'callback' de
 * l'onglet 'propriétés'
 * limité aux entries de position et
 * dimensions ; on passe en paramètre le
 * n° du champ concerné
 * **************************************/
void editor_react_properties_entries(GtkEditable *editable, gint champ)
{
   gint count;
   GList *node;
   gdouble x,y,w,h,ratio,coeff_redim,th,tv;
   gchar *valeur;
   gboolean contrainte_ratio=FALSE;
 
    contrainte_ratio=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbuttonConstrainProperties));
	ratio=wmem/hmem;
	switch (champ)
	  {
		  case FIELD_X:
		    {
				/* lire la valeur de l'entrée */
				valeur=(gchar *)gtk_entry_get_text(GTK_ENTRY(entryXPosProperties));
				x=atof(valeur)+EDITOR_CANVAS_MARGINS;	
                if ((pSelectedItemsList) && (x>=0))
	                {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
                        {          	                          
                          item_move_to((GooCanvasItem *)(node->data),x-x1sel,0);
                        } /* for */ 	
					 x1sel=x1sel+(x-x1sel);
					 x2sel=x1sel+wsel;
					 selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);
				    }/* endif */	
				gtk_widget_set_sensitive(buttonRazDimProperties,TRUE);	
				PropertiesChange=PROP_CANCEL_POS;
				gtk_entry_set_text(GTK_ENTRY(entryXPosProperties),valeur);
				// g_free interdit sur *valeur, car stocké par Gtk 				
				break;
			}
		  case FIELD_Y:
		    {
				valeur=(gchar *)gtk_entry_get_text(GTK_ENTRY(entryYPosProperties));
				y=atof(valeur)+EDITOR_CANVAS_MARGINS;		   
                if ((pSelectedItemsList)&&(y>=0))
	                {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
                        {          	                          
                          item_move_to((GooCanvasItem *)(node->data),0,y-y1sel);
                        } /* for */ 
					  y1sel=y1sel+(y-y1sel);
					  y2sel=y1sel+hsel;
					 selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);					  	
				    }
				gtk_widget_set_sensitive(buttonRazDimProperties,TRUE);	
				PropertiesChange=PROP_CANCEL_POS;
				gtk_entry_set_text(GTK_ENTRY(entryYPosProperties),valeur);
				break;
			}
		  case FIELD_W:
		    {
				valeur=(gchar *)gtk_entry_get_text(GTK_ENTRY(entryWidthProperties));
				w=atof(valeur);
#ifdef DEBUG
				printf("New height: %f\n", w);
#endif
				coeff_redim=w/wmem;
				if (contrainte_ratio)
				{
#ifdef DEBUG
					printf("Ratio enabled.\n");
#endif
				   h=w/ratio;
				}
				/* calculer la nouvelle taille */
#ifdef DEBUG
                printf("wmem: %f hmem: %f\n", wmem, hmem);
#endif DEBUG
  	  	        th=resize_compute(wmem,w);
  	  	        tv=resize_compute(hmem,h);
#ifdef DEBUG
                printf("Computed size: %fx%f\n", th, tv);
#endif
				if (pSelectedItemsList)
	                {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++)	
					    {
							item_resize((GooCanvasItem *)(node->data),x1sel,y1sel,th,tv);
					    }/* next*/
					  wsel=wsel*coeff_redim;
					  if (contrainte_ratio)
					     hsel=wsel/ratio;
					  x2sel=x1sel+wsel;
					  y2sel=y1sel+hsel;	 
					  selection_box(rootKF,x1sel,y1sel,x2sel,y2sel);	 	
					}/* endif*/   
				gtk_entry_set_text(GTK_ENTRY(entryWidthProperties),
				            (gchar *)g_strdup_printf("%d",(gint)wsel));
                gtk_entry_set_text(GTK_ENTRY(entryHeightProperties),
				            (gchar *)g_strdup_printf("%d",(gint)hsel));
				gtk_widget_set_sensitive(buttonRazDimProperties,TRUE);	
				PropertiesChange=PROP_CANCEL_SIZE;				
				break;
			}
		  case FIELD_H:
		    {
				valeur=(gchar *)gtk_entry_get_text(GTK_ENTRY(entryHeightProperties));
				h=atof(valeur);
#ifdef DEBUG
				printf("New height: %f\n", h);
#endif
				coeff_redim=h/hmem;
				if (contrainte_ratio)
				{
#ifdef DEBUG
                    printf("Ratio enabled.\n");
#endif
				   w=h*ratio;
				}
				/* calculer la nouvelle taille */
                /*th is > 1 if wmem < w, th = w/wmem*/
                /*tv is > 1 if hmem < h, tv = h/hmem*/
  	  	        th=resize_compute(wmem,w);
  	  	        tv=resize_compute(hmem,h);
#ifdef DEBUG
                printf("Computed size: %fx%f\n", th, tv);
#endif
				if (pSelectedItemsList)
	                {
					  goo_canvas_item_remove(pDrawings);/* effacer rectangle de sélection */
	  	              for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++)	
					    {
							item_resize((GooCanvasItem *)(node->data),x1sel,y1sel,th,tv);
					    }/* next*/
					  hsel=hsel*coeff_redim;	
					  if (contrainte_ratio)
					     wsel=hsel*ratio;
					  x2sel=x1sel+wsel;
					  y2sel=y1sel+hsel;	 
					  selection_box(rootKF,x1sel,y1sel,x2sel,y2sel); 
					}/* endif */  
				gtk_entry_set_text(GTK_ENTRY(entryWidthProperties),
				            (gchar *)g_strdup_printf("%d",(gint)wsel));
                gtk_entry_set_text(GTK_ENTRY(entryHeightProperties),
				            (gchar *)g_strdup_printf("%d",(gint)hsel));
				gtk_widget_set_sensitive(buttonRazDimProperties,TRUE);	
				PropertiesChange=PROP_CANCEL_SIZE;	 
				break;
			}
		  default:printf("cas non prévu ! \n"); 			
	  }/* end switch */
      
      /*We saved the new value into wmem and hmem, for the next time ^^*/
      wmem = w;
      hmem = h;
}
/**************************************
 * appels callbacks des fonctions de
 * rotation arbitraire ;
 * attention : GooCanvas fonctionne
 * en degrés TRIGO, et ma fonction 
 * rotate fonctionne en radians.
 * ************************************/
void editor_rotate(gdouble angle)
 {
  gint count;
  GList *node;
  gdouble radians;
  
  if (angle==90)
     angle=PI_PERSO/2;
  if (angle==-90)
     angle=-PI_PERSO/2;	
  if (angle==180)
     angle=PI_PERSO;
  if (angle==270)
     angle=PI_PERSO+PI_PERSO/2;	  
  if (pSelectedItemsList)
       {
		   	   goo_canvas_item_remove(pDrawings);	
	           for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
                {          	                          
                  item_rotate((GooCanvasItem *)(node->data),angle);
                } /* for */ 
  	  	       g_list_free(pSelectedItemsList);
               pSelectedItemsList=NULL; 
			   SelectedArea=FALSE;
       }
}
/**** manipulation de la pile des images ****/
/**********************
 * RAZ pile des images
 * ********************/
void raz_pile_images(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
	
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		g_free(((stack_images *)(node->data))->nom);
		// g_free(((stack_images *)(node->data))->complete_path); 
		g_object_unref(((stack_images *)(node->data))->pixb); 
        g_free(((stack_images *)(node->data)));
      }/* for */
   g_slist_free (list);
}
/*********************************
 * ajouter un item sur la pile
 * des images
 * ******************************/
GSList  *add_pile_images(GSList *list, gchar *titre, GdkPixbuf *pix)
{
 GSList *temp;
 stack_images *temp_data;
 
 printf("pile image titre:%s\n",titre);
 temp_data = g_malloc(sizeof(stack_images));
 temp_data->nom = g_strdup(titre); 
 temp_data->pixb = pix;
 // temp_data->complete_path=g_strdup_printf("%s",path_to_image);
 temp = g_slist_append (list,temp_data);
 return temp;	
}

/**********************************
 * fonction ad_hoc pour obtenir
 * un pointeur sur la GSList de
 * la pile des images
 * *******************************/
GSList *editor_get_images_stack(void)
{
	return pile_images;
}

/***********************************
 extraire une sous-chaîne
 attention : il faut libérer
 la mémoire APRES avec 
g_free (output);
***********************************/
gchar *g_substr ( const gchar* string, gint start,gint  end)
{
  gsize len = (end - start + 1);
  gchar *output = g_malloc0 (len + 1);
  return g_utf8_strncpy (output, &string[start], len);
}

void editor_item_set_name(GooCanvasItem *item,gchar *name, gchar *path)
{
	gchar *titre;
	
	titre=g_strdup_printf("%s%d",name,ItemCounter);
	if (path!=NULL)
	   {
		   titre=g_strconcat (titre,path, NULL); ;
	   }
	g_object_set(G_OBJECT(item),"title",titre,NULL);
	printf("sous editeur --titre:%s<\n",titre);
	//g_free(titre); NO !!!!
}
/****************************************
 * convertir données couleurs
 * séparées RGB+A (en valeur flottante 0-1)
 * vers du 32 bits RGB ;
 * thanks to : Damon Chaplin (goocanvas)
 * ***************************************/
guint
convert_color_to_rgba (gdouble red, gdouble green, gdouble blue, gdouble alpha)
{
  guint red_byte, green_byte, blue_byte, alpha_byte;

  red_byte = red * 256;
  red_byte -= red_byte >> 8;

  green_byte = green * 256;
  green_byte -= green_byte >> 8;

  blue_byte = blue * 256;
  blue_byte -= blue_byte >> 8;

  alpha_byte = alpha * 256;
  alpha_byte -= alpha_byte >> 8;

  return (red_byte << 24) + (green_byte << 16) + (blue_byte << 8) + alpha_byte;
}
/*************************************
 * fonction pour savoir si le 
 * dessin a été modifié
 * **********************************/
gboolean editor_get_modified(void)
{
	  return fModifiedDrawing;
}
/*************************************
 * fonction pour mettre si le 
 * dessin a été modifié
 * **********************************/
void editor_set_modified(gboolean state)
{
	  fModifiedDrawing=state;
}
/*************************************
 * fonction pour mémoriser l'état de
 * l'item root
 * **********************************/
void editor_set_root_item(GooCanvasItem *root)
{
	rootEditorItems=root;
}
/************************************
 * remettre en blanc le fond de page
 * **********************************/
void editor_wipe_page(void)
{
	 g_object_set(itemPage,
				       "fill-color-rgba",
					   convert_color_to_rgba((gdouble)(1.0),
					                         (gdouble)(1.0), 
											 (gdouble)(1.0),
											 (gdouble)(1.0)),NULL);
}
/************************************
 * lire s'il existe une sélection 
 * **********************************/
gboolean editor_get_selection_state(void)
{
	if (pSelectedItemsList!=NULL)
	   return TRUE;
	else
	   return FALSE;
}
/*************************************
 * effacer le contenu de la sélection
 * **********************************/
void selection_raz(void)
{
  gint count;
  GList *node;
  
  if (pSelectedItemsList!=NULL)
	{
	  
	  for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
        {          	
			pSelectedItemsList=g_list_remove(pSelectedItemsList,node->data);                          
        } /* for */         
	  g_list_free(pSelectedItemsList);
	  pSelectedItemsList=NULL;
	}/* endif list!=NULL*/	
  /* bloquer l'accès aux propriétés */
  gtk_widget_set_sensitive (scrolledwindowProperties,FALSE);	
}
/*************************************
 * ajouter un nouvel item, en tenant
 * compte de son NOM pour incrémenter
 * intelligemment le compteur d'objets
 * ***********************************/
void editor_add_new_item(gchar *title)
{
  gchar *compteur;
  gint count;
    
  compteur=g_substr(title,1,strlen(title)-1);
  count=atoi(compteur);
  if (count>ItemCounter)
    ItemCounter=count;		
}
void editor_inc_item_counter(void)
{
	ItemCounter++;
}
/***************************
 * détruit liste des items,
 * en particulier des 
 * images
 * *************************/
void editor_reset_item_counter(void)
{
	ItemCounter=0;
	raz_pile_images(pile_images);
	pile_images=NULL;
}
/*****/
void editor_set_page_width(gdouble w)
{
  editor_canvas_max_width=w;
}	
void editor_set_page_height(gdouble h)
{
  editor_canvas_max_height=h;
}	
void editor_set_default_page_width(gdouble w)
{
  editor_default_width=w;
}	
void editor_set_default_page_height(gdouble h)
{
  editor_default_height=h;
}
void editor_set_default_margins(gdouble h)
{
  editor_default_margins=h;
}
gdouble editor_get_default_page_width(void)
{
  return editor_default_width;
}	
gdouble editor_get_default_page_height(void)
{
  return editor_default_height;
}
gdouble editor_get_default_margins(void)
{
  return editor_default_margins;
}
/******************************
 *  redimensionner la page 
 * ***************************/
void editor_page_resize(gdouble w,gdouble h, gdouble m)
{
  g_object_set(itemPage,"x",(gdouble)EDITOR_CANVAS_MARGINS,
                   "y",(gdouble)EDITOR_CANVAS_MARGINS,
				   "width", w, "height", h,NULL);	
 g_object_set(itemBack,"width", (gdouble)1024+2*EDITOR_CANVAS_MARGINS,
                "height",(gdouble)1024+2*EDITOR_CANVAS_MARGINS,NULL);
  
}
/********************
 * changer couleur
 * de fond de la
 * scène
 * *****************/
void editor_set_page_color(gint r,gint v, gint b)
{
	PageColor.red=r;
	PageColor.green=v;
	PageColor.blue=b;
	g_object_set(itemPage,"fill-color",gdk_color_to_string(&PageColor),NULL);
}
/* par défaut */
void editor_set_default_page_color(gint r,gint v, gint b)
{
	editor_defaut_page_color.red=r;
	editor_defaut_page_color.green=v;
	editor_defaut_page_color.blue=b;
}
GdkColor editor_get_default_page_color(void )
{
	return editor_defaut_page_color;
}
gdouble editor_get_page_width(void)
 {
 	return  editor_canvas_max_width;
 }
gdouble editor_get_page_height(void)
 {
 	return  editor_canvas_max_height;
 }  
gdouble editor_get_page_margins(void)
{
	return (gdouble) EDITOR_CANVAS_MARGINS;
}
GdkColor editor_get_page_color(void)
{
	return  PageColor;
}
/***********************
 * 
 * menus PopUp 
 * EDITION clipboard
 * 
 *********************/
/***************************
 * copier vers le clipboard
 * ************************/
void editor_copy_items(void)
{
	clipboard_copy(CLIP_ITEMS,pSelectedItemsList,x1sel,y1sel);	
}
void cb_menu_editor_copy (GtkWidget *p_widget, gpointer user_data)
{
	editor_copy_items();
}
/******************************
 * coller depuis le clipboard
 * ***************************/
void editor_paste_items(void)
{
	gint i;
	gdouble x,y;
	GList *node;
	
	if (clipboard_get_atom()!=CLIP_ITEMS)
	   return;/* rien à copier */
    /* copy current selection to undo buffer */
	undo_store_datas(UNDO_PASTE_ITEMS, NULL,pSelectedItemsList,x1sel,y1sel,0);	   
	/* calculer la position réelle de la souris */
	x=correct_x(xev);
    y=correct_y(yev);	   
	/* faire une boucle pour décalquer le contenu */  
    clipboard_paste(CLIP_ITEMS,rootKF,x,y); 	
}
void cb_menu_editor_paste (GtkWidget *p_widget, GdkEventButton *event)
{
 editor_paste_items();
}
/****************************
 * couper : retire les items
 * mais les copie en mémoire
 * *************************/
void editor_cut_items(void)
{
    GList *node;
	gint    nIndex, count;
	gchararray title;	
	
	if (SelectedArea==FALSE)
	  return;
	if (pSelectedItemsList==NULL)
	   return;
	   
	/* copier dans le clipboard le contenu actuel de la sélection */
	
	clipboard_copy(CLIP_ITEMS,pSelectedItemsList,x1sel,y1sel);
	/* copy current selection to undo buffer */
	undo_store_datas(UNDO_CUT_ITEMS, NULL,pSelectedItemsList,x1sel,y1sel,0);	   
	/* effacer la sélection */   
	for (nIndex = 0; node = g_list_nth (pSelectedItemsList, nIndex); nIndex++) 
	   {
		   /* lire le titre de l'item */
			g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL); 
			if (title!=NULL)   	
			   {
				   goo_canvas_item_remove((GooCanvasItem *)(node->data));
			   }
          	count++;
       }	
   /* supprimer sélection et cadre */
   goo_canvas_item_remove(pDrawings);
   SelectedArea=FALSE;
   selection_raz();
   pSelectedItemsList=NULL;  		
}
void cb_menu_editor_cut (GtkWidget *p_widget, gpointer user_data)
{
	editor_cut_items();
}
/**********************************
 * redraw the color preview of
 * the current selected color
 * set, on the notification area
 * of the editor ;
 * input : 2 pointers on drawing
 *  areas
 * *******************************/
void editor_editeur_set_color_set_preview(GtkWidget *win1, GdkColor TmpPpColor,GtkWidget *win2, GdkColor TmpApColor)
{
  cairo_t *cr1,*cr2;
  //GdkColor TmpPpColor;/* foreground */
  //GdkColor TmpApColor;/* background */
  
  if (win1==NULL)
      return;
  if (win2==NULL)
      return;
	  
  cr1 = gdk_cairo_create(win1->window);	
  cr2 = gdk_cairo_create(win2->window);	
  cairo_set_source_rgb(cr1,(gdouble)(TmpPpColor.red/65535.0),
                         (gdouble)(TmpPpColor.green/65535.0),
						 (gdouble)(TmpPpColor.blue/65535.0));
  cairo_rectangle(cr1,0,0,12,20);
  cairo_fill(cr1);
  
  cairo_set_source_rgb(cr2,(gdouble)(TmpApColor.red/65535.0),
                         (gdouble)(TmpApColor.green/65535.0),
						 (gdouble)(TmpApColor.blue/65535.0));
  cairo_rectangle(cr2,0,0,12,20);
  cairo_fill(cr2);
   
  cairo_destroy(cr1);
  cairo_destroy(cr2);
}
/* store pointers for preview */
void editor_store_preview(GtkWidget *da1, GtkWidget *da2)
{
	preview_Pp=da1;
	preview_Ap=da2;
}
/*********************************
 * fonction commune pour choisir 
 * les couleurs et l'alpha
 * *******************************/
void choix_couleurs(void)
{
	gint count;
	GList *node;
	gint reponse;
    GtkWidget *dialog_colors;
	gchararray title;
	 
	dialog_colors=create_dialogColorsAlpha(PenColor,FillColor,PageColor,Alpha);
	reponse=gtk_dialog_run(GTK_DIALOG(dialog_colors)); 
	 /* traitement de la réponse */
    if (reponse==GTK_RESPONSE_OK) 
      {		  
        Alpha=dlg_editeur_get_alpha();
		PenColor=dlg_editeur_get_scolor();
		FillColor=dlg_editeur_get_fcolor();
		/* store datas for undo color change */
		undo_store_datas(UNDO_COLOR_CHANGE, NULL,pSelectedItemsList,x1sel,y1sel,0);	
		/* change the color preview of the notification area */
		editor_editeur_set_color_set_preview(preview_Pp,FillColor, preview_Ap, PenColor);
	    /* passer la couleur */
	    if (pSelectedItemsList)
	     {
	  	  for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
            {   
			  /* faire un tesst sur l'item */
			  g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);
			  if ((strncmp("A",title,1)!=0)&&(strncmp("K",title,1)!=0)&&(strncmp("F",title,1)!=0))
			    {	                                 
                  g_object_set((GooCanvasItem *)(node->data),
				      "stroke-color-rgba",
					  convert_color_to_rgba((gdouble)(PenColor.red/65535.0),
					                         (gdouble)(PenColor.green/65535.0), 
											 (gdouble)(PenColor.blue/65535.0),
											 (gdouble)(Alpha/100.0)),
					  NULL);
				  g_object_set((GooCanvasItem *)(node->data),
				       "fill-color-rgba",
					   convert_color_to_rgba((gdouble)(FillColor.red/65535.0),
					                         (gdouble)(FillColor.green/65535.0), 
											 (gdouble)(FillColor.blue/65535.0),
											 (gdouble)(Alpha/100.0)),
					   NULL);	  
				}/* if pas ITEM A, c'est à dire bordure */
            } /* for */         
	     }/* endif list!=NULL*/
	  }/* réponse OK */
	gtk_widget_destroy(dialog_colors);	
}
/***********************************
 * choix de la couleur de fond de
 * page depuis le menu déroulant
 * ********************************/
void editor_set_page_color_menu(void)
{
	gchar *choix;
	gboolean retour;
	
	choix=dlg_couleur(DLGEDITEUR_COLORPAGE_TITLE);
	if (choix)
	  {
	    retour=gdk_color_parse(choix,&PageColor);
	    /* passer la couleur */
	   g_object_set(itemPage,"fill-color",gdk_color_to_string(&PageColor),NULL);	   
	  }	
}
/**********************************
 * fonction de callback du choix 
 * de la couleur du fond de page
 * *******************************/
gboolean cb_menu_color_page(GtkWidget *pBtn, gpointer data)
{
	gchar *choix;
	gboolean retour;
	
	choix=dlg_couleur(DLGEDITEUR_COLORPAGE_TITLE);
	if (choix)
	  {
	    retour=gdk_color_parse(choix,&PageColor);
	    /* passer la couleur */
	   g_object_set(itemPage,"fill-color",gdk_color_to_string(&PageColor),NULL);	   
	  }
    return TRUE;
}
/* couleur de contour */
gboolean cb_menu_color_foreground (GtkWidget *p_widget, gpointer user_data)
{
    choix_couleurs();
    return TRUE;
}
/*********************************
 * choix de la fonte
 * *******************************/
void cb_menu_font_select(GtkWidget *p_widget, gpointer user_data)
{
	gint count;
	GList *node;
	gchararray title;
	
	FntDesc=dlg_fontes(gettext("Font choice by menu"));
	if (FntDesc)
	   {
	   	  if (pSelectedItemsList)
	        {
			 /* store datas for undo color change */
		     undo_store_datas(UNDO_FONT_CHANGE, NULL,pSelectedItemsList,x1sel,y1sel,0);		
	  	     for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
               {    
			      g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);	
			      if (strncmp("T",title,1)==0)	      	                          
                       g_object_set((GooCanvasItem *)(node->data),"font-desc",FntDesc,NULL);
               } /* for */         
	        }/* endif list!=NULL*/
	   }/* endif gntdesc */
}
/***********************************
 * fonction testant si la sélection
 * courante contient un item texte ;
 * si oui, renvoit un pointeur sur
 * cet item
 * *********************************/
GooCanvasItem *item_is_text(GList *selection)
{
	GList *node;
	gint    nIndex, count;
	gchararray title;
	
	if (SelectedArea==FALSE)
	  return NULL;
	/* passer en revue la sélection */
	if (selection==NULL)
	  return NULL;
	for (nIndex = 0; node =g_list_nth (selection, nIndex); nIndex++) 
          {      
			/* lire le titre de l'item */
			g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);
			if(title != NULL)
			{
				if (strncmp("T",title,1)==0)   	
				   {
					   return (GooCanvasItem *)(node->data);
				   }
				count++;
			}          
		  }
	return NULL;
}
/*********************************
 * éditer une zone de texte
 * ******************************/
gboolean cb_text_edit(GtkWidget *p_widget, gpointer user_data)
{
	gchararray texte;
	GooCanvasItem *item;
	GooCanvasBounds area;
		
	if (rootKF==NULL)
		   return TRUE;
	/* passer en mode saisie de texte dans une fenêtre */
	/* récupérer le contenu actuel */
	item=item_is_text(pSelectedItemsList);
	if (item==NULL)
	   return;
	g_object_get(item,"text",&texte,NULL);
	gchar *texte_saisi=dlg_saisie_text(gettext("Modify text"),texte);
	if (texte_saisi!=NULL)
	  	  {
	  	     g_object_set(G_OBJECT(item),"text",texte_saisi,NULL);	
			 goo_canvas_item_remove(pDrawings);	
			 goo_canvas_item_get_bounds(item,&area);	
			 x1drw=area.x1;
			 y1drw=area.y1;
			 x2drw=area.x2;
			 y2drw=area.y2;
			 x1sel=x1drw;y1sel=y1drw;
			 x2sel=x2drw;y2sel=y2drw;
		     wsel=x2sel-x1sel;
		     hsel=y2sel-y1sel;
			 selection_box(rootKF,x1drw,y1drw,x2drw,y2drw);
			 return TRUE;
	  	  }
	fdrawing=FALSE;/* pour éviter de changer en cas de nouvel outil !*/ 		
	return TRUE;
}
/***************************
 * monter et descendre des
 * objets sélectionnés ;
 * attention : met au
 * sommet ou en bas, 
 * directement
 * *************************/
void editor_raise(GList *selection)
{
    GList *node;
	gint    nIndex, count;
	gchararray title;	
	
	if (SelectedArea==FALSE)
	  return;
	if (selection==NULL)
	   return;
	for (nIndex = 0; node = g_list_nth (selection, nIndex); nIndex++) 
	   {
		   /* lire le titre de l'item */
			g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL); 
			if (title!=NULL)   	
			   {
				   goo_canvas_item_raise((GooCanvasItem *)(node->data),NULL);
			   }
          	count++;
       }
}
/*****************/
void editor_lower(GList *selection)
{
    GList *node;
	gint    nIndex, count;
	gchararray title;	
	
	if (SelectedArea==FALSE)
	  return;
	if (selection==NULL)
	   return;
	for (nIndex = 0; node = g_list_nth (selection, nIndex); nIndex++) 
	   {
		   /* lire le titre de l'item */
			g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL); 
			if (title!=NULL)   	
			   {
				   goo_canvas_item_lower((GooCanvasItem *)(node->data),NULL);
			   }
          	count++;
       }
}
/*************************
 * annuler la sélection
 * **********************/
void editor_unselect(void)
{ 
	if (SelectedArea==FALSE)
	  return;
	if (pSelectedItemsList==NULL)
	   return;
   /* supprimer sélection et cadre */
   goo_canvas_item_remove(pDrawings);
   SelectedArea=FALSE;
   selection_raz();
   pSelectedItemsList=NULL;  	
}
/***************************
 * sélectionner tous les 
 * objets 
 * ***********************/
void editor_select_all(void)
{
	if (pSelectedItemsList!=NULL)
	   {
		 /* supprimer sélection et cadre */
         SelectedArea=FALSE;
         selection_raz();
         pSelectedItemsList=NULL;  
	   }
	/* encadrer tous les objets */
	x1drw=0;
	y1drw=0;
	x2drw=editor_get_page_width()+2*editor_get_page_margins();
	y2drw=editor_get_page_height()+2*editor_get_page_margins();
	if (rootKF!=NULL)
	  { 
		pDrawings=NULL;
	    pSelectedItemsList=get_items_selected(rootKF,x1drw,y1drw,x2drw,y2drw);   
		if (pSelectedItemsList!=NULL)
		  {
		    SelectedArea=TRUE;
		    x1sel=x1drw;y1sel=y1drw;
		    x2sel=x2drw;y2sel=y2drw;
		    wsel=x2sel-x1sel;
		    hsel=y2sel-y1sel;
			/* gérer les "entries" des propriétés */
			editor_set_entries(pSelectedItemsList,x1sel,y1sel,wsel,hsel);   
		  }
	  }
}
/************************
 * dupliquer les objets
 * de la sélection
 * *********************/
void editor_duplicate(void)
{
	if (SelectedArea==FALSE)
	  return;
	if (pSelectedItemsList==NULL)
	   return;	
	/* effacer le cadre */
	if (pDrawings!=NULL)
	    goo_canvas_item_remove(pDrawings);
	/* copier vers clipboard */
	editor_copy_items();
	/* coller en place aux coordonnées x+1,y+1 */
	xev=x1sel+1;
	yev=y1sel+1;
	editor_paste_items();   
	SelectedArea=FALSE;
    selection_raz();
    pSelectedItemsList=NULL; 	
}
/*************************
 * supprimer le contenu de
 * la sélection
 * **********************/
void editor_delete_items(void)
{
    GList *node;
	gint    nIndex, count;
	gchararray title;	
	
	if (SelectedArea==FALSE)
	  return;
	if (pSelectedItemsList==NULL)
	   return;
	for (nIndex = 0; node = g_list_nth (pSelectedItemsList, nIndex); nIndex++) 
	   {
		   /* lire le titre de l'item */
			g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL); 
			if (title!=NULL)   	
			   {
				   goo_canvas_item_remove((GooCanvasItem *)(node->data));
			   }
          	count++;
       }	
   /* supprimer sélection et cadre */
   goo_canvas_item_remove(pDrawings);
   SelectedArea=FALSE;
   selection_raz();
   pSelectedItemsList=NULL;  	
}
/************************
 * supprimer le groupe
 * d'objets sélectionnés.
 * Attention :pas
 * d'annulation possible
 * **********************/
void cb_menu_editor_delete(GSList *selection)
{
  editor_delete_items(); 	
}
/**************/
void cb_menu_style_dash (GtkWidget *p_widget, gpointer user_data)
{
}
void cb_menu_object_up (GtkWidget *p_widget, gpointer user_data)
{
	editor_raise(pSelectedItemsList);
}
void cb_menu_object_down (GtkWidget *p_widget, gpointer user_data)
{
	editor_lower(pSelectedItemsList);
}
void cb_menu_object_group (GtkWidget *p_widget, gpointer user_data)
{
}
void cb_menu_object_ungroup (GtkWidget *p_widget, gpointer user_data)
{
}
/******************************
 * menu POp Up de l'éditeur
 * entièrement local
 * ***************************/ 
void editor_popup_menu_(GooCanvasItem  *gitem, GdkEventButton *event, gdouble x, gdouble y)
{
    GtkWidget *menu, *item, *sep1, *sep2, *sep3;
    
	
	xev=event->x;
	yev=event->y;
	/* clipboard vide ou non ? */
	fClipboardFull=FALSE;
	if (clipboard_get_atom()==CLIP_ITEMS)
	   fClipboardFull=TRUE;
    /* tester si objet en dessous */
	menu = gtk_menu_new (); 	
	item = gtk_menu_item_new_with_mnemonic ((gettext("_Copy")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_editor_copy), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	/* est-ce un objet ? */
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	
    item = gtk_menu_item_new_with_mnemonic ((gettext("Paste")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_editor_paste),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	/* est-ce un objet ? */
	if (!fClipboardFull)
	    gtk_widget_set_sensitive(item,FALSE);
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("C_ut")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_editor_cut), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	/* est-ce un objet ? */
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
		
	item = gtk_menu_item_new_with_mnemonic ((gettext("Remove")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_editor_delete), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	/* est-ce un objet ? */
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	sep1 = gtk_separator_menu_item_new();
	
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep1);
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("Color and op_acity")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_color_foreground), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	/*
	item = gtk_menu_item_new_with_mnemonic (("Couleur de _remplissage"));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_color_fill), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	*/
	item = gtk_menu_item_new_with_mnemonic ((gettext("_Page color")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_color_page), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("Dash _style")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_style_dash), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	
    item = gtk_menu_item_new_with_mnemonic ((gettext("F_ont choice")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_font_select),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("Edit _text")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_text_edit),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);  
	else
	   {
		   /* tester si la sélection contient du texte */
		   if (item_is_text(pSelectedItemsList)==NULL)
		      gtk_widget_set_sensitive(item,FALSE); 
	   }
		  
	sep2 = gtk_separator_menu_item_new();
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep2);
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("Raise object(s)")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_object_up),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);   
	
	item = gtk_menu_item_new_with_mnemonic ((gettext("Lower object(s)")));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_object_down),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);    
	/*     
	item = gtk_menu_item_new_with_mnemonic (("_Grouper"));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_object_group),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);      
	    
    item = gtk_menu_item_new_with_mnemonic (("Dégroup_er"));
	g_signal_connect_swapped (item, "activate",G_CALLBACK (cb_menu_object_ungroup),NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	if (!pSelectedItemsList)
	    gtk_widget_set_sensitive(item,FALSE);     
	 */   
	gtk_widget_show_all (menu); 
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, 
	               NULL,/* pointeur datas */
	               event->button, /* bouton de la souris, automatique */
	               event->time); /* moment du clic, automatique */
}		      
/**********************************
 * repasser l'éditeur en mode de
 * sélection, en préselectionnant
 * le bouton
 * *******************************/
void set_mode_select(gboolean effacer)
{
  if ((fdrawing)&&(effacer))
     goo_canvas_item_remove(pDrawings);
  EditorMode=EDITOR_MODE_SELECT;
  fdrawing=FALSE;
  fmoved=FALSE;
  frotation=FALSE;
  fresizing=FALSE;
  pEditorModeItem=gtk_toolbar_get_nth_item(GTK_TOOLBAR(pToolbarEditor),EDITOR_TOOL_SELECT);/* 2 car item de rang 3 !*/
  gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(pEditorModeItem),TRUE);
  if (SelectedArea)
   {
    	SelectedArea=FALSE;
      	selection_raz();
        pSelectedItemsList=NULL;
   }
}
/*********************************
 * correction d'une abscisse pour
 * rentrer dans l'espace canvas
 * ******************************/
gdouble correct_x(gdouble x)
{
	gdouble t;
	
	t=x;
	if (t<0)
	  t=0;
	  /*
	if (t>editor_canvas_max_width)
	  t=editor_canvas_max_width;*/
	return t;
}
/*********************************
 * correction d'une ordonnée pour
 * rentrer dans l'espace canvas
 * ******************************/
gdouble correct_y(gdouble y)
{
	gdouble t;
	
	t=y;
	if (t<0)
	  t=0;
	  /*
	if (t>editor_canvas_max_height)
	  t=editor_canvas_max_height;*/
	return t;
}
/**********************************
 * fonction pour calculer la
 * nouvelle dimension horizontale
 * après redimensionnement
 * *******************************/
gdouble resize_compute(gdouble before, gdouble after)
{
	gdouble t;
	t= after/before;/* >1 = plus grand ; <1 = plus petit */
	return t;
}
/*******************************
 * calculer la rotation d'une
 * abscisse selon la valeur
 * initiale et l'angle fournis
 * *****************************/
gdouble x_rotate(gdouble x, gdouble y, gdouble angle)
{
	gdouble t;
	
	t = (x * cos(angle)) - (y * sin(angle));
	return t;
}
/*******************************
 * calculer la rotation d'une
 * ordonnée selon la valeur
 * initiale et l'angle fournis
 * *****************************/
gdouble y_rotate(gdouble x, gdouble y, gdouble angle)
{
	gdouble t;
	
	t = x * sin(angle) +  y * cos(angle);
	return t;
}
/********************************
 * fonction pour calculer la
 * valeur de la rotation compte
 * tenu de la position de la souris
 * (xm, ym) et du centre de l'objet
 * (xc, yc)
 * *********************************/
gdouble rotate_compute(gdouble xm, gdouble ym, gdouble xc, gdouble yc)
{
	gdouble angle=0;

	angle=(gdouble)(atan2((ym-yc),(xm-xc)));/* fourni en radians */
	//angle=(180*angle)/3.1416;
	//printf("angle=%f\n",(180*angle)/3.1416); // 90° =pi/2
	return angle;
}
/**********************************
 * rectangle pour rotation
 * 
 * *******************************/
void rotate_box(GooCanvasItem *ancetre, gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t, gdouble angle)
{
	gdouble t,xc,yc;
	gdouble xg1,yg1,xg2,yg2,xd1,yd1,xd2,yd2; /* coordonnées après rotation */
	const gdouble vers_radian = PI_PERSO / 360;/* ou 180 pour des 1/4 quadrants */
    static const double dashed1[] = {4.0, 2.0};
    static int len1  = sizeof(dashed1) / sizeof(dashed1[0]);
    GooCanvasLineDash *style_trait; 
    gchar *svgpath;	
	 
    style_trait=goo_canvas_line_dash_newv(len1,(gdouble *)dashed1);
    SelectedArea=TRUE;
    /* vérifier les valeurs des coins */
    if (x2t<x1t)
       {
       	t=x1t;x1t=x2t;x2t=t;
	   }
	if (y2t<y1t)
	   {
	   	t=y1t;y1t=y2t;y2t=t;
	   }
	   	   
	/* calculer les projections */
	xc=(x2t+x1t)/2;
	yc=(y2t+y1t)/2;
	
    /* changement d'origine */
    x1t=x1t-xc;
    x2t=x2t-xc;
    y1t=y1t-yc;
    y2t=y2t-yc;    
    
    /* coins sup. gauche et inf. droit */
    xg1 = x_rotate(x1t,y1t,angle); //(x1t * cos(angle)) - (y1t * sin(angle));
	yg1 = y_rotate(x1t,y1t,angle); //(x1t * sin(angle)) +  (y1t * cos(angle));
    xd2 = x_rotate(x2t,y2t,angle); //(x2t * cos(angle)) - (y2t * sin(angle));
	yd2 = y_rotate(x2t,y2t,angle); //(x2t * sin(angle)) +  (y2t * cos(angle));
	/* coins inf. gauche et sup. droit */
	xg2=x_rotate(x1t,y2t,angle); //(x1t * cos(angle)) - (y2t * sin(angle));
	yg2=y_rotate(x1t, y2t,angle); //(x1t * sin(angle)) +  (y2t * cos(angle));
	xd1 = x_rotate(x2t,y1t,angle); //(x2t * cos(angle)) - (y1t * sin(angle));
	yd1 = y_rotate(x2t,y1t,angle); //(x2t * sin(angle)) +  (y1t * cos(angle));

	/* tracé rectangle de sélection */

	 svgpath=g_strdup_printf("M %d %d L %d %d M %d %d L %d %d M %d %d L %d %d L %d %d L %d %d L %d %d M %d %d  A 4 4 0 1 1 %d %d A 4 4 0 1 1 %d %d z",
	           (gint)xc-4,(gint)yc,
	           (gint)xc+4,(gint)yc,
	           (gint)xc,(gint)yc-4,
	           (gint)xc,(gint)yc+4,
			   (gint)(xg1+xc),(gint)(yg1+yc),
			   (gint)(xd1+xc),(gint)(yd1+yc),
			   (gint)(xd2+xc),(gint)(yd2+yc),
			   (gint)(xg2+xc),(gint)(yg2+yc),
			   (gint)(xg1+xc),(gint)(yg1+yc),
			   (gint)(xc+8),(gint)yc,
			   (gint)(xc-8),(gint)yc,
			   (gint)(xc+8),(gint)yc);		 
     pDrawings = goo_canvas_path_new(ancetre,svgpath,
	                            "stroke-color", "blue",
	                            "line-width", 2.0,
	                             "line-dash", style_trait,
	                             NULL);         
	 g_free(svgpath);		 		 			 		 
}    
/**********************************
 * rectangle pour redimensionner
 * correction intégrée des
 * coordonnées
 * *******************************/
void resize_box(GooCanvasItem *ancetre, gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t)
{
    gdouble t;
    static const double dashed1[] = {1.0, 1.0};
    static int len1  = sizeof(dashed1) / sizeof(dashed1[0]);
    GooCanvasLineDash *style_trait;  
    GooCanvasItem *rect1, *rect2;
    gchar *svgpath;	
    
    style_trait=goo_canvas_line_dash_newv(len1,(gdouble *)dashed1);
    //SelectedArea=TRUE;
    /* vérifier les valeurs des coins */
    if (x2t<x1t)
       {
       	t=x1t;
       	x1t=x2t;
       	x2t=t;
	   }
	if (y2t<y1t)
	   {
	   	t=y1t;
	   	y1t=y2t;
	   	y2t=t;
	   }
	
	 svgpath=g_strdup_printf("M %d %d L %d %d L %d %d L %d %d  L %d %d z M %d %d L %d %d L %d %d L %d %d L %d %d z",
	           (gint)x1t,(gint)y1t,
			   (gint)x2t,(gint)y1t,
			   (gint)x2t,(gint)y2t,
			   (gint)x1t,(gint)y2t,
			   (gint)x1t,(gint)y1t,/* small rectangle */
			   (gint)x2t-6,(gint)y2t-6,
			   (gint)x2t+3,(gint)y2t-6,
			   (gint)x2t+3,(gint)y2t+3,
			   (gint)x2t-6,(gint)y2t+3,
			   (gint)x2t-6,(gint)y2t-6);

	 pDrawings = goo_canvas_path_new(ancetre,
			                     svgpath,
	                            "stroke-color", "blue",
	                            "line-width", 2.0,
	                             "line-dash", style_trait,
	                             NULL);         
	 g_free(svgpath);
}
/**********************************
 * rectangle de sélection en cours
 * locale
 * *******************************/
void selection_box(GooCanvasItem *ancetre, gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t)
{
    gdouble t;
    static const double dashed1[] = {4.0, 4.0};
    static int len1  = sizeof(dashed1) / sizeof(dashed1[0]);
    GooCanvasLineDash *style_trait;  
    
    style_trait=goo_canvas_line_dash_newv(len1,(gdouble *)dashed1);
    SelectedArea=TRUE;
    /* vérifier les valeurs des coins */
    if (x2t<x1t)
       {
       	t=x1t;x1t=x2t;x2t=t;
	   }
	if (y2t<y1t)
	   {
	   	t=y1t;y1t=y2t;y2t=t;
	   } 	 
	
    pDrawings=goo_canvas_rect_new(ancetre,
       x1t,y1t,x2t-x1t,y2t-y1t,
	   "line-width", 2.0,
	   "radius-x", 2.0,
	   "radius-y", 2.0,
	   "stroke-color", "blue",
	   "line-dash", style_trait,
	   NULL);		
	/* autoriser les propriétés */
    gtk_widget_set_sensitive (scrolledwindowProperties,TRUE);   	
}
/**********************************
 * fonction pour tester quel(s)
 * élément(s) sont dans le
 * rectangle de sélection ; renvoit
 * une GSlist
 * *******************************/
GList *get_items_selected(GooCanvasItem *item,
                           gdouble x1z, gdouble y1z, gdouble x2z, gdouble y2z)
{
	GList *tmp_list=NULL;
	GList *node;
	GooCanvasBounds area;
	gint    nIndex, count;
	gdouble xlb, ylb, xrb, yrb,t; /* bounds */
	GooCanvasItem *parent;
	
	if (pDrawings!=NULL)
	    goo_canvas_item_remove(pDrawings);
	fdrawing=FALSE;
	/* pseudos valeurs pour les extremités */
	xlb = 128000;
	ylb = 128000;
	xrb = 0;
	yrb =0;
	/* passage de l'aire de test */
	area.x1=x1z;
	area.y1=y1z;
	area.x2=x2z;
	area.y2=y2z;
	tmp_list=goo_canvas_get_items_in_area(goo_canvas_item_get_canvas(item),
	                                     &area,TRUE,FALSE,FALSE);
	/* compter le nombre de valeurs */
	count=0;
	if (tmp_list)
	  {
	  	for (nIndex = 0; node = g_list_nth (tmp_list, nIndex); nIndex++) 
          {          	
          	count++;
          	/* effacer items n'appartenant pas à l'image active !*/
          	parent=goo_canvas_item_get_parent((GooCanvasItem *)(node->data));	
	        if ((parent==NULL)||(parent!=rootKF))
	          {
          	    tmp_list=g_list_remove(tmp_list,node->data);
          	    count--;
			  }
		  }
	  }/* endif */	
	if (count==0)
	    return NULL;	
    /* sortir la liste */
     if (tmp_list)
   	   {   	   	
   	   	for (nIndex = 0; node = g_list_nth (tmp_list, nIndex); nIndex++) 
          {
          	goo_canvas_item_get_bounds((GooCanvasItem *)(node->data),&area); 
            if (area.x1<xlb)
               xlb=area.x1;
            if (area.y1<ylb)
               ylb=area.y1;
            if (area.x2>xrb)
               xrb=area.x2;
            if (area.y2>yrb)
               yrb=area.y2;	
          }/* for */   	     	   	
	   }/* endif*/ 
    /* mettre à jour cadre de sélection */
    x1drw=xlb;
    y1drw=ylb;
    x2drw=xrb;
    y2drw=yrb;
     
    if (tmp_list)
      {
       selection_box(item,x1drw ,y1drw, x2drw, y2drw);
      }
	return tmp_list;
}
/***********************************
 * fonction maison pour trouver
 * en un clic si un item est présent
 * sous le pointeur
 * *********************************/
GList *get_item_here(GooCanvasItem *item)
{
   GList *tmp_list=NULL;
   GList *node=NULL;
   GooCanvasBounds area;
   GooCanvasItem *child,*found;
   gint i,j,enfants,total_trouve;
   gchararray title;
   gboolean trouve=FALSE;
   
   if (pDrawings)
       goo_canvas_item_remove(pDrawings);
   fdrawing=FALSE;
  printf("//////////////////////\n");
   /* lire tous les items appartenant à rootKF */
   enfants= goo_canvas_item_get_n_children(rootKF);
   //printf("rootKF a %d enfants \n",enfants);
   
   /* faire une boucle, et chercher quels sont ces enfants */
   if (enfants==0)
     {
       x1drw=-1;
	   x2drw=-1;
	   y1drw=-1;
	   y2drw=-1;
	   return NULL;
     }
   i=enfants-1;
   while ((!trouve)&&(i>=0))
      {
		  child=goo_canvas_item_get_child(rootKF,i);
		  g_object_get(child,"title",&title,NULL);
		  /* récupérer les bounds */
		  goo_canvas_item_get_bounds(child,&area);
		  /* si l'enfant est contenu dans les bounds, alors c'est trouvé ! */
		  if ((x1drw>area.x1)&&(x1drw<area.x2)
		     &&(y1drw>area.y1)&&(y1drw<area.y2))
			   {
				   //printf("trouvé pour l'enfant %d nommé %s  !",i,title);
				   trouve=TRUE; 
                   found = g_malloc(sizeof((GooCanvasItem *)child));
                   tmp_list=g_list_append (tmp_list,child);
				   x1drw=area.x1;y1drw=area.y1;
				   x2drw=area.x2;y2drw=area.y2;  
				   tmp_list=goo_canvas_get_items_in_area(goo_canvas_item_get_canvas(rootKF),
	                                     &area,TRUE,FALSE,FALSE);  
				   /* il faut nettoyer cette liste de tout ce qui n'appartient pas à ce calque /image-clé */
				   for (j=0;node=g_list_nth(tmp_list,j);j++)
				       {
						   if (goo_canvas_item_get_parent (((GooCanvasItem *) (node->data)))!=rootKF)
						      tmp_list=g_list_remove(tmp_list,(node->data));
					   }
			   }
	      i--;	  
	  }/* wend */
   total_trouve= g_list_length (tmp_list); 
   //printf("************** trouvé %d objet(s)\n",total_trouve);
   while (g_list_length (tmp_list)>1)
     {
		tmp_list=g_list_remove(tmp_list,g_list_nth_data(tmp_list,0)); 
	 }
	//printf("reste après nettoyage :%d\n",g_list_length (tmp_list)); 
   if (tmp_list)
       selection_box(rootKF, x1drw, y1drw, x2drw, y2drw);
   return tmp_list;   	
}
/**********************************
 * fonction pour sélectionner un
 * élément par simple-clic
 * à l'intérieur
 * elle se contente de recalculer 
 * x1drw ... y2drw
 * *******************************/
GList *get_item_at(GooCanvasItem *item)
{
	GList *tmp_list=NULL;
	gint    nIndex, count;
	GList *node;
	GooCanvasBounds area;	
	GooCanvasItem *parent;
	gchararray title;
	gint visi;
	
	goo_canvas_item_remove(pDrawings);
	fdrawing=FALSE;
	tmp_list=goo_canvas_get_items_at(goo_canvas_item_get_canvas(rootKF),
	           x1drw,y1drw,FALSE);
	/* sortir la liste */
	/* note : si la liste sort plus de 1 élément, alors il ne s'agit pas d'un item de dessin,
	 * et la fonction doit renvoyer NULL  
	 * mais attention : si c'est le fond, un seul élément aussi !!! */
	/* compter le nombre d'éléments : si 1 seul, quitter car il s'agit du fond */
	count=0;
	if (tmp_list)
	  {
	  	for (nIndex = 0; node = g_list_nth (tmp_list, nIndex); nIndex++) 
          {
          	count++;
			g_object_get((GooCanvasItem *)(node->data),"title",&title,
			             "visibility",&visi,NULL);
			printf("titre <%s<\n",title);
          	/* effacer items n'appartenant pas à l'image active !*/
          	parent=goo_canvas_item_get_parent((GooCanvasItem *)(node->data));	
	        if ((strncmp(title,"A",1)==0)||(strncmp(title,"F",1)==0)
			    ||(visi==GOO_CANVAS_ITEM_HIDDEN)||(visi==GOO_CANVAS_ITEM_INVISIBLE)
				||(parent!=rootKF))
	          {
          	     tmp_list=g_list_remove(tmp_list,node->data);
          	     count--;
			  }
			/* visible ou pas ? */
			printf("visi=%d\n",visi);
		  }
	  }/* endif */
printf("nombre éléments %d \n",count);
	if (count<1)	 
	     return NULL;	  
	/* à priori, on connaît maintenant l'indice de l'item : 0 */  
	node = g_list_nth (tmp_list, 0);

	goo_canvas_item_get_bounds((GooCanvasItem *)(node->data), &area); 
	x1drw=area.x1;
    y1drw=area.y1;
    x2drw=area.x2;
    y2drw=area.y2;    
    if (tmp_list)
       selection_box(item, x1drw, y1drw, x2drw, y2drw);
    /* on nettoie la liste en ôtant tous les items sauf le premier, de rang zéro */   
    if (tmp_list)
	  {
	  	while (count>0)
	  	  {
	  	  	 node = g_list_nth (tmp_list, count);
	  	  	 tmp_list = g_list_delete_link (tmp_list, node);
	  	  	 count--;
		  }/* wend */
	  }/* endif */	  	
    return tmp_list; 	  
}
/**********************************************
 * fonction rotate : on donne en entrée un
 * pointeur sur l'item, puis l'angle de rotation
 * en radians ; ce dernier est converti en degrés
 * pour être compatible avec Cairo ;
 * la fonction de rotation de Goo est ENFIN
 * utilisable ; attention : sens horaire !
 * ********************************************/
void item_rotate(GooCanvasItem *item, gdouble angle)
{
	gchararray title;
	gdouble xc,yc;
	gdouble x1t,y1t,width,height;/* pour les rectangles */
	gdouble cx,cy,rx,ry; /* pour les ellipses */
	gdouble current_angle;/* in order to cancel the current rotation if exists */
	gboolean resultat;
	gdouble xt,yt,ech;
	GooCanvasPoints *l_pts;
	GooCanvasBounds area;
	
	const gdouble vers_degre = 180/ PI_PERSO; 
				
	angle=(gdouble)(angle*vers_degre);
	g_object_get(item,"title",&title,NULL);
	if (title==NULL)
         return;
	  else printf("titre=%s\n",title);
	resultat= goo_canvas_item_get_simple_transform(item,&xt,&yt,&ech,&current_angle);  
	if (resultat)
	   angle=angle-current_angle;
    if (strncmp("R",title,1)==0)
     {
       g_object_get(item,"x",&x1t,"y",&y1t,"width",&width,"height",&height,NULL);
       /* changement origine */       
       xc=x1t+(width/2);
       yc=y1t+(height/2);              
       goo_canvas_item_rotate(item,angle,xc,yc);
       return;
     }
	if (strncmp("B",title,1)==0)
     {
       g_object_get(item,"x",&x1t,"y",&y1t,"width",&width,"height",&height,NULL);
       /* changement origine */       
       xc=x1t+(width/2);
       yc=y1t+(height/2);              
       goo_canvas_item_rotate(item,-angle,xc,yc);
	   return;
     }
    if (strncmp("C",title,1)==0)
     {       
       g_object_get(item,"center_x",&cx,"center_y",&cy,"radius_x",&rx,"radius_y",&ry,NULL);       
       goo_canvas_item_rotate(item,angle,cx,cy);
       return;
     }
    if (strncmp("T",title,1)==0)
     {
       g_object_get(item,"x",&xc,"y",&yc,"width",&width,NULL);
       xc=xc+(width/2);
       goo_canvas_item_rotate(item,angle,xc,yc);
       return;
     }  
    if (strncmp("L",title,1)==0)
     {
       g_object_get(item,"x",&x1t,"y",&y1t,"width",&width,"height",&height,NULL);
       /* changement origine */       
       xc=x1t+(width/2);
       yc=y1t+(height/2);              
       goo_canvas_item_rotate(item,angle,xc,yc);
       goo_canvas_item_get_bounds(item, &area);			 
	   return;
	 }	 
	if (strncmp("U",title,1)==0)
     {/* outline object */
       g_object_get(item,"x",&x1t,"y",&y1t,"width",&width,"height",&height,NULL);
       /* changement origine */       
       xc=x1t+(width/2);
       yc=y1t+(height/2);              
       goo_canvas_item_rotate(item,angle,xc,yc);
       return;	 
	 }
	if (strncmp("P",title,1)==0)
     {/* path filled object */
	 printf("rotation path \n");
       g_object_get(item,"x",&x1t,"y",&y1t,"width",&width,"height",&height,NULL);
       /* changement origine */       
       xc=x1t+(width/2);
       yc=y1t+(height/2);              
       goo_canvas_item_rotate(item,angle,xc,yc);
       return;	 
	 }	 
 return;	
}
/***************************************************
 * fonction resize ; celle de goocanvas est
 * valable, mais ne tient pas compte des bords
 * ce qui oblige à la corriger malgré tout ;
 * on a ajouté les paramètres xorh,yorg pour
 * déplacer les objets en cas de sélection multiple.
 * *************************************************/
void item_resize(GooCanvasItem *item, gdouble xorg,gdouble yorg,gdouble h, gdouble v)
{
	gchararray title;
	gdouble x,y,xf,yf,width,height;/* pour les rectangles */
	gdouble cx,cy,rx,ry; /* pour les ellipses */
	GooCanvasPoints *l_pts;
		
	
    g_object_get(item,"title",&title,NULL);
    if (title==NULL)
      return; 
    if (strncmp("R",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
	   x=xorg+((x-xorg)*h);
	   y=yorg+((y-yorg)*v);	   
       g_object_set(item,"x",x,"y",y,"width", width*h, "height", height*v, NULL);
       return;
     }
	if (strncmp("B",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
	   x=xorg+((x-xorg)*h);
	   y=yorg+((y-yorg)*v);	
       g_object_set(item,"x",y,"y",y, "scale_to_fit",TRUE,"width", width*h, "height", height*v, NULL);
       return;
     }
    if (strncmp("C",title,1)==0)
     {       
       g_object_get(item,"center_x",&cx,"center_y",&cy,"radius_x",&rx,"radius_y",&ry,NULL);   
	   cx=xorg+((cx-xorg)*h);
	   cy=yorg+((cy-yorg)*v);    
       g_object_set(item,"center_x",cx,"center_y",cy,"radius_x",rx*h,"radius_y",ry*v,NULL);
       return;
     } 
    if (strncmp("L",title,1)==0)
     {
       l_pts= goo_canvas_points_new(2);/* lignes simples */
       g_object_get(item,"points",&l_pts,NULL);
	   x=(gdouble)l_pts->coords[0];
	   y=(gdouble)l_pts->coords[1];
	   xf=(gdouble)l_pts->coords[2];
	   yf=(gdouble)l_pts->coords[3];
	   x=xorg+((x-xorg)*h);
	   y=yorg+((y-yorg)*v);
	   xf=xorg+((xf-xorg)*h);
	   yf=yorg+((yf-yorg)*v);	     
       l_pts->coords[0]=x;
       l_pts->coords[1]=y;
       l_pts->coords[2]=xf;
       l_pts->coords[3]=yf;
       g_object_set(item,"points",l_pts,NULL);       
       goo_canvas_points_unref(l_pts); 
       return;
     }
	if (strncmp("U",title,1)==0)
     {/* outline object */
	   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
	   x=xorg+((x-xorg)*h);
	   y=yorg+((y-yorg)*v);	   
       g_object_set(item,"x",x,"y",y,"width", width*h, "height", height*v, NULL);
       return;
	 }
	if (strncmp("P",title,1)==0)
     {/* path filled object */
	   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
	   x=xorg+((x-xorg)*h);
	   y=yorg+((y-yorg)*v);	   
       g_object_set(item,"x",x,"y",y,"width", width*h, "height", height*v, NULL);
       return;
	 }	 
 }
/**********************************************
 * fonction move object, car 
 * goocanvas ne connaît pas !
 * on passe en paramètre un
 * pointeur sur l'objet, duquel on
 * a fixé des propriétés utiles :
 * title
 * (C, R, L, T)
 * après, pour un rectangle, on connaît
 * l'origine x,y, trivial !
 * pour l'ellipse, on connaît le centre
 * ATTENTION : il fautr tenir compte du fait
 * que l'objet a pu subir une rotation de son
 * système d'axes
 * 
/**********************************************/
void item_move_to(GooCanvasItem *item, gdouble xtran, gdouble ytran)
{
	gchararray title;
    gdouble angle;
	gdouble x,y;/* for rectangles */
	gdouble xc,yc; /* for ellipses */
	gdouble width, height;
	gdouble xt,yt,echelle,rota;/* pour tenir compte des transformations */
	GooCanvasPoints *l_pts;
	GooCanvasItem *parent;
	gboolean transforme=FALSE;

	/* si l'ancêtre de l'item n'est pas le groupe courant, sortir */
	parent=goo_canvas_item_get_parent(item);	
	if (parent==NULL)
		 return;
	if (parent!=rootKF)
	    return;

	g_object_get(item,"title",&title,"width",&width,"height",&height,NULL);
    /* récupérer l'angle de rotation courant */
	transforme=goo_canvas_item_get_simple_transform(item,&xt,&yt,&echelle,&rota);
  
    /* switcher selon le type d'objet , récupérer anciennes coordonnées */
    /* détruire ancienne position,  recréer à la nouvelle position */
    //printf("initiale :%s\n",&title[0]);
    if (strncmp("R",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       g_object_set(item,"x", x+xtran, "y", y+ytran, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;
     }
    if (strncmp("C",title,1)==0)
     {
       g_object_get(item,"center_x",&xc,"center_y",&yc,NULL);
	   goo_canvas_item_rotate(item,-rota,xc,yc);/* annuler rotation */
       g_object_set(item,"center_x",xc+xtran,"center_y",yc+ytran,NULL);
	   g_object_get(item,"center_x",&xc,"center_y",&yc,NULL);
	   goo_canvas_item_rotate(item,rota,xc,yc);/* remettre rotation */
       return;
     } 
    if (strncmp("L",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */			 
       l_pts= goo_canvas_points_new(2);/* lignes simples */
       g_object_get(item,"points",&l_pts,NULL);    
       l_pts->coords[0]=(gdouble)l_pts->coords[0]+xtran;
       l_pts->coords[1]=(gdouble)l_pts->coords[1]+ytran;
       l_pts->coords[2]=(gdouble)l_pts->coords[2]+xtran;
       l_pts->coords[3]=(gdouble)l_pts->coords[3]+ytran;
       g_object_set(item,"points",l_pts,NULL);       
       goo_canvas_points_unref(l_pts); 
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */	   
       return;
     }
    if (strncmp("T",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       g_object_set(item,"x", x+xtran, "y", y+ytran, NULL);
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */
       return;
     } 
	if (strncmp("B",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */	   
       g_object_set(item,"x", x+xtran, "y", y+ytran, NULL);
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */	   
       return;
     }   
	if (strncmp("U",title,1)==0)
     {/* outline object */
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       g_object_set(item,"x", x+xtran, "y", y+ytran, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;	   
	 }
	if (strncmp("P",title,1)==0)
     {/* path filled object */
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       g_object_set(item,"x", x+xtran, "y", y+ytran, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;	   
	 }       
}
/*********************************
 * placer du texte 
 * ******************************/
void texte(GooCanvasItem *ancetre, gdouble x1t, gdouble y1t,gchar *text)
{
	gchar *str, *str1;
	
	str= g_strdup_printf ("%s",gdk_color_to_string(&PenColor));  
    str1 = g_strdup_printf ("%s",gdk_color_to_string(&FillColor));  
	pDrawings = goo_canvas_text_new (rootKF, text, x1t, y1t, -1,
				   GTK_ANCHOR_CENTER,
				   "font", "Sans 20",
				   "fill-color", str1,
				   "stroke-color",str,
				   NULL);
	g_object_set(G_OBJECT(pDrawings),"title","T",NULL);			
	g_free(str);	  
	g_free(str1); 
	fModifiedDrawing=TRUE;
}
/********************************
 * placer une bitmap
 * ******************************/
void bitmap(GooCanvasItem *ancetre, gdouble x1t, gdouble y1t, GdkPixbuf *pb)
{
	pDrawings=goo_canvas_image_new(rootKF,pb,x1drw,y1drw,NULL);
	g_object_set(G_OBJECT(pDrawings),"title","B",NULL);
	fModifiedDrawing=TRUE;	
}
/*********************************
 * fonctions de tracés implicites
 * tenant compte des valeurs
 * courantes des positions de la
 * souris ;
 * en entrée, pointeur sur la 
 * racine du GROUPE d'items
 * du calque
 * locale
 * ******************************/
void line(GooCanvasItem *ancetre, gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t, gdouble epl)
{
    gchar *str;
    str= g_strdup_printf ("%s",gdk_color_to_string(&FillColor));    
    pDrawings=goo_canvas_polyline_new_line(rootKF,
       x1t,y1t,
	   x2t,y2t,
	   "line-width",epl, 
	   "stroke-color", str,
	   NULL);	
	g_object_set(G_OBJECT(pDrawings),"title","L",NULL);	
	g_free(str);	
	fModifiedDrawing=TRUE;
}
/*************************************************
 * obtain a SVG path sequence from internal path
 * description
 * **********************************************/
gchar *ella_path_to_svg_path(GSList *list, gboolean remplir)
{
	gchar *path=NULL;
	gchar *part;
    GSList *temp;
	gint i=0;
	gdouble x1,y1;
	
	if ((list!=NULL) && (g_slist_length(list)>0))
	  {
       /* we pass all elements to path string */
	   /* first element, a M instruction */
	   temp=g_slist_nth (list, i);
	   path=g_strdup_printf("M %d,%d ",(gint)((ElemPath *)(temp->data))->x,(gint)((ElemPath *)(temp->data))->y);
	   x1=((ElemPath *)(temp->data))->x;
	   y1=((ElemPath *)(temp->data))->y;								
	   /* other elements , L instruction */
	   for(i=1;temp=g_slist_nth (list, i);i++)
	     {
		   part=g_strdup_printf("L %d,%d ",(gint)((ElemPath *)(temp->data))->x,(gint)((ElemPath *)(temp->data))->y);
		   /* concat a new string */
		   path=g_strconcat (path,part , NULL); 
		   g_free(part);
	     }  
	   //if (remplir)
	     //{/* we must add and element to close the path's outline */
		   part=g_strdup_printf("L %d,%d ",(gint)x1,(gint)y1);
		   path=g_strconcat (path,part , NULL); 
		   g_free(part);		   	
	    // }	 		  
	  }	
	return path;
}
/*********************************************************
 * draw a path
 * input : the ancestor, a GSList of points values, a flag
 *     to fill or not
 * ******************************************************/
void polypath(GooCanvasItem *ancetre,GSList *list,gdouble epl, gboolean remplir)
{
    gchar *str,*str1,*path,*part;
    GSList *temp;
	gint i=0;
	gdouble x1,y1;
			
	if (list==NULL)
	   return;
	if (g_slist_length(list)<1)
	   return;
    str= g_strdup_printf ("%s",gdk_color_to_string(&PenColor));  
    str1 = g_strdup_printf ("%s",gdk_color_to_string(&FillColor)); 	  
	path=ella_path_to_svg_path(list, remplir);	
	if (remplir)
	   {
		   pDrawings= goo_canvas_path_new (rootKF,path,"line-width",epl, 
	            "stroke-color", str,"fill-color",str1,NULL);// if specified a fill color = auto fill
		   g_object_set(G_OBJECT(pDrawings),"title","P",NULL);			
	   }
	else
	   { pDrawings= goo_canvas_path_new (rootKF,path,"line-width",epl, 
	            "stroke-color", str1,NULL);// if specified a fill color = auto fill
		 g_object_set(G_OBJECT(pDrawings),"title","U",NULL);
	   }			
	g_free(path);							
	g_free(str);
	g_free(str1);    
	fModifiedDrawing=TRUE;
	fdrawing=TRUE;	
}
/****************************
 * rectangle en coordonnées 
 * naturelles des 4 coins
 * * locale
 * **************************/
void rectangle(GooCanvasItem *ancetre, 
               gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t, 
               gdouble epl, gdouble xr, gdouble yr)
{
    gchar *str,*str1;
    gchar *rx,*ry;
    gchar *lw;
    gdouble t;
    
    /* vérifier les valeurs des coins */
    if (x2t<x1t)
       {
       	t=x1t;
       	x1t=x2t;
       	x2t=t;
	   }
	if (y2t<y1t)
	   {
	   	t=y1t;
	   	y1t=y2t;
	   	y2t=t;
	   }	
    str= g_strdup_printf ("%s",gdk_color_to_string(&PenColor));  
    str1 = g_strdup_printf ("%s",gdk_color_to_string(&FillColor));    
    pDrawings=goo_canvas_rect_new(rootKF,
       x1t,y1t,
	   x2t-x1t,y2t-y1t,
	   "line-width", epl,
	   "radius-x", xr,
	   "radius-y", yr,
	   "stroke-color", str,
	   "fill-color", str1,
	   NULL);	
	g_object_set(G_OBJECT(pDrawings),"title","R",NULL);	
	g_free(str);
	g_free(str1);
	fModifiedDrawing=TRUE;
}
/****************************
 * cercle en coordonnées 
 * naturelles des 4 coins
 * * locale
 * **************************/
void cercle(GooCanvasItem *ancetre, 
            gdouble x1t,gdouble y1t,gdouble x2t,gdouble y2t, 
            gdouble epl)
{
	gchar *str,*str1;
    gdouble rx,ry;
    gchar *lw;
    gdouble t;
    
    /* vérifier les valeurs des coins */
    /*
    if (x2t<x1t)
       {
       	t=x1t;
       	x1t=x2t;
       	x2t=t;
	   }
	if (y2t<y1t)
	   {
	   	t=y1t;
	   	y1t=y2t;
	   	y2t=t;
	   }
	/* corriger les rayons en vérifiant que la bounding rentre dans l'espace admis */
	rx= x2t-x1t;
	/*
	if ((x1t+rx)>(EDITOR_CANVAS_MARGINS+editor_canvas_max_width))
	  {
	  	rx=EDITOR_CANVAS_MARGINS+editor_canvas_max_width-x1t;
	  }
	if ((x1t-rx)<EDITOR_CANVAS_MARGINS)
	  {
	  	rx=x1t-EDITOR_CANVAS_MARGINS;
	  }*/  
	ry=(y2t-y1t);
	/*
	if ((y1t+ry)>(EDITOR_CANVAS_MARGINS+editor_canvas_max_height))
	  {
	  	ry=EDITOR_CANVAS_MARGINS+editor_canvas_max_height-y1t;
	  }
	if ((y1t-ry)<EDITOR_CANVAS_MARGINS)
	  {
	  	ry=y1t-EDITOR_CANVAS_MARGINS;
	  }  */
    str= g_strdup_printf ("%s",gdk_color_to_string(&PenColor));  
    str1 = g_strdup_printf ("%s",gdk_color_to_string(&FillColor)); 
    /* passage des coordonnées naturelles à centre+rayon */
    //printf("x1t %f  y1t %f \n",x1t,y1t);
    pDrawings =  goo_canvas_ellipse_new(rootKF, x1t,y1t,
                  rx, ry,
                   "line-width", epl,				   
	               "stroke-color", str,
	               "fill-color", str1,
	               NULL);
	g_object_set(G_OBJECT(pDrawings),"title","C",NULL);
	g_free(str);
	g_free(str1); 
	fModifiedDrawing=TRUE;  
}
/**************************************************/
/* callbacks de la fenêtre de l'éditeur vectoriel */
/**************************************************/
/******************************************
 * callback for keypress
 * ***************************************/

gboolean
on_editor_key_press_event              (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        GtkWidget *win)
{
  gint keyh; /* stores the GDK key code */
	if (EditorMode==EDITOR_MODE_PATH && event->type == GDK_KEY_PRESS)
	   {
		    keyh = event->keyval;
			switch (keyh)
			  {
				  case GDK_Escape:
				    {	/* store the new path object in data base */
						/* close path, but not fill path */
						goo_canvas_item_remove(pDrawings);
						/* add the first item to close the path but without fill */
						
						polypath(rootGroup,ListElemPath,LineWidth, FALSE);
						editor_inc_item_counter();
	   	                editor_item_set_name(pDrawings ,"U",
						      g_strdup_printf("%s%s","#",ella_path_to_svg_path(ListElemPath, FALSE)));
						undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);	  
						g_slist_free (ListElemPath);
						ListElemPath=NULL;
						NbElemPath=0;
						fdrawing=FALSE;
						return TRUE;
						break;
					}
				  case GDK_Return:
				    {	/* close path and fill , store the new path object in data base */
					printf("je mets en place un P \n");
					goo_canvas_item_remove(pDrawings);
						polypath(rootGroup,ListElemPath,LineWidth, TRUE);
						editor_inc_item_counter();
	   	                editor_item_set_name(pDrawings ,"P",
						        g_strdup_printf("%s%s","#",ella_path_to_svg_path(ListElemPath, TRUE)));
						undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);		
						g_slist_free (ListElemPath);
						ListElemPath=NULL;
						NbElemPath=0;
						fdrawing=FALSE;
						return TRUE;
						break;
					}
				  default:
				    {
						printf("touche non gérée \n");
					}		
			  }/* end switch */
	   }
    /* If the user is resizing an object with his mouse*/
    else if(EditorMode==EDITOR_MODE_RESIZE)
    {
        /* Get the gdk key code, see /usr/include/gtk-2.0/gdk/gdkkeysyms-compat.h */
        keyh = event->keyval;
        switch (keyh)
        {
            /* if he press control, right or left*/
            case GDK_Control_R:
            case GDK_Control_L:
            {
                /* force the object ratio*/
                if(event->type == GDK_KEY_PRESS)
                {
                    force_ratio = 1;
                }
                else
                {
                    force_ratio = 0;
                }
                break;
            }
            default: ;
        }
    }
	return FALSE;
}
/**********************************
 * déplacement de la souris
 * *******************************/
gboolean cb_editor_mouse_move(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win)
{	
	gdouble angle;
	GSList *node;
	gint i;	
	ElemPath *new_elem;
	
	if (fDontRespondToClick)
	   return TRUE;
	fmoved=TRUE; /* la souris a bougé */
    x2drw=correct_x((gdouble)event->x_root);
	y2drw=correct_y((gdouble)event->y_root);	

    /* we show the mouse position in editor's info zone */
	 gtk_label_set_text (GTK_LABEL(labelMousePos), g_strdup_printf("%d,%d",(gint)x2drw,(gint)y2drw));
	/* gérer en fonction du mode en cours */
    if ((EditorMode==EDITOR_MODE_SELECT)&&(fdrawing))
	  {	/* mode sélection : tracer le rectangle de sélection */	       
	       goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */       
	       selection_box(item, x1drw,y1drw,x2drw,y2drw);	       
	  	   return TRUE;
	  }
	if ((EditorMode==EDITOR_MODE_MOVE)&&(fdrawing)&&(SelectedArea))
	  {	  	    
	       goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */  
	       /* vérifier que le nouveau tracé entre dans les limites du dessin */
		   if ((x2drw-x_delta)<0)
		      x2drw=x_delta+1;
		   if ((y2drw-y_delta)<0)
		      y2drw=y_delta+1;	  
	       if ((x2drw+wsel-x_delta)>(editor_canvas_max_width+2*EDITOR_CANVAS_MARGINS))
	           x2drw= editor_canvas_max_width-wsel+2*EDITOR_CANVAS_MARGINS-x_delta;	          
	       if ((y2drw+hsel-y_delta)>(editor_canvas_max_height+2*EDITOR_CANVAS_MARGINS))
	           y2drw=editor_canvas_max_height-hsel+2*EDITOR_CANVAS_MARGINS-y_delta;	   
		   x2drw=x2drw-x_delta;
		   y2drw=y2drw-y_delta;	 	       	 
	       selection_box(item, x2drw,y2drw,x2drw+wsel,y2drw+hsel);	       
	  	   return TRUE;
	  }/* endif move */
	if ((EditorMode==EDITOR_MODE_RESIZE)&&(fdrawing)&&(SelectedArea)&&(fresizing))
	  {	  	    
        goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */  
        /* vérifier que le nouveau tracé entre dans les limites du dessin */  	
        if (x2drw<x1sel+1)
            x2drw=x1sel+1;
        if (y2drw<y1sel+1)
            y2drw=y1sel;
        if(force_ratio == 1)
        {
            gdouble ratio=wmem/hmem;
            gdouble w = x2drw - x1sel;
            gdouble h=w/ratio;
            y2drw = h + y1sel;
        }
        resize_box(item, x1sel,y1sel,x2drw,y2drw);	       
        return TRUE;
	  }/* enfif resize */  
	if ((EditorMode==EDITOR_MODE_ROTATE)&&(fdrawing)&&(frotation))
	  {	  	    
	       goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */  
	       /* calculer l'angle à partir de la position de la souris (quel quadrant ? */       
	       angle = rotate_compute(x2drw,y2drw,(x1sel+x2sel)/2,(y1sel+y2sel)/2);
		   gtk_label_set_text (GTK_LABEL(labelRotationAngle), g_strdup_printf("%.2f°",(180*angle)/PI_PERSO));
	       rotate_box(item,x1sel,y1sel,x2sel,y2sel,angle); 
	  	   return TRUE;
	  }  
    if ((EditorMode==EDITOR_MODE_LINE)&&(fdrawing))
	  {
	  	goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */
	  	line(rootGroup, x1drw,y1drw,x2drw,y2drw, LineWidth);
	  	return TRUE;
	  }
	if ((EditorMode==EDITOR_MODE_PATH)&&(fdrawing))
	  {
	    if (NbElemPath>0)
	       {				  
			  fdrawing=TRUE; 
			  /* we add a lasr temporary element */
			  new_elem = g_malloc(sizeof(ElemPath));
		      new_elem->x=x2drw;
		      new_elem->y=y2drw;
		      ListElemPath=g_slist_append(ListElemPath,new_elem);				  			 			 	 
			  goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */
		      polypath(rootGroup,ListElemPath,LineWidth, FALSE);
			  i=g_slist_length(ListElemPath);
			  ListElemPath=g_slist_remove(ListElemPath,g_slist_nth_data(ListElemPath,i-1));
			  /* we remove the last temporary element */
		   }	
	  	return TRUE;
	  }  
    if ((EditorMode==EDITOR_MODE_BOX)&&(fdrawing))
	  {
	  	goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */
	  	rectangle(rootGroup,x1drw,y1drw,x2drw,y2drw, LineWidth, XLineRound, YLineRound);
	  	return TRUE;
	  } 
	if ((EditorMode==EDITOR_MODE_CIRCLE)&&(fdrawing))
	  {
	  	goo_canvas_item_remove(pDrawings);/* effacer ancien tracé */
	  	cercle(rootGroup,x1drw,y1drw,x2drw,y2drw, LineWidth);
	  	return TRUE;
	  }    
	return TRUE;
}
/****************************
 * relâchement du bouton de
 * la souris sur l'éditeur
 * *************************/
gboolean cb_editor_button_release(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win)
{
	gdouble t, th,tv, angle;      
    gint count;   
    GList *node;
	GdkPixbuf *pb;
	gchar *titre;
	ElemPath *new_elem;
 	
	fDontRespondToClick=FALSE;
    if (event->button==3) /* right click */
   	     return TRUE;
    fdrawing=FALSE;       
    /* clic sans vrai déplacement, et mode selection en cours ? */
    if (EditorMode==EDITOR_MODE_SELECT)
      {	/* corriger les éventuelles inversions */
	    if (x2drw<x1drw)
		      {	t=x2drw;x2drw=x1drw;x1drw=t;  }
	    if (y2drw<y1drw)
		      {	t=y2drw;y2drw=y1drw;y1drw=t; }  
      	SelectedArea=FALSE;
      	fmoved=FALSE;
      	selection_raz();
		 
      	if ((x1drw==x2drw)&&(y1drw==y2drw))		
      	   {	/* chercher dans quel élément se trouve le pointeur car la souris n'a pas bougé */  		  	
			 if (rootKF!=NULL)
			   {
      	  	    //pSelectedItemsList=get_item_at(rootKF);				 
				pSelectedItemsList=get_item_here(rootKF);		 
			   }			  
		   }
		else
		  {	/* appeler la fonction de test de zone car la souris a bougé */		  		   		   
			if (rootKF!=NULL)
			   {
		          pSelectedItemsList=get_items_selected(rootKF,x1drw,y1drw,x2drw,y2drw);
			   }
			else
			   {   /* effacer la sélection */
			       if (fdrawing)
			            goo_canvas_item_remove(pDrawings);
				   SelectedArea=FALSE;
				   fmoved=FALSE;
				   fdrawing=FALSE;				   
			   }
		   }/* else pas déplacement souris*/		   				   			  
		if (pSelectedItemsList)
		  {		    		  	
		    x1sel=x1drw;y1sel=y1drw;
		    x2sel=x2drw;y2sel=y2drw;
		    wsel=x2sel-x1sel;hsel=y2sel-y1sel;
		    SelectedArea=TRUE;
			/* gérer les "entries" des propriétés */
			editor_set_entries(pSelectedItemsList,x1sel,y1sel,wsel,hsel);
			/* gérer les "entries" des styles couleur et épaisseur ligne */
			NbElementsSelectedItemsList=g_list_length(pSelectedItemsList);
			printf("la liste comprend:%d éléments \n",NbElementsSelectedItemsList);
			editor_set_entries_styles(pSelectedItemsList,NbElementsSelectedItemsList);
			undo_reset();
			gtk_label_set_text (GTK_LABEL(labelMouseTips),gettext("> Left-click and drag to move selection"));
		  }	
	    return TRUE;	
	  }/* endif mode select */	  
	if (EditorMode==EDITOR_MODE_MOVE)
	  {	  
		cursor_normal(win); 
		if (fmoved)	  
		   {
				goo_canvas_item_remove(pDrawings);
				x2drw=correct_x((gdouble)event->x_root)-x_delta;
				y2drw=correct_y((gdouble)event->y_root)-y_delta;	
				/* vérifier que le nouveau tracé entre dans les limites du dessin */
				if (x2drw+wsel>editor_canvas_max_width+2*EDITOR_CANVAS_MARGINS)
					{
						x2drw= editor_canvas_max_width-wsel+2*EDITOR_CANVAS_MARGINS;	 						
					}
				if (y2drw+hsel>editor_canvas_max_height+2*EDITOR_CANVAS_MARGINS)
					{
						y2drw=editor_canvas_max_height-hsel+2*EDITOR_CANVAS_MARGINS;							           	
					}  
				/* autoplacement */
				if (fAutoPlacement==TRUE)
						  { 
						   x2drw=get_autoplaced_coordinate(x2drw, AutoPLacementScale);  
						   y2drw=get_autoplaced_coordinate(y2drw, AutoPLacementScale);         
					      }	
				/* faire la translation */
				if (pSelectedItemsList)
					{
						undo_store_datas(UNDO_MOVING, NULL,pSelectedItemsList,x1sel,y1sel,0);
						for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
							{          	                          
								item_move_to((GooCanvasItem *)(node->data),x2drw-x1sel,y2drw-y1sel);
							} /* for */         
						g_list_free(pSelectedItemsList);
						pSelectedItemsList=NULL;
					}/* endif list!=NULL*/
				SelectedArea=FALSE;
				x1sel=0;y1sel=0;x2sel=0;y2sel=0;/* annule la sélection */
				set_mode_select(TRUE); 
			}
	    else
		    {   /* if not moved, it's a call for changing pointer mode ! */
				SelectedArea=TRUE;
				fdrawing=TRUE;
				goo_canvas_item_remove(pDrawings);
				/* switch between reisze mode and rotation mode */
				EditorMode=EDITOR_MODE_RESIZE;
                /*Reset the force _ratio variable, which is set to 1 when ctrl is pressed
                 * (to keep ratio!!)*/
                force_ratio = 0;
				fModifiedDrawing=FALSE;
				/* changing the selection box to resize-box */
			    resize_box(rootKF,x1sel,y1sel,x2sel,y2sel); 
				gtk_label_set_text (GTK_LABEL(labelMouseTips),gettext("> Left-click and drag to size selection ; hit <CTRL> to maintain ratio"));
			}
       return TRUE;
	  } /* endif move */
  	if (EditorMode==EDITOR_MODE_RESIZE)
  	  {
		cursor_normal(win); 
		if (fmoved)
		   {
				if (fresizing)
					goo_canvas_item_remove(pDrawings);
				if ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel)&&(SelectedArea))
					{  	  	    
					/* calculer la nouvelle taille */
					th=resize_compute(wsel,x2drw-x1sel);
					tv=resize_compute(hsel,y2drw-y1sel);
					/* transformer les objets */
					if (pSelectedItemsList)
						{
							undo_store_datas(UNDO_SIZING, NULL,pSelectedItemsList,x1sel,y1sel,0);
							for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
								{          	                          
									item_resize((GooCanvasItem *)(node->data),x1sel,y1sel,th,tv);
								} /* for */         
							g_list_free(pSelectedItemsList);
							pSelectedItemsList=NULL;        
						}/* endif list!=NULL*/
					}
				  fdrawing=FALSE;
				  set_mode_select(TRUE);
	        }/* endif fmoved */
		else
		   {
			   /* because the mouse isn't moved, we change the mode to 'rotation' */
			 if  ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel)&&(SelectedArea)) 
			  {
			    goo_canvas_item_remove(pDrawings);
			    SelectedArea=TRUE;
			    fdrawing=TRUE;
			    frotation=FALSE;
			    EditorMode=EDITOR_MODE_ROTATE;
			    fModifiedDrawing=FALSE;
			    angle = 0;
		        gtk_label_set_text (GTK_LABEL(labelRotationAngle), "-------");
				gtk_label_set_text (GTK_LABEL(labelMouseTips),gettext("> Left-click and drag to rotate selection"));
	            rotate_box(item,x1sel,y1sel,x2sel,y2sel,angle); 		  
		      }
			 else
			  {
				  if (pSelectedItemsList)
					{
						g_list_free(pSelectedItemsList);
						pSelectedItemsList=NULL;
					}/* endif list!=NULL*/
				SelectedArea=FALSE;
				x1sel=0;y1sel=0;x2sel=0;y2sel=0;/* annule la sélection */
				set_mode_select(TRUE);	
				gtk_label_set_text (GTK_LABEL(labelMouseTips),gettext("> left-click and drag to select objects"));				
			  } 
		   }/* else if fmoved */
	   return TRUE;			   
	  }/* endif resize */   
	if (EditorMode==EDITOR_MODE_ROTATE)
  	  {
		cursor_normal(win);  
		if (fmoved)
		  {
			if (frotation)
				goo_canvas_item_remove(pDrawings);
			fdrawing=FALSE;
			frotation=FALSE;
			/* réaliser la transformation */
			angle = rotate_compute(x2drw,y2drw,(x1sel+x2sel)/2,(y1sel+y2sel)/2);
			if (pSelectedItemsList)
				{
                    undo_store_datas(UNDO_ROTATING, NULL,pSelectedItemsList,x1sel,y1sel,angle);
					for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
						{          	                          
							item_rotate((GooCanvasItem *)(node->data),angle);
						} /* for */ 
					g_list_free(pSelectedItemsList);
					pSelectedItemsList=NULL; 
				}
			SelectedArea=FALSE;
			x1sel=0;y1sel=0;x2sel=0;y2sel=0;/* annule la sélection */
			set_mode_select(TRUE);
		}/* endif fmoved */
	   else
	    {
			/* we switch to selection mode because the mouse isn't moved */
			if (frotation)
				goo_canvas_item_remove(pDrawings);
            if (pSelectedItemsList)
					{
						g_list_free(pSelectedItemsList);
						pSelectedItemsList=NULL;
					}/* endif list!=NULL*/
			SelectedArea=FALSE;
			x1sel=0;y1sel=0;x2sel=0;y2sel=0;/* annule la sélection */
			set_mode_select(TRUE);	      
		}
	   return TRUE;		
	  }/* endif rotation */
	if (EditorMode==EDITOR_MODE_TEXT)
	  { 
	    if (rootKF==NULL)
		   return TRUE;
	  	/* passer en mode saisie de texte dans une fenêtre */
	  	gchar *texte_saisi=dlg_saisie_text(gettext("Type here your text "),"");
	  	if (texte_saisi!=NULL)
	  	  {
	  	    texte(rootGroup,x1drw,y1drw,texte_saisi);
			editor_inc_item_counter();	
	  	    editor_item_set_name(pDrawings ,"T",NULL);
			fdrawing=FALSE;/* pour éviter de changer en cas de nouvel outil !*/ 
			set_keyframe();
			undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);
	  	  }
	  	else
		  fdrawing=TRUE;	
	  	return TRUE;
	  }
	 if (EditorMode==EDITOR_MODE_IMAGE)
	  {  
	    if (rootKF==NULL)
		   return TRUE;
	  	/* passer en mode sélection d'image dans une fenêtre */
	  	pb=dlg_import_image((gint)editor_get_page_width(),(gint)editor_get_page_height());	  	
		if (pb!=NULL)
		   {
			 bitmap(rootGroup,x1drw,y1drw,pb);
	  	     editor_inc_item_counter();	
	  	     editor_item_set_name(pDrawings ,"B", NULL);			 
			 /* mettre sur la pile des images */
			 g_object_get(pDrawings,"title",&titre,NULL);
	         pile_images=add_pile_images(pile_images,titre, pb);
			 undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);
			 //g_free(titre); INTERDIT !
		   }
		fdrawing=FALSE;/* pour éviter de changer en cas de nouvel outil !*/ 
	  	return TRUE;
	  }
	 if (EditorMode==EDITOR_MODE_BOX)
	   {  
		 if (rootKF==NULL)
		   return TRUE;
		 if ((x1drw==x2drw)&&(y1drw==y2drw))
		   {
			 goo_canvas_item_remove(pDrawings);  
      	     return TRUE; /*car fausse manoeuvre */
		   }
	   	 editor_inc_item_counter();
	   	 editor_item_set_name(pDrawings ,"R",NULL);
		 undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);
	   	 return TRUE;
	   } 
	 if (EditorMode==EDITOR_MODE_LINE)
	   {
		 if (rootKF==NULL)
		   return TRUE;
		 if ((x1drw==x2drw)&&(y1drw==y2drw))
		   {
			 goo_canvas_item_remove(pDrawings);  
      	     return TRUE; /*car fausse manoeuvre */
		   }
	   	 editor_inc_item_counter();
	   	 editor_item_set_name(pDrawings ,"L",NULL);
		 undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);
	   	 return TRUE;
	   } 
	  if (EditorMode==EDITOR_MODE_PATH)
	   {
		 if (rootKF==NULL)
		   return TRUE;
		 xmem_path=x2drw;
		 ymem_path=y2drw;
		 NbElemPath++;
		 new_elem = g_malloc(sizeof(ElemPath));
		 new_elem->x=x2drw;
		 new_elem->y=y2drw;
		 ListElemPath=g_slist_append(ListElemPath,new_elem);
		 //printf("la liste a maintenant %d élements \n",g_slist_length(ListElemPath));		
		 if (NbElemPath>0)
		   {			
					if (NbElemPath>1)
			            goo_canvas_item_remove(pDrawings); 			   
			fdrawing=TRUE;   
		    polypath(rootGroup,ListElemPath,LineWidth, FALSE);
			// undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL);
		   }		
	   	 return TRUE;
	   }   
	 if (EditorMode==EDITOR_MODE_CIRCLE)
	   { 
		 if (rootKF==NULL)
		   return TRUE;  
		 if ((x1drw==x2drw)&&(y1drw==y2drw))
		   {
			 goo_canvas_item_remove(pDrawings);  
      	     return TRUE; /*car fausse manoeuvre */
		   }
	   	 editor_inc_item_counter();
	   	 editor_item_set_name(pDrawings ,"C",NULL);
		 undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);
	   	 return TRUE;
	   } 
 /* traiter le cas où la souris n'a pas bougé ! */	  
 SelectedArea=FALSE;
 if (!fmoved)
     goo_canvas_item_remove(pDrawings);/* détruire début de dessin si la souris n'a pas bougé*/	     
 return TRUE;
}
/***********************************
 * clic bouton souris
 * item pointe sur la racine des
 * dessins de l'éditeur
 * ********************************/
gboolean cb_editor_button_press(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win)
{
   GooCanvasItem  *tmp;
   gboolean reponse;
     
    gtk_label_set_text (GTK_LABEL(labelRotationAngle), "-------");
    fDontRespondToClick=FALSE;
	fmoved=FALSE; /* la souris n'a pas encore bougé */
	/* conversion des coordonnées plus correction */
	x1drw=correct_x((gdouble)event->x_root);
	y1drw=correct_y((gdouble)event->y_root);	
	x2drw=x1drw;y2drw=y1drw;	
	/* double click */	
	if (event->type==GDK_2BUTTON_PRESS)
	   {
		   if ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel)&&(SelectedArea))
		      {
				  /* there is a selection, so we must answer to the double click */
				  /* if the first item is a text, we call the text editor */
				  if (item_is_text(pSelectedItemsList)!=NULL)
				      {
						  reponse= cb_text_edit(NULL,NULL);
						  SelectedArea=TRUE;
						  fDontRespondToClick=TRUE;
						  EditorMode=EDITOR_MODE_SELECT;
						  return TRUE;
					  }
				  /* if the first item isn't a text item, we call the 'color' dialog' */
				 // choix_couleurs();
				 // SelectedArea=FALSE;
      	         // fmoved=FALSE;
      	         // selection_raz();
				  return TRUE;
			  }
		   return TRUE;
	   } 	
	/* right click */   
	if (event->button==3)
       {
   	     editor_popup_menu_(item, event, correct_x((gdouble)event->x), correct_y((gdouble)event->y));
   	     return TRUE;
       }
	if ((SelectedArea)&&(EditorMode==EDITOR_MODE_SELECT))
		     goo_canvas_item_remove(pDrawings);
	
	if (EditorMode==EDITOR_MODE_SELECT)
	  {
	  	/* trouver si un item vient d'être cliqué */
	  	   if ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel)&&(SelectedArea))
	  	     {	  	     
	  	        EditorMode=EDITOR_MODE_MOVE;
				x_delta=x1drw-x1sel;y_delta=y1drw-y1sel;
	  	        x1drw=x1sel;y1drw=y1sel;
	  	        x2drw=x2sel;y2drw=y2sel;	
				cursor_hand(win);					  	     	     
		     }/* endif */
		   if (rootKF==NULL)
		         return TRUE;  	 
	       fdrawing=TRUE;
	       selection_box(rootKF, x1drw,y1drw,x2drw,y2drw);
	  	   return TRUE;
	  }/* endif mode select */
	
    if ((EditorMode==EDITOR_MODE_RESIZE)&&(SelectedArea))
	  {
	  	/* si dans la bonne zone, continuer, sinon quitter */
		 goo_canvas_item_remove(pDrawings);
	  	 if ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel))
	  	   {
	         fdrawing=TRUE;
	         fresizing=TRUE;
	         frotation=FALSE;
	         resize_box(rootKF,x1sel,y1sel,x1sel+x2sel-x1sel,y1sel+y2sel-y1sel); 
			 cursor_resize(win);
	  	     return TRUE;
		   }	
		fresizing=FALSE;
		return TRUE;	
	  }/* endif mode resize */
	if ((EditorMode==EDITOR_MODE_ROTATE)&&(SelectedArea))
	  {  /* si dans la bonne zone, continuer, sinon quitter */	    
	  	 if ((x1drw>x1sel)&&(x1drw<x2sel)&&(y1drw>y1sel)&&(y1drw<y2sel))
	  	   {
			 goo_canvas_item_remove(pDrawings);  
	         fdrawing=TRUE;
	         frotation=TRUE;
	         rotate_box(rootKF,x1sel,y1sel,x1sel+wsel,y1sel+hsel,0); 
			 cursor_rotate(win);
		   }
		 else
		   {		  
			set_mode_select(FALSE);// il ne faut pas enlever le dessin !
			fdrawing=FALSE;	    
		   }
	    return TRUE;
	  }/* endif mode rotate */
	if (EditorMode==EDITOR_MODE_LINE)
	  {
		if (rootKF==NULL)
		   return TRUE;  
	  	fdrawing=TRUE;
	  	line(rootGroup, x1drw,y1drw,x2drw,y2drw, LineWidth);
	  	return TRUE;
	  }
	if (EditorMode==EDITOR_MODE_PATH)
	  {
		if (rootKF==NULL)
		   return TRUE;  
	  	fdrawing=FALSE;
	  	//line(rootGroup, x1drw,y1drw,x2drw,y2drw, LineWidth);
	  	return TRUE;
	  }  
	if (EditorMode==EDITOR_MODE_BOX)
	  {
		if (rootKF==NULL)
		  {
		   return TRUE;
	      }
	  	fdrawing=TRUE;
	  	rectangle(rootGroup,x1drw,y1drw,x2drw,y2drw, LineWidth , XLineRound, YLineRound);
	  	return TRUE;
	  }  	  
    if (EditorMode==EDITOR_MODE_CIRCLE)
	  {
		if (rootKF==NULL)
		   return TRUE;  
	  	fdrawing=TRUE;	  	
	  	cercle(rootGroup,x1drw,y1drw,x2drw,y2drw, LineWidth);
	  	return TRUE;
	  }
    if (EditorMode==EDITOR_MODE_TEXT)
	  {
		if (rootKF==NULL)
		   return TRUE;  
	  	fdrawing=TRUE;
	  	return TRUE;
	  }
	 if (EditorMode==EDITOR_MODE_IMAGE)
	  {
		if (rootKF==NULL)
		   return TRUE;  
	  	fdrawing=TRUE;
	  	return TRUE;
	  }
	fdrawing=FALSE;
	return TRUE;
}
/*********************************
 * fonction pour insérer un objet
 * bitmap par DnD
 * ******************************/
void editor_add_bitmap_dnd(gint x,gint y,GdkPixbuf *pb)
{
	gchar *titre;
	
	if (rootKF==NULL)
		   return;
	if (pb!=NULL)
		   {
			 bitmap(rootGroup,x,y,pb);
	  	     editor_inc_item_counter();	
	  	     editor_item_set_name(pDrawings ,"B", NULL);			 
			 /* mettre sur la pile des images */
			 g_object_get(pDrawings,"title",&titre,NULL);
	         pile_images=add_pile_images(pile_images,titre, pb);
			 fModifiedDrawing=TRUE;
			 //g_free(titre); INTERDIT !
		   }	
}
/**************************************
 * add a symbol (path object) by DnD
 * ***********************************/
void editor_add_symbol_dnd(gint x,gint y,gchar *path)
{
 gchar *str,*str1;
 	
	if (rootKF==NULL)
		   return;
    str= g_strdup_printf ("%s",gdk_color_to_string(&PenColor));  
    str1 = g_strdup_printf ("%s",gdk_color_to_string(&FillColor));		   
	pDrawings= goo_canvas_path_new (rootKF,path,"line-width",LineWidth, 
	            "stroke-color", str,"fill-color",str1,NULL);// if specified a fill color = auto fill
    g_object_set(G_OBJECT(pDrawings),"title","P","x",(gdouble)x,"y",(gdouble)y,NULL);
	g_free(str);
	g_free(str1);    
	fModifiedDrawing=TRUE;
	fdrawing=FALSE;	
	editor_inc_item_counter();
	editor_item_set_name(pDrawings ,"P",g_strdup_printf("%s%s","#",(gchar *)path));
	undo_store_datas(UNDO_NEW_DRAWING, pDrawings, NULL,-1,-1,0);	
		   	
}
/**********************************
 * fonction de callback du choix 
 * de la couleur du tracé
 * *******************************/
gboolean cb_color_trace(GtkWidget *pBtn, gpointer data)
{
	if ((EditorMode==EDITOR_MODE_BOX) ||
	    (EditorMode==EDITOR_MODE_CIRCLE) ||
		(EditorMode==EDITOR_MODE_TEXT) ||
		(EditorMode==EDITOR_MODE_LINE) ||
		(EditorMode==EDITOR_MODE_PATH) ||
		 (EditorMode==EDITOR_MODE_IMAGE)
	    )
		   cursor_normal(pWinMemo);
	EditorMode=EDITOR_MODE_SELECT;	   
    choix_couleurs();
	pEditorModeItem=gtk_toolbar_get_nth_item(GTK_TOOLBAR(pToolbarEditor),EDITOR_TOOL_SELECT);/* 2 car item de rang 3 !*/
    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(pEditorModeItem),TRUE);
    return TRUE;
}

/**********************************
 * fonction de callback du choix 
 * du style du texte
 * *******************************/
gboolean cb_style_texte(GtkWidget *pBtn, gpointer data)
{
    gint count;
	GList *node;
	PangoFontDescription *FntDesc=NULL;
	gchararray title;
	
	if ((EditorMode==EDITOR_MODE_BOX) ||
	    (EditorMode==EDITOR_MODE_CIRCLE) ||
		(EditorMode==EDITOR_MODE_TEXT) ||
		(EditorMode==EDITOR_MODE_LINE) ||
		(EditorMode==EDITOR_MODE_PATH) ||
		 (EditorMode==EDITOR_MODE_IMAGE)
	    )
	cursor_normal(pWinMemo);
	EditorMode=EDITOR_MODE_SELECT;	 
	pEditorModeItem=gtk_toolbar_get_nth_item(GTK_TOOLBAR(pToolbarEditor),EDITOR_TOOL_SELECT);/* 2 car item de rang 3 !*/
    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(pEditorModeItem),TRUE);
	FntDesc=dlg_fontes(gettext("Font choice "));
	if (FntDesc)
	   {
	   	  if (pSelectedItemsList)
	        {
	  	      for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
               {          	                          
				g_object_get((GooCanvasItem *)(node->data),"title",&title,NULL);
				printf("titre dans dlg fontes>%s<\n",title);
				if (strncmp("T",title,1)==0)
                   g_object_set((GooCanvasItem *)(node->data),"font-desc",FntDesc,NULL);
               } /* for */         
	     }/* endif list!=NULL*/
		 //g_free(FntDesc);
	   }/* endif gntdesc */
    return TRUE;
} 
/**********************************
 * fonction callback :
 * en entrée, le pointeur sur le
 * radiobutton, et le n° du
 * bouton
 * très important : les seules
 * macros admises sont les
 * "classiques" et non pas
 * les spécifiques avec _tool !!
 * *******************************/

gboolean cb_toolbar_toggled(GtkWidget *pBtn, gpointer data)
{
	GSList *pList;
	gint count;	

	if (fFirstCall==FALSE)
	   {
		   fFirstCall=TRUE;
		   return FALSE;
	   }

	fFirstCall=!fFirstCall;
	/* clear cursor if we are among one of the drawing modes */
	if ((EditorMode==EDITOR_MODE_BOX) ||
	    (EditorMode==EDITOR_MODE_CIRCLE) ||
		(EditorMode==EDITOR_MODE_TEXT) ||
		(EditorMode==EDITOR_MODE_LINE) ||
		(EditorMode==EDITOR_MODE_PATH) ||
		 (EditorMode==EDITOR_MODE_IMAGE)
	    )
		{
			SelectedArea=FALSE;
	        cursor_normal(pWinMemo);
	    }
	/* Recuperation de la liste des boutons */
    pList = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(data));
    /* Parcours de la liste */
    count=MAX_TOOLS;
    while(pList)
      { /* Le bouton est il selectionne */
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pList->data)))
         {                        
            /* On met la liste a NULL pour sortir de la boucle */
            pList = NULL;
         }
        else
         {  /* NON -> on passe au bouton suivant */
            pList = g_slist_next(pList);
            count--;
         }
      }

	switch (count)
	   {
		   case 1:{EditorMode=EDITOR_MODE_SELECT;break;}
		   case 2:{EditorMode=EDITOR_MODE_LINE;break;}
		   case 3:{EditorMode=EDITOR_MODE_PATH;break;}
		   case 4:{EditorMode=EDITOR_MODE_BOX;break;}
		   case 5:{EditorMode=EDITOR_MODE_CIRCLE;break;}
		   case 6:{EditorMode=EDITOR_MODE_TEXT;break;}
		   case 7:{EditorMode=EDITOR_MODE_IMAGE;break;}
		   default:{break;}
	   }/* end switch */
	printf("sélection en cours :%d\n",EditorMode);
	
	if (count==1) // new position of mode select button, sorry !
	   EditorMode=EDITOR_MODE_SELECT;
    if ((EditorMode<=8)&&(SelectedArea)&&(EditorMode!=EDITOR_MODE_ROTATE)&&(EditorMode!=EDITOR_MODE_RESIZE)  )
	    {
		  fdrawing=FALSE;
		  selection_raz();
		  goo_canvas_item_remove(pDrawings);
		  return TRUE;
		}
    if ((EditorMode==EDITOR_MODE_RESIZE)&&(SelectedArea))
       {
       	 fdrawing=TRUE;
       	 goo_canvas_item_remove(pDrawings);
	     resize_box(rootEditorItems,x1drw,y1drw,x1drw+x2sel-x1sel,y1drw+y2sel-y1sel);  
		 return TRUE;   
	   }
   if ((EditorMode==EDITOR_MODE_ROTATE)&&(SelectedArea))
       {
       	 fdrawing=TRUE;
       	 goo_canvas_item_remove(pDrawings);
	     rotate_box(rootEditorItems,x1drw,y1drw,x1drw+x2sel-x1sel,y1drw+y2sel-y1sel,0);  
		 return TRUE;   
	   }	
   if (EditorMode==EDITOR_MODE_LINE)
     cursor_line(pWinMemo);   
   if (EditorMode==EDITOR_MODE_PATH)
     cursor_path(pWinMemo);	 
   if (EditorMode==EDITOR_MODE_TEXT)
     cursor_text(pWinMemo); 	 
   if (EditorMode==EDITOR_MODE_CIRCLE)
     cursor_circle(pWinMemo);  
   if (EditorMode==EDITOR_MODE_IMAGE)
     cursor_image(pWinMemo); 	 
   if (EditorMode==EDITOR_MODE_BOX)
     cursor_box(pWinMemo); 	 
   return TRUE;	 
}
/**************************************
 * initialisation du canvas de
 * l'éditeur
 * entrée : scrolledwindow à laquelle
 * est accroché le canvas,
 * sortie : pointeur sur le nouveau
 * root du canvas ainsi créé.
 * locale sauf pour les constantes
 * ***********************************/
GooCanvasItem *init_canvas_editor(GtkWidget *win, GtkWidget *label, GtkWidget *label_mouse, GtkWidget *label_tips)
{
	static GtkWidget *canvas;	
	GooCanvasItem *item;
	static GooCanvasItem *root;
	gchar *str;
	
	 /* store a pointer on teh scrolledwindow to cursors */
    pWinMemo=win;
	labelRotationAngle =label;/* to store an external widget */
	labelMousePos=label_mouse;
    labelMouseTips=label_tips;
	canvas = goo_canvas_new ();
	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 
	                       EDITOR_MAX_WIDTH+2*EDITOR_CANVAS_MARGINS, 
	                       EDITOR_MAX_HEIGHT+2*EDITOR_CANVAS_MARGINS);
	
    gtk_widget_show (canvas);
    gtk_container_add (GTK_CONTAINER (win), canvas);
    root = goo_canvas_get_root_item (GOO_CANVAS (canvas));  
	/* récupérer les valeurs par défaut */
	
    /* tracer un bord gris */                       
	/*itemBack= goo_canvas_rect_new (root,
	                0, 0, editor_canvas_max_width+2*EDITOR_CANVAS_MARGINS,
	                 editor_canvas_max_height+2*EDITOR_CANVAS_MARGINS,
				   "line-width", 10.0,"radius-x", 0.0,"radius-y", 0.0,
				   "stroke-color", "darkgrey","fill-color", "lightgrey",
				   NULL);   */
	//g_object_set(G_OBJECT(itemBack),"title","A",NULL);	/* arrière plan */
    /* tracer le rectangle de fond pour qu'il soit accessible en propriétés */
	itemPage= goo_canvas_rect_new (root,
	                EDITOR_CANVAS_MARGINS,EDITOR_CANVAS_MARGINS, 
	                editor_canvas_max_width,editor_canvas_max_height,
				   "line-width", 0.5,"radius-x", 0.0,
				   "radius-y", 0.0,"stroke-color", "black",
				   "fill-color", "white",
				   NULL);	
    g_object_set(G_OBJECT(itemPage),"title","F",NULL);	/* arrière plan */
	
    /* couleur de fond */
    str= g_strdup_printf ("%s",gdk_color_to_string(&PageColor));
    g_object_set(G_OBJECT(itemPage),"fill-color",str,NULL);
    g_free(str);	
  return root;
}
/**************************************
 * création du groupe d'items propres
 * à un calque donné
 * on passe en entrée un pointeur sur
 * l'item "root" du canvas
 * retourne un pointeur sur le groupe
 * associé
 * locale
 * ***********************************/
GooCanvasItem *init_items_layer(GooCanvasItem *pere)
{
	GooCanvasItem *ancetre;
	
	ancetre = goo_canvas_group_new(pere,NULL);/* crée vide, donc NULL en paramètre */
	g_object_set(ancetre, "title", "K", NULL);
	return ancetre;
}
/***************************************
 * création de la barre d'outils ;
 * retourne un pointeur sur la barre
 * d'outils nouvellement créée 
 * entièrement locale
 * ************************************/
GtkWidget *toolbar_init(GtkWidget *win)
{
 GtkWidget *pToolbar;
 GtkToolItem *pToolItem;
 GSList *pListToolItems;
 GdkPixbuf *pPix;
 GtkWidget *iconw;
 GtkWidget *pLabelTitre;
 GtkWidget *pLabelCouleurs;
 gchar *chaine;  
  
 
  /* Creation de la barre d'outils */
  pToolbar = gtk_toolbar_new();
  gtk_toolbar_set_orientation(GTK_TOOLBAR(pToolbar),GTK_ORIENTATION_VERTICAL);
  /* 1er séparateur */
  pToolItem = gtk_separator_tool_item_new();
  gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);/* -1 ajout à la suite */
  /* insertion des item par la nouvelle méthode GTK 2.12 */
   
   /* 1er bouton : sélection */
   pToolItem= gtk_radio_tool_button_new(NULL);
   //pEditorModeItem=pToolItem;
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);/* -1 ajout à la suite */
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)pointeur_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("select an object or a group"));
   gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(pToolItem),TRUE);/* flèche sélection outil par défaut */
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);
   
   /* 2ème bouton : zoom */
   /*
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
  /*
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)zoom_plus_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,"agrandir une partie de l'image");
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);
   */
   /* 3ème bouton */
   
  // pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
  /* pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)redim_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,"redimensionner un objet");
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);
   /* bouton rotation */
   //pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   /*pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)rotation_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,"rotation d'un objet");
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);  
 
   /* outil lignes */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)lignes_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Draw a line "));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);   
   /* path tool (i.e polylines ) */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)tool_path_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Draw a path "));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);    
   /* outil rectangles */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)rectangle_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Draw a box or a rectangle"));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);   
   
   /* outil cercles */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)cercles_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Draw a circle or an ellipse"));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);   
   
   /* outil texte */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)texte_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("insert text"));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);   
   
   /* outil insertion d'images */
   pListToolItems = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(pToolItem));/* récupère le pointeur sur le groupe */
   pToolItem= gtk_radio_tool_button_new(pListToolItems);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)images_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("insert an image"));
   g_signal_connect(G_OBJECT(pToolItem), "toggled", G_CALLBACK(cb_toolbar_toggled), pToolItem);      
   /* outil couleur de tracé : boutons normaux, réponse au clic */
   
   pToolItem= gtk_tool_button_new(NULL, NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)couleurs_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Choose colors and opacity for the current drawings"));
   g_signal_connect(G_OBJECT(pToolItem), "clicked", G_CALLBACK(cb_color_trace), NULL);   

   /* outil couleur de remplissage : boutons normaux, réponse au clic */
   /*
   pToolItem= gtk_tool_button_new(NULL, NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)couleur_fill_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,"choisir la couleur de remplissage");
   g_signal_connect(G_OBJECT(pToolItem), "clicked", G_CALLBACK(cb_color_fill), NULL);   
   */
   /* outil style de tracé : boutons normaux, réponse au clic */
   /*
   pToolItem= gtk_tool_button_new(NULL, NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)style_trace_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,"choisir le style des traces");
   g_signal_connect(G_OBJECT(pToolItem), "clicked", G_CALLBACK(cb_style_trace), NULL);   
*/
  /* outil style de texte : boutons normaux, réponse au clic */
   pToolItem= gtk_tool_button_new(NULL, NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);
   pPix = gdk_pixbuf_new_from_xpm_data((const char **)style_texte_xpm);  
   iconw = gtk_image_new_from_pixbuf(pPix);
   g_object_unref(pPix);
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(pToolItem), iconw); 
   gtk_tool_item_set_tooltip_text(pToolItem,gettext("Choose writing style (font, font size)"));
   g_signal_connect(G_OBJECT(pToolItem), "clicked", G_CALLBACK(cb_style_texte), NULL);        
   /*  séparateur des outils de dessin */
   pToolItem = gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,8);/*  ajout position 5 */
   /*  séparateur final */
   pToolItem = gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,-1);/* -1 ajout à la suite */
   
   /* titre de la palette d'outils, avec styles Pango */
   pToolItem =  gtk_tool_item_new ();
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,0);/*  ajout position début*/   
   pLabelTitre = gtk_label_new ("");
   chaine = g_strdup_printf("%s",gettext("Tools"));
   gtk_label_set_markup(GTK_LABEL(pLabelTitre), chaine);
   g_free(chaine);
   gtk_label_set_use_markup(GTK_LABEL(pLabelTitre), TRUE);
   gtk_widget_show (pLabelTitre);
   gtk_container_add (GTK_CONTAINER (pToolItem), pLabelTitre);

   /*  séparateur des outils de couleurs */
   //pToolItem = gtk_separator_tool_item_new();
   //gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,6);/*  ajout position 5 */        
   /* titre pour les couleurs, avec styles Pango */
   pToolItem =  gtk_tool_item_new ();
   gtk_toolbar_insert(GTK_TOOLBAR(pToolbar),pToolItem,10);/*  ajout position début*/  
   pLabelCouleurs = gtk_label_new ("");
   chaine = g_strdup_printf("%s",gettext("<span face=\"sans 8\">styles</span>"));
   gtk_label_set_markup(GTK_LABEL(pLabelCouleurs), chaine);
   g_free(chaine);
   gtk_label_set_use_markup(GTK_LABEL(pLabelCouleurs), TRUE);
   gtk_widget_show (pLabelCouleurs);
   gtk_container_add (GTK_CONTAINER (pToolItem), pLabelCouleurs);
   /* Affichage uniquement des icones */
   gtk_toolbar_set_style(GTK_TOOLBAR(pToolbar),GTK_TOOLBAR_ICONS);
   gtk_toolbar_set_icon_size(GTK_TOOLBAR(pToolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
   gtk_toolbar_set_show_arrow(GTK_TOOLBAR(pToolbar),TRUE);/* pour étendre la toolbar si nécessaire */
  return pToolbar;
}
/******************************************
 * moveto for alignment
 * a parameter =-1 means don't move
 * ***************************************/
void item_set_position_to(GooCanvasItem *item, gdouble xtran, gdouble ytran, gint mode_h_align, gint mode_v_align)
{
	gchararray title;
    gdouble angle;
	gdouble x,y;/* for rectangles */
	gdouble xc,yc; /* for ellipses */
	gdouble xbis,ybis;/* local computation */
	gdouble width, height;
	gdouble xt,yt,echelle,rota;/* pour tenir compte des transformations */
	GooCanvasPoints *l_pts;	
	GooCanvasItem *parent;
	gboolean transforme=FALSE;
	GooCanvasBounds area;
	
	/* si l'ancêtre de l'item n'est pas le groupe courant, sortir */
	parent=goo_canvas_item_get_parent(item);	
	if (parent==NULL)
		 return;
	if (parent!=rootKF)
	    return;

	/* récupérer les bounds, et les déplacer */
	goo_canvas_item_get_bounds(item, &area);	
	g_object_get(item,"title",&title,"width",&width,"height",&height,NULL);
    /* récupérer l'angle de rotation courant */
	transforme=goo_canvas_item_get_simple_transform(item,&xt,&yt,&echelle,&rota);
     
    if (strncmp("R",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
	   xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;	  
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;	  
	     }
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;
     }
    if (strncmp("C",title,1)==0)
     {
       g_object_get(item,"center_x",&xc,"center_y",&yc,NULL);
	   goo_canvas_item_rotate(item,-rota,xc,yc);/* annuler rotation */
       xbis=xc;
	   if (xtran>=0)
	     {
	       xbis=xtran+(width/2);
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;
	     }
	   ybis=yc;
	   if (ytran>=0)
	     {
	       ybis=ytran+(height/2);	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;
	     }
       g_object_set(item,"center_x", xbis, "center_y", ybis, NULL);
	   g_object_get(item,"center_x",&xc,"center_y",&yc,NULL);
	   goo_canvas_item_rotate(item,rota,xc,yc);/* remettre rotation */
       return;
     } 
    if (strncmp("L",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */			 
       xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;	  
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;	  
	     }	
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */	   
       return;
     }
    if (strncmp("T",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran+(area.x2-area.x1)/2;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-(area.x2-area.x1);
		   if (mode_h_align==2)/* center */
		      xbis=xbis-(area.x2-area.x1)/2;
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran+(area.y2-area.y1)/2;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-(area.y2-area.y1);
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-(area.y2-area.y1)/2;
	     }
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */
       return;
     } 
	if (strncmp("B",title,1)==0)
     {
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */	 
	   xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;	  
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;	  
	     }
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);/* remettre rotation */	   
       return;
     }   
	if (strncmp("U",title,1)==0)
     {/* outline object */
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
	   xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;	  
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;	  
	     }	
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;	   
	 }
	if (strncmp("P",title,1)==0)
     {/* path filled object */
       g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2);              
       goo_canvas_item_rotate(item,-rota,xt,yt);/* annuler rotation */
       xbis=x;
	   if (xtran>=0)
	     {
	       xbis=xtran;
		   if (mode_h_align==1)/* right */
		      xbis=xbis-width;
		   if (mode_h_align==2)/* center */
		      xbis=xbis-width/2;	  
	     }
	   ybis=y;
	   if (ytran>=0)
	     {
	       ybis=ytran;	
		   if (mode_v_align==1)/* down */
		      ybis=ybis-height;
  		   if (mode_v_align==2)/* mid */
		      ybis=ybis-height/2;	  
	     }
       g_object_set(item,"x", xbis, "y", ybis, NULL);
	   /* remettre l'angle */
	   g_object_get(item,"x",&x,"y",&y,NULL);
	   xt=x+(width/2);
       yt=y+(height/2); 
	   goo_canvas_item_rotate(item,rota,xt,yt);
       return;	   
	 }       
}
/******************************************
 * set alignment of items
 * ***************************************/
gboolean editor_set_align(gint h, gint v, gint z)
{
	gboolean retour=FALSE;
	gdouble x,y,w,ht;
	gint count;
    GList *node;
				
    if (pSelectedItemsList)
	     {	
			 /* two cases : align IN selection, OR align IN scenery */
			switch (z)
			  {
				  case 0:/* scenery */
				    {
						x=editor_get_page_margins();
						y=editor_get_page_margins();
						w=editor_get_page_width();
						ht=editor_get_page_height();
						break;
					}
				  case 1:/* selection*/
				    {
						x=x1sel;
						y=y1sel;
						w=wsel;
						ht=hsel;
						break;
					}	
				  default:;	
			  }/* end switch*/
			  if ((h!=0)||(v!=0))
			      undo_store_datas(UNDO_ALIGN_ITEMS, NULL,pSelectedItemsList,x1sel,y1sel,0);
			 /* now, FOUR cases of horizontal align AND FOUR cases of vertical align */
			 /* horizontally */
			 if (h!=0)
			   {				 
				  for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
					{    
						switch (h)
						  {       
							case 1:/* right */
							  {
							    item_set_position_to((GooCanvasItem *)(node->data),x+w-1,-1,h,v);
							    break;
							  }
							case 2:/* center*/
							  {
							    item_set_position_to((GooCanvasItem *)(node->data),x+w/2,-1,h,v);
							    break;
							  }
							case 3:/* left*/
							  {    
						        item_set_position_to((GooCanvasItem *)(node->data),x,-1,h,v);
								break;
							  }
							default:;  
						  }/* end switch */
					} /* for */         		     	  				  
			   }/* endif h!=0*/
			 /* vertically */ 
			 if (v!=0)
			   {
				 for (count = 0; node = g_list_nth (pSelectedItemsList, count); count++) 
					{           
						switch (v)
						  {       
							case 1:/* low */
							  {
							    item_set_position_to((GooCanvasItem *)(node->data),-1,y+ht-1,h,v);
							    break;
							  }
							case 2:/* mid*/
							  {
							    item_set_position_to((GooCanvasItem *)(node->data),-1,y+ht/2,h,v);
							    break;
							  }
							case 3:/* up*/
							  {    
						        item_set_position_to((GooCanvasItem *)(node->data),-1,y,h,v);
								break;
							  }
							default:;  
						  }/* end switch */												
					} /* for */
			   }/*endif v!=0*/
             g_list_free(pSelectedItemsList);
			 pSelectedItemsList=NULL;				
			 SelectedArea=FALSE;
			 x1sel=0;y1sel=0;x2sel=0;y2sel=0;/* cancel selection */			 
             goo_canvas_item_remove(pDrawings);
			 fdrawing=FALSE;				
			 set_mode_select(TRUE);			   
			 retour=TRUE;
		 }/* endif pSelectedItemList!=NULL*/
	return retour;
}
