/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/


/****************************
 * bibliothèque d'exportation
 * vers le format SWF
 * **************************/
//translations
#include <locale.h>
#include <libintl.h>

#include "math.h"
#include <gtk/gtk.h>
#include <goocanvas.h>
#include "ellasyms.h"
#include "pango/pangofc-font.h"
#include "pango/pangofc-fontmap.h"

/* constantes des effets :
 1 mouvement, 2 zoom, 4 rotation, 8 luminance,
			   * 16 alpha */
 
/* structure spéciale pour la pile d'items */
typedef struct
      {
		gint layer; /* calque d'appartenance */
        gint x; /* position en pixels */
        gint y;
		gint cx;/* cercle*/
		gint cy;
		gint rx;
		gint ry;
		gint w;/* pour gérer le zooming */
		gint h;
        gint zoom;/* de 0 à x % */
        gdouble rotation;/* en degrés */    
        gchar *titre;/* nom de l'objet */  
		gboolean fDel;/* flag pour savoir s'il a subi un .del */  
      } stack_data_type;
/* structure spéciale pour optimiser l'usage des fontes */
typedef struct
      {
		  gchar *file_font;
		  gchar *nom_font;
	  } stack_font;
	  	  
GSList *pile_items_export;
GSList *pile_fontes_export;
GSList *pile_sound;

/* variables globales */
gint stack_object_width,stack_object_height;
gdouble stack_object_rotation;
static gint snd_item_count=0;
static GtkWidget *pOnAWidget;
/*******************************************
 * renvoyer un chemin d'accès à un fichier
 * duquel on a retiré l'éventuelle
 * extension 
 * ****************************************/
gchar *file_str_sub_extension (const gchar *s, guint start, guint end)
{
   gchar *new_s = NULL;

   if (s != NULL && start < end)
   {
/* (1)*/
      new_s = g_malloc (sizeof (*new_s) * (end - start + 2));
      if (new_s != NULL)
      {
         gint i;

/* (2) */
         for (i = start; i <= end; i++)
         {
/* (3) */
            new_s[i-start] = s[i];
         }
         new_s[i-start] = '\0';
      }
      else
      {
         fprintf (stderr, "Memoire insuffisante\n");
         exit (0);
      }
   }
   return new_s;
}
/********************************
 * convertir les angles horaires
 * de goocanvas vers les angles
 * antihoraires swf
 * *****************************/
gdouble goo_to_swf_angle(gdouble a)
{
	return (gdouble) (a-360.0);
}
/**********************************
 * recoder les RC en escapes
 * *******************************/
gchar *str_replace_rc (const char *string)
{
   gchar *strip = NULL;

   if (string != NULL)
   {
      strip = malloc (2*sizeof (*strip) * (strlen (string) + 1));
      if (strip != NULL)
      {
         gint i, j;

         for (i = 0, j = 0; string[i]; i++)
         {
            if (string[i] == '\n')
            {
           
                  strip[j] = '\\';
				  j++;
				  strip[j] = 'n';
                  j++;
            }
            else
            {
               strip[j] = string[i];
               j++;
            }
         }
        strip[j] = '\0';
      }
      else
      {
         fprintf (stderr, "Memoire insuffisante\n");
         exit (EXIT_FAILURE);
      }
   }
   return strip;
}
/**********************************
 * calculer la valeur du zoom
 * pour l'export swf en cas d'effet
 * de zooming ;
 * résultat entier en %
 * *******************************/
gint compute_zoom_value_x(gdouble debut, gdouble fin)
{
   gdouble z;
   
   z=fin/debut*100;
   return (gint)z;
}
/**********************************
 * calculer la valeur du zoom
 * pour l'export swf en cas d'effet
 * de zooming ;
 * résultat entier en %
 * *******************************/
gint compute_zoom_value_y(gdouble debut, gdouble fin)
{
   gdouble z;
   
   z=fin/debut*100;
   return (gint)z;
}
/**********************************
 * correction des marges pour
 * l'exportation
 * *******************************/
gint correct_margins_x(gdouble valeur)
{
	return (gint)valeur-EDITOR_CANVAS_MARGINS;
}
gint correct_margins_y(gdouble valeur)
{
	return (gint)valeur-EDITOR_CANVAS_MARGINS;
}
/***************************
 * bricolages pour gérer
 * les images
 * ************************/
/*************************************
 * chercher si un item a été taggé
 * retourne un pointeur sur
 * son item pour le modifier, ou
 * NULL dans le cas contraire
 * **********************************/
stack_data_type *find_tagged_item_on_stack(GSList *pile,gchar *nom_item)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   stack_data_type *item=NULL;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return NULL;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
	if (nom_item==NULL)
	  return NULL;
   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if ( (strcmp(((stack_data_type *)(temp->data))->titre,nom_item)==0)
		   &&(((stack_data_type *)(temp->data))->fDel==TRUE))
          {          	
			trouve=TRUE;
			item=((stack_data_type *)(temp->data));
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */	    			  
   return item;
}

/************************************
 * chercher un item dont le
 * titre est connu dans la pile
 * des images ; si trouvé, renvoit
 * l'adresse du pixbuf, sinon NULL
 * *********************************/
GdkPixbuf *find_item_on_images_stack(GSList *pile,gchar *titre)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   GdkPixbuf *pb=NULL;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return NULL;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
   if (titre==NULL)
	  return NULL;	
   /* début boucle */
	temp = g_slist_nth (pile, j);				  
	while ((temp!=NULL)&&(trouve==FALSE))
	  { 
		if (  strcmp(   ((stack_images *)(temp->data))->nom,titre)==0   )
          {          	
			trouve=TRUE;
			pb=((stack_images *)(temp->data))->pixb;
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */			  
   return pb;
}
/************************************************************
 * Find the 'real' file font for a font name and size
 *
 * in :  a pointer on a widget to build the PangoContext
 *       a gchar like this pattern 'sans 12' 
 * out : a gchar with a path to the file
 * 
 * **********************************************************/
gchar *get_font_file(GtkWidget *w, gchar *fonte)
{
    PangoContext *Pct;
    PangoFont *Pft;
    PangoFontDescription *font_desc=NULL;
    FcPattern *pattern = NULL;
    FcChar8 *sToRealFontFile=NULL;

    if (fonte==NULL)
	   fonte=g_strdup_printf("%s","sans 12");
    /* first, we retrieve the current pango context of the window */
    Pct=gtk_widget_get_pango_context(GTK_WIDGET(w));
    /* 2nd, we get a PangoFontDescription for a gchar describing the font face with size, eg 'sans 12' */
    font_desc=pango_font_description_from_string(fonte);
    /* we get a pointer on a pango font */
    Pft=pango_context_load_font(Pct,font_desc);  
    /* we get from the pangofont description the pattern which will be tested by fc-config */  
    g_object_get (Pft, "pattern", &pattern, NULL);
    if (pattern) 
       {
          FcPatternGetString (pattern, FC_FILE,0, &sToRealFontFile);
          // printf("chaine:%s\n",sToRealFontFile);
       }	
	
	return sToRealFontFile;
}
/**********************
 * RAZ pile des fontes
 * ********************/
void raz_pile_fontes(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		g_free(((stack_font *)(node->data))->file_font); 
		g_free(((stack_font *)(node->data))->nom_font); 
        g_free(((stack_font *)(node->data)));
      }/* for */
   g_slist_free (list);
}
/*******************************
 * get a pointer on the sound
 * stack from exterior modules
 * ****************************/
GSList *get_sound_stack_pointer(void)
{
	return pile_sound;
}
/**********************
 * clear sound files
 * stack
 * ********************/
void raz_pile_sound(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		g_free(((stack_sound *)(node->data))->snd_file); 
		g_free(((stack_sound *)(node->data))->item_snd_name);
        g_free(((stack_sound *)(node->data)));
      }/* for */
   g_slist_free (list);
}
/**********************
 RAZ de la pile
**********************/
void raz_pile(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		g_free(((stack_data_type *)(node->data))->titre); 
        g_free(((stack_data_type *)(node->data)));
      }/* for */
   g_slist_free (list);
}
/*********************************
 * add an sound item on the
 * sound stack
 * ******************************/
GSList  *add_pile_sound(GSList *list, gchar *fich,gboolean playing_now,gchar *item_snd)
{
 GSList *temp;
 stack_sound *temp_data;
 
 temp_data = g_malloc(sizeof(stack_sound));
 temp_data->snd_file = g_strdup(fich); 
 temp_data->item_snd_name=g_strdup(item_snd);
 temp_data->playing=playing_now;
 temp = g_slist_append (list,temp_data);
 return temp;	
}
/*********************************
 * ajouter un item sur la pile
 * des fontes
 * ******************************/
GSList  *add_pile_fontes(GSList *list, gchar *fich, gchar *fonte)
{
 GSList *temp;
 stack_font *temp_data;
 
 //printf("je vais ajouter :%s<>%s<\n",fich,fonte);
 temp_data = g_malloc(sizeof(stack_font));
 temp_data->file_font = g_strdup(fich); 
 temp_data->nom_font = g_strdup(fonte);
 temp = g_slist_append (list,temp_data);
 return temp;	
}
/*********************************
 * ajouter un item sur la pile
 * ******************************/
GSList  *add_pile(GSList *list, gint calque,gchar *name, gint x, gint y, gint rx, gint ry,
                        gint w,gint h,gdouble rota)
{
 GSList *temp;

 stack_data_type *temp_data;
 temp_data = g_malloc(sizeof(stack_data_type));
 temp_data->layer=calque;
 temp_data->fDel=FALSE;/* flag .del déjà ou pas */
 temp_data->x=x;
 temp_data->y=y;
 temp_data->w=w;
 temp_data->h=h;
 temp_data->rotation=rota;
 temp_data->titre = g_strdup(name); 
 temp = g_slist_append (list,temp_data);
 return temp;	
}
/*************************************
 * get the swftools reference of a
 * valid sound item 
 * *********************************/
gchar *get_last_sound_item(GSList *pile)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   gchar *nom=NULL;
      
   trouve=FALSE;
   j=0;
  
	  
   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (    ((stack_sound *)(temp->data))->playing==TRUE  )
          {          	
			trouve=TRUE;
			nom=g_strdup_printf("%s",((stack_sound *)(temp->data))->item_snd_name);
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */				  
   return nom;
}
/************************************
 * find a sound item on sound stack
 * **********************************/
gchar *find_sound_item_on_stack(GSList *pile,gchar *fich)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   gchar *nom=NULL;
      
   trouve=FALSE;
   j=0;
  
	if (fich==NULL)
	  {
		  printf("paramètre nul \n");
	  return NULL;
      }
	  
   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (  strcmp(   ((stack_sound *)(temp->data))->snd_file,fich)==0   )
          {          	
			trouve=TRUE;
			nom=g_strdup_printf("%s",((stack_sound *)(temp->data))->snd_file);
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */				  
   return nom;
}
/************************************
 * chercher un item dont le
 * titre est connu dans la pile
 * des fontes ; si trouvé, renvoit
 * le nom connu par swfc, sinon NULL
 * *********************************/
gchar *find_item_on_font_stack(GSList *pile,gchar *fich_font)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   gchar *nom=NULL;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return NULL;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
	if (fich_font==NULL)
	  return NULL;

   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (  strcmp(   ((stack_font *)(temp->data))->file_font,fich_font)==0   )
          {          	
			trouve=TRUE;
			nom=g_strdup_printf("%s",((stack_font *)(temp->data))->nom_font);
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */			  
   return nom;
}
/******************************
 * chercher un item dont le
 * titre est connu dans la pile
 * ****************************/
gboolean find_item_on_stack(GSList *pile,gchar *nom_item)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return FALSE;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
	if (nom_item==NULL)
	  return FALSE;
	  
	printf("find pile titre >%s<\n",nom_item);	
   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (  strcmp(   ((stack_data_type *)(temp->data))->titre,nom_item)==0   )
          {          	
			trouve=TRUE;
			/* poser des variables globales de width/height pour aller plus vite !*/
			stack_object_width=((stack_data_type *)(temp->data))->w;
			stack_object_height=((stack_data_type *)(temp->data))->h;
			stack_object_rotation=((stack_data_type *)(temp->data))->rotation;
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */				  
   return trouve;
}
/******************************
 * conversion des couleurs
 * GOOCANVAS sur 32 bits vers 
 * chaînes couleurs
 * en 8 bits 
 * **************************/
gchar *strgdk8bits(guint32 couleur)
{
	gint16 r,v,b;
	gchar *temp=NULL;
	
	/* passage de 32 bits RGBA en 16 bits composantes */
    r=(couleur & 0xFF000000)>>24;
    v=(couleur & 0x00FF0000)>>16;
    b=(couleur & 0x0000FF00)>>8;
	/* passage de 16 bits en 8 bits par plan */
	temp=g_strdup_printf("#%.2x%.2x%.2x",r,v,b);  	
	return temp;	
}
/**************************************
 * tracer une ellipse en utilisant
 * de l'actionscript 
 * ***********************************/
void swf_ellipse(FILE *f,gchar *title,gint x,gint y,gint radiusX,gint radiusY)
{
	gdouble controlX, controlY, anchorX, anchorY, angle;
	gint i;
	gdouble dr = 2 - cos(3.1416 / 8);
	gdouble contraX = radiusX * dr;
	gdouble contraY = radiusY * dr;
	gint ellipse_mcx = x;
	gint ellipse_mcy = y;
	
    fprintf(f,".outline %s_outline:\n",title);		
	fprintf(f,"moveTo %d,%d\n", radiusX,0);

	for(i = 1; i <= 8; i++) 
	{
		angle = i * (3.1416 / 4);
		controlX = contraX * cos(angle - (3.1416 / 8));
		controlY = contraY * sin(angle - (3.1416 / 8));
		anchorX = radiusX * cos(angle);
		anchorY = radiusY * sin(angle);
		fprintf(f,"curveTo %d, %d, %d, %d \n",(gint)controlX,(gint) controlY,
		               (gint) anchorX,(gint) anchorY);
	}	
	fprintf(f,".end\n");
}
/**************************************
 * exporter les données d'un item
 * vers du swf (Flash)
 * avec aiguillage interne intelligent
 * il faut passer un pointeur sur la
 * pile des items déjà traités ou à
 * traiter
 * ***********************************/
void exporte_item_swf(FILE *f, gint calque,guint cell,
           GSList *pile, GooCanvasItem *item,GSList *list_im, guint effets)
{
  gchararray title,texte,fonte;
  guint32 fcolor,scolor;
  gdouble lw;
  gdouble angle;
  gdouble x,y,xt,yt,echelle,width, height;/* pour les rectangles */
  gdouble xc,yc,rx,ry; /* pour les ellipses */
  GooCanvasPoints *l_pts;
  gboolean item_find;
  gboolean trouve,erreur,transfo;
  gchar *font_file,*image_file;
  PangoFontDescription *fnt;
  gdouble valeur_alpha,valeur_rotation;
  gchar *font_name;
  GdkPixbuf *pb;
  stack_data_type *cherche;
  gboolean reaffiche=FALSE;/* flag pour remettre à l'écran un objet caché par .del */
  gchar *commande,*cmd_base,*cmd_alpha,*cmd_zoom,*cmd_rota;
  gchar *texte_rectifie;
  gchar *path,*diese, *titre_court;
  
  if (item==NULL)
    return;
  g_object_get(item,"title",&title,NULL);
  if (title==NULL)
    return;
  commande=NULL;
  cmd_base=NULL;
  cmd_alpha=NULL;
  cmd_zoom=NULL;
  cmd_rota=NULL;
  g_object_get(item, "fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,"line-width",&lw,NULL);
  transfo=goo_canvas_item_get_simple_transform(item,&xt,&yt,&echelle,&valeur_rotation);				
  valeur_alpha=(fcolor & 0x000000000000FF)/256.0*100;
  /* maintenant que nous avons le titre de l'item, est-il déjà
   * dans la pile d'items ? */
  /* be careful ! title must be corrected for P&U objects ! */ 
  if ( (strncmp(title,"U",1)==0) || (strncmp(title,"P",1)==0) )
	 {
	   diese= g_strrstr(title, "#");
	   if (diese!=NULL)
	     {
			valeur_alpha=(scolor & 0x000000000000FF)/256.0*100; 
			titre_court= file_str_sub_extension (title, 0, diese-title-1);  
			trouve=find_item_on_stack(pile_items_export,titre_court);  
		 }
	 }
  else
    /* other items */
	 {
		 trouve=find_item_on_stack(pile_items_export,title); 
	 } 
  /* si absent, on ajoute et on enregistre dans le fichier */
  if (trouve==TRUE)
    {
      /* prendre certaines valeurs critiques pour des comparaisons */	  
	  reaffiche=FALSE;
	  /* l'objet a-t-il été "taggé" car caché ? */
	  if ( (strncmp(title,"U",1)==0) || (strncmp(title,"P",1)==0) )
	     {
	       diese= g_strrstr(title, "#");
	       if (diese!=NULL)
	          {
				  cherche=find_tagged_item_on_stack(pile,titre_court);
				  g_free(titre_court);
			  }
		 }
	  else 
	       cherche=find_tagged_item_on_stack(pile,title);
	  if (cherche!=NULL)
	      {	  cherche->fDel=FALSE;
			  reaffiche=TRUE;
		  }
	   /* aiguillage selon type d'item */		
	  if (strncmp(title,"R",1)==0)
        {
     	  /* rectangle */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);   	
		  /* si interpollation, choix ? */
		  if (cell==ELLA_FRAME_KEY_USED)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else   
		       fprintf(f,".jump %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
		  if (cell==12)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else 
			  {
				cmd_base =g_strdup_printf(".change %s %d %d ",
				         title,correct_margins_x(x)+(gint)(width/2),
						 correct_margins_y(y)+(gint)(height/2));
				 commande=g_strconcat("",cmd_base,NULL);		 
				/* à coups de strcat après des tests */
				if ((effets & 16)==16) /* alpha */
				  {
					   cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
				       commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				  }
				if ((effets & 2)== 2) /* zoom */
				     {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					 }
				if ((effets & 4 )== 4) /* rotation */
				     {		
						cmd_rota=g_strdup_printf("pin=center rotate=%d ",(gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }
		        fprintf(f,"%s\n",commande);
				g_free(commande);
				g_free(cmd_base);	
				if (cmd_alpha!=NULL)
				    g_free(cmd_alpha);
				if (cmd_zoom!=NULL)
				    g_free(cmd_zoom);	
				if (cmd_rota!=NULL)
				    g_free(cmd_rota);	
		      }
	    }/* end rectangle */
     if (strncmp(title,"P",1)==0)
        {
     	  /* filled path */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);   
		  diese= g_strrstr(title, "#");
	      if (diese!=NULL)
	         {
			    titre_court= file_str_sub_extension (title, 0, diese-title-1);   
		      }	
		  /* si interpollation, choix ? */
		  if (cell==ELLA_FRAME_KEY_USED)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else   
		       fprintf(f,".jump %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
		  if (cell==12)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else 
			  {
				cmd_base =g_strdup_printf(".change %s %d %d ",
				         titre_court,correct_margins_x(x)+(gint)(width/2),
						 correct_margins_y(y)+(gint)(height/2));
				 commande=g_strconcat("",cmd_base,NULL);		 
				/* à coups de strcat après des tests */
				if ((effets & 16)==16) /* alpha */
				  {
					   cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
				       commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				  }
				if ((effets & 2)== 2) /* zoom */
				     {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					 }
				if ((effets & 4 )== 4) /* rotation */
				     {		
						cmd_rota=g_strdup_printf("pin=center rotate=%d ",(gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }
		        fprintf(f,"%s\n",commande);
				g_free(titre_court);
				g_free(commande);
				g_free(cmd_base);	
				if (cmd_alpha!=NULL)
				    g_free(cmd_alpha);
				if (cmd_zoom!=NULL)
				    g_free(cmd_zoom);	
				if (cmd_rota!=NULL)
				    g_free(cmd_rota);	
		      }
	    }/* end filled path */		
      if (strncmp(title,"U",1)==0)
        { /* outline path */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);   
		  diese= g_strrstr(title, "#");
	      if (diese!=NULL)
	         {
			    titre_court= file_str_sub_extension (title, 0, diese-title-1);   
		      }	
		  /* si interpollation, choix ? */
		  if (cell==ELLA_FRAME_KEY_USED)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else   
		       fprintf(f,".jump %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
		  if (cell==12)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d\n",titre_court,correct_margins_x(x)+(gint)width/2,correct_margins_y(y)+(gint)height/2);
			else 
			  {
				cmd_base =g_strdup_printf(".change %s %d %d ",
				         titre_court,correct_margins_x(x)+(gint)(width/2),
						 correct_margins_y(y)+(gint)(height/2));
				 commande=g_strconcat("",cmd_base,NULL);		 
				/* à coups de strcat après des tests */
				if ((effets & 16)==16) /* alpha */
				  {
					   cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
				       commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				  }
				if ((effets & 2)== 2) /* zoom */
				     {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					 }
				if ((effets & 4 )== 4) /* rotation */
				     {		
						cmd_rota=g_strdup_printf("pin=center rotate=%d ",(gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }
		        fprintf(f,"%s\n",commande);
				g_free(titre_court);
				g_free(commande);
				g_free(cmd_base);	
				if (cmd_alpha!=NULL)
				    g_free(cmd_alpha);
				if (cmd_zoom!=NULL)
				    g_free(cmd_zoom);	
				if (cmd_rota!=NULL)
				    g_free(cmd_rota);	
		      }
	    }/* end filled path */			
	   if (strncmp(title,"B",1)==0)
        {
     	  /* bitmap */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);    	
		  /* si interpollation, choix ? */
		  if (cell==ELLA_FRAME_KEY_USED)
		      if (reaffiche)
			       fprintf(f,".put %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,
				                    correct_margins_y(y)+(gint)height/2);
			  else
		           fprintf(f,".jump %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,
				             correct_margins_y(y)+(gint)height/2);
		  if (cell==12)
		        if (reaffiche)
				    fprintf(f,".put %s %d %d\n",title,correct_margins_x(x)+(gint)width/2,
					         correct_margins_y(y)+(gint)height/2);
				else
				  {
					cmd_base=g_strdup_printf(".change %s %d %d\n",
					            title,correct_margins_x(x)+(gint)(width/2),
								correct_margins_y(y)+(gint)(height/2));	
				    commande=g_strconcat("",cmd_base,NULL);				
				    if ((effets & 16)==16) /* alpha */
					  {
					    cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);	
						commande=g_strconcat(cmd_base,cmd_alpha,NULL);
					  }
				    if ((effets & 2)== 2) /* zoom */
				      {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					  }					   			
				    if ((effets & 4 )== 4) /* rotation */
				       {		
						cmd_rota=g_strdup_printf("pin=center rotate=%d ",(gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					   }					  
					fprintf(f,"%s\n",commande);
					g_free(commande);
					g_free(cmd_base);	
					if (cmd_alpha!=NULL)
						g_free(cmd_alpha);	
				    if (cmd_rota!=NULL)
				        g_free(cmd_rota);								
				  }
	    }/* bitmap */
	  /* cercle */
	  if (strncmp(title,"C",1)==0)
       {
     	/* cercle */
     	g_object_get(item,"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);
		 /* si interpollation, choix ? */
		if (cell==ELLA_FRAME_KEY_USED)
		  if (reaffiche)
		     fprintf(f,".put %s pin=center %d %d\n",title,correct_margins_x(xc),correct_margins_y(yc));
		  else
		    fprintf(f,".jump %s %d %d\n",title,correct_margins_x(xc),correct_margins_y(yc));
		if (cell==12)
		  if (reaffiche)
		     fprintf(f,".put %s pin=center %d %d\n",title,correct_margins_x(xc),correct_margins_y(yc));
		  else
		    {
					cmd_base=g_strdup_printf(".change %s %d %d ",
					            title,correct_margins_x(xc),correct_margins_y(yc));	
				    commande=g_strconcat("",cmd_base,NULL);								
				    if ((effets & 16)==16) /* alpha */
					  {
					   cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
					   commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				      }
				    if ((effets & 2)== 2) /* zoom */
				     {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					 }			
				   if ((effets & 4 )== 4) /* rotation */
				     {		
						 printf("valeur rotation cercle %f\n",valeur_rotation);
						cmd_rota=g_strdup_printf(" pin=center rotate=%d ",
						        (gint) 90);
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }					 		
					fprintf(f,"%s\n",commande);
					g_free(commande);
					g_free(cmd_base);	
					if (cmd_alpha!=NULL)
						g_free(cmd_alpha);	
					if (cmd_zoom!=NULL)
						g_free(cmd_zoom);	
					if (cmd_rota!=NULL)
						g_free(cmd_rota);														  
		    }
	   }
	 /* ligne */
	 if (strncmp(title,"L",1)==0)
      {
       l_pts= goo_canvas_points_new(2);/* lignes simples */
       g_object_get(item,"points",&l_pts,"width",&width,"height",&height,NULL);
	   if (cell==ELLA_FRAME_KEY_USED)
	       if (reaffiche)
		       fprintf(f,".put %s x=%d y=%d\n",title,
			       correct_margins_x(l_pts->coords[0])+(gint)width/2,
			       correct_margins_y(l_pts->coords[1])+(gint)height/2);
		   else
		     fprintf(f,".jump %s x=%d y=%d\n",title,
			 correct_margins_x(l_pts->coords[0])+(gint)width/2,
			 correct_margins_y(l_pts->coords[1])+(gint)height/2);
		if (cell==12)
		   if (reaffiche)
		       fprintf(f,".put %s x=%d y=%d \n",title,
			            correct_margins_x(l_pts->coords[0])+(gint)width/2,
			            correct_margins_y(l_pts->coords[1])+(gint)height/2);
		   else
		     {
			   cmd_base=g_strdup_printf(".change %s %d %d\n",title,
			       correct_margins_x(l_pts->coords[0])+(gint)width/2,
				   correct_margins_y(l_pts->coords[1])+(gint)height/2);	  
		       commande=g_strconcat("",cmd_base,NULL);
			   if ((effets & 16)==16) /* alpha */
			      {
			               cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
			               commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				  }
			   if ((effets & 2)== 2) /* zoom */
				     {				
						cmd_zoom=g_strdup_printf(" scalex=%d%% scaley=%d%%",
						compute_zoom_value_x((gdouble) stack_object_width,
						    width),
						compute_zoom_value_y((gdouble) stack_object_height,
						    height)	); 
						commande=g_strconcat(cmd_base,cmd_zoom,NULL);
					 }	
			   if ((effets & 4 )== 4) /* rotation */
				     {		
						cmd_rota=g_strdup_printf(" pin=center rotate=%d ",
						        (gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }					 		   
			   fprintf(f,"%s\n",commande);
			   g_free(commande);
			   g_free(cmd_base);	
			   if (cmd_alpha!=NULL)
						g_free(cmd_alpha);	 
			   if (cmd_zoom!=NULL)
						g_free(cmd_zoom);	
			   if (cmd_rota!=NULL)
						g_free(cmd_rota);														  						  
			 }
       goo_canvas_points_unref(l_pts);
	 }/* ligne */ 
	 /* texte */
	if (strncmp(title,"T",1)==0)
      {
     	/* texte */
        g_object_get(item,"x",&x,"y",&y,"text",&texte,"font",&fonte,NULL);
       	if (cell==ELLA_FRAME_KEY_USED)
		    if (reaffiche)
			   fprintf(f,".put %s %d %d  pin=center \n",title,
		        correct_margins_x(x),correct_margins_y(y));  
			else
		        fprintf(f,".jump %s %d %d\n",title,correct_margins_x(x),correct_margins_y(y));
		if (cell==12)
		    if (reaffiche)
			    fprintf(f,".put %s %d %d  pin=center \n",title,
		        correct_margins_x(x),correct_margins_y(y));
			else	
			  {	
					cmd_base=g_strdup_printf(".change %s %d %d\n",
					            title,correct_margins_x(x),correct_margins_y(y));	
									/* à coups de strcat après des tests */
				    commande=g_strconcat("",cmd_base,NULL);									
				    if ((effets & 16)==16) /* alpha */
					  {
					   cmd_alpha=g_strdup_printf(" alpha=%d%%",(gint)valeur_alpha);					   
					   commande=g_strconcat(cmd_base,cmd_alpha,NULL);
				      }
		            if ((effets & 4 )== 4) /* rotation */
				     {		
						cmd_rota=g_strdup_printf("rotate=%d pin=center ",
						     (gint) goo_to_swf_angle(valeur_rotation));
						commande=g_strconcat(cmd_base,cmd_rota,NULL); 		
					 }					
					fprintf(f,"%s\n",commande);
					g_free(commande);
					g_free(cmd_base);	
					if (cmd_alpha!=NULL)
						g_free(cmd_alpha);				  
			  }
	  }
    }
  else
    {
	 /* 1er affichage d'un objet */	
	  if (strncmp(title,"R",1)==0)
        { /* rectangle */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
		  pile_items_export=add_pile(pile_items_export,calque, 
		                  title, x, y,0,0,(gint)width,(gint)height,valeur_rotation);    
		  fprintf(f,".box %s %d %d color=%s fill=%s\n",
		            title,(gint)width,(gint)height,
					strgdk8bits(scolor),strgdk8bits(fcolor));
		  if (valeur_rotation!=0)
		       fprintf(f,".put %s %d %d alpha=%d%% pin=center rotate=%d\n",title,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha,
								   (gint)goo_to_swf_angle(valeur_rotation));		  
		  else			
		       fprintf(f,".put %s %d %d alpha=%d%% pin=center \n",title,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha);
	    }/* rectangle */
	  if (strncmp(title,"P",1)==0)/* outline */
        { 
		   diese= g_strrstr(title, "#");
	       if (diese!=NULL)
	         {
			    path=file_str_sub_extension (title, diese-title+1,strlen(title)-1);	 
				titre_court= file_str_sub_extension (title, 0, diese-title-1);         	            
     	        g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
                pile_items_export=add_pile(pile_items_export,calque, 
		                 titre_court, x, y,0,0,(gint)width,(gint)height,valeur_rotation);    
		        fprintf(f,".outline %s_outline:\n",titre_court);	
		        fprintf(f,"%s\n",path);						      
	            fprintf(f,".end\n");	/* end of outline declaration */			  
		        fprintf(f,".filled %s  outline=%s_outline color=%s fill=%s\n",titre_court,titre_court,
	                              strgdk8bits(scolor),strgdk8bits(fcolor));		       
				if (valeur_rotation!=0)
		             fprintf(f,".put %s %d %d alpha=%d%% pin=center rotate=%d\n",titre_court,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha,
								   (gint)goo_to_swf_angle(valeur_rotation));		  
		        else			
		            fprintf(f,".put %s %d %d alpha=%d%% pin=center \n",titre_court,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha);		 
				g_free(path);	
				g_free(titre_court);				   
			}/* endif path correct */
	    }/* path */	
	  if (strncmp(title,"U",1)==0)/* outline */
        { 
		   diese= g_strrstr(title, "#");
	       if (diese!=NULL)
	         {
			    path=file_str_sub_extension (title, diese-title+1,strlen(title)-1);	 
				titre_court= file_str_sub_extension (title, 0, diese-title-1);         	            
     	        g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
                pile_items_export=add_pile(pile_items_export,calque, 
		                 titre_court, x, y,0,0,(gint)width,(gint)height,valeur_rotation);    
		        fprintf(f,".outline %s_outline:\n",titre_court);	
		        fprintf(f,"%s\n",path);						      
	            fprintf(f,".end\n");	/* end of outline declaration */			  
		        fprintf(f,".filled %s  outline=%s_outline color=%s fill=none\n",titre_court,titre_court,
	                              strgdk8bits(scolor));		       						         
				if (valeur_rotation!=0)
		             fprintf(f,".put %s %d %d alpha=%d%% pin=center rotate=%d\n",titre_court,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha,
								   (gint)goo_to_swf_angle(valeur_rotation));		  
		        else			
		            fprintf(f,".put %s %d %d alpha=%d%% pin=center \n",titre_court,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)valeur_alpha);		 
				g_free(path);	
				g_free(titre_court);				   
			}/* endif path correct */
	    }/* outline */	

      if (strncmp(title,"B",1)==0)
        {
     	  /* bitmap */
     	  g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
		  pile_items_export=add_pile(pile_items_export,calque,title, x, y,0,0,
		                    (gint)width,(gint)height,valeur_rotation); 
		  /* exporter le pixbuf vers le dossier caché */  
		  image_file=g_strdup_printf("%s/.ella/%s%s",getenv("HOME"),title,".png");
		  pb=find_item_on_images_stack(list_im,title);
		  erreur = gdk_pixbuf_save (pb, image_file, "png", NULL,NULL);  
		  fprintf(f,".png %s \"%s\" \n",title,image_file);	
		  if (valeur_rotation!=0)
		       fprintf(f,".put %s %d %d alpha=%d%% pin=center rotate=%d\n",title,
								   correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)100,
								   (gint)goo_to_swf_angle(valeur_rotation));		  
		  else		  
		       fprintf(f,".put %s %d %d alpha=%d%% pin=center\n",title,
			                       correct_margins_x(x)+(gint)(width/2),
		                           correct_margins_y(y)+(gint)(height/2),(gint)100);
		  g_free(image_file); 
	    }/* bitmap */
	 if (strncmp(title,"C",1)==0)
       {
     	/* cercle */
     	g_object_get(item,"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,
				"width",&width,"height",&height,NULL);	
		pile_items_export=add_pile(pile_items_export,calque, 
		                  title, (gint)xc,(gint) yc,(gint) rx,(gint)ry,
						  (gint)width,(gint)height,valeur_rotation);
		/* pour faire une ellipse il faut une outline */				  
		swf_ellipse(f,title,(gint)xc,(gint)yc,(gint)rx,(gint)ry);
	    fprintf(f,".filled %s  outline=%s_outline color=%s fill=%s\n",title,title,
	                              strgdk8bits(scolor),strgdk8bits(fcolor));
		fprintf(f,".put %s x=%d y=%d pin=center alpha=%d%s",title,
				(gint)(correct_margins_x(xc)),(gint)(correct_margins_y(yc)),
				(gint)valeur_alpha,"%");		
		if (valeur_rotation!=0)
		    fprintf(f," rotate=%d ",(gint)goo_to_swf_angle(valeur_rotation));
		fprintf(f,"\n");							  
	   }	
	/* ligne */
	 if (strncmp(title,"L",1)==0)
      {
       l_pts= goo_canvas_points_new(2);/* lignes simples */
       g_object_get(item,"points",&l_pts,"width",&width,"height",&height,NULL);
	   pile_items_export=add_pile(pile_items_export,calque, 
		                  title, (gint)l_pts->coords[0],(gint) l_pts->coords[1],
						  (gint) l_pts->coords[2],(gint)l_pts->coords[3],
						  (gint)width,(gint)height,
						  valeur_rotation);
	   x=correct_margins_x(l_pts->coords[0])+width/2;
	   y=correct_margins_y(l_pts->coords[1])+height/2;				  
       fprintf(f,".outline %s_outline:\n",title);	
	   fprintf(f,"M %d %d L %d %d \n",(correct_margins_x(l_pts->coords[0])),
	                                 correct_margins_y( l_pts->coords[1]),
						  (correct_margins_x(l_pts->coords[2])),
						  correct_margins_y(l_pts->coords[3]));
       fprintf(f,".end\n");
	   fprintf(f,".filled %s  outline=%s_outline color=%s fill=%s\n",title,title,
	                              strgdk8bits(scolor),strgdk8bits(fcolor));
       if (valeur_rotation!=0)
	       fprintf(f,".put %s x=%d y=%d alpha=%d%% pin=center rotate=%d\n",title,(gint)x,(gint)y,
		                   (gint)100,(gint)goo_to_swf_angle(valeur_rotation));
	   else							  
			fprintf(f,".put %s  x=%d y=%d alpha=%d%% pin=center\n",title,(gint)x,(gint)y,(gint)100);					                
       goo_canvas_points_unref(l_pts);
	 }/* ligne */   
	 /* texte */
	if (strncmp(title,"T",1)==0)
      {
     	/* texte */
        g_object_get(item,"x",&x,"y",&y,"text",&texte,"font",&fonte,
		             "width",&width,"height",&height,NULL);
		/* ôter les caractères RC du texte */
		texte_rectifie=g_strdup_printf("%s",str_replace_rc (texte));		
	    pile_items_export=add_pile(pile_items_export,calque, 
		                  title, x, y,0,0,(gint)width,(gint)height,valeur_rotation);
        font_file=get_font_file(GTK_WIDGET(pOnAWidget),(gchar *)fonte);
		fnt=pango_font_description_from_string(fonte);	
		/* on cherche si la fonte est déjà utilisée */
		font_name=find_item_on_font_stack(pile_fontes_export,font_file);
		if (font_name==NULL)
		   {	
		      font_name=g_strdup_printf("fnt_%s",title);
			  pile_fontes_export=add_pile_fontes(pile_fontes_export,font_file,font_name);
			  fprintf(f,".font %s filename=\"%s\"\n",font_name,font_file);
		   }	
		 else
		    printf("impossible de trouver la fonte ! \n");  	 				
		/* attention, ne PAS utiliser g_str_escape ! */
		fprintf(f,".text %s font=%s text=\"%s\" color=%s size=%d%%\n",
		                title,font_name,
						texte_rectifie,
						strgdk8bits(fcolor),
						(gint)((pango_font_description_get_size (fnt)/ PANGO_SCALE)*2.1));/* paramètre 2.1 trouvé empiriquement */
		fprintf(f,".put %s %d %d  pin=center alpha=%d%%\n",title,
		        correct_margins_x(x),correct_margins_y(y),(gint)valeur_alpha); 
		g_free(font_name);
		g_free(texte_rectifie);
		//g_free(font_file); ne pas le freer !
	  }
	}/* elseif item sur pile */   
  return; 
}


/* exporter le script pour swfc */
//FILE *file_swf;

/* ouvrir un fichier pour sauvegarder, format Ella */
FILE *ouvrir_fichier_ella_sauver(gchar *nom)
{
	FILE *f;
	
	f=fopen(nom,"w");
	return f;
}
/************************************************* 
 ouvrir un fichier pour charger, format Ella 
 * **********************************************/
FILE *ouvrir_fichier_ella_charger(gchar *nom)
{
	FILE *f;
	
	f=fopen(nom,"r");
	return f;
}
/********************************************* 
 * ouvrir le fichier d'export de script "sc" 
 ********************************************/
FILE *ouvrir_fichier_swf_export(gchar *nom,GtkWidget *w)
{
	FILE *f;
	
	f=fopen(nom,"w");
	/* init index of sound items */
	snd_item_count=0;
	/* initialiser la pile dse items*/
	pile_items_export=NULL;
	pile_fontes_export=NULL;
	pile_sound=NULL;
	pOnAWidget=w;
	return f;
}
/******************************
 * conversion vers couleurs
 * en 8 bits (on part
 * d'une GdkColor)
 * **************************/
gchar *gdk8bits(GdkColor *couleur)
{
	gchar *temp=NULL;
	
	temp=g_strdup_printf("#%.2x%.2x%.2x",(gint)(255*(couleur->red/65535.0)),(gint)(255*(couleur->green/65535.0)), 
	                (gint)(255*(couleur->blue/65535.0)));  	
	return temp;	
}
/******************************
 * fermer le fichier courant
 * en y mettant le trailer
 * attendu par swfc
 * ***************************/
void fermer_fichier_swf_export(FILE *f)
{
 fprintf(f,".end # End swf movie\n");	
 fclose(f);
 /* effacer toute la pile des items */
 raz_pile(pile_items_export);
 raz_pile_fontes(pile_fontes_export);
 raz_pile_sound(pile_sound);
 pile_items_export=NULL;
 pile_fontes_export=NULL;
 pile_sound=NULL;
}
/******************************
 * fermer le fichier courant
 * 
 * ***************************/
void fermer_fichier_ella(FILE *f)
{	
 fprintf(f,"[FILEND]\n");/* obligatoire pour éviter le bug en relecture */
 fclose(f);
}
/*******************************
 * exporter l'en-tête du
 * fichier courant, APRES
 * qu'il ait été ouvert
 * ****************************/
void exporter_en_tete_swf(FILE *f,
                      gchar *nom_fich_swf, 
                      guint width, guint height, 
                      guint version_flash, 
                      guint fps,
                      GdkColor couleur_fond, gint marg_x,gint marge_y)
{
	/* attention : couleurs codées en 8 bits sous Flash, contre 16 bits en Gdk ! */
	fprintf(f,".flash filename=\"%s\" version=%d fps=%d  bbox=%dx%d background=\"%s\"\n",
	           nom_fich_swf,
	           version_flash,
	           fps,
	           width,height,
	           gdk8bits(&couleur_fond)
	           );
	           
}
/*******************************
 * enregistrer l'en-tête du
 * fichier courant, APRES
 * qu'il ait été ouvert
 * ****************************/
void sauver_en_tete_ella(FILE *f,
                      guint width, guint height, 
                      guint version_flash, 
                      guint fps,
                      GdkColor couleur_fond)
{
 fprintf(f,"[VERSIO]1\n");
 fprintf(f,"[PGEWID]%d\n",width);
 fprintf(f,"[PGEHEI]%d\n",height);
 fprintf(f,"[MOVFRQ]%d\n",fps);
 fprintf(f,"[PGEBGR]%d\n",couleur_fond.red);
 fprintf(f,"[PGEBGG]%d\n",couleur_fond.green);
 fprintf(f,"[PGEBGB]%d\n",couleur_fond.blue);	           
}
/**************************************
 * créer un item
 * 
 * ***********************************/
void add_item(GooCanvasItem *group,gchar *title, 
              gdouble x, gdouble y, gdouble w, gdouble h,gdouble rx, gdouble ry,
              gdouble xf, gdouble yf,
              gdouble lw,
              guint fill_color,
              guint stroke_color, gchararray texte, gchararray fonte,
			  gdouble rota, gchar *path)
{
	GooCanvasItem *nouveau;
	GdkPixbuf *pb;
	gdouble xt,yt,width,height;
	
   if (group==NULL)
	   return;
   if (lw<=0)
	   lw=1.0;
	/* aiguiller selon le titre */
    if (strncmp("R",title,1)==0)
     {
       nouveau=goo_canvas_rect_new(group,x,y,w,h,"stroke-color-rgba",stroke_color,
                  "fill-color-rgba" ,fill_color,
				  "line-width",lw,NULL);
       g_object_set(G_OBJECT(nouveau),"title",title,NULL);		      
	   if (rota!=0)
	            goo_canvas_item_rotate(nouveau,rota,x+(w/2),y+(h/2));
       return;
     }
    if (strncmp("B",title,1)==0)
     {
		pb=find_item_on_images_stack(pile_images,title); 
		nouveau=goo_canvas_image_new(group,pb,x,y,NULL);
		g_object_set(G_OBJECT(nouveau),"title",title,NULL);	
		if (rota!=0)
		   {
			   g_object_get(nouveau,"width",&width,"height",&height,NULL);
			   goo_canvas_item_rotate(nouveau,rota,x+(width/2),y+(height/2));
		   }
        return;
	 }
    if (strncmp("C",title,1)==0)
     {
       nouveau=goo_canvas_ellipse_new(group,x,y,rx,ry,"stroke-color-rgba",stroke_color,
                    "fill-color-rgba" ,fill_color,
					"line-width",lw,NULL);
       g_object_set(G_OBJECT(nouveau),"title",title,NULL); 
	   if (rota!=0)
	       goo_canvas_item_rotate(nouveau,rota,x,y);
       return;
     } 
    if (strncmp("L",title,1)==0)
     {	
       nouveau=goo_canvas_polyline_new_line(group,
                   x,y,
	               xf,yf,
	               "fill-color-rgba" ,fill_color,
				   "stroke-color-rgba",fill_color,"line-width",lw,NULL);	              
       g_object_set(G_OBJECT(nouveau),"title",title,NULL);
	   if (rota!=0)
	       {
			  g_object_get(nouveau,"width",&width,"height",&height,NULL); 
			  goo_canvas_item_rotate(nouveau,rota,x+(width/2),y+(height/2));
		   }
       return;
     }
    if (strncmp("T",title,1)==0)
     {
       nouveau=goo_canvas_text_new (group, texte, x, y, -1,
				   GTK_ANCHOR_CENTER,
				   "font", fonte,
				    "fill-color-rgba" ,fill_color,"stroke-color-rgba",stroke_color,
					"line-width",lw,NULL);	 
       g_object_set(G_OBJECT(nouveau),"title",title,NULL);
	   if (rota!=0)
	       {
			  goo_canvas_item_rotate(nouveau,rota,x,y);
		   }
       return;
     }      
   if (strncmp("U",title,1)==0)
     { 
		    nouveau= goo_canvas_path_new (group,path,"line-width",lw, 
	            "stroke-color-rgba", stroke_color,NULL);
            g_object_set(G_OBJECT(nouveau),"title",g_strdup_printf("%s#%s",title,path),NULL);
			/* then we size the object */
			g_object_set(G_OBJECT(nouveau),"x",x,"y",y,"width",w,"height",h,NULL);
	        if (rota!=0)
			     goo_canvas_item_rotate(nouveau,rota,x,y);
       return;
     }   	 
   if (strncmp("P",title,1)==0)
     {
	         nouveau= goo_canvas_path_new (group,path,"line-width",lw, 
	            "stroke-color-rgba", stroke_color,"fill-color-rgba",fill_color,NULL);
             g_object_set(G_OBJECT(nouveau),"title",g_strdup_printf("%s#%s",title,path),NULL);
			 /* then we size the object */
			 g_object_set(G_OBJECT(nouveau),"x",x,"y",y,"width",w,"height",h,NULL);
	         if (rota!=0)
			     goo_canvas_item_rotate(nouveau,rota,x,y);
       return;
     } 	 
} 
/**************************************
 * enregistrer les données d'un item
 * avec aiguillage interne intelligent
 * ***********************************/
void encode_item(FILE *f, GooCanvasItem *item)
{
  gchararray title,texte,fonte;
  guint32 fcolor,scolor;
  gdouble lw;
  gdouble angle;
  gdouble x,y,width, height;/* pour les rectangles */
  gdouble xc,yc,rx,ry; /* pour les ellipses */
  GooCanvasPoints *l_pts;
  gdouble valeur_alpha,xt,yt,echelle,rota;	
  gboolean transforme;
  gchar *path,*diese,*titre_court;
  
  if (item==NULL)
     return;
  g_object_get(item,"title",&title,NULL);
  if (title==NULL)
     return; /* à défaut de mieux ! */
  g_object_get(item, "fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,"line-width",&lw,NULL);
  transforme=goo_canvas_item_get_simple_transform(item,&xt,&yt,&echelle,&rota);

  /* exporter données selon type item */
  if (strncmp(title,"R",1)==0)
     {
     	/* rectangle */
		fprintf(f,"[ITMNEW]%s\n",title);
     	g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
     	fprintf(f,"[ITMXPS]%f\n",x);
     	fprintf(f,"[ITMYPS]%f\n",y);
     	fprintf(f,"[ITMWID]%f\n",width);
     	fprintf(f,"[ITMHEI]%f\n",height);     	
	 }    
   if (strncmp(title,"B",1)==0)
     {
     	/* bitmap */
		fprintf(f,"[ITMNEW]%s\n",title);
     	g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
     	fprintf(f,"[ITMXPS]%f\n",x);
     	fprintf(f,"[ITMYPS]%f\n",y);
     	fprintf(f,"[ITMWID]%f\n",width);
     	fprintf(f,"[ITMHEI]%f\n",height);     	
	 } 
  if (strncmp(title,"C",1)==0)
     {
     	/* cercle */
		fprintf(f,"[ITMNEW]%s\n",title);
     	g_object_get(item,"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,NULL);
     	fprintf(f,"[ITMCTX]%f\n",xc);
     	fprintf(f,"[ITMCTY]%f\n",yc);
     	fprintf(f,"[ITMRDX]%f\n",rx);
     	fprintf(f,"[ITMRDY]%f\n",ry);     	
	 }       
    if (strncmp(title,"L",1)==0)
     {
     	/* ligne */
	   fprintf(f,"[ITMNEW]%s\n",title);	
       l_pts= goo_canvas_points_new(2);/* lignes simples */
       g_object_get(item,"points",&l_pts,NULL);
       fprintf(f,"[ITMXPS]%f\n",l_pts->coords[0]);
       fprintf(f,"[ITMYPS]%f\n",l_pts->coords[1]);
       fprintf(f,"[ITMXPE]%f\n",l_pts->coords[2]);
       fprintf(f,"[ITMYPE]%f\n",l_pts->coords[3]);	              
       goo_canvas_points_unref(l_pts);
	   fcolor=scolor;
	 }
	if (strncmp(title,"T",1)==0)
     {
     	/* texte */
		fprintf(f,"[ITMNEW]%s\n",title);
        g_object_get(item,"x",&x,"y",&y,"text",&texte,"font",&fonte,NULL);
        fprintf(f,"[ITMXPS]%f\n",x);
     	fprintf(f,"[ITMYPS]%f\n",y);
		/* attention, remplacer RC par \n */
     	fprintf(f,"[ITMTXT]%s\n",g_strescape(texte,NULL));
     	fprintf(f,"[ITMFNT]%s\n",fonte);
	 }	
	if (strncmp(title,"U",1)==0)
     {/* outline object */	
	    diese= g_strrstr(title, "#");
		titre_court= file_str_sub_extension (title, 0, diese-title-1);  
		fprintf(f,"[ITMNEW]%s\n",titre_court); 			    
		g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
     	fprintf(f,"[ITMXPS]%f\n",x);
     	fprintf(f,"[ITMYPS]%f\n",y);
     	fprintf(f,"[ITMWID]%f\n",width);
     	fprintf(f,"[ITMHEI]%f\n",height);   		
		path=file_str_sub_extension (title, diese-title+1,strlen(title)-1);
		fprintf(f,"[ITMPTH]%s\n",path);
		g_free(path);  	 
		g_free(titre_court);
	 }
	if (strncmp(title,"P",1)==0)
     {/* path object */	 
	    diese= g_strrstr(title, "#");
	    titre_court= file_str_sub_extension (title, 0, diese-title-1);  
		fprintf(f,"[ITMNEW]%s\n",titre_court); 	
	    g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL);
     	fprintf(f,"[ITMXPS]%f\n",x);
     	fprintf(f,"[ITMYPS]%f\n",y);
     	fprintf(f,"[ITMWID]%f\n",width);
     	fprintf(f,"[ITMHEI]%f\n",height); 		
		path=file_str_sub_extension (title, diese-title+1,strlen(title)-1);
		fprintf(f,"[ITMPTH]%s\n",path);
		g_free(path);   
		g_free(titre_court);  	
	 }	 
  /* codes couleurs communs *
   * conversion :   */
   fprintf(f,"[ITMFCR]%d\n",(fcolor & 0xFF000000)>>24);
   fprintf(f,"[ITMFCG]%d\n",(fcolor & 0x00FF0000)>>16);
   fprintf(f,"[ITMFCB]%d\n",(fcolor & 0x0000FF00)>>8);
   valeur_alpha=(fcolor & 0x000000000000FF)/255.0*100;
   if (strncmp(title,"B",1)==0)
         valeur_alpha=100;
   fprintf(f,"[ITMFAL]%d\n",(gint)valeur_alpha);
   fprintf(f,"[ITMSCR]%d\n",(scolor & 0xFF000000)>>24);
   fprintf(f,"[ITMSCG]%d\n",(scolor & 0x00FF0000)>>16);
   fprintf(f,"[ITMSCB]%d\n",(scolor & 0x0000FF00)>>8);
   valeur_alpha=(scolor & 0x000000000000FF)/255.0*100;
   if (strncmp(title,"B",1)==0)
        valeur_alpha=100;   
   fprintf(f,"[ITMSAL]%d\n",(gint)valeur_alpha);
   /* taille de la ligne */
   fprintf(f,"[ITMLIW]%f\n",lw);
   /* valeur rotation */
   fprintf(f,"[ITMROT]%f\n",rota);   
  /* code fin séquence */
  fprintf(f,"[ITMEND]\n");
}
/****************************************
 * save the sound description of an 
 * sound item, with the keyframe_list
 * and keyframe index (PC coordinates)
 * as parameters
 * *************************************/
void put_sound_item_to_file(FILE *f,GSList *list, guint kf)
{      
	GSList  *temp;
	effects_data_type *temp_data;
	gint    nIndex;
	gboolean sortir=FALSE;
    gboolean trouve=FALSE;
	gchar *fich_son;

	/* récupérer un pointeur sur le groupe d'items de l'image-clé précédente */
	nIndex=0;
	do
	  {
	  	temp = g_slist_nth (list, nIndex);
	  	if (temp==NULL)
	  	   sortir=TRUE;
	  	if (temp!=NULL)
	  	  {
	  	    if (((effects_data_type *)(temp->data))->keyframe==kf)
             {          	
          	   sortir=TRUE;
			   trouve=TRUE;
             }/*endif kf*/
		  }/*endif temp!NULL*/
		nIndex++;
	  }
	while (!sortir);
    if (!trouve)
       return;
    /* output the sound item */
	if (((effects_data_type *)(temp->data))->sound==TRUE)
	   {	   
		   fich_son=g_strescape(g_strdup_printf("%s",((effects_data_type *)(temp->data))->sound_file),NULL);
		   fprintf(f,"[ITMSND]%s\n",fich_son);
		   /* this item is already on stack ? */
		   if ( find_sound_item_on_stack(pile_sound,fich_son) ==NULL)
		       pile_sound=add_pile_sound(pile_sound,fich_son,FALSE,NULL);
 	   }
}
/*************************************
 * sortir vers un fichier tous les
 * items d'une image-clé, fournie en
 * paramètre coordonnées PC ; autre
 * paramètre : une keyframes_list
 * et le pointeur, sur un fichier
 * **********************************/
void put_items_to_file(FILE *f,GSList *list, guint kf)
{      
	GSList  *temp;
	effects_data_type *temp_data;
	gint    nIndex;
	gboolean sortir=FALSE;
	GooCanvasItem *source, *dest, *item;
	gchararray title;

	/* récupérer un pointeur sur le groupe d'items de l'image-clé précédente */
	nIndex=0;
	do
	  {
	  	temp = g_slist_nth (list, nIndex);
	  	if (temp==NULL)
	  	   sortir=TRUE;
	  	if (temp!=NULL)
	  	  {
	  	    source=((effects_data_type *)(temp->data))->rootKF; 
	  	    if (((effects_data_type *)(temp->data))->keyframe==kf)
             {          	
          	   sortir=TRUE;
             }/*endif kf*/
		  }/*endif temp!NULL*/
		nIndex++;
	  }
	while (!sortir);
	if (goo_canvas_item_get_n_children (source)==0)
	    return;	
	
    /* boucle pour transférer les items */
	nIndex=0;
	while (nIndex<goo_canvas_item_get_n_children (source))		
	   {
	   	 item=goo_canvas_item_get_child(source,nIndex);
	   	 if (item!=NULL)
	   	   {
              encode_item(f,item);
		   }	
		 nIndex++;	
	   }/* wend */	
}
/**********************************
 * sauvegarder les interpolations
 * d'une image clé de type 
 * "mouvement"
 * ********************************/
void put_transitions_to_file(FILE *f, GSList *list, gint kf)
{
   gint    nIndex,total;
   GSList *temp;
   gboolean trouve=FALSE;
   	
	/* vérifier que l'image-clé existe ! */
   if (list==NULL)
        return;	
   if (kf<0)
        return;
  /* on cherche le nombre total actuel d'éléments de la liste */ 
   total=g_slist_length(list);	
   nIndex=0;
   while ((!trouve)&&(nIndex<total))
     {
		temp = g_slist_nth (list, nIndex);
		if (((effects_data_type *)(temp->data))->keyframe==kf)
		   {
			   trouve=TRUE;
			   /* sortir toutes les valeurs */
			   if  (((effects_data_type *)(temp->data))->effect_motion)
			      fprintf(f, "[MOTIOF]1\n");
			   else
			      fprintf(f,"[MOTIOF]0\n");
			   if  (((effects_data_type *)(temp->data))->effect_zooming)
			     {
			      fprintf(f, "[ZOOMIF]1\n");
			     }
			   else
			      fprintf(f,"[ZOOMIF]0\n");	  
			   if  (((effects_data_type *)(temp->data))->effect_lumi)
			      fprintf(f, "[LUMINF]1\n");
			   else
			      fprintf(f,"[LUMINF]0\n");	  
			   if  (((effects_data_type *)(temp->data))->effect_rotation)
			      fprintf(f, "[ROTATF]1\n");
			   else
			      fprintf(f,"[ROTATF]0\n");		  
			   if  (((effects_data_type *)(temp->data))->effect_alpha)
			      fprintf(f, "[ALPHAF]1\n");
			   else
			      fprintf(f,"[ALPHAF]0\n");				  
			   if  (((effects_data_type *)(temp->data))->effect_morphing)
			      fprintf(f, "[MORPHF]1\n");
			   else
			      fprintf(f,"[MORPHF]0\n");		  
		   }
	   nIndex++;	   
	 }   	/* wend */
}
/***********************************
 * enregistrer le contenu de tous 
 * les calques ; en entrée, pointeur
 * sur la liste des calques,
 * suivi du nombre total réel de
 * calques
 * *******************************/
void sauver_donnees_ella(FILE *f, GtkWidget *pListLayers, gint nb_layers, gint maxframes)
{   
 GtkTreeModel *model;
 GtkTreePath *path;
 gchar *path1, *nom_calque;
 GtkTreeIter pIter;
 gint count, j,k, max_actives_frames,layer_type;
 guint cell;
 gboolean visible;
 GSList *keyframes_list;
	
 for (count=0;count<nb_layers;count++)
  {
	/* switch  between drawing layers and sound layers */
  	/* pointer sur les données du calque */
  	model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListLayers));  
  	path1=g_strdup_printf ("%d:0",count);/* pointe sur la 1ère case du contenu du layer */
  	path=gtk_tree_path_new_from_string(path1);      
    gtk_tree_model_get_iter(model, &pIter, path);
    /* lire le nom du calque */
    gtk_tree_model_get (model, &pIter, LAYER_NAME_COLUMN, &nom_calque,
	                    LAYER_VISIBLE_COLUMN,&visible,LAYER_TYPE,&layer_type,-1);/* 0=rang colonne du nom, 1=rang visibilité */
	if (layer_type==DRAWING_LAYER)					
	         fprintf(f,"[LYRNEW]\n");					
	else
	         fprintf(f,"[LYRSND]\n");
    fprintf(f,"[LYRNME]%s\n",g_strescape(nom_calque,NULL));	
    if (visible)
             fprintf(f,"[LYRVIS]1\n");
    else
             fprintf(f,"[LYRVIS]0\n");
    gtk_tree_model_get(model, &pIter,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list, -1);/* 5=rang du pointeur sur les images_clés */
    /* libérer la mémoire */
    g_free(nom_calque);
    g_free(path1);
    gtk_tree_path_free(path);
    /* chercher le nombre total d'images (trames) du calque */
    max_actives_frames=0;
    for (j=N_COLUMN;j<N_COLUMN+maxframes;j++)
       {
       	 path1=g_strdup_printf ("%d:%d",count,j);
         path=gtk_tree_path_new_from_string(path1);      
         gtk_tree_model_get_iter(model, &pIter, path);
         gtk_tree_model_get (model, &pIter,j,&cell,  -1); 
         if (cell!=0)
             max_actives_frames=j-N_COLUMN;
         g_free(path1);
         gtk_tree_path_free(path);
	   }/* next j */
    fprintf(f,"[LYRMXF]%d\n",max_actives_frames+1);
    /* sauvegarder les données des keyframes : on ne fait le test que jusqu'à max_actives_frames*/
    for (j=N_COLUMN;j<=N_COLUMN+max_actives_frames;j++)
       {
       	 path1=g_strdup_printf ("%d:%d",count,j);
         path=gtk_tree_path_new_from_string(path1);      
         gtk_tree_model_get_iter(model, &pIter, path);
         gtk_tree_model_get (model, &pIter,j,&cell,  -1); 
         if (cell!=0)
		   {
             if (cell==ELLA_FRAME_KEY_EMPTY)
                  fprintf(f,"[KEYBLK]%d\n",j-N_COLUMN);/* image clé vide */
			 if (cell==ELLA_FRAME_KEY_USED_FOR_SOUND)
			      {
					  fprintf(f,"[KEYSND]%d\n",j-N_COLUMN);/* keyframe with a sound */
					  /* now, we're saving the sound file */
					  put_sound_item_to_file(f,keyframes_list,j-N_COLUMN);
				  }	  
             if ((cell==ELLA_FRAME_KEY_USED)||(cell==12))
               {
                 if (cell==ELLA_FRAME_KEY_USED)				 
				     fprintf(f,"[KEYUSD]%d\n",j-N_COLUMN);/* image clé pleine */
				  if (cell==12)	
				    /* image clé pleine avec mouvement*/ 	
				    {		 
				      fprintf(f,"[KEYMVT]%d\n",j-N_COLUMN);
					  /* sauvegarder les données des mouvements */
					  put_transitions_to_file(f, keyframes_list, j-N_COLUMN);
				    }
                 /* sauver les objets contenus dans l'image-clé */
                 put_items_to_file( f,keyframes_list,j-N_COLUMN);
               }/* 4&8*/
		   }/* !=0 */
         g_free(path1);
         gtk_tree_path_free(path);
	   }/* next */    
  }/* next count*/
}
/***********************************
 * fonction ajoutant un objet
 * donné, dont le nom est fourni
 * comme une chaîne, sur la pile
 * des objets en cours d'exportation
 * idée : 28/12/2009
 * *pile = liste spécifique pour export
 * *list = pointeur sur le groupe d'items
 * *nom = nom de l'item cherché
 * *****************************************/
void item_on_stack(FILE *f, gint calque,
               guint cell, GSList *pile, GooCanvasItem *list,GSList *list_im,
			   guint effets, gint layer_type)
{
	gboolean trouve;
	gint j,nIndex;
	GSList  *temp,*node;
	effects_data_type *temp_data;
	GooCanvasItem *source, *dest, *item;
	
	j=0;
	trouve =FALSE;
	temp=NULL;
	
	/* before all tests, is it a sound layer ? So, we don't care about the 'list' variable */
	if (layer_type==SOUND_LAYER)	   
		   return;/* endif sound layer */
	if (list==NULL)
	    return; /* liste d'items vide, donc rien à faire ! */
	if ((goo_canvas_item_get_n_children (list)==0)&&(layer_type==DRAWING_LAYER))
	   {
		/* it's a blank keyframe : so, we must delete all drawings */
		for (j=0;node=g_slist_nth (pile_items_export, j);j++)
		   {
			   if (((stack_data_type *)(node->data))->layer==calque)
			     {
					if  (((stack_data_type *)(node->data))->fDel==FALSE)
			            fprintf(f,".del %s\n",((stack_data_type *)(node->data))->titre);
			        /* et il faut le "tager" sur la liste ! */
			        ((stack_data_type *)(node->data))->fDel=TRUE;
			     }
		   }		   
	    return;/* groupe d'items vide, donc rien à faire ! */	
	   }/* endif blank drawings layer */
	/* lire tout le groupe d'items et agir selon la situation */	
	nIndex=0;
	while (nIndex<goo_canvas_item_get_n_children (list))		
	   {
	   	 item=goo_canvas_item_get_child(list,nIndex);
	   	 if (item!=NULL)
	   	   {			 
             exporte_item_swf(f,calque,cell,pile,item,list_im, effets);
		   }	
		 nIndex++;	
	   }/* wend */
}
/***************************************
 * a return function : output a gchar*
 * on a sound file
 * *************************************/
gchar *get_current_sound_file(GSList *list, guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
	GSList  *temp;
	effects_data_type *temp_data;
	gchar *texte_file;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {       
			   	
          	  if (((effects_data_type *)(temp->data))->sound  ==TRUE)
			     {
			       trouve=TRUE;
				   texte_file=g_strdup_printf("%s",((effects_data_type *)(temp->data))->sound_file);
				   return  texte_file;
			     }
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return texte_file;
}
/**********************************
 * a test function : verify if the
 * current keyframe contains a
 * sound
 * *******************************/
gboolean get_valid_sound_keyframe(GSList *list,guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
	GSList  *temp;
	effects_data_type *temp_data;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {       
			   	
          	  if (((effects_data_type *)(temp->data))->sound  ==TRUE)
			      trouve=TRUE;
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return trouve;
}
/**********************************
 * tester si l'image courante est
 * une image-clé ; si OUI,
 * la fonction renvoit un pointeur
 * valide sur le groupe d'items,
 * sinon elle renvoit NULL ;
 * le paramètre est en coordonnées
 * PC
 * attention : bug au 25/12/2009
 * si l'image est vide !
 * GSList correspond à keyframes_list
 * ***********************************/
GooCanvasItem *get_valid_keyframe(GSList *list,guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
	GooCanvasItem *item=NULL;
	GSList  *temp;
	effects_data_type *temp_data;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	item=NULL;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {          	
          	  item=((effects_data_type *)(temp->data))->rootKF; 
			  trouve=TRUE;
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return item;
}
/**********************************
 * tester si l'image courante est
 * une image-clé ; si OUI,
 * la fonction renvoit un ensemble
 * de bits allumés ou éteints
 * indiquant les effets activés
 * le paramètre est en coordonnées
 * PC.
 * GSList correspond à keyframes_list
 * ***********************************/
guint get_transitions_keyframe(GSList *list,guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
    guint effets=0;
	GSList  *temp;
	effects_data_type *temp_data;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {          	
			  trouve=TRUE;
			  /* passer les bits : 0 rien,
			   * 1 mouvement, 2 zoom, 4 rotation, 8 luminance,
			   * 16 alpha */
			  if (((effects_data_type *)(temp->data))->effect_motion)
			    effets = effets | 1;
			  if (((effects_data_type *)(temp->data))->effect_zooming)
			    effets = effets | 2;
			  if (((effects_data_type *)(temp->data))->effect_rotation)
			    effets = effets | 4;
			  if (((effects_data_type *)(temp->data))->effect_lumi)
			    effets = effets | 8;
			  if (((effects_data_type *)(temp->data))->effect_alpha)
			    effets = effets | 16;				
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return effets;
}
/**********************************
 * parcourir TOUS les calques,
 * et en exporter le contenu
 * en entrée : pointeur sur
 * le widget des calques 
 * ATTENTION : coordonnées PC !
 * fournies en sortie des fonctions
 * servant à chercher les keyframes
 * **********************************/
void exporter_calques_vers_swf(FILE *f,
                              GtkWidget *pListLayers,
							  gint nb_layers,
							  gint maxframes,
                              GSList *pile_img)
{
 GtkTreeModel *model;
 GtkTreePath *path,*path3;
 gchar *path1,*path2;
 GtkTreeIter pIter;
 gint count, j, ligne, type_layer;
 guint cell;
 GSList *keyframes_list;
 GooCanvasItem *groupe;
 gboolean frame_posee;/* flag indiquant si le repère de keyframes a déjà été utilisé */
 guint effets=0;
 gchar *fichier_son;
 
 for (j=0;j<maxframes;j++)
   {
	 frame_posee=FALSE;
     for (ligne=0;ligne<nb_layers;ligne++)
       {
   	    /* pointer sur les données du calque */
  	    model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListLayers));  
		path1=g_strdup_printf ("%d:0",ligne);/* pointe sur la 1ère case du contenu du layer */
		path=gtk_tree_path_new_from_string(path1);      
		gtk_tree_model_get_iter(model, &pIter, path);				
		/* récupérer la liste des images_clés = keyframe_list */
		gtk_tree_model_get(model, &pIter,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list,LAYER_TYPE,&type_layer, -1);/*LAYER_MOVEMENT_INDEX== 5=rang du pointeur sur les images_clés */
		if (type_layer==SOUND_LAYER)
		   {
			    path2=g_strdup_printf ("%d:%d",ligne,j);
			    path3=gtk_tree_path_new_from_string(path2);      
                gtk_tree_model_get_iter(model, &pIter, path3);
                gtk_tree_model_get (model, &pIter,j+N_COLUMN,&cell,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list,  -1);       
				if (cell==ELLA_FRAME_KEY_USED_FOR_SOUND)
				    {
						if (get_valid_sound_keyframe(keyframes_list,j))
						  {
							/* on pose donc une image-clé pour le compilateur swfc*/
							if (frame_posee==FALSE)
								{
									fprintf(f,".frame %d\n",j+1);/* tj >0 pour swfc */
									frame_posee=TRUE;
								}	
							/* we take the sound file associated with this keyframe */
							fichier_son=g_strdup_printf("%s", get_current_sound_file(keyframes_list,j)); 
							/* if the sound file is already on stack, we stop the sound */
							if (pile_sound!=NULL)
							   {
								   // .stop "nom_son"
								   fprintf(f,".stop %s\n", get_last_sound_item(pile_sound));
								   // pop from the sound_stack 
								   raz_pile_sound(pile_sound);
								   pile_sound=NULL;
							   }
							/* else, we push the file on file stack , create the .sound and start playing */
							snd_item_count++;
							pile_sound=add_pile_sound(pile_sound,fichier_son,TRUE,g_strdup_printf("%s%d","W",snd_item_count));
							fprintf(f,".sound W%d \"/tmp/%s\"\n",snd_item_count,fichier_son);
							fprintf(f,".play W%d loop=1 \n",snd_item_count);
							//item_on_stack(f,ligne, cell,pile_items_export,groupe,pile_img,effets,type_layer);   		
							g_free(fichier_son);
					      }/* endif valid sound keyframe */
					}       
                gtk_tree_path_free(path3);
			    g_free(path2);
		   }/* endif sound layer */
		/* tester si image courante est une image-clé */
		effets=get_transitions_keyframe(keyframes_list,j);
		groupe=get_valid_keyframe(keyframes_list,j);
		/* gérer les items si c'est une image-clé non vide*/
		if (groupe!=NULL)
		   {
			/* on dispose d'un pointeur sur l'ensemble des items sur ce calque et pour CETTE image-clé */
			/* récupérer contenu cellule pour savoir si mouvement */
			path2=g_strdup_printf ("%d:%d",ligne,j);
			path3=gtk_tree_path_new_from_string(path2);      
            gtk_tree_model_get_iter(model, &pIter, path3);
            gtk_tree_model_get (model, &pIter,j+N_COLUMN,&cell,  -1);          
            gtk_tree_path_free(path3);
			g_free(path2);
			/* on pose donc une image-clé pour le compilateur swfc*/
			if (frame_posee==FALSE)
			  {
			     fprintf(f,".frame %d\n",j+1);/* tj >0 pour swfc */
			     frame_posee=TRUE;
			  }			
			/* maintenant, chercher et définir tous les items seulement si pas déjà définis */			
			item_on_stack(f,ligne, cell,pile_items_export,groupe,pile_img,effets,type_layer);
		   	 /* si l'indice est =0, c'est sans doute l'initialisation ! */
		   }/* endif keyframe!=NULL */
		 else
		    /* is it a blank keyframe or perhaps a 'sound' keyframe ? */
			{
				//printf("pas trouvé de groupe donc test ic vide et ic sound \n");
				path2=g_strdup_printf ("%d:%d",ligne,j);
			    path3=gtk_tree_path_new_from_string(path2);      
                gtk_tree_model_get_iter(model, &pIter, path3);
                gtk_tree_model_get (model, &pIter,j+N_COLUMN,&cell,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list,  -1);  
				if (cell==ELLA_FRAME_KEY_EMPTY)
				   printf("trouvé ic vide position %d:%d\n",ligne,j);     			       
                gtk_tree_path_free(path3);
			    g_free(path2);
			}
		g_free(path1);
		gtk_tree_path_free(path);
	   }/* next ligne */
   }/* next j */
  /* we must the sound server at the end */
  if (pile_sound!=NULL)
							   {
								   // .stop "nom_son"
								   fprintf(f,".stop %s\n", get_last_sound_item(pile_sound));
								   // pop from the sound_stack 
								   raz_pile_sound(pile_sound);
								   pile_sound=NULL;
							   }  
}
/**********************************
 * parcourir TOUS les calques,
 * et en exporter les images
 * bitmaps ;
 * en entrée : pointeur sur
 * le widget des calques 
 * ATTENTION : coordonnées PC !
 * fournies en sortie des fonctions
 * servant à chercher les keyframes
 * **********************************/
void exporter_bitmaps_vers_ella(FILE *f,
                              GtkWidget *pListLayers,
							  gint nb_layers,
							  gint maxframes,
                              GSList *pile_img)
{
 GtkTreeModel *model;
 GtkTreePath *path,*path3;
 gchar *path1,*path2;
 GtkTreeIter pIter;
 gint count, j, ligne,type_layer;
 guint cell;
 GSList *keyframes_list;
 GooCanvasItem *groupe;
 gboolean frame_posee;/* flag indiquant si le repère de keyframes a déjà été utilisé */
 guint effets=0;
 
 for (j=0;j<maxframes;j++)
   {
	 frame_posee=FALSE;
     for (ligne=0;ligne<nb_layers;ligne++)
       {
   	    /* pointer sur les données du calque */
  	    model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListLayers));  
		path1=g_strdup_printf ("%d:0",ligne);/* pointe sur la 1ère case du contenu du layer */
		path=gtk_tree_path_new_from_string(path1);      
		gtk_tree_model_get_iter(model, &pIter, path);				
		/* récupérer la liste des images_clés = keyframe_list */
		gtk_tree_model_get(model, &pIter,5,(gpointer)&keyframes_list, LAYER_TYPE,&type_layer,-1);/* 5=rang du pointeur sur les images_clés */
		/* tester si image courante est une image-clé */
		/* l'erreur est sur la fonction ci-après */
		effets=get_transitions_keyframe(keyframes_list,j);		
		groupe=get_valid_keyframe(keyframes_list,j);
		/* gérer les items si c'est une image-clé */
		if (groupe!=NULL)
		   {
			/* on dispose d'un pointeur sur l'ensemble des items 
			 sur ce calque et pour CETTE image-clé */
			/* récupérer contenu cellule pour savoir si mouvement */
			path2=g_strdup_printf ("%d:%d",ligne,j);
			path3=gtk_tree_path_new_from_string(path2);      
            gtk_tree_model_get_iter(model, &pIter, path3);
            gtk_tree_model_get (model, &pIter,j+N_COLUMN,&cell,  -1);  
			//printf("cell %d\n",cell);           
            gtk_tree_path_free(path3);
			g_free(path2);
			/* on pose donc une image-clé pour le compilateur swfc*/
			if (frame_posee==FALSE)
			  {
			   fprintf(f,".frame %d\n",j+1);/* tj >0 pour swfc */
			   frame_posee=TRUE;
			  }			
			/* les afficher */
			 //  printf("layer %d / frame= %d \n",ligne,j);
			/* maintenant, chercher et définir tous les items seulement 
			 si pas déjà définis */
			item_on_stack(f,ligne, cell,pile_items_export, groupe,pile_img,effets,type_layer);
		   	 /* si l'indice est =0, c'est sans doute l'initialisation ! */
		   }/* endif keyframe!=NULL */
		g_free(path1);
		gtk_tree_path_free(path);
	   }/* next ligne */
   }/* next j */
}

/***********************************
 dialogue de sauvegarde
***********************************/

GtkWidget *create_file_save_dialog (void)
{
  GtkWidget *file_save_dialog;
  GtkWidget *dialog_vbox7;
  GtkWidget *dialog_action_area7;
  GtkWidget *button10;
  GtkWidget *button11;

  GtkFileFilter *filter = gtk_file_filter_new ();
  gtk_file_filter_add_pattern (filter, "*.ella");
  gtk_file_filter_set_name (filter,"ella");

  file_save_dialog = gtk_file_chooser_dialog_new ("Enregistrer sous ...", NULL,
             GTK_FILE_CHOOSER_ACTION_SAVE,NULL,NULL, NULL);
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(file_save_dialog),filter);
  gtk_container_set_border_width (GTK_CONTAINER (file_save_dialog), 5);
  gtk_window_set_position (GTK_WINDOW (file_save_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (file_save_dialog), TRUE);
  gtk_window_set_icon_name (GTK_WINDOW (file_save_dialog), "gtk-save");
  gtk_window_set_type_hint (GTK_WINDOW (file_save_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox7 = GTK_DIALOG (file_save_dialog)->vbox;
  gtk_widget_show (dialog_vbox7);

  dialog_action_area7 = GTK_DIALOG (file_save_dialog)->action_area;
  gtk_widget_show (dialog_action_area7);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area7), GTK_BUTTONBOX_END);

  button10 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button10);
  gtk_dialog_add_action_widget (GTK_DIALOG (file_save_dialog), button10, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (button10, GTK_CAN_DEFAULT);

  button11 = gtk_button_new_from_stock ("gtk-save");
  gtk_widget_show (button11);
  gtk_dialog_add_action_widget (GTK_DIALOG (file_save_dialog), button11, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (button11, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button11);
  return file_save_dialog;
}
/********************************
 File Dialog item
 ********************************/
 GtkWidget *create_DialogOpenFile (gchar *titre,
            gchar *filtre1,gchar *filtre2,gchar *filtre3)
{
  GtkWidget *DialogOpenFile;
  GtkWidget *dialog_vbox2;
  GtkWidget *dialog_action_area2;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkFileFilter *filter = gtk_file_filter_new ();

  DialogOpenFile = gtk_file_chooser_dialog_new ("Ouvrir ...", NULL,
               GTK_FILE_CHOOSER_ACTION_OPEN,NULL,NULL, NULL);
  if (filtre1!=NULL)
  gtk_file_filter_add_pattern (filter, filtre1);
  if (filtre2!=NULL)
    gtk_file_filter_add_pattern (filter, filtre2);
  if (filtre3!=NULL)
    gtk_file_filter_add_pattern (filter, filtre3);
  gtk_file_filter_set_name (filter,"ella");
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(DialogOpenFile),filter);

  gtk_container_set_border_width (GTK_CONTAINER (DialogOpenFile), 5);
  gtk_window_set_position (GTK_WINDOW (DialogOpenFile), GTK_WIN_POS_CENTER);
  gtk_window_set_type_hint (GTK_WINDOW (DialogOpenFile), GDK_WINDOW_TYPE_HINT_DIALOG);
gtk_window_set_icon_name (GTK_WINDOW (DialogOpenFile), "gtk-open");

  dialog_vbox2 = GTK_DIALOG (DialogOpenFile)->vbox;
  gtk_widget_show (dialog_vbox2);

  dialog_action_area2 = GTK_DIALOG (DialogOpenFile)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogOpenFile), button1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  button2 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button2);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogOpenFile), button2, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) DialogOpenFile, "destroy",
                    G_CALLBACK (gtk_widget_destroy),
                    NULL);
  g_signal_connect ((gpointer) button1, "clicked",
                    G_CALLBACK (gtk_true),
                    NULL);

  gtk_widget_grab_default (button2);
  return DialogOpenFile;
}
/********************************
 Export Dialog item
 ********************************/
 GtkWidget *create_DialogExportFile (gchar *titre,
            gchar *filtre1,gchar *filtre2,gchar *filtre3)
{
  GtkWidget *DialogExportFile;
  GtkWidget *dialog_vbox2;
  GtkWidget *dialog_action_area2;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkFileFilter *filter = gtk_file_filter_new ();

  DialogExportFile = gtk_file_chooser_dialog_new ("Exporter ...", NULL,
            GTK_FILE_CHOOSER_ACTION_SAVE,NULL,NULL, NULL);
  if (filtre1!=NULL)
  gtk_file_filter_add_pattern (filter, filtre1);
  if (filtre2!=NULL)
    gtk_file_filter_add_pattern (filter, filtre2);
  if (filtre3!=NULL)
    gtk_file_filter_add_pattern (filter, filtre3);
  gtk_file_filter_set_name (filter,"exporter");
  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(DialogExportFile),filter);

  gtk_container_set_border_width (GTK_CONTAINER (DialogExportFile), 5);
  gtk_window_set_position (GTK_WINDOW (DialogExportFile), GTK_WIN_POS_CENTER);
  gtk_window_set_type_hint (GTK_WINDOW (DialogExportFile), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_icon_name (GTK_WINDOW (DialogExportFile), "gtk-save");

  dialog_vbox2 = GTK_DIALOG (DialogExportFile)->vbox;
  gtk_widget_show (dialog_vbox2);

  dialog_action_area2 = GTK_DIALOG (DialogExportFile)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogExportFile), button1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  button2 = gtk_button_new_from_stock ("gtk-save");
  gtk_widget_show (button2);
  gtk_dialog_add_action_widget (GTK_DIALOG (DialogExportFile), button2, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) DialogExportFile, "destroy",
                    G_CALLBACK (gtk_widget_destroy),
                    NULL);
  g_signal_connect ((gpointer) button1, "clicked",
                    G_CALLBACK (gtk_true),
                    NULL);

  gtk_widget_grab_default (button2);
  return DialogExportFile;
}
/*******************************
 fonction copie en binaire
 copie de source vers dest
*******************************/
int copier_fichier(char const * const source, char const * const destination)
{
    FILE* fSrc;
    FILE* fDest;
    char buffer[512];
    int NbLus;
    
    if ((fSrc = fopen(source, "rb")) == NULL)
    {
        return 1;
    }
    
    if ((fDest = fopen(destination, "wb")) == NULL)
    {
        fclose(fSrc);
        return 2;
    }
    
    while ((NbLus = fread(buffer, 1, 512, fSrc)) != 0)
        fwrite(buffer, 1, NbLus, fDest);
    
    fclose(fDest);
    fclose(fSrc);
    
    return 0;
}
