/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
 
//translations
#include <locale.h>
#include <libintl.h>
#define _(x) gettext(x)

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <goocanvas.h>

#include "callbacks.h"
#include "timeline.h"
#include "editeur.h"
#include "ellasyms.h"
#include "biblio.h"
#include "clipboard.h"

/***************************
 * icones de certains menus
 * ************************/
#include "./icones_rcs/morphing.xpm"
#include "./icones_rcs/import_symbol.xpm"
#include "./icones_rcs/splash_screen.xpm"
#include "./icones_rcs/aimant.xpm"
#include "./icones_rcs/w3svg16x16.xpm"
#include "./icones_rcs/javascript16x16.xpm"
#include "./icones_rcs/remove_interpola16x16.xpm"
#include "./icones_rcs/set_keyframe_to_blank.xpm"
#include "./icones_rcs/remove_frames_to_cursor.xpm"
#include "./icones_rcs/remove_frames_from_cursor.xpm"
#include "./icones_rcs/add_frames_from_cursor.xpm"
#include "./icones_rcs/remove_keyframe.xpm"
#include "./icones_rcs/remove_selection16x16.xpm"
#include "./icones_rcs/add_frames_to_cursor.xpm"
#include "./icones_rcs/new_keyframe.xpm"
#include "./icones_rcs/new_blank_keyframe.xpm"
#include "./icones_rcs/interpola16x16.xpm"
#include "./icones_rcs/layer_rename_16x16.xpm"
#include "./icones_rcs/select_all.xpm"
#include "./icones_rcs/deselect_all.xpm"
#include "./icones_rcs/duplicate_item_16x16.xpm"
#include "./icones_rcs/layer_new_16x16.xpm"
#include "./icones_rcs/layer_delete_16x16.xpm"
#include "./icones_rcs/layer_delete_32x32.xpm"
#include "./icones_rcs/layer_new_32x32.xpm"
#include "./icones_rcs/duplicate_layer_16x16.xpm"
#include "./icones_rcs/duplicate_layer_32x32.xpm"
#include "./icones_rcs/flash.xpm"
#include "./icones_rcs/rotate.xpm"
#include "./icones_rcs/export.xpm"
#include "./icones_rcs/import.xpm"
/***************************
 * icones des propriétés
 * *************************/
#include "./icones_rcs/icone_cadenas_ferme.xpm"
#include "./icones_rcs/icone_reload.xpm"
/****************************
 * icones des effets de
 * transitions 
 * *************************/
#include "./icones_rcs/alpha.xpm"
#include "./icones_rcs/interpola.xpm"
#include "./icones_rcs/rota.xpm"
#include "./icones_rcs/zoom.xpm"
/* logo du programme */
#include "./icones_rcs/logo.xpm"


/* variables "extern" */

#include "interface_editor.h"
/* canvas */
GtkWidget *canvasEditor;

GooCanvasItem *itemPage; /* pointeur sur le fond de page pour en contrôler les propriétés */
GooCanvasItem *itemBack; /*fond de page */
gint LayerSelected; /* calque actuellement sélectionné, coordonnées ordinateur 0-max */ 

GooCanvasItem *rootGroup;/* source du groupe de chaque calque */ 
GooCanvasItem *rootKF; /* source des dessins de chaque image-clé */

gboolean fModifiedDrawing = FALSE;/* flag pour savoir si le dessin a été modifié, et agir en conséquence */

GooCanvasItem *pDrawings;/* pointeur sur le dessin en cours, obligé de le mettre extern à cause des
sélections */

GooCanvasItem *rootEditorItems; /* racine de tous les dessins de l'éditeur */ 
GSList *keyframes_list=NULL;/* source de la GSList des images-clés de chaque calque */
GtkWidget *pListViewLayers;/* pointeur sur le magasin (store) contenant les calques (layers) */
gint LayerCount=0; /* nombre total de calques en cours, comptage humain */
  gboolean SelectedArea=FALSE; /* indique si une sélection est en cours */							
  GList *pSelectedItemsList=NULL; /* liste des items, mais VALIDE, sans le fond et l'origine */ 
/* fin variables externes */  
  GtkWidget *ScrolledWindowTimeLine;
  GtkWidget *ScrolledWindowLayersList;
  GtkWidget *scrolledWindowRuler;
 /* main window geometry */
 static gint x_fen,y_fen,w_fen,h_fen; 
 static   guint SplahScreenId=0;
 
/*******************************
 * callbacks intermédiaires
 * permettant un appel à
 * des fonctions étendues sur
 * plusieurs modules
 * ****************************/
/********************************
 * callback pour le Dnd, en 
 * réception
 * *****************************/
void
view_onDragDataReceived(GtkWidget *wgt, GdkDragContext *context, gint x, gint y,
                        GtkSelectionData *seldata, guint info, guint time,
                        gpointer userdata)
{
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  GtkTreeIter   iter;
  gchar *nom,*path_datas;
  /* Always initialise a GValue with 0 */
  GValue value={0,};
  GValue value_atom={0,};
  GdkPixbuf *pb=NULL;
  GdkPixbufFormat *fmt;
  GError *error=NULL;
  gint width,height,e_w,e_h,atom;
 
 selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW(pListViewBiblio) );
 
 /* aller chercher l'objet */
 gtk_tree_selection_set_mode ( selection, GTK_SELECTION_SINGLE );
 if ( gtk_tree_selection_get_selected ( selection, &model, &iter ) )
	{
	   gtk_tree_model_get (model, &iter, -1);
	   gtk_tree_model_get_value(GTK_TREE_MODEL (pBiblio),&iter,BIBLIO_PATH_NAME_COLUMN,&value);
	   gtk_tree_model_get_value(GTK_TREE_MODEL (pBiblio),&iter,BIBLIO_ATOM,&value_atom);
	   nom=(gchar*) g_value_get_string(&value);
	   atom= g_value_get_int(&value_atom); 
	   printf("voici le nom dropé :%s<\n",nom);	  
	   /* lancer le chargement de l'objet et sa mise en place sur la scène */
	   /* vérifier contenu et validité du fichier */
	   /* si fichier son, passer à la suite ! */
	   switch (atom)
	    {
			case BIBLIO_ATOM_BITMAP:
		    case BIBLIO_ATOM_SVG:
			    {
					fmt=gdk_pixbuf_get_file_info(nom,&width,&height);

					if (fmt==NULL)
					  {
						printf(gettext("erreur lecture ! \n"));
						g_value_unset(&value);
						g_value_unset(&value_atom);
						return;
					  }
					/* contraindre les dimensions !!! */
					e_w=(gint)editor_get_page_width();
					e_h=(gint)editor_get_page_height();
					if ((width>e_w)||(height>e_h))
						{
						  width=e_w;
						  height=e_h;
						}
					pb=gdk_pixbuf_new_from_file_at_scale(nom,width,height,TRUE,&error);	 
					editor_add_bitmap_dnd(x,y,pb);
					if (fModifiedDrawing)
						{
						  set_keyframe();
						  /* passer le flag 'utilisé' */
						  gtk_list_store_set(GTK_LIST_STORE(model),
									&iter, BIBLIO_VISIBLE_COLUMN,
									TRUE,-1);
						}
					break;
				}/* end case bitmap & svg */
			case BIBLIO_ATOM_WAV:
			   {
				   printf(gettext("demande DnD son ! \n"));
				   /* mettre un message d'erreur */
				   break;
			   }	
			case BIBLIO_ATOM_SYMBOL:
			   {
				   printf(gettext("demande DnD symbole ! \n"));
				   fmt=gdk_pixbuf_get_file_info(nom,&width,&height);
                   if (fmt==NULL)
					  {
						printf(gettext("erreur lecture ! \n"));
						g_value_unset(&value);
						g_value_unset(&value_atom);
						return;
					  }
				   /* we must constrain size of paths */				   
				   e_w=(gint)editor_get_page_width();
				   e_h=(gint)editor_get_page_height();
				   if ((width>e_w)||(height>e_h))
						{
						  width=e_w;
						  height=e_h;
						}	  
				   /* we must 'parse' the svg file in order to obtain a path */
				   path_datas=biblio_parse_svg_file(nom);
				   /* then a new path object is created */
				   if (path_datas!=NULL)
				     {
				        editor_add_symbol_dnd(x,y,path_datas);
					 }
				   /* we put an 'used' flag in library for this symbol */
				   if (fModifiedDrawing)
						{
						  set_keyframe();
						  /* passer le flag 'utilisé' */
						  gtk_list_store_set(GTK_LIST_STORE(model),
									&iter, BIBLIO_VISIBLE_COLUMN,
									TRUE,-1);
						}
				   break;
			   }   
			default:printf(gettext("rien a faire !\n"));
		}/* end switch */   	
	   g_value_unset(&value);
	   g_value_unset(&value_atom);
    }
}
/********************************
 * callback pour le Dnd, en 
 * réception
 * *****************************/
void
view_onDragDataReceivedTimeline(GtkWidget *wgt, GdkDragContext *context, gint x, gint y,
                        GtkSelectionData *seldata, guint info, guint time,
                        gpointer userdata)
{
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  GtkTreeIter   iter;
  gchar *nom;
  /* Always initialise a GValue with 0 */
  GValue value={0,};
  GValue value_atom={0,};
  gint atom;
 
 selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW(pListViewBiblio) );
 
 /* aller chercher l'objet */
 gtk_tree_selection_set_mode ( selection, GTK_SELECTION_SINGLE );
 if ( gtk_tree_selection_get_selected ( selection, &model, &iter ) )
	{
	   gtk_tree_model_get (model, &iter, -1);
	   gtk_tree_model_get_value(GTK_TREE_MODEL (pBiblio),&iter,BIBLIO_PATH_NAME_COLUMN,&value);
	   gtk_tree_model_get_value(GTK_TREE_MODEL (pBiblio),&iter,BIBLIO_ATOM,&value_atom);
	   nom=(gchar*) g_value_get_string(&value);
	   atom= g_value_get_int(&value_atom); 
 
	   /* lancer le chargement de l'objet et sa mise en place sur la scène */
	   /* vérifier contenu et validité du fichier */
	   /* si fichier son, passer à la suite ! */
	   switch (atom)
	    {
			case BIBLIO_ATOM_BITMAP:
		    case BIBLIO_ATOM_SVG:		
			case BIBLIO_ATOM_WAV:
			   {
				   /* voir où nous sommes sur la timeline */
				   timeline_add_sound_DnD(nom);
				   /* passer le flag 'utilisé' */
				   gtk_list_store_set(GTK_LIST_STORE(model),
									&iter, BIBLIO_VISIBLE_COLUMN,
									TRUE,-1);
				   break;
			   }	
			default:printf(gettext("rien a faire !\n"));
		}/* end switch */   	
	   g_value_unset(&value);
	   g_value_unset(&value_atom);
    }
}
/***********************************
 * clic bouton souris
 * item pointe sur la racine des
 * dessins de l'éditeur
 * ********************************/
gboolean cb_editeur_button_press(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win)
{
	cb_editor_button_press(item,
		      target,
		      event,
		      win);

	return TRUE;
}    
/***********************************
 * lâché bouton souris
 * item pointe sur la racine des
 * dessins de l'éditeur
 * SURTOUT NE PAS ENLEVER !
 * ********************************/
gboolean cb_editeur_button_release(GooCanvasItem  *item,
		      GooCanvasItem  *target,
		      GdkEventButton *event,
		      GtkWidget        *win)
{
	cb_editor_button_release(rootKF,target,event,win);
    /* si modif, intervenir sur le dessin de la timeline */
    if (fModifiedDrawing)
      {
		 printf(gettext("je modifie ! \n"));
         set_keyframe();
	  }
	return TRUE;
}    

/****************************/
GtkWidget *create_window_ella (void)
{
  GtkWidget *window_ella;
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  GtkWidget *menuitem1;
  GtkWidget *menuitem1_menu;
  GtkWidget *nouveau1;
  GtkWidget *ouvrir1;
  GtkWidget *recent_files1;
  GtkWidget *image3796;
  GtkWidget *recent_files1_menu;
  GtkWidget *recent1;
  GtkWidget *recent2;
  GtkWidget *recent3;
  GtkWidget *recent4;
  GtkWidget *recent5;
  GtkWidget *recent6;
  gpointer *pDatasRecentFilesMenu;
  GtkWidget *s__parateur15;
  GtkWidget *enregistrer1;
  GtkWidget *enregistrer_sous1;
  GtkWidget *separatormenuitem1;
  GtkWidget *importer1;
  GtkWidget *Imgimporter1;
  GtkWidget *importer1_menu;
  GtkWidget *une_image_bitmap1;
  GtkWidget *une_image_vectorielle1;
  GtkWidget *ImgImportSymbol;
  GtkWidget *s__parateur22;
  GtkWidget *import_symbol;
  GtkWidget *exporter1;
  GtkWidget *Imgexporter1;
  GtkWidget *exporter1_menu;
  GtkWidget *Imgflash;
  GtkWidget *animation_flash1;
  GtkWidget *animation_svg1;
  GtkWidget *ImgSvg;
  GtkWidget *ImgJavaScript;
  GtkWidget *animation_javascript1;
  GtkWidget *s__parateur1;
  GtkWidget *options1;
  GtkWidget *image903;
  GtkWidget *s__parateur18;
  GtkWidget *quitter1;
  GtkWidget *menuitem2;
  GtkWidget *menuitem2_menu;
  GtkWidget *copier1;
  GtkWidget *coller1;
  GtkWidget *s__parateur4;
  GtkWidget *couper1;
  GtkWidget *s__parateur16;
  GtkWidget *annuler1;
  GtkWidget *image_menu_undo;
  GtkWidget *calques1;
  GtkWidget *calques1_menu;
  GtkWidget *nouveau_calque1;
  GtkWidget *nouveau_calque_sonore1;
  GtkWidget *image3878;
  GtkWidget *s__parateur22Layers;  
  GtkWidget *dupliquer_calque1;
  GtkWidget *Imgdupliquer_calque;
  GtkWidget *renommer_calque1;
  GtkWidget *ImgLayerRename;
  GtkWidget *s__parateur12;
  GtkWidget *remonter_le_calque_sur_la_pile1;
  GtkWidget *image905;
  GtkWidget *descendre_le_calque_sur_la_pile1;
  GtkWidget *image906;
  GtkWidget *s__parateur11;
  GtkWidget *supprimer_calque1;
  GtkWidget *imageSupprimerCalque;
  GtkWidget *objets_1;
  GtkWidget *objets_1_menu;
  GtkWidget *dupliquer1;
  GtkWidget *ImgDupliquer;
  GtkWidget *deplacer;
  GtkWidget *s__parateur7;
  GtkWidget *monter;
  GtkWidget *descendre;
  GtkWidget *s__parateur10;
  GtkWidget *rotation1;
  GtkWidget *Imgrotation1;
  GtkWidget *image908;
  GtkWidget *rotation1_menu;
  GtkWidget *_90sens_horaire1;
  GtkWidget *_90_anti_horaire1;  
  GtkWidget *separateur22_rotation;
  GtkWidget *_180_sens_horaire1;
  GtkWidget *_270_sens_horaire1;  
  
  GtkWidget *s__parateur5;
  GtkWidget *align_items1;
  GtkWidget *image_menu_align;
  GtkWidget *s__parateur23_align;
  GtkWidget *tout_selectionner1;
  GtkWidget *ImgToutSelectionner;
  GtkWidget *deselectionner_tout1;


  GtkWidget *image910;
  GtkWidget *s__parateur8;
  GtkWidget *supprimer1;
  GtkWidget *image911;
  GtkWidget *trames1;
  GtkWidget *trames1_menu;
  GtkWidget *inserer_trames;
  GtkWidget *AddFramesToCursor;
  GtkWidget *ajouter_apres_la_position_courante1;
  GtkWidget *ImgAddFromCursor;
  GtkWidget *s__parateur14;
  GtkWidget *supprimer2;
  GtkWidget *ImgRemoveToCursor;
  GtkWidget *supprimer_apres_la_position_courante1;
  GtkWidget *ImgRemoveAfterCursor;
  GtkWidget *supprimer_les_trames_selectionnees1;
  GtkWidget *ImgRemoveSelectedFrames;
  GtkWidget *s__parateur13;
  GtkWidget *poser_une_image_cle1;

  GtkWidget *ImgNewKeyframe;
  GtkWidget *poser_une_image_cle_vide1;
  GtkWidget *ImgNewBlankKeyframe;
  GtkWidget *creer_une_interpolation_de_mouvement1;
  GtkWidget *ImgNewInterpola;
  GtkWidget *s__parateur20;
  GtkWidget *retirer_une_image_cle1;
  GtkWidget *ImgRemoveKeyframe;
  GtkWidget *mettre_a_blanc_image_cle1;
  GtkWidget *imageSetKeyframeToBlank;
  GtkWidget *objets1;
  GtkWidget *objets1_menu;
  GtkWidget *taille1;
  GtkWidget *image922;
  GtkWidget *couleur_de_la_page1;
  GtkWidget *image923;
  GtkWidget *s__parateur19;
  GtkWidget *format_animation1;
  GtkWidget *image924;
  GtkWidget *visualisation;
  GtkWidget *visualisation_menu;
  GtkWidget *jouer_animation1;
  GtkWidget *image925;
  GtkWidget *s__parateur17;
  GtkWidget *editeur_zoom_avant;
  GtkWidget *image926;
  GtkWidget *editeur_zoom_arriere;
  GtkWidget *image927;
  GtkWidget *page1;
  GtkWidget *page1_menu;
  GtkWidget *a_propos1;
  GtkWidget *image928;
  GtkWidget *s__parateur3;
  GtkWidget *touches_de_fonction1;
  GtkWidget *image929;
  GtkWidget *utilisation_de_la_souris1; 
 GtkWidget *image930;
 /* horizontal toolbar, detachable */
  GtkWidget *handleboxHorizToolbox;
  GtkWidget *toolbar1;
  GtkIconSize tmp_toolbar_icon_size;
  GtkWidget *toolitem1;
  GtkWidget *buttonNew;
  GtkWidget *image8;
  GtkWidget *toolitem2;
  GtkWidget *buttonOpen;
  GtkWidget *image9;
  GtkWidget *toolitem3;
  GtkWidget *buttonSave;
  GtkWidget *image10;
  GtkWidget *separatortoolitem3;
  GtkWidget *toolitem4;
  GtkWidget *buttonCopy;
  GtkWidget *image11;
  GtkWidget *toolitem5;
  GtkWidget *buttonPaste;
  GtkWidget *image12;
  GtkWidget *toolitem6;
  GtkWidget *buttonCut;
  GtkWidget *image13;
  GtkWidget *toolitem7;
   GtkWidget *separatortoolitem4a;
  GtkWidget *buttonUndo;
  GtkWidget *image14;
   GtkWidget *toolitem8;
   GtkWidget *buttonRedo;
  GtkWidget *image1379;   
  GtkWidget *separatortoolitem4;
  GtkWidget *hbox1;
  /* éditeur */
  GtkWidget *HandleBoxEditor;
  GtkWidget *toolbarEditor;
  GtkWidget *notebook1;
  GtkWidget *vbox16;
  GtkWidget *ScrolledWindowEditor;
  GtkWidget *statusbarEditor;
  GtkWidget *label11;
  GtkWidget *frame1;
  GtkWidget *alignment3;
  GtkWidget *vbox9;
  GtkWidget *ScrolledWindowPlayer;
  GtkWidget *scrolledwindow1;
  GtkWidget *viewportPlayer;
  GtkWidget *hseparator2;
  GtkWidget *hbox18;
    GtkWidget *labelEditorRotation;
   GtkWidget *label51EditorCouleurPP;
  GtkWidget *drawingarea1EditorCouleurPP;
  GtkWidget *label52CouleurAP;
  GtkWidget *drawingarea2CouleurAP;
  GtkWidget *vseparator4Editor;
  GtkWidget *labelEditorMousePosition;
  GtkWidget *vseparator3bEditor;
  GtkWidget *labelEditorHelp;  
  /* player */
  GtkWidget *buttonPlayerPlay;   
  GtkWidget *buttonPlayerStop;
  GtkWidget *buttonPlayerPause;
  GtkWidget *vseparator3Editor;
  GtkWidget *labelEditorPosition;
  GtkWidget *labelEditorTimePosition;
  GtkWidget *label12;
  GtkWidget *vbox15;
  GtkWidget *frameTimeline;
  GtkWidget *label37;
  GtkWidget *imageLayerDuplicate;
  GtkWidget *imageLayerNew;
  GtkWidget *imageLayerDelete;
  GtkWidget *statusbarEllaMain;
  GtkAccelGroup *accel_group;
  GtkTooltips *tooltips;

  GtkWidget *tableTimeline;
  GtkWidget *hbox2,*hbox3;
  GtkWidget *image6,*image7,*image8t,*image9t;
  GtkWidget *vseparator4,*vseparator5,*vseparator21;
  GtkWidget *labelImagePosition;
  GtkObject *spinbuttonMoviePosition_adj;
  GtkWidget *image1352;
  GtkWidget *image1353;
  GtkWidget *image1351;
  GtkWidget *imageNouveauCalque;
  GtkWidget *image1346;


  GtkWidget * s__parateur21;
  GtkWidget *image1575;
  GtkWidget *image1576;
  GtkWidget *importer_son;
  GtkWidget *image1577;
  GtkWidget *buttonZoomMoins;
  GtkWidget *imageZoomMoins;
  GtkWidget *buttonZoomPlus;
  GtkWidget *imageZoomPlus; 
  /* notebook 2 */
  GtkWidget *vseparator1;
  GtkWidget *vseparator2;
  GtkWidget *vboxNotebookProperties;
  GtkWidget *notebook2;

  /* onglet propriétés */
  GtkWidget *labelOngletProperties;
   GtkWidget *viewportProperties;
    GtkWidget *hseparator7Properties; 
  GtkWidget *vbox18Properties;
  GtkWidget *label50Properties;
  GtkWidget *tablePropertiesPosition;
  GtkWidget *imagePropertiesLock;
  GtkWidget *hbox20Properties;
  GtkWidget *label51Properties;
  GtkWidget *label52Properties;
  GtkWidget *hbox21Properties;
  GtkWidget *label54Properties;
  GtkWidget *label55Properties;
  GtkWidget *imageRedoProperties;
  GtkWidget *hbox31Properties;
  GtkWidget *hseparator5Properties;
  /* partie 'styles' */
  GtkWidget *tablePropertiesColors;
  GtkWidget *labelPropertiesFColor;
  GtkWidget *vseparatorTablePropertiesColor;
  GtkWidget *label51PropertiesSColor;
  GtkWidget *hboxPropertiesRFcolor;
  GtkWidget *labelPropertiesRFColor;
  GtkObject *spinbuttonPropertiesRFColor_adj;
  GtkWidget *spinbuttonPropertiesRFColor;
  GtkWidget *hboxPropertiesVFColor;
  GtkWidget *labelPropertiesVFColor;
  GtkObject *spinbuttonPropertiesVFColor_adj;
  GtkWidget *spinbuttonPropertiesVFColor;
  GtkWidget *hboxPropertiesBFColor;
  GtkObject *spinbuttonPropertiesBFColor_adj;
  GtkWidget *spinbuttonPropertiesBFColor;
  GtkWidget *hboxPropertiesAFColor;
  GtkWidget *labelPropertiesBFColor;
  GtkObject *spinbuttonPropertiesAFColor_adj;
  GtkWidget *spinbuttonPropertiesAFColor;
  GtkWidget *hboxPropertiesRSColor;
  GtkWidget *labelPropertiesRSColor;
  GtkObject *spinbuttonPropertiesRSColor_adj;
  GtkWidget *spinbuttonPropertiesRSColor;
  GtkWidget *hboxPropertiesVSColor;
  GtkWidget *labelPropertiesVSColor;
  GtkObject *spinbuttonPropertiesVSColor_adj;
  GtkWidget *spinbuttonPropertiesVSColor;
  GtkWidget *labelPropertiesBScolor;
  GtkObject *spinbuttonPropertiesBSColor_adj;
  GtkWidget *spinbuttonPropertiesBSColor;
  GtkWidget *hboxPropertiesASColor;
  GtkWidget *labelPropertiesASColor;
  GtkObject *spinbuttonPropertiesASColor_adj;
  GtkWidget *spinbuttonPropertiesASColor;
  GtkWidget *framePropertiesFColor;
  GtkWidget *alignment7;
  GtkWidget *drawingareaPropertiesFColor;
  GtkWidget *framePropertiesSColor;
  GtkWidget *alignment8;
  GtkWidget *drawingareaPropertiesSColor;
  GtkWidget *vbox20Properties;
  GtkWidget *hbox35PropertiesLines;
  GtkWidget *label50PropertiesLinesWidth;
  GtkObject *spinbuttonPropertiesLineWidth_adj;
  GtkWidget *spinbuttonPropertiesLineWidth;
  GtkWidget *buttonPropertiesUndoStyleChange;
  GtkWidget *imageUndoStyles;
  GtkWidget *hseparator7PropertiesStyle;  
  GtkWidget *labelTitreCouleursProperties;  
  
  //GtkWidget *scrolledwindowTransitions;
 GtkWidget *viewport3;
  GtkWidget *vboxTransitions;
  GtkWidget *labelInterpolation;
  GtkWidget *hboxInterpola;
  GtkWidget *labelActiverInterpola;
    GtkWidget *imageTransitionInterpola;

  GtkWidget *hseparator7t;
  GtkWidget *labelZoom;
  GtkWidget *hboxZoom;
  GtkWidget *labelActiverZooming;
    GtkWidget *imageTransitionZooming;

  GtkWidget *hseparator8t;
  GtkWidget *labelRotation;
  GtkWidget *hboxRotation;
  GtkWidget *labelActiverRotation;
    GtkWidget *imageTransitionRotation;

  GtkWidget *hseparator10t;
  GtkWidget *labelAlpha;
  GtkWidget *hbox35Alpha;
  GtkWidget *label7ActiverAlpha;
  GtkWidget *imageTransitionAlpha;
  GtkWidget *hseparator7t_morphing;
  GtkWidget *labelMorphing;
  GtkWidget *hbox35Morphing;
  GtkWidget *imageTransitionMorphing;
  GtkWidget *labelActivateMorphing;

  
  GtkWidget *labelTransitions;
  
  GtkWidget *scrolledwindowEffects;
  GtkWidget *viewport2;
  GtkWidget *vbox19;
  /* fin transitions */
   GtkWidget *scrolledwindowBiblio;
  GtkWidget *hseparator6;
  GtkWidget *hbox32;
  GtkWidget *buttonBiblioArchive;
  GtkWidget *alignment5;
  GtkWidget *hbox33;
  GtkWidget *image1610;
  GtkWidget *buttonBiblioOpenArchive;
  GtkWidget *alignment6;
  GtkWidget *hbox34;
  GtkWidget *image1611;
 // GtkWidget *vseparator2;
  GtkWidget *buttonBiblioRemove;
  GtkWidget *image1612;
  GtkWidget *label49;
  GdkPixbuf *ico; 
   
  tooltips = gtk_tooltips_new ();

  accel_group = gtk_accel_group_new ();

  window_ella = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window_ella), "Ella");
  gtk_window_set_position (GTK_WINDOW (window_ella), GTK_WIN_POS_CENTER);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window_ella), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 0);

  menuitem1 = gtk_menu_item_new_with_mnemonic (gettext("_File"));
  gtk_widget_show (menuitem1);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);

  menuitem1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menuitem1_menu);

  nouveau1 = gtk_image_menu_item_new_from_stock ("gtk-new", accel_group);
  gtk_widget_show (nouveau1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), nouveau1);

  ouvrir1 = gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
  gtk_widget_show (ouvrir1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), ouvrir1);
/* recent files */
 recent_files1 = gtk_image_menu_item_new_with_mnemonic (gettext("Recent files"));
  gtk_widget_show (recent_files1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), recent_files1);

  image3796 = gtk_image_new_from_stock ("gtk-index", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image3796);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (recent_files1), image3796);

  recent_files1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (recent_files1), recent_files1_menu);

  recent1 = gtk_menu_item_new_with_mnemonic ("_1");
  gtk_widget_show (recent1);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent1);
  gtk_widget_set_sensitive (recent1, FALSE);
  
  recent2 = gtk_menu_item_new_with_mnemonic ("_2");
  gtk_widget_show (recent2);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent2);
gtk_widget_set_sensitive (recent2, FALSE);

  recent3 = gtk_menu_item_new_with_mnemonic ("_3");
  gtk_widget_show (recent3);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent3);
gtk_widget_set_sensitive (recent3, FALSE);

  recent4 = gtk_menu_item_new_with_mnemonic ("_4");
  gtk_widget_show (recent4);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent4);
gtk_widget_set_sensitive (recent4, FALSE);

  recent5 = gtk_menu_item_new_with_mnemonic ("_5");
  gtk_widget_show (recent5);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent5);
gtk_widget_set_sensitive (recent5, FALSE);

  recent6 = gtk_menu_item_new_with_mnemonic ("_6");
  gtk_widget_show (recent6);
  gtk_container_add (GTK_CONTAINER (recent_files1_menu), recent6);
gtk_widget_set_sensitive (recent6, FALSE);

  s__parateur15 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur15);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), s__parateur15);
  gtk_widget_set_sensitive (s__parateur15, FALSE);

  enregistrer1 = gtk_image_menu_item_new_from_stock ("gtk-save", accel_group);
  gtk_widget_show (enregistrer1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), enregistrer1);

  enregistrer_sous1 = gtk_image_menu_item_new_from_stock ("gtk-save-as", accel_group);
  gtk_widget_show (enregistrer_sous1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), enregistrer_sous1);

  separatormenuitem1 = gtk_separator_menu_item_new ();
  gtk_widget_show (separatormenuitem1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separatormenuitem1);
  gtk_widget_set_sensitive (separatormenuitem1, FALSE);

  importer1 = gtk_image_menu_item_new_with_mnemonic(gettext("I_mport in the library"));
  gtk_widget_show (importer1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), importer1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)import_xpm); 
  Imgimporter1 = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (Imgimporter1);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (importer1), Imgimporter1);
  importer1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (importer1), importer1_menu);

  une_image_bitmap1 = gtk_image_menu_item_new_with_mnemonic (gettext("Image _bitmap"));
  gtk_widget_show (une_image_bitmap1);
  gtk_container_add (GTK_CONTAINER (importer1_menu), une_image_bitmap1);
  image1575 = gtk_image_new_from_stock ("gtk-orientation-portrait", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1575);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (une_image_bitmap1), image1575);

  une_image_vectorielle1 = gtk_image_menu_item_new_with_mnemonic (gettext("Ve_ctorial Drawing"));
  gtk_widget_show (une_image_vectorielle1);
  gtk_container_add (GTK_CONTAINER (importer1_menu), une_image_vectorielle1);
  image1576 = gtk_image_new_from_stock ("gtk-page-setup", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1576);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (une_image_vectorielle1), image1576);
  
  s__parateur21 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur21);
  gtk_container_add (GTK_CONTAINER (importer1_menu), s__parateur21);
  gtk_widget_set_sensitive (s__parateur21, FALSE);
  
  importer_son = gtk_image_menu_item_new_with_mnemonic (gettext("Sound or music"));
  gtk_widget_show (importer_son);
  gtk_container_add (GTK_CONTAINER (importer1_menu), importer_son);
  image1577 = gtk_image_new_from_stock ("gtk-cdrom", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1577);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (importer_son), image1577);

  s__parateur22 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur22);
  gtk_container_add (GTK_CONTAINER (importer1_menu), s__parateur22);
  gtk_widget_set_sensitive (s__parateur22, FALSE);
  import_symbol = gtk_image_menu_item_new_with_mnemonic (gettext("Import a symbo_l"));
  gtk_widget_show (import_symbol);
  gtk_container_add (GTK_CONTAINER (importer1_menu), import_symbol);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)import_symbol_xpm); 
  ImgImportSymbol=gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgImportSymbol);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (import_symbol), ImgImportSymbol);
  
/* export */  
  exporter1 = gtk_image_menu_item_new_with_mnemonic(gettext("E_xport the project"));
  gtk_widget_show (exporter1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), exporter1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)export_xpm); 
  Imgexporter1 = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (Imgexporter1);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (exporter1), Imgexporter1);
  exporter1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (exporter1), exporter1_menu);

  animation_flash1 = gtk_image_menu_item_new_with_mnemonic (gettext("as _Flash animation"));
  gtk_widget_show (animation_flash1);
  gtk_container_add (GTK_CONTAINER (exporter1_menu), animation_flash1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)flash_xpm); 
  Imgflash = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (Imgflash);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (animation_flash1), Imgflash);  

  animation_svg1 = gtk_image_menu_item_new_with_mnemonic (gettext("as S_VG animations"));
  gtk_widget_show (animation_svg1);
  gtk_container_add (GTK_CONTAINER (exporter1_menu), animation_svg1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)w3svg16x16_xpm); 
  ImgSvg = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgSvg);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (animation_svg1), ImgSvg);

  animation_javascript1 = gtk_image_menu_item_new_with_mnemonic (gettext("as _JavaScript animation"));
  gtk_widget_show (animation_javascript1);
  gtk_container_add (GTK_CONTAINER (exporter1_menu), animation_javascript1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)javascript16x16_xpm); 
  ImgJavaScript = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgJavaScript);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (animation_javascript1), ImgJavaScript); 
 gtk_widget_set_sensitive (animation_javascript1, FALSE);
 
  s__parateur1 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), s__parateur1);
  gtk_widget_set_sensitive (s__parateur1, FALSE);

  options1 = gtk_image_menu_item_new_with_mnemonic (gettext("O_ptions and settings"));
  gtk_widget_show (options1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), options1);

  image903 = gtk_image_new_from_stock ("gtk-preferences", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image903);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (options1), image903);

  s__parateur18 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur18);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), s__parateur18);
  gtk_widget_set_sensitive (s__parateur18, FALSE);

  quitter1 = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
  gtk_widget_show (quitter1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), quitter1);
/* edit */
  menuitem2 = gtk_menu_item_new_with_mnemonic (gettext("E_dit"));
  gtk_widget_show (menuitem2);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem2);

  menuitem2_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menuitem2_menu);

  copier1 = gtk_image_menu_item_new_from_stock ("gtk-copy", accel_group);
  gtk_widget_show (copier1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), copier1);

  coller1 = gtk_image_menu_item_new_from_stock ("gtk-paste", accel_group);
  gtk_widget_show (coller1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), coller1);

  s__parateur4 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur4);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), s__parateur4);
  gtk_widget_set_sensitive (s__parateur4, FALSE);

  couper1 = gtk_image_menu_item_new_from_stock ("gtk-cut", accel_group);
  gtk_widget_show (couper1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), couper1);

  s__parateur16 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur16);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), s__parateur16);
  gtk_widget_set_sensitive (s__parateur16, FALSE);
/* undo */
  annuler1 = gtk_image_menu_item_new_with_mnemonic (gettext("Nothing to _undo"));
  gtk_widget_show (annuler1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), annuler1);
  gtk_widget_add_accelerator (annuler1, "activate", accel_group,
                              GDK_Z, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  image_menu_undo = gtk_image_new_from_stock ("gtk-undo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image_menu_undo);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (annuler1), image_menu_undo);

  /* layers */


  calques1 = gtk_menu_item_new_with_mnemonic (gettext("_Layers"));
  gtk_widget_show (calques1);
  gtk_container_add (GTK_CONTAINER (menubar1), calques1);

  calques1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (calques1), calques1_menu);

  calques1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (calques1), calques1_menu);

  nouveau_calque1 = gtk_image_menu_item_new_with_mnemonic (gettext("_New drawing layer"));
  gtk_widget_show (nouveau_calque1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), nouveau_calque1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)layer_new_16x16_xpm); 
  imageNouveauCalque = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageNouveauCalque);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (nouveau_calque1), imageNouveauCalque);   
  
  nouveau_calque_sonore1 = gtk_image_menu_item_new_with_mnemonic (gettext("New _sound layer"));
  gtk_widget_show (nouveau_calque_sonore1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), nouveau_calque_sonore1);

  image3878 = gtk_image_new_from_stock ("gtk-cdrom", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image3878);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (nouveau_calque_sonore1), image3878);

  s__parateur22Layers = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur22Layers);
  gtk_container_add (GTK_CONTAINER (calques1_menu), s__parateur22Layers);
  gtk_widget_set_sensitive (s__parateur22Layers, FALSE);
  
  dupliquer_calque1 = gtk_image_menu_item_new_with_mnemonic (gettext("Duplicate layer"));
  gtk_widget_show (dupliquer_calque1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), dupliquer_calque1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)duplicate_layer_16x16_xpm); 
  Imgdupliquer_calque = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (Imgdupliquer_calque);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (dupliquer_calque1), Imgdupliquer_calque); 

  renommer_calque1 = gtk_image_menu_item_new_with_mnemonic (gettext("Rename layer"));
  gtk_widget_show (renommer_calque1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), renommer_calque1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)layer_rename_16x16_xpm); 
  ImgLayerRename = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgLayerRename);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (renommer_calque1), ImgLayerRename);

  s__parateur12 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur12);
  gtk_container_add (GTK_CONTAINER (calques1_menu), s__parateur12);
  gtk_widget_set_sensitive (s__parateur12, FALSE);

  remonter_le_calque_sur_la_pile1 = gtk_image_menu_item_new_with_mnemonic (gettext("Raise the layer in the layer stack"));
  gtk_widget_show (remonter_le_calque_sur_la_pile1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), remonter_le_calque_sur_la_pile1);
  image905 = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image905);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (remonter_le_calque_sur_la_pile1), image905);

  descendre_le_calque_sur_la_pile1 = gtk_image_menu_item_new_with_mnemonic (gettext("Lower the layer in the layer stack"));
  gtk_widget_show (descendre_le_calque_sur_la_pile1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), descendre_le_calque_sur_la_pile1);
  image906 = gtk_image_new_from_stock ("gtk-go-down", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image906);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (descendre_le_calque_sur_la_pile1), image906);

  s__parateur11 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur11);
  gtk_container_add (GTK_CONTAINER (calques1_menu), s__parateur11);
  gtk_widget_set_sensitive (s__parateur11, FALSE);

  supprimer_calque1 = gtk_image_menu_item_new_with_mnemonic (gettext("Remove the layer"));
  gtk_widget_show (supprimer_calque1);
  gtk_container_add (GTK_CONTAINER (calques1_menu), supprimer_calque1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)layer_delete_16x16_xpm); 
  imageSupprimerCalque = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageSupprimerCalque);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer_calque1), imageSupprimerCalque);
/* objets - items */
  objets_1 = gtk_menu_item_new_with_mnemonic (gettext("_Objects "));
  gtk_widget_show (objets_1);
  gtk_container_add (GTK_CONTAINER (menubar1), objets_1);
  objets_1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (objets_1), objets_1_menu);

  dupliquer1 = gtk_image_menu_item_new_with_mnemonic (gettext("D_uplicate"));
  gtk_widget_show (dupliquer1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), dupliquer1);
  gtk_widget_add_accelerator (dupliquer1, "activate", accel_group,
                              GDK_D, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)duplicate_item_16x16_xpm); 
  ImgDupliquer = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgDupliquer);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (dupliquer1), ImgDupliquer);

  s__parateur7 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur7);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), s__parateur7);
  gtk_widget_set_sensitive (s__parateur7, FALSE);

  monter = gtk_image_menu_item_new_with_mnemonic (gettext("_Raise"));
  gtk_widget_show (monter);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), monter);
  gtk_widget_add_accelerator (monter, "activate", accel_group,
                              GDK_G, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1352 = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1352);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (monter), image1352);

  descendre = gtk_image_menu_item_new_with_mnemonic (gettext("_Lower"));
  gtk_widget_show (descendre);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), descendre);
  gtk_widget_add_accelerator (descendre, "activate", accel_group,
                              GDK_U, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image1353 = gtk_image_new_from_stock ("gtk-go-down", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image1353);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (descendre), image1353);

  s__parateur10 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur10);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), s__parateur10);
  gtk_widget_set_sensitive (s__parateur10, FALSE);

  rotation1 = gtk_image_menu_item_new_with_mnemonic (gettext("Rotate"));
  gtk_widget_show (rotation1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), rotation1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)rotate_xpm); 
  Imgrotation1 = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (Imgrotation1);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (rotation1), Imgrotation1);
 
  rotation1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (rotation1), rotation1_menu);

  _90sens_horaire1 = gtk_menu_item_new_with_mnemonic (gettext("90 \302\260 _clockwise"));
  gtk_widget_show (_90sens_horaire1);
  gtk_container_add (GTK_CONTAINER (rotation1_menu), _90sens_horaire1);

  _90_anti_horaire1 = gtk_menu_item_new_with_mnemonic (gettext("90 \302\260 c_ounterclockwise"));
  gtk_widget_show (_90_anti_horaire1);
  gtk_container_add (GTK_CONTAINER (rotation1_menu), _90_anti_horaire1);
  separateur22_rotation = gtk_separator_menu_item_new ();
  gtk_widget_show (separateur22_rotation);
  gtk_container_add (GTK_CONTAINER (rotation1_menu), separateur22_rotation);
  gtk_widget_set_sensitive (separateur22_rotation, FALSE);

  _180_sens_horaire1 = gtk_menu_item_new_with_mnemonic (gettext("180\302\260 cl_ockwise"));
  gtk_widget_show (_180_sens_horaire1);
  gtk_container_add (GTK_CONTAINER (rotation1_menu), _180_sens_horaire1);

  _270_sens_horaire1 = gtk_menu_item_new_with_mnemonic (gettext("270\302\260 cloc_e"));
  gtk_widget_show (_270_sens_horaire1);
  gtk_container_add (GTK_CONTAINER (rotation1_menu), _270_sens_horaire1);

  s__parateur5 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur5);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), s__parateur5);
  gtk_widget_set_sensitive (s__parateur5, FALSE);
   align_items1 = gtk_image_menu_item_new_with_mnemonic (gettext("_Align items"));
  gtk_widget_show (align_items1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), align_items1);
  image_menu_align = gtk_image_new_from_stock ("gtk-justify-fill", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image_menu_align);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (align_items1), image_menu_align);

  s__parateur23_align = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur23_align);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), s__parateur23_align);
  gtk_widget_set_sensitive (s__parateur23_align, FALSE);

  tout_selectionner1 = gtk_image_menu_item_new_with_mnemonic (gettext("_Select all"));
  gtk_widget_show (tout_selectionner1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), tout_selectionner1);
  gtk_widget_add_accelerator (tout_selectionner1, "activate", accel_group,
                              GDK_A, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)select_all_xpm); 
  ImgToutSelectionner = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (ImgToutSelectionner);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (tout_selectionner1), ImgToutSelectionner);

  deselectionner_tout1 = gtk_image_menu_item_new_with_mnemonic (gettext("_Deselect all"));
  gtk_widget_show (deselectionner_tout1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), deselectionner_tout1);
  gtk_widget_add_accelerator (deselectionner_tout1, "activate", accel_group,
                              GDK_A, (GdkModifierType) GDK_CONTROL_MASK | GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)deselect_all_xpm);
  
  GtkWidget *ImgDeselectionnerTout = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico); 
  gtk_widget_show (ImgDeselectionnerTout);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (deselectionner_tout1), ImgDeselectionnerTout);

   s__parateur8 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur8);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), s__parateur8);
  gtk_widget_set_sensitive (s__parateur8, FALSE);

  supprimer1 = gtk_image_menu_item_new_with_mnemonic (gettext("Remove"));
  gtk_widget_show (supprimer1);
  gtk_container_add (GTK_CONTAINER (objets_1_menu), supprimer1);
  gtk_widget_add_accelerator (supprimer1, "activate", accel_group,
                              GDK_Delete, (GdkModifierType) 0,
                              GTK_ACCEL_VISIBLE);
  image911 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image911);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer1), image911);
/* frames */
  trames1 = gtk_menu_item_new_with_mnemonic (gettext("_Frames"));
  gtk_widget_show (trames1);
  gtk_container_add (GTK_CONTAINER (menubar1), trames1);

  trames1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (trames1), trames1_menu);

  inserer_trames = gtk_image_menu_item_new_with_mnemonic (gettext("Add up the current position"));
  gtk_widget_show (inserer_trames);
  gtk_container_add (GTK_CONTAINER (trames1_menu), inserer_trames);
  gtk_widget_add_accelerator (inserer_trames, "activate", accel_group,
                              GDK_F5, (GdkModifierType) 0,
                              GTK_ACCEL_VISIBLE);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)add_frames_to_cursor_xpm);							  
  AddFramesToCursor = gtk_image_new_from_pixbuf(ico); 
  gtk_widget_show (AddFramesToCursor);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (inserer_trames), AddFramesToCursor);

  ajouter_apres_la_position_courante1 = gtk_image_menu_item_new_with_mnemonic (gettext("Add from the current position"));
  gtk_widget_show (ajouter_apres_la_position_courante1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), ajouter_apres_la_position_courante1);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)add_frames_from_cursor_xpm);
  ImgAddFromCursor = gtk_image_new_from_pixbuf(ico);
  gtk_widget_show (ImgAddFromCursor);
  g_object_unref(ico);  
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (ajouter_apres_la_position_courante1), ImgAddFromCursor);

  s__parateur14 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur14);
  gtk_container_add (GTK_CONTAINER (trames1_menu), s__parateur14);
  gtk_widget_set_sensitive (s__parateur14, FALSE);

  supprimer2 = gtk_image_menu_item_new_with_mnemonic (gettext("Delete to the current position"));
  gtk_widget_show (supprimer2);
  gtk_container_add (GTK_CONTAINER (trames1_menu), supprimer2);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)remove_frames_to_cursor_xpm);
  ImgRemoveToCursor = gtk_image_new_from_pixbuf(ico); 
  gtk_widget_show (ImgRemoveToCursor);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer2), ImgRemoveToCursor);

  supprimer_apres_la_position_courante1 = gtk_image_menu_item_new_with_mnemonic (gettext("Delete after the current position"));
  gtk_widget_show (supprimer_apres_la_position_courante1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), supprimer_apres_la_position_courante1);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)remove_frames_from_cursor_xpm);
  ImgRemoveAfterCursor = gtk_image_new_from_pixbuf(ico); 
  gtk_widget_show (ImgRemoveAfterCursor);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer_apres_la_position_courante1), ImgRemoveAfterCursor);

  supprimer_les_trames_selectionnees1 = gtk_image_menu_item_new_with_mnemonic (gettext("Remove the selected frames"));
  gtk_widget_show (supprimer_les_trames_selectionnees1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), supprimer_les_trames_selectionnees1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)remove_selection16x16_xpm);
  ImgRemoveSelectedFrames = gtk_image_new_from_pixbuf(ico); 
  gtk_widget_show (ImgRemoveSelectedFrames);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer_les_trames_selectionnees1), ImgRemoveSelectedFrames);

  s__parateur13 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur13);
  gtk_container_add (GTK_CONTAINER (trames1_menu), s__parateur13);
  gtk_widget_set_sensitive (s__parateur13, FALSE);

  poser_une_image_cle1 = gtk_image_menu_item_new_with_mnemonic (gettext("Place a key image"));
  gtk_widget_show (poser_une_image_cle1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), poser_une_image_cle1);
  gtk_widget_add_accelerator (poser_une_image_cle1, "activate", accel_group,
                              GDK_F6, (GdkModifierType) 0,
                              GTK_ACCEL_VISIBLE);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)new_keyframe_xpm);
  ImgNewKeyframe = gtk_image_new_from_pixbuf(ico); 
  gtk_widget_show (ImgNewKeyframe);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (poser_une_image_cle1), ImgNewKeyframe);

  poser_une_image_cle_vide1 = gtk_image_menu_item_new_with_mnemonic (gettext("Place an empty key image"));
  gtk_widget_show (poser_une_image_cle_vide1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), poser_une_image_cle_vide1);
  gtk_widget_add_accelerator (poser_une_image_cle_vide1, "activate", accel_group,
                              GDK_F7, (GdkModifierType) 0,
                              GTK_ACCEL_VISIBLE);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)new_blank_keyframe_xpm);
  ImgNewBlankKeyframe = gtk_image_new_from_pixbuf(ico); 
  g_object_unref(ico);  
  gtk_widget_show (ImgNewBlankKeyframe);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (poser_une_image_cle_vide1), ImgNewBlankKeyframe);

  creer_une_interpolation_de_mouvement1 = gtk_image_menu_item_new_with_mnemonic (gettext("Create an interpolation"));
  gtk_widget_show (creer_une_interpolation_de_mouvement1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), creer_une_interpolation_de_mouvement1);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)interpola16x16_xpm);
  ImgNewInterpola = gtk_image_new_from_pixbuf(ico);
  gtk_widget_show (ImgNewInterpola);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (creer_une_interpolation_de_mouvement1), ImgNewInterpola);

  s__parateur20 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur20);
  gtk_container_add (GTK_CONTAINER (trames1_menu), s__parateur20);
  gtk_widget_set_sensitive (s__parateur20, FALSE);

  retirer_une_image_cle1 = gtk_image_menu_item_new_with_mnemonic (gettext("Remove a key image"));
  gtk_widget_show (retirer_une_image_cle1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), retirer_une_image_cle1);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)remove_keyframe_xpm);
  ImgRemoveKeyframe =  gtk_image_new_from_pixbuf(ico);
  gtk_widget_show (ImgRemoveKeyframe);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (retirer_une_image_cle1), ImgRemoveKeyframe);

  mettre_a_blanc_image_cle1 = gtk_image_menu_item_new_with_mnemonic (gettext("Set the image key empty"));
  gtk_widget_show (mettre_a_blanc_image_cle1);
  gtk_container_add (GTK_CONTAINER (trames1_menu), mettre_a_blanc_image_cle1);
  ico=gdk_pixbuf_new_from_xpm_data((const char **)set_keyframe_to_blank_xpm);
  gtk_tooltips_set_tip (tooltips, mettre_a_blanc_image_cle1,
    gettext( "Allow tho wipe the objects of an key frame, for example to build jumps animations"), NULL);
  gtk_widget_add_accelerator (mettre_a_blanc_image_cle1, "activate", accel_group,
                              GDK_F6, (GdkModifierType) GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);
  imageSetKeyframeToBlank = gtk_image_new_from_pixbuf(ico);
  gtk_widget_show (imageSetKeyframeToBlank);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mettre_a_blanc_image_cle1), imageSetKeyframeToBlank);
  
  GtkWidget *supprimer_interpolation;
  GtkWidget *imageRemoveInterpolation;
  supprimer_interpolation = gtk_image_menu_item_new_with_mnemonic (gettext("Remove the interpolation"));
  gtk_widget_show(supprimer_interpolation);
  gtk_container_add (GTK_CONTAINER (trames1_menu), supprimer_interpolation);
  ico=gdk_pixbuf_new_from_xpm_data((const char **) remove_interpola16x16_xpm);
  imageRemoveInterpolation=gtk_image_new_from_pixbuf(ico);
  gtk_widget_show (imageRemoveInterpolation);
  g_object_unref(ico);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (supprimer_interpolation), imageRemoveInterpolation);

  objets1 = gtk_menu_item_new_with_mnemonic (gettext("Fo_rmat"));
  gtk_widget_show (objets1);
  gtk_container_add (GTK_CONTAINER (menubar1), objets1);

  objets1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (objets1), objets1_menu);

  taille1 = gtk_image_menu_item_new_with_mnemonic (gettext("Scene size"));
  gtk_widget_show (taille1);
  gtk_container_add (GTK_CONTAINER (objets1_menu), taille1);

  image922 = gtk_image_new_from_stock ("gtk-orientation-portrait", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image922);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (taille1), image922);

  couleur_de_la_page1 = gtk_image_menu_item_new_with_mnemonic (gettext("Scene background"));
  gtk_widget_show (couleur_de_la_page1);
  gtk_container_add (GTK_CONTAINER (objets1_menu), couleur_de_la_page1);

  image923 = gtk_image_new_from_stock ("gtk-select-color", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image923);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (couleur_de_la_page1), image923);

  s__parateur19 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur19);
  gtk_container_add (GTK_CONTAINER (objets1_menu), s__parateur19);
  gtk_widget_set_sensitive (s__parateur19, FALSE);

  format_animation1 = gtk_image_menu_item_new_with_mnemonic (gettext("Film footage"));
  gtk_widget_show (format_animation1);
  gtk_container_add (GTK_CONTAINER (objets1_menu), format_animation1);
  gtk_widget_add_accelerator (format_animation1, "activate", accel_group,
                              GDK_M, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image924 = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image924);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (format_animation1), image924);

  visualisation = gtk_menu_item_new_with_mnemonic (gettext("_Visualization"));
  gtk_widget_show (visualisation);
  gtk_container_add (GTK_CONTAINER (menubar1), visualisation);

  visualisation_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (visualisation), visualisation_menu);

  jouer_animation1 = gtk_image_menu_item_new_with_mnemonic (gettext("Play the film"));
  gtk_widget_show (jouer_animation1);
  gtk_container_add (GTK_CONTAINER (visualisation_menu), jouer_animation1);
  gtk_widget_add_accelerator (jouer_animation1, "activate", accel_group,
                              GDK_Return, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image925 = gtk_image_new_from_stock ("gtk-media-play", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image925);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (jouer_animation1), image925);

  s__parateur17 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur17);
  gtk_container_add (GTK_CONTAINER (visualisation_menu), s__parateur17);
  gtk_widget_set_sensitive (s__parateur17, FALSE);

  editeur_zoom_avant = gtk_image_menu_item_new_with_mnemonic (gettext("Pasteboard zoom up"));
  gtk_widget_show (editeur_zoom_avant);
  gtk_container_add (GTK_CONTAINER (visualisation_menu), editeur_zoom_avant);
  gtk_widget_add_accelerator (editeur_zoom_avant, "activate", accel_group,
                              GDK_KP_Add, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image926 = gtk_image_new_from_stock ("gtk-zoom-in", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image926);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (editeur_zoom_avant), image926);

  editeur_zoom_arriere = gtk_image_menu_item_new_with_mnemonic (gettext("Pasteboard zoom down"));
  gtk_widget_show (editeur_zoom_arriere);
  gtk_container_add (GTK_CONTAINER (visualisation_menu), editeur_zoom_arriere);
  gtk_widget_add_accelerator (editeur_zoom_arriere, "activate", accel_group,
                              GDK_KP_Subtract, (GdkModifierType) GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  image927 = gtk_image_new_from_stock ("gtk-zoom-out", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image927);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (editeur_zoom_arriere), image927);

  page1 = gtk_menu_item_new_with_mnemonic ("_?");
  gtk_widget_show (page1);
  gtk_container_add (GTK_CONTAINER (menubar1), page1);

  page1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (page1), page1_menu);

  a_propos1 = gtk_image_menu_item_new_with_mnemonic (gettext("About Ella"));
  gtk_widget_show (a_propos1);
  gtk_container_add (GTK_CONTAINER (page1_menu), a_propos1);

  image928 = gtk_image_new_from_stock ("gtk-info", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image928);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (a_propos1), image928);

  s__parateur3 = gtk_separator_menu_item_new ();
  gtk_widget_show (s__parateur3);
  gtk_container_add (GTK_CONTAINER (page1_menu), s__parateur3);
  gtk_widget_set_sensitive (s__parateur3, FALSE);

  touches_de_fonction1 = gtk_image_menu_item_new_with_mnemonic (gettext("_Help"));
  gtk_widget_show (touches_de_fonction1);
  gtk_container_add (GTK_CONTAINER (page1_menu), touches_de_fonction1);

  image929 = gtk_image_new_from_stock ("gtk-help", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image929);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (touches_de_fonction1), image929);

  
/* upper toolbar */
 handleboxHorizToolbox = gtk_handle_box_new ();
  gtk_widget_show (handleboxHorizToolbox);
  gtk_box_pack_start (GTK_BOX (vbox1), handleboxHorizToolbox, FALSE, TRUE, 0);
  
   toolbar1 = gtk_toolbar_new ();
  gtk_widget_show (toolbar1);
  gtk_container_add (GTK_CONTAINER (handleboxHorizToolbox), toolbar1);
  //gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_BOTH);
  tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1));

  toolitem1 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem1);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem1);

  buttonNew = gtk_button_new ();
  gtk_widget_show (buttonNew);
  gtk_container_add (GTK_CONTAINER (toolitem1), buttonNew);
  gtk_tooltips_set_tip (tooltips, buttonNew, gettext("Create a new animation project"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonNew), GTK_RELIEF_NONE);

  image8 = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image8);
  gtk_container_add (GTK_CONTAINER (buttonNew), image8);

  toolitem2 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem2);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem2);

  buttonOpen = gtk_button_new ();
  gtk_widget_show (buttonOpen);
  gtk_container_add (GTK_CONTAINER (toolitem2), buttonOpen);
  gtk_tooltips_set_tip (tooltips, buttonOpen, gettext("Open an existing project"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonOpen), GTK_RELIEF_NONE);

  image9 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image9);
  gtk_container_add (GTK_CONTAINER (buttonOpen), image9);

  toolitem3 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem3);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem3);

  buttonSave = gtk_button_new ();
  gtk_widget_show (buttonSave);
  gtk_container_add (GTK_CONTAINER (toolitem3), buttonSave);
  gtk_tooltips_set_tip (tooltips, buttonSave, gettext("Save the current project"), NULL);
gtk_button_set_relief (GTK_BUTTON (buttonSave), GTK_RELIEF_NONE);

  image10 = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image10);
  gtk_container_add (GTK_CONTAINER (buttonSave), image10);

  separatortoolitem3 = (GtkWidget*) gtk_separator_tool_item_new ();
  gtk_widget_show (separatortoolitem3);
  gtk_container_add (GTK_CONTAINER (toolbar1), separatortoolitem3);

  toolitem4 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem4);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem4);

  buttonCopy = gtk_button_new ();
  gtk_widget_show (buttonCopy);
  gtk_container_add (GTK_CONTAINER (toolitem4), buttonCopy);
  gtk_tooltips_set_tip (tooltips, buttonCopy, gettext("Copy"), NULL);
gtk_button_set_relief (GTK_BUTTON (buttonCopy), GTK_RELIEF_NONE);

  image11 = gtk_image_new_from_stock ("gtk-copy", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image11);
  gtk_container_add (GTK_CONTAINER (buttonCopy), image11);

  toolitem5 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem5);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem5);

  buttonPaste = gtk_button_new ();
  gtk_widget_show (buttonPaste);
  gtk_container_add (GTK_CONTAINER (toolitem5), buttonPaste);
  gtk_tooltips_set_tip (tooltips, buttonPaste, gettext("Paste"), NULL);
gtk_button_set_relief (GTK_BUTTON (buttonPaste), GTK_RELIEF_NONE);

  image12 = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image12);
  gtk_container_add (GTK_CONTAINER (buttonPaste), image12);

  toolitem6 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem6);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem6);

  buttonCut = gtk_button_new ();
  gtk_widget_show (buttonCut);
  gtk_container_add (GTK_CONTAINER (toolitem6), buttonCut);
  gtk_tooltips_set_tip (tooltips, buttonCut, gettext("Cut"), NULL);
gtk_button_set_relief (GTK_BUTTON (buttonCut), GTK_RELIEF_NONE);

  image13 = gtk_image_new_from_stock ("gtk-cut", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image13);
  gtk_container_add (GTK_CONTAINER (buttonCut), image13);
  
separatortoolitem4a = (GtkWidget *) gtk_separator_tool_item_new ();
  gtk_widget_show (separatortoolitem4a);
  gtk_container_add (GTK_CONTAINER (toolbar1), separatortoolitem4a);


  toolitem7 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem7);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem7);

/* undo */
  buttonUndo = gtk_button_new ();
  gtk_widget_show (buttonUndo);
  gtk_container_add (GTK_CONTAINER (toolitem7), buttonUndo);
  gtk_tooltips_set_tip (tooltips, buttonUndo, gettext("Undo"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonUndo), GTK_RELIEF_NONE); 
  image14 = gtk_image_new_from_stock ("gtk-undo", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image14);
  gtk_container_add (GTK_CONTAINER (buttonUndo), image14);
  gtk_widget_set_sensitive(buttonUndo,FALSE);

  toolitem8 = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (toolitem8);
  gtk_container_add (GTK_CONTAINER (toolbar1), toolitem8);
/* redo */
  buttonRedo = gtk_button_new ();
  gtk_widget_show (buttonRedo);
  gtk_container_add (GTK_CONTAINER (toolitem8), buttonRedo);
  gtk_tooltips_set_tip (tooltips, buttonRedo,gettext("Redo"), NULL);
  gtk_button_set_relief (GTK_BUTTON (buttonRedo), GTK_RELIEF_NONE);
  image1379 = gtk_image_new_from_stock ("gtk-redo", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1379);
  gtk_container_add (GTK_CONTAINER (buttonRedo), image1379);
  gtk_widget_set_sensitive(buttonRedo,FALSE);

  separatortoolitem4 = (GtkWidget*) gtk_separator_tool_item_new ();
  gtk_widget_show (separatortoolitem4);
  gtk_container_add (GTK_CONTAINER (toolbar1), separatortoolitem4);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
  
  undo_store_variables(annuler1,buttonUndo );
/* left (editor) toolbar */
  HandleBoxEditor = gtk_handle_box_new ();
  gtk_handle_box_set_handle_position  (GTK_HANDLE_BOX(HandleBoxEditor),GTK_POS_TOP);
  gtk_widget_show (HandleBoxEditor);
  gtk_box_pack_start (GTK_BOX (hbox1), HandleBoxEditor, FALSE, TRUE, 0);
  
  pToolbarEditor = toolbar_init (ScrolledWindowEditor); 
  gtk_widget_show (pToolbarEditor);
  gtk_container_set_border_width (GTK_CONTAINER (pToolbarEditor), 4);
  //gtk_box_pack_start (GTK_BOX (hbox1), pToolbarEditor, FALSE, FALSE, 0);
  gtk_container_add (GTK_CONTAINER (HandleBoxEditor), pToolbarEditor);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (hbox1), notebook1, TRUE, TRUE, 0);

  vbox16 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox16);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox16);

  ScrolledWindowEditor = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (ScrolledWindowEditor);
  gtk_box_pack_start (GTK_BOX (vbox16), ScrolledWindowEditor, TRUE, TRUE, 0);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ScrolledWindowEditor), GTK_SHADOW_IN);
 /* items of the editor's statusbar */
  GtkWidget *hbox35EditorBas;
  hbox35EditorBas = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox35EditorBas);
  gtk_box_pack_start (GTK_BOX (vbox16), hbox35EditorBas, FALSE, FALSE, 0);
  GtkWidget *imageAimant;
  ico = gdk_pixbuf_new_from_xpm_data((const char **)aimant_xpm); 
  imageAimant = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageAimant);  
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), imageAimant, FALSE, FALSE, 0);
  GtkWidget *checkbuttonEditorAttractionMagnetic;
  checkbuttonEditorAttractionMagnetic = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonEditorAttractionMagnetic);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), checkbuttonEditorAttractionMagnetic, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonEditorAttractionMagnetic, 
     gettext("Click here in order to activate the 'autoplacing' of objets on editor's drawing area"), NULL);
  GtkWidget *vseparatorEditorBasavantRotation;
  vseparatorEditorBasavantRotation = gtk_vseparator_new ();
  gtk_widget_show (vseparatorEditorBasavantRotation);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), vseparatorEditorBasavantRotation, FALSE, FALSE, 0);

  labelEditorRotation = gtk_label_new ("------");
  gtk_widget_show (labelEditorRotation);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), labelEditorRotation, FALSE, FALSE, 0);  
  GtkWidget *vseparator3HboxEditorBas;
  vseparator3HboxEditorBas = gtk_vseparator_new ();
  gtk_widget_show (vseparator3HboxEditorBas);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), vseparator3HboxEditorBas, FALSE, FALSE, 0);
  label51EditorCouleurPP = gtk_label_new (gettext("<small>FC:</small>"));
  gtk_widget_show (label51EditorCouleurPP);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), label51EditorCouleurPP, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label51EditorCouleurPP), TRUE);

  drawingarea1EditorCouleurPP = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea1EditorCouleurPP);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), drawingarea1EditorCouleurPP, FALSE, TRUE, 0);
  gtk_widget_set_size_request (drawingarea1EditorCouleurPP, 12, 12);

  label52CouleurAP = gtk_label_new (gettext("<small>OC:</small>"));
  gtk_widget_show (label52CouleurAP);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), label52CouleurAP, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label52CouleurAP), TRUE);

  drawingarea2CouleurAP = gtk_drawing_area_new ();
  gtk_widget_show (drawingarea2CouleurAP);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), drawingarea2CouleurAP, FALSE, TRUE, 0);
  gtk_widget_set_size_request (drawingarea2CouleurAP, 12, 12);

  editor_store_preview(drawingarea1EditorCouleurPP, drawingarea2CouleurAP);
 
  vseparator4Editor = gtk_vseparator_new ();
  gtk_widget_show (vseparator4Editor);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), vseparator4Editor, FALSE, TRUE, 0);
  gtk_widget_set_size_request (vseparator4Editor, 5, -1); 
  labelEditorMousePosition = gtk_label_new (gettext("x:---/y:---"));
  gtk_widget_show (labelEditorMousePosition);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), labelEditorMousePosition, FALSE, FALSE, 0);

  vseparator3bEditor = gtk_vseparator_new ();
  gtk_widget_show (vseparator3bEditor);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), vseparator3bEditor, FALSE, TRUE, 0);
  gtk_widget_set_size_request (vseparator3bEditor, 5, -1);

  labelEditorHelp = gtk_label_new ("------------------");
  gtk_widget_show (labelEditorHelp);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), labelEditorHelp, FALSE, FALSE, 0);
  
  
  statusbarEditor = gtk_statusbar_new ();
  gtk_widget_show (statusbarEditor);
  gtk_box_pack_start (GTK_BOX (hbox35EditorBas), statusbarEditor, TRUE, TRUE, 0);
  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbarEditor), FALSE);

  label11 = gtk_label_new (gettext("Editor"));
  gtk_widget_show (label11);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label11);

/* lecteur */
  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_container_add (GTK_CONTAINER (notebook1), frame1);
  //gtk_widget_set_size_request (frame1, 640, 340);
  gtk_frame_set_shadow_type (GTK_FRAME (frame1), GTK_SHADOW_IN);

  alignment3 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment3);
  gtk_container_add (GTK_CONTAINER (frame1), alignment3);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment3), 0, 0, 12, 0);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (alignment3), vbox9);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox9), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

  viewportPlayer = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewportPlayer);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewportPlayer);

 hseparator2 = gtk_hseparator_new ();
 gtk_widget_show (hseparator2);
 gtk_box_pack_start (GTK_BOX (vbox9), hseparator2, FALSE, FALSE, 0);
 gtk_widget_set_size_request (hseparator2, 5, 5);

  hbox18 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox18);
  gtk_box_pack_start (GTK_BOX (vbox9), hbox18, FALSE, FALSE, 0);


  buttonPlayerPlay = gtk_button_new_from_stock ("gtk-media-play");
  gtk_widget_show (buttonPlayerPlay);
  gtk_box_pack_start (GTK_BOX (hbox18), buttonPlayerPlay, FALSE, FALSE, 0);
  buttonPlayerPause = gtk_button_new_from_stock ("gtk-media-pause");
  gtk_widget_show (buttonPlayerPause);
  gtk_box_pack_start (GTK_BOX (hbox18), buttonPlayerPause, FALSE, FALSE, 0);  
  buttonPlayerStop = gtk_button_new_from_stock ("gtk-media-stop");
  gtk_widget_show (buttonPlayerStop);
  gtk_box_pack_start (GTK_BOX (hbox18), buttonPlayerStop, FALSE, FALSE, 0);
  vseparator3Editor = gtk_vseparator_new ();
  gtk_widget_show (vseparator3Editor);
  gtk_box_pack_start (GTK_BOX (hbox18), vseparator3Editor, FALSE, TRUE, 0);
  gtk_widget_set_size_request (vseparator3Editor, 74, -1);
  labelEditorPosition = gtk_label_new (gettext("<b>Position :</b>"));
  gtk_widget_show (labelEditorPosition);
  gtk_box_pack_start (GTK_BOX (hbox18), labelEditorPosition, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelEditorPosition), TRUE);

  //labelEditorTimePosition = gtk_label_new ("<big><b><i>0,00secs</i></b></big>");
  labelEditorTimePosition = gtk_label_new(NULL);
  gtk_label_set_markup (GTK_LABEL(labelEditorTimePosition),
	 "<span font_desc=\"Digit 12\" foreground=\"#00FF00\" background=\"#000000\"><big><b>0,00secs</b></big></span>");
  gtk_widget_show (labelEditorTimePosition);
  gtk_box_pack_start (GTK_BOX (hbox18), labelEditorTimePosition, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelEditorTimePosition), TRUE);
  label12 = gtk_label_new (gettext("Player"));
  gtk_widget_show (label12);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label12);

/* notebook 2 */

vseparator1 = gtk_vseparator_new ();
  gtk_widget_show (vseparator1);
  gtk_box_pack_start (GTK_BOX (hbox1), vseparator1, FALSE, FALSE, 0);

  vboxNotebookProperties = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vboxNotebookProperties);
  gtk_box_pack_start (GTK_BOX (hbox1), vboxNotebookProperties, FALSE, FALSE, 0);

  notebook2 = gtk_notebook_new ();
  gtk_widget_show (notebook2);
  gtk_box_pack_start (GTK_BOX (vboxNotebookProperties), notebook2, TRUE, TRUE, 0);
/* propriétés */
  scrolledwindowProperties = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowProperties);
       gtk_widget_set_sensitive (scrolledwindowProperties,FALSE);
  gtk_container_add (GTK_CONTAINER (notebook2), scrolledwindowProperties);
  viewportProperties = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewportProperties);
  gtk_container_add (GTK_CONTAINER (scrolledwindowProperties), viewportProperties);
  vbox18Properties = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox18Properties);
  gtk_container_add (GTK_CONTAINER (viewportProperties), vbox18Properties);
  hseparator7Properties = gtk_hseparator_new ();
  gtk_widget_show (hseparator7Properties);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), hseparator7Properties, FALSE, TRUE, 0);


  label50Properties = gtk_label_new (gettext("<b>Position and size selection\n(pixels)</b>"));
  gtk_widget_show (label50Properties);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), label50Properties, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label50Properties), TRUE);    
  gtk_misc_set_alignment (GTK_MISC (label50Properties), 0, 0.5);

  tablePropertiesPosition = gtk_table_new (3, 4, FALSE);
  gtk_widget_show (tablePropertiesPosition);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), tablePropertiesPosition, FALSE, TRUE, 0);

  label51Properties = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label51Properties),
     gettext("<small>horizontal</small>"));   
  gtk_widget_show (label51Properties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), label51Properties, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entryXPosProperties = gtk_entry_new ();
  gtk_widget_show (entryXPosProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), entryXPosProperties, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_tooltips_set_tip (tooltips, entryXPosProperties, gettext("Enter the horizontal position of the object"), NULL);
  gtk_entry_set_max_length (GTK_ENTRY (entryXPosProperties), 5);
  gtk_entry_set_text (GTK_ENTRY (entryXPosProperties), "");
  gtk_entry_set_invisible_char (GTK_ENTRY (entryXPosProperties), 9679);
  gtk_entry_set_width_chars (GTK_ENTRY (entryXPosProperties), 5);
  

  label52Properties = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label52Properties),
     gettext("<small>vertical</small>"));   
  gtk_widget_show (label52Properties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), label52Properties, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entryYPosProperties = gtk_entry_new ();
  gtk_widget_show (entryYPosProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), entryYPosProperties, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_tooltips_set_tip (tooltips, entryYPosProperties, gettext("Enter the vertical position of the object"), NULL);
  gtk_entry_set_max_length (GTK_ENTRY (entryYPosProperties), 5);
  gtk_entry_set_text (GTK_ENTRY (entryYPosProperties), "");
  gtk_entry_set_invisible_char (GTK_ENTRY (entryYPosProperties), 9679);
  gtk_entry_set_width_chars (GTK_ENTRY (entryYPosProperties), 5);

  label54Properties = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label54Properties),
     gettext("<small>Width</small>")); 
  gtk_widget_show (label54Properties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), label54Properties, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entryWidthProperties = gtk_entry_new ();
  gtk_widget_show (entryWidthProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), entryWidthProperties, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_tooltips_set_tip (tooltips, entryWidthProperties, gettext("Width of the object (pixels)"), NULL);
  gtk_entry_set_max_length (GTK_ENTRY (entryWidthProperties), 5);
  gtk_entry_set_text (GTK_ENTRY (entryWidthProperties), "");
  gtk_entry_set_invisible_char (GTK_ENTRY (entryWidthProperties), 9679);
  gtk_entry_set_width_chars (GTK_ENTRY (entryWidthProperties), 5);


  label55Properties = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (label55Properties),
     gettext("<small>Height</small>"));
  gtk_widget_show (label55Properties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), label55Properties, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0); 
  entryHeightProperties = gtk_entry_new ();
  gtk_widget_show (entryHeightProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), entryHeightProperties, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_tooltips_set_tip (tooltips, entryHeightProperties, gettext("Height of the object (pixels)"), NULL);
  gtk_entry_set_max_length (GTK_ENTRY (entryHeightProperties), 5);
  gtk_entry_set_text (GTK_ENTRY (entryHeightProperties), "");
  gtk_entry_set_invisible_char (GTK_ENTRY (entryHeightProperties), 9679);
  gtk_entry_set_width_chars (GTK_ENTRY (entryHeightProperties), 5);

  buttonRazDimProperties = gtk_button_new ();
  gtk_widget_show (buttonRazDimProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), buttonRazDimProperties, 3, 4, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request (buttonRazDimProperties, 28, 28);
  gtk_tooltips_set_tip (tooltips, buttonRazDimProperties, gettext("Reset the postion and the size of the object"), NULL);

  ico = gdk_pixbuf_new_from_xpm_data((const char **)icone_reload_xpm); 
  imageRedoProperties = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageRedoProperties);
  gtk_container_add (GTK_CONTAINER (buttonRazDimProperties), imageRedoProperties);

  checkbuttonConstrainProperties = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonConstrainProperties);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), checkbuttonConstrainProperties, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbuttonConstrainProperties),TRUE);					
  gtk_tooltips_set_tip (tooltips, checkbuttonConstrainProperties, gettext("Keep ratio"), NULL);

  ico = gdk_pixbuf_new_from_xpm_data((const char **)icone_cadenas_ferme_xpm);
  imagePropertiesLock = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imagePropertiesLock);
  gtk_table_attach (GTK_TABLE (tablePropertiesPosition), imagePropertiesLock, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  hseparator5Properties = gtk_hseparator_new ();
  gtk_widget_show (hseparator5Properties);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), hseparator5Properties, FALSE, TRUE, 0);

 labelTitreCouleursProperties = gtk_label_new (gettext("<b>Selection style :</b>"));
  gtk_widget_show (labelTitreCouleursProperties);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), labelTitreCouleursProperties, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelTitreCouleursProperties), TRUE);
  gtk_misc_set_alignment (GTK_MISC (labelTitreCouleursProperties), 0, 0.5);

  tablePropertiesColors = gtk_table_new (8, 3, FALSE);
  gtk_widget_show (tablePropertiesColors);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), tablePropertiesColors, FALSE, FALSE, 0);

  labelPropertiesFColor = gtk_label_new (gettext("<small>Filing color :</small>"));
  gtk_widget_show (labelPropertiesFColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), labelPropertiesFColor, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelPropertiesFColor), TRUE);
  gtk_misc_set_alignment (GTK_MISC (labelPropertiesFColor), 0, 0.5);

  framePropertiesFColor = gtk_frame_new (NULL);
  gtk_widget_show (framePropertiesFColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), framePropertiesFColor, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (framePropertiesFColor), GTK_SHADOW_IN);

  alignment7 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment7);
  gtk_container_add (GTK_CONTAINER (framePropertiesFColor), alignment7);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment7), 0, 0, 12, 0);

  drawingareaPropertiesFColor = gtk_drawing_area_new ();
  gtk_container_add (GTK_CONTAINER (alignment7), drawingareaPropertiesFColor);
  gtk_widget_set_size_request (drawingareaPropertiesFColor, 24, 24);
  gtk_tooltips_set_tip (tooltips, drawingareaPropertiesFColor, gettext("Click on this area to select a color"), NULL);

  hboxPropertiesRFcolor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesRFcolor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesRFcolor, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  labelPropertiesRFColor = gtk_label_new ("R:");
  gtk_widget_show (labelPropertiesRFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesRFcolor), labelPropertiesRFColor, FALSE, FALSE, 0);

  spinbuttonPropertiesRFColor_adj = gtk_adjustment_new (255, 0, 255, 1, 1, 0);
  spinbuttonPropertiesRFColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesRFColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesRFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesRFcolor), spinbuttonPropertiesRFColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesRFColor, gettext("Red value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesRFColor), TRUE);

  hboxPropertiesVFColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesVFColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesVFColor, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesVFColor = gtk_label_new ("V:");
  gtk_widget_show (labelPropertiesVFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVFColor), labelPropertiesVFColor, FALSE, FALSE, 0);

  spinbuttonPropertiesVFColor_adj = gtk_adjustment_new (255, 0, 255, 1, 1, 0);
  spinbuttonPropertiesVFColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesVFColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesVFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVFColor), spinbuttonPropertiesVFColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesVFColor, gettext("Green value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesVFColor), TRUE);

  hboxPropertiesBFColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesBFColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesBFColor, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesVFColor = gtk_label_new ("B:");
  gtk_widget_show (labelPropertiesVFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesBFColor), labelPropertiesVFColor, FALSE, FALSE, 0);

  spinbuttonPropertiesBFColor_adj = gtk_adjustment_new (255, 0, 255, 1, 1, 0);
  spinbuttonPropertiesBFColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesBFColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesBFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesBFColor), spinbuttonPropertiesBFColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesBFColor, gettext("Blue value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesBFColor), TRUE);

  hboxPropertiesAFColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesAFColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesAFColor, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesBFColor = gtk_label_new ("A:");
  gtk_widget_show (labelPropertiesBFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesAFColor), labelPropertiesBFColor, FALSE, FALSE, 0);

  spinbuttonPropertiesAFColor_adj = gtk_adjustment_new (100, 0, 100, 1, 1, 0);
  spinbuttonPropertiesAFColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesAFColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesAFColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesAFColor), spinbuttonPropertiesAFColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesAFColor, 
   gettext("A value near 0 means 'transparent', a value near 100 means 'opaque'"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesAFColor), TRUE);

  label51PropertiesSColor = gtk_label_new (gettext("<small>Stroke/line color :</small>"));
  gtk_widget_show (label51PropertiesSColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), label51PropertiesSColor, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_use_markup (GTK_LABEL (label51PropertiesSColor), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label51PropertiesSColor), 0, 0.5);

  framePropertiesSColor = gtk_frame_new (NULL);
  gtk_widget_show (framePropertiesSColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), framePropertiesSColor, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_frame_set_label_align (GTK_FRAME (framePropertiesSColor), 0.5, 0.5);
  gtk_frame_set_shadow_type (GTK_FRAME (framePropertiesSColor), GTK_SHADOW_IN);

  alignment8 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment8);
  gtk_container_add (GTK_CONTAINER (framePropertiesSColor), alignment8);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment8), 0, 0, 12, 0);

  drawingareaPropertiesSColor = gtk_drawing_area_new ();
  gtk_container_add (GTK_CONTAINER (alignment8), drawingareaPropertiesSColor);
  gtk_widget_set_size_request (drawingareaPropertiesSColor, 24, 24);
  gtk_tooltips_set_tip (tooltips, drawingareaPropertiesSColor,
   gettext( "Click here to invoke the color picker"), NULL);
  vbox20Properties = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox20Properties);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), vbox20Properties, FALSE, FALSE, 0);
  
  hboxPropertiesRSColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesRSColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesRSColor, 2, 3, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  labelPropertiesRSColor = gtk_label_new ("R:");
  gtk_widget_show (labelPropertiesRSColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesRSColor), labelPropertiesRSColor, FALSE, FALSE, 0);

  spinbuttonPropertiesRSColor_adj = gtk_adjustment_new (0, 0, 255, 1, 1, 0);
  spinbuttonPropertiesRSColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesRSColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesRSColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesRSColor), spinbuttonPropertiesRSColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesRSColor, gettext("Red value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesRSColor), TRUE);

  hboxPropertiesVSColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesVSColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesVSColor, 2, 3, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesVSColor = gtk_label_new ("V:");
  gtk_widget_show (labelPropertiesVSColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVSColor), labelPropertiesVSColor, FALSE, FALSE, 0);

  spinbuttonPropertiesVSColor_adj = gtk_adjustment_new (0, 0, 255, 1, 1, 0);
  spinbuttonPropertiesVSColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesVSColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesVSColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVSColor), spinbuttonPropertiesVSColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesVSColor, gettext("Green value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesVSColor), TRUE);

  hboxPropertiesVSColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesVSColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesVSColor, 2, 3, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesBScolor = gtk_label_new ("B:");
  gtk_widget_show (labelPropertiesBScolor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVSColor), labelPropertiesBScolor, FALSE, FALSE, 0);

  spinbuttonPropertiesBSColor_adj = gtk_adjustment_new (0, 0, 255, 1, 1, 0);
  spinbuttonPropertiesBSColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesBSColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesBSColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesVSColor), spinbuttonPropertiesBSColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesBSColor, gettext("Blue value (0 to 255)"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesBSColor), TRUE);

  hboxPropertiesASColor = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxPropertiesASColor);
  gtk_table_attach (GTK_TABLE (tablePropertiesColors), hboxPropertiesASColor, 2, 3, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  labelPropertiesASColor = gtk_label_new ("A:");
  gtk_widget_show (labelPropertiesASColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesASColor), labelPropertiesASColor, FALSE, FALSE, 0);

  spinbuttonPropertiesASColor_adj = gtk_adjustment_new (100, 0, 100, 1, 1, 0);
  spinbuttonPropertiesASColor = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesASColor_adj), 1, 0);
  gtk_widget_show (spinbuttonPropertiesASColor);
  gtk_box_pack_start (GTK_BOX (hboxPropertiesASColor), spinbuttonPropertiesASColor, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesASColor,
   gettext( "A value near 0 means 'transparent', a value near 100 means 'opaque'"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesASColor), TRUE);

  hbox35PropertiesLines = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox35PropertiesLines);
  gtk_box_pack_start (GTK_BOX (vbox18Properties), hbox35PropertiesLines, FALSE, FALSE, 0);

  label50PropertiesLinesWidth = gtk_label_new (gettext("<small>Stroke width :</small>"));
  gtk_widget_show (label50PropertiesLinesWidth);
  gtk_box_pack_start (GTK_BOX (hbox35PropertiesLines), label50PropertiesLinesWidth, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label50PropertiesLinesWidth), TRUE);

  spinbuttonPropertiesLineWidth_adj = gtk_adjustment_new (1, 0, 32, 1, 1, 0);
  spinbuttonPropertiesLineWidth = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonPropertiesLineWidth_adj), 1, 2);
  gtk_widget_show (spinbuttonPropertiesLineWidth);
  gtk_box_pack_start (GTK_BOX (hbox35PropertiesLines), spinbuttonPropertiesLineWidth, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, spinbuttonPropertiesLineWidth, gettext("Stroke or line width"), NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonPropertiesLineWidth), TRUE);

  buttonPropertiesUndoStyleChange = gtk_button_new ();
  gtk_widget_show (buttonPropertiesUndoStyleChange);
  gtk_box_pack_start (GTK_BOX (hbox35PropertiesLines), buttonPropertiesUndoStyleChange, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonPropertiesUndoStyleChange, gettext("Reset selection style"), NULL);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)icone_reload_xpm); 
  imageUndoStyles = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageUndoStyles);
  gtk_container_add (GTK_CONTAINER (buttonPropertiesUndoStyleChange), imageUndoStyles);  
 
  hseparator7PropertiesStyle = gtk_hseparator_new ();
  gtk_widget_show (hseparator7PropertiesStyle);
  gtk_box_pack_start (GTK_BOX (vbox20Properties), hseparator7PropertiesStyle, FALSE, FALSE, 0);

  /* fin 'styles' */

  labelOngletProperties = gtk_label_new (gettext("Properties"));
  gtk_widget_show (labelOngletProperties);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 0), labelOngletProperties);

/* onglet transitions */

  scrolledwindowTransitions = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowTransitions);
  gtk_container_add (GTK_CONTAINER (notebook2), scrolledwindowTransitions);
  
  viewport3 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport3);
  gtk_container_add (GTK_CONTAINER (scrolledwindowTransitions), viewport3);

  vboxTransitions = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vboxTransitions);
  gtk_container_add (GTK_CONTAINER (viewport3), vboxTransitions);

  labelInterpolation = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (labelInterpolation),
     gettext("<big>Interpolation\nmovement :</big>"));
  gtk_widget_show (labelInterpolation);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), labelInterpolation, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelInterpolation), 0, 0.5);

  hboxInterpola = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxInterpola);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hboxInterpola, FALSE, FALSE, 0);

  ico = gdk_pixbuf_new_from_xpm_data((const char **)interpola_xpm); 
  imageTransitionInterpola = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imageTransitionInterpola);
  gtk_box_pack_start (GTK_BOX (hboxInterpola), imageTransitionInterpola, FALSE, FALSE, 0);
  
  labelActiverInterpola = gtk_label_new (gettext("Enabled :"));
  gtk_widget_show (labelActiverInterpola);
  gtk_box_pack_start (GTK_BOX (hboxInterpola), labelActiverInterpola, FALSE, FALSE, 0);

  checkbuttonInterpola = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonInterpola);
  gtk_box_pack_start (GTK_BOX (hboxInterpola), checkbuttonInterpola, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonInterpola, gettext("Check this case if you want a weet objects displace between the late key image and the current key image"), NULL);

  hseparator7t = gtk_hseparator_new ();
  gtk_widget_show (hseparator7t);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hseparator7t, FALSE, FALSE, 0);

  labelZoom = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (labelZoom),gettext("<big>Zoom :</big>"));    
  gtk_widget_show (labelZoom);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), labelZoom, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelZoom), 0, 0.5);

  hboxZoom = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxZoom);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hboxZoom, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)zoom_xpm); 
  imageTransitionZooming = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imageTransitionZooming); 
  gtk_box_pack_start (GTK_BOX (hboxZoom), imageTransitionZooming,FALSE, FALSE, 0); 

  labelActiverZooming = gtk_label_new (gettext("Enabled :"));
  gtk_widget_show (labelActiverZooming);
  gtk_box_pack_start (GTK_BOX (hboxZoom), labelActiverZooming, FALSE, FALSE, 0);

  checkbuttonZoom = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonZoom);
  gtk_box_pack_start (GTK_BOX (hboxZoom), checkbuttonZoom, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonZoom, gettext("Check this case if you want a zomm effect between the late key image and the current key image"), NULL);

  hseparator8t = gtk_hseparator_new ();
  gtk_widget_show (hseparator8t);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hseparator8t, FALSE, FALSE, 0);

  labelRotation = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (labelRotation),gettext("<big>Rotate :</big>")); 
  gtk_widget_show (labelRotation);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), labelRotation, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelRotation), 0, 0.5);

  hboxRotation = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxRotation);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hboxRotation, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)rota_xpm); 
  imageTransitionRotation = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imageTransitionRotation); 
  gtk_box_pack_start (GTK_BOX (hboxRotation), imageTransitionRotation, FALSE, FALSE, 0); 

  labelActiverRotation = gtk_label_new (gettext("Enabled :"));
  gtk_widget_show (labelActiverRotation);
  gtk_box_pack_start (GTK_BOX (hboxRotation), labelActiverRotation, FALSE, FALSE, 0);

  checkbuttonRotation = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonRotation);
  gtk_box_pack_start (GTK_BOX (hboxRotation), checkbuttonRotation, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonRotation, "Check this case to have an object rotate the late key image and the current key image (0-360). Counterclockwise", NULL);



  hseparator10t = gtk_hseparator_new ();
  gtk_widget_show (hseparator10t);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hseparator10t, FALSE, FALSE, 0);

  labelAlpha = gtk_label_new (NULL);
  gtk_label_set_markup (GTK_LABEL (labelAlpha),gettext("<big>Alpha :</big>"));
  gtk_widget_show (labelAlpha);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), labelAlpha, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelAlpha), 0, 0.5);

  hbox35Alpha = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox35Alpha);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hbox35Alpha, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)alpha_xpm); 
  imageTransitionAlpha = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imageTransitionAlpha); 
  gtk_box_pack_start (GTK_BOX (hbox35Alpha), imageTransitionAlpha, FALSE, FALSE, 0);  
  label7ActiverAlpha = gtk_label_new (gettext("Enabled :"));
  gtk_widget_show (label7ActiverAlpha);
  gtk_box_pack_start (GTK_BOX (hbox35Alpha), label7ActiverAlpha, FALSE, FALSE, 0);

  checkbuttonAlphaT = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonAlphaT);
  gtk_box_pack_start (GTK_BOX (hbox35Alpha), checkbuttonAlphaT, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonAlphaT, gettext("Check this case to have an alpha effect during the objects animation"), NULL);

 /* morphing */
  hseparator7t_morphing = gtk_hseparator_new ();
  gtk_widget_show (hseparator7t_morphing);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hseparator7t_morphing, FALSE, FALSE, 0);
  labelMorphing = gtk_label_new (gettext("<big>Morphing :</big>"));
  gtk_widget_show (labelMorphing);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), labelMorphing, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelMorphing), TRUE);
  gtk_misc_set_alignment (GTK_MISC (labelMorphing), 0, 0.5);
  hbox35Morphing = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox35Morphing);
  gtk_box_pack_start (GTK_BOX (vboxTransitions), hbox35Morphing, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)morphing_xpm); 
  imageTransitionMorphing = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);
  gtk_widget_show (imageTransitionMorphing);
  gtk_box_pack_start (GTK_BOX (hbox35Morphing), imageTransitionMorphing, FALSE, FALSE, 0);
  labelActivateMorphing = gtk_label_new (gettext("Activate :"));
  gtk_widget_show (labelActivateMorphing);
  gtk_box_pack_start (GTK_BOX (hbox35Morphing), labelActivateMorphing, FALSE, FALSE, 0);
  checkbuttonMorphing = gtk_check_button_new_with_mnemonic ("");
  gtk_widget_show (checkbuttonMorphing);
  gtk_box_pack_start (GTK_BOX (hbox35Morphing), checkbuttonMorphing, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, checkbuttonMorphing,
      gettext("Click here in order to add a morphing effect between two symbols and tow keyframes"), NULL);

  labelTransitions = gtk_label_new (gettext("Transitions"));
  gtk_widget_show (labelTransitions);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 1), labelTransitions);

  scrolledwindowEffects = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowEffects);
  gtk_container_add (GTK_CONTAINER (notebook2), scrolledwindowEffects);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowEffects), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport2 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport2);
  gtk_container_add (GTK_CONTAINER (scrolledwindowEffects), viewport2);

  vbox19 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox19);
  gtk_container_add (GTK_CONTAINER (viewport2), vbox19);

/* biblio */
  scrolledwindowBiblio = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindowBiblio);
  gtk_box_pack_start (GTK_BOX (vbox19), scrolledwindowBiblio, TRUE, TRUE, 0);

  hseparator6 = gtk_hseparator_new ();
  gtk_widget_show (hseparator6);
  gtk_box_pack_start (GTK_BOX (vbox19), hseparator6, FALSE, FALSE, 0);

  hbox32 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox32);
  gtk_box_pack_start (GTK_BOX (vbox19), hbox32, FALSE, FALSE, 0);

  buttonBiblioArchive = gtk_button_new ();
  gtk_widget_show (buttonBiblioArchive);
  gtk_box_pack_start (GTK_BOX (hbox32), buttonBiblioArchive, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonBiblioArchive, 
         gettext("Archive the library content"),
		  NULL);

  alignment5 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment5);
  gtk_container_add (GTK_CONTAINER (buttonBiblioArchive), alignment5);

  hbox33 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox33);
  gtk_container_add (GTK_CONTAINER (alignment5), hbox33);

  image1610 = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1610);
  gtk_box_pack_start (GTK_BOX (hbox33), image1610, FALSE, FALSE, 0);

  buttonBiblioOpenArchive = gtk_button_new ();
  gtk_widget_show (buttonBiblioOpenArchive);
  gtk_box_pack_start (GTK_BOX (hbox32), buttonBiblioOpenArchive, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonBiblioOpenArchive,
		gettext("Load an archive of the library"),
		 NULL);

  alignment6 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment6);
  gtk_container_add (GTK_CONTAINER (buttonBiblioOpenArchive), alignment6);

  hbox34 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox34);
  gtk_container_add (GTK_CONTAINER (alignment6), hbox34);

  image1611 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1611);
  gtk_box_pack_start (GTK_BOX (hbox34), image1611, FALSE, FALSE, 0);

  vseparator2 = gtk_vseparator_new ();
  gtk_widget_show (vseparator2);
  gtk_box_pack_start (GTK_BOX (hbox32), vseparator2, TRUE, TRUE, 0);
  gtk_widget_set_size_request (vseparator2, 12, -1);

  buttonBiblioRemove = gtk_button_new ();
  gtk_widget_show (buttonBiblioRemove);
  gtk_box_pack_start (GTK_BOX (hbox32), buttonBiblioRemove, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonBiblioRemove,
	gettext("Remove an item from the library"),
	 NULL);

  image1612 = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1612);
  gtk_container_add (GTK_CONTAINER (buttonBiblioRemove), image1612);

  label49 = gtk_label_new (gettext("Library"));
  gtk_widget_show (label49);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 2), label49);
  
/* fin notebook 2 */

  frameTimeline = gtk_frame_new (NULL);
  gtk_widget_show (frameTimeline);
  gtk_box_pack_start (GTK_BOX (vbox1), frameTimeline, FALSE, FALSE, 0);
  //gtk_widget_set_size_request (frameTimeline, -1, 200);
  gtk_frame_set_shadow_type (GTK_FRAME (frameTimeline), GTK_SHADOW_OUT);

  label37 = gtk_label_new (gettext("Pasteboard :"));
  gtk_widget_show (label37);
  gtk_frame_set_label_widget (GTK_FRAME (frameTimeline), label37);
  gtk_label_set_use_markup (GTK_LABEL (label37), TRUE);
 
  statusbarEllaMain = gtk_statusbar_new ();
  gtk_widget_show (statusbarEllaMain);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbarEllaMain, FALSE, TRUE, 0);
  /* ajouter éléments interface timeline */

/* partie à transposer */
  tableTimeline = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (tableTimeline);
  //gtk_widget_set_size_request(tableTimeline,-1,240);
  gtk_container_add (GTK_CONTAINER (frameTimeline), tableTimeline);
  ScrolledWindowTimeLine = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (ScrolledWindowTimeLine);
  gtk_table_attach (GTK_TABLE (tableTimeline), ScrolledWindowTimeLine, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ScrolledWindowTimeLine), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ScrolledWindowTimeLine), GTK_SHADOW_OUT);
  
  ScrolledWindowLayersList = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (ScrolledWindowLayersList);
  gtk_table_attach (GTK_TABLE (tableTimeline), ScrolledWindowLayersList, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (ScrolledWindowLayersList), 1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ScrolledWindowLayersList), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ScrolledWindowLayersList), GTK_SHADOW_OUT);
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_table_attach (GTK_TABLE (tableTimeline), hbox3, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);

  buttonLayerNew = gtk_button_new ();
  gtk_widget_show (buttonLayerNew);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonLayerNew, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)layer_new_32x32_xpm); 
  imageLayerNew = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageLayerNew);
  gtk_container_add (GTK_CONTAINER (buttonLayerNew), imageLayerNew);
  gtk_tooltips_set_tip (tooltips, buttonLayerNew,
     gettext("Create a new layer"), NULL);
  
  buttonLayerDuplicate = gtk_button_new();
  gtk_widget_show (buttonLayerDuplicate);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonLayerDuplicate, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)duplicate_layer_32x32_xpm); 
  imageLayerDuplicate = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageLayerDuplicate);
  gtk_container_add (GTK_CONTAINER (buttonLayerDuplicate), imageLayerDuplicate);
  gtk_tooltips_set_tip (tooltips, buttonLayerDuplicate,
     gettext("Duplicate the selected layer"), NULL);

  buttonLayerUp = gtk_button_new ();
  gtk_widget_show (buttonLayerUp);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonLayerUp, FALSE, FALSE, 0);
  image7 = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image7);
  gtk_container_add (GTK_CONTAINER (buttonLayerUp), image7);
  gtk_tooltips_set_tip (tooltips, buttonLayerUp,
     gettext("Raise the selected layer"), NULL);
	 
  buttonLayerDown = gtk_button_new ();
  gtk_widget_show (buttonLayerDown);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonLayerDown, FALSE, FALSE, 0);
  image8t = gtk_image_new_from_stock ("gtk-go-down", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image8t);
  gtk_container_add (GTK_CONTAINER (buttonLayerDown), image8t);
  gtk_tooltips_set_tip (tooltips, buttonLayerDown,
     gettext("Lower the selected layer"), NULL); 
  
 vseparator4 = gtk_vseparator_new ();
 gtk_widget_show (vseparator4);
 gtk_box_pack_start (GTK_BOX (hbox3), vseparator4, FALSE, FALSE, 0);
 gtk_widget_set_size_request (vseparator4, 5, 5);
  
  buttonLayerRemove = gtk_button_new ();
  gtk_widget_show (buttonLayerRemove);
  gtk_box_pack_start (GTK_BOX (hbox3), buttonLayerRemove, FALSE, FALSE, 0);
  ico = gdk_pixbuf_new_from_xpm_data((const char **)layer_delete_32x32_xpm); 
  imageLayerDelete = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  gtk_widget_show (imageLayerDelete);
  gtk_container_add (GTK_CONTAINER (buttonLayerRemove), imageLayerDelete);
  gtk_tooltips_set_tip (tooltips, buttonLayerRemove,
     gettext("Remove the current layer"), NULL);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_table_attach (GTK_TABLE (tableTimeline), hbox2, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);					
   vseparator5 = gtk_vseparator_new ();
   gtk_widget_show (vseparator5);

  gtk_box_pack_start (GTK_BOX (hbox2), vseparator5, FALSE, FALSE, 0);
  gtk_widget_set_size_request (vseparator5, 5, 5);
  
  /* zoom for timeline */
  
  buttonZoomMoins = gtk_button_new ();
  gtk_widget_show (buttonZoomMoins);
  gtk_box_pack_start (GTK_BOX (hbox2), buttonZoomMoins, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonZoomMoins, gettext("Zoom out"), NULL);
  imageZoomMoins = gtk_image_new_from_stock ("gtk-zoom-out", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (imageZoomMoins);
  gtk_container_add (GTK_CONTAINER (buttonZoomMoins), imageZoomMoins);
  buttonZoomPlus = gtk_button_new ();
  gtk_widget_show (buttonZoomPlus);
  gtk_box_pack_start (GTK_BOX (hbox2), buttonZoomPlus, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, buttonZoomPlus,gettext("Zoom in"), NULL);
  imageZoomPlus = gtk_image_new_from_stock ("gtk-zoom-in", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (imageZoomPlus);
  gtk_container_add (GTK_CONTAINER (buttonZoomPlus), imageZoomPlus);
  
  /* timeline position */
  labelImagePosition = gtk_label_new (gettext("Position :"));
  gtk_widget_show (labelImagePosition);
  gtk_box_pack_start (GTK_BOX (hbox2), labelImagePosition, FALSE, FALSE, 0);
  spinbuttonMoviePosition_adj = gtk_adjustment_new (1, 1, MAX_FRAMES, 1, 1, 0);
  spinbuttonMoviePosition = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonMoviePosition_adj), 1, 0);
  gtk_widget_show (spinbuttonMoviePosition);
  gtk_box_pack_start (GTK_BOX (hbox2), spinbuttonMoviePosition, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonMoviePosition), TRUE);
  vseparator21= gtk_vseparator_new ();
  gtk_widget_show (vseparator21);
  gtk_box_pack_start (GTK_BOX (hbox2), vseparator21, FALSE, FALSE, 0);
  gtk_widget_set_size_request (vseparator21, 5, 5);
  GtkWidget *labelImageCalque = gtk_label_new (gettext("Layer :"));
  gtk_widget_show (labelImageCalque);
  gtk_box_pack_start (GTK_BOX (hbox2), labelImageCalque, FALSE, FALSE, 0);
  labelNumeroCalque = gtk_label_new ("1");
  gtk_widget_show (labelNumeroCalque);
  gtk_box_pack_start (GTK_BOX (hbox2), labelNumeroCalque, FALSE, FALSE, 0);
  vseparator2 = gtk_vseparator_new ();
  gtk_widget_show (vseparator2);
  gtk_box_pack_start (GTK_BOX (hbox2), vseparator2, FALSE, FALSE, 0);
  gtk_widget_set_size_request (vseparator2, 5, 5);
  GtkWidget *labelTimePosition = gtk_label_new (gettext("Time : "));
  gtk_widget_show (labelTimePosition);
  gtk_box_pack_start (GTK_BOX (hbox2), labelTimePosition, FALSE, FALSE, 0);
  GtkWidget *labelTimeLinePosition = gtk_label_new (gettext("00:00 secs"));
  gtk_widget_show (labelTimeLinePosition);
  gtk_box_pack_start (GTK_BOX (hbox2), labelTimeLinePosition, FALSE, FALSE, 0);
  GtkWidget *vseparator3 = gtk_vseparator_new ();
  gtk_widget_show (vseparator3);
  gtk_box_pack_start (GTK_BOX (hbox2), vseparator3, FALSE, FALSE, 0);
  gtk_widget_set_size_request (vseparator3, 5, 5);
  GtkWidget * labelFrequence = gtk_label_new (gettext("Frequency : "));
  gtk_widget_show (labelFrequence);
  gtk_box_pack_start (GTK_BOX (hbox2), labelFrequence, FALSE, FALSE, 0);
  
  GtkObject *spinbuttonMovieFrequency_adj = gtk_adjustment_new (timeline_get_default_movie_freq(), 1, 30, 1, 1, 0);
  spinbuttonMovieFrequency = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonMovieFrequency_adj), 1, 0);
  gtk_widget_show (spinbuttonMovieFrequency);
  gtk_box_pack_start (GTK_BOX (hbox2), spinbuttonMovieFrequency, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbuttonMovieFrequency), TRUE);
  
  GtkWidget *labelImagesSecs = gtk_label_new (gettext("frames/sec"));
  gtk_widget_show (labelImagesSecs);
  gtk_box_pack_start (GTK_BOX (hbox2), labelImagesSecs, FALSE, FALSE, 0);
  
  scrolledWindowRuler = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledWindowRuler);
  gtk_widget_set_size_request(scrolledWindowRuler,-1,32);
  gtk_table_attach (GTK_TABLE (tableTimeline), scrolledWindowRuler, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_widget_set_size_request (scrolledWindowRuler, -1, RULER_HEIGHT+6);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledWindowRuler), GTK_POLICY_NEVER, 
                           GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledWindowRuler), GTK_SHADOW_OUT);
  
  GtkWidget *labelLayers = gtk_label_new (gettext("Layers :"));
  gtk_widget_show (labelLayers);
  gtk_widget_set_size_request(labelLayers,-1,32);
  gtk_table_attach (GTK_TABLE (tableTimeline), labelLayers, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelLayers), 0, 0);

  /*** fin transpo */
 
  /* get in ONE pointer the adresses of the 6 sub menus for recent files */
  pDatasRecentFilesMenu=g_new(gpointer,6);
  pDatasRecentFilesMenu[0]=recent1;
  pDatasRecentFilesMenu[1]=recent2;
  pDatasRecentFilesMenu[2]=recent3;
  pDatasRecentFilesMenu[3]=recent4;
  pDatasRecentFilesMenu[4]=recent5;
  pDatasRecentFilesMenu[5]=recent6;
  cb_init_menus_recent_files(pDatasRecentFilesMenu);
  /* main window geometry */
  gpointer *data_geo=g_new(gpointer, 6);
  data_geo[0]=window_ella;
  data_geo[1]=&x_fen;
  data_geo[2]=&y_fen;
  data_geo[3]=&w_fen;
  data_geo[4]=&h_fen;

  g_signal_connect ((gpointer) window_ella, "delete_event",
                    G_CALLBACK (on_window_ella_delete_event),
                    data_geo);
  g_signal_connect ((gpointer) nouveau1, "activate",
                    G_CALLBACK (on_nouveau1_activate),
                    window_ella);
  g_signal_connect ((gpointer) ouvrir1, "activate",
                    G_CALLBACK (on_ouvrir1_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent1, "activate",
                    G_CALLBACK (on_recent1_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent2, "activate",
                    G_CALLBACK (on_recent2_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent3, "activate",
                    G_CALLBACK (on_recent3_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent4, "activate",
                    G_CALLBACK (on_recent4_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent5, "activate",
                    G_CALLBACK (on_recent5_activate),
                    window_ella);
  g_signal_connect ((gpointer) recent6, "activate",
                    G_CALLBACK (on_recent6_activate),
                    window_ella);					
  g_signal_connect ((gpointer) enregistrer1, "activate",
                    G_CALLBACK (on_enregistrer1_activate),
                    window_ella);
  g_signal_connect ((gpointer) enregistrer_sous1, "activate",
                    G_CALLBACK (on_enregistrer_sous1_activate),
                    window_ella);
  g_signal_connect ((gpointer) importer1, "activate",
                    G_CALLBACK (on_importer1_activate),
                    NULL);
  g_signal_connect ((gpointer) une_image_bitmap1, "activate",
                    G_CALLBACK (on_une_image_bitmap1_activate),
                    NULL);
  g_signal_connect ((gpointer) une_image_vectorielle1, "activate",
                    G_CALLBACK (on_une_image_vectorielle1_activate),
                    NULL);
  g_signal_connect ((gpointer) importer_son, "activate",
                    G_CALLBACK (on_importer_son_activate),
                    NULL);	
  g_signal_connect ((gpointer) import_symbol, "activate",
                    G_CALLBACK (on_import_symbol_activate),
                    NULL);									
  g_signal_connect ((gpointer) exporter1, "activate",
                    G_CALLBACK (on_exporter1_activate),
                    NULL);
  g_signal_connect ((gpointer) animation_flash1, "activate",
                    G_CALLBACK (on_animation_flash1_activate),
                    window_ella);
  g_signal_connect ((gpointer) animation_svg1, "activate",
                    G_CALLBACK (on_animation_svg1_activate),
                    window_ella);
  g_signal_connect ((gpointer) animation_javascript1, "activate",
                    G_CALLBACK (on_comme_animation_javascript1_activate),
                    window_ella);					
  g_signal_connect ((gpointer) options1, "activate",
                    G_CALLBACK (on_options1_activate),
                    NULL);
  g_signal_connect ((gpointer) quitter1, "activate",
                    G_CALLBACK (on_quitter1_activate),
                    data_geo);
/* inferior horizontal notification of editor */
 g_signal_connect  ((gpointer) drawingarea1EditorCouleurPP	, "expose_event",
                          G_CALLBACK (editor_react_notification_area_fillcolor_expose_event),
                           NULL);
 g_signal_connect  ((gpointer) drawingarea2CouleurAP	, "expose_event",
                          G_CALLBACK (editor_react_notification_area_strokecolor_expose_event),
                           NULL);
						   
				
  /* pass more than one var */
  
  gpointer *data_edition = g_new (gpointer, 4);
  data_edition[0] = copier1;
  data_edition[1] = coller1;
  data_edition[2]=  couper1;
  data_edition[3]=  annuler1;					
  g_signal_connect ((gpointer) menuitem2, "activate",
                    G_CALLBACK (on_edition_activate),
                    data_edition);					
					
  g_signal_connect ((gpointer) copier1, "activate",
                    G_CALLBACK (on_copier1_activate),
                    NULL);
  g_signal_connect ((gpointer) coller1, "activate",
                    G_CALLBACK (on_coller1_activate),
                    NULL);
  g_signal_connect ((gpointer) couper1, "activate",
                    G_CALLBACK (on_couper1_activate),
                    NULL);
  g_signal_connect ((gpointer) annuler1, "activate",
                    G_CALLBACK (on_annuler1_activate),
                    NULL);
  g_signal_connect ((gpointer) calques1, "activate",
                    G_CALLBACK (on_calques1_activate),
                    NULL);
  g_signal_connect ((gpointer) nouveau_calque1, "activate",
                    G_CALLBACK (on_nouveau_calque1_activate),
                    NULL);
  g_signal_connect ((gpointer) nouveau_calque_sonore1, "activate",
                    G_CALLBACK (on_nouveau_calque_sonore1_activate),
                    NULL);					
  g_signal_connect ((gpointer) dupliquer_calque1, "activate",
                    G_CALLBACK (on_dupliquer_calque1_activate),
                    NULL);
  g_signal_connect ((gpointer) renommer_calque1, "activate",
                    G_CALLBACK (on_renommer_calque1_activate),
                    NULL);
  g_signal_connect ((gpointer) remonter_le_calque_sur_la_pile1, "activate",
                    G_CALLBACK (on_remonter_le_calque_sur_la_pile1_activate),
                    NULL);
  g_signal_connect ((gpointer) descendre_le_calque_sur_la_pile1, "activate",
                    G_CALLBACK (on_descendre_le_calque_sur_la_pile1_activate),
                    NULL);
  g_signal_connect ((gpointer) supprimer_calque1, "activate",
                    G_CALLBACK (on_supprimer_calque1_activate),
                    NULL);
  g_signal_connect ((gpointer) dupliquer1, "activate",
                    G_CALLBACK (on_dupliquer1_activate), 
                    NULL);
/*					
  g_signal_connect ((gpointer) deplacer, "activate",
                    G_CALLBACK (on_deplacer_activate),
                    NULL);*/
  g_signal_connect ((gpointer) monter, "activate",
                    G_CALLBACK (on_monter_activate),
                    NULL);
  g_signal_connect ((gpointer) descendre, "activate",
                    G_CALLBACK (on_descendre_activate),
                    NULL);
//  g_signal_connect ((gpointer) redimensionner1, "activate",
  //                  G_CALLBACK (on_redimensionner1_activate),
    //                NULL);
  g_signal_connect ((gpointer) rotation1, "activate",
                    G_CALLBACK (on_rotation1_activate),
                   NULL);
  g_signal_connect ((gpointer) _90sens_horaire1, "activate",
                    G_CALLBACK (on_90sens_horaire1_activate),
                    NULL);
  g_signal_connect ((gpointer) _90_anti_horaire1, "activate",
                    G_CALLBACK (on_90_anti_horaire1_activate),
                    NULL);		
  g_signal_connect ((gpointer) _180_sens_horaire1, "activate",
                    G_CALLBACK (on_180_sens_horaire1_activate),
                    NULL);
  g_signal_connect ((gpointer) _270_sens_horaire1, "activate",
                    G_CALLBACK (on_270_sens_horaire1_activate),
                    NULL);	
  g_signal_connect ((gpointer) align_items1, "activate",
                    G_CALLBACK (on_align_items1_activate),
                    NULL);											   
  g_signal_connect ((gpointer) tout_selectionner1, "activate",
                    G_CALLBACK (on_tout_selectionner1_activate),
                    NULL);
  g_signal_connect ((gpointer) deselectionner_tout1, "activate",
                    G_CALLBACK (on_deselectionner_tout1_activate),
                    NULL);
//  g_signal_connect ((gpointer) proprietes1, "activate",
  //                  G_CALLBACK (on_proprietes1_activate),
    //                NULL);
  g_signal_connect ((gpointer) supprimer1, "activate",
                    G_CALLBACK (on_supprimer1_activate),
                    NULL);
 /* passer plusieurs vars */
  
  gpointer *data_trames = g_new (gpointer, 10);
  data_trames[0] = inserer_trames;
  data_trames[1] = ajouter_apres_la_position_courante1;
  data_trames[2]=  supprimer2;
  data_trames[3]=  supprimer_apres_la_position_courante1;					
  data_trames[4]= poser_une_image_cle1;
  data_trames[5]= poser_une_image_cle_vide1;
  data_trames[6]= creer_une_interpolation_de_mouvement1;
  data_trames[7]= retirer_une_image_cle1;
  data_trames[8]= mettre_a_blanc_image_cle1;
  data_trames[9]= supprimer_interpolation;
  
  g_signal_connect ((gpointer) trames1, "activate",
                    G_CALLBACK (on_trames1_activate),
                    data_trames);
  g_signal_connect ((gpointer) inserer_trames, "activate",
                    G_CALLBACK (on_inserer_trames_activate),
                    NULL);
  g_signal_connect ((gpointer) ajouter_apres_la_position_courante1, "activate",
                    G_CALLBACK (on_ajouter_apres_la_position_courante1_activate),
                    NULL);
  g_signal_connect ((gpointer) supprimer2, "activate",
                    G_CALLBACK (on_supprimer2_activate),
                    NULL);
  g_signal_connect ((gpointer) supprimer_apres_la_position_courante1, "activate",
                    G_CALLBACK (on_supprimer_apres_la_position_courante1_activate),
                    NULL);
  g_signal_connect ((gpointer) supprimer_les_trames_selectionnees1, "activate",
                    G_CALLBACK (on_supprimer_les_trames_selectionnees1_activate),
                    NULL);
  g_signal_connect ((gpointer) poser_une_image_cle1, "activate",
                    G_CALLBACK (on_poser_une_image_cle1_activate),
                    NULL);
  g_signal_connect ((gpointer) poser_une_image_cle_vide1, "activate",
                    G_CALLBACK (on_poser_une_image_cle_vide1_activate),
                    NULL);
  g_signal_connect ((gpointer) creer_une_interpolation_de_mouvement1, "activate",
                    G_CALLBACK (on_creer_une_interpolation_de_mouvement1_activate),
                    NULL);
  g_signal_connect ((gpointer) retirer_une_image_cle1, "activate",
                    G_CALLBACK (on_retirer_une_image_cle1_activate),
                    NULL);
  g_signal_connect ((gpointer) mettre_a_blanc_image_cle1, "activate",
                    G_CALLBACK (on_mettre_a_blanc_image_cle1_activate),
                    NULL);					
  g_signal_connect ((gpointer) supprimer_interpolation, "activate",
                    G_CALLBACK (on_remove_interpola_activate),
                    NULL);					
					
					
  g_signal_connect ((gpointer) taille1, "activate",
                    G_CALLBACK (on_taille1_activate),
                    NULL);
  g_signal_connect ((gpointer) couleur_de_la_page1, "activate",
                    G_CALLBACK (on_couleur_de_la_page1_activate),
                    NULL);
  g_signal_connect ((gpointer) format_animation1, "activate",
                    G_CALLBACK (on_format_animation1_activate),
                    NULL);
  g_signal_connect ((gpointer) visualisation, "activate",
                    G_CALLBACK (on_visualisation_activate),
                    NULL);
  /* passer deux vars */
  
  gpointer *data = g_new (gpointer, 4);
  data[0] = notebook1;
  data[1] = viewportPlayer;
  data[2] =  labelEditorTimePosition;
  data[3] = buttonPlayerPlay;
  
  g_signal_connect ((gpointer) jouer_animation1, "activate",
                    G_CALLBACK (on_jouer_animation1_activate),
                    data);
  g_signal_connect ((gpointer) editeur_zoom_avant, "activate",
                    G_CALLBACK (on_editeur_zoom_avant_activate),
                    NULL);
  g_signal_connect ((gpointer) editeur_zoom_arriere, "activate",
                    G_CALLBACK (on_editeur_zoom_arriere_activate),
                    NULL);
  g_signal_connect ((gpointer) a_propos1, "activate",
                    G_CALLBACK (on_a_propos1_activate),
                    NULL);
  g_signal_connect ((gpointer) touches_de_fonction1, "activate",
                    G_CALLBACK (on_aide_activate),
                    NULL);
  g_signal_connect ((gpointer) buttonZoomMoins, "clicked",
                    G_CALLBACK (on_editeur_zoom_arriere_activate),
                    NULL);
  g_signal_connect ((gpointer) buttonZoomPlus, "clicked",
                    G_CALLBACK (on_editeur_zoom_avant_activate),
                    NULL);					
/* boutons onglet 'propriétés' */
   /* on connecte à la même fonction contenue dans editor.c */
      g_signal_connect((gpointer) entryXPosProperties, "activate",
                            G_CALLBACK (editor_react_properties_entries),
                            (gint*)FIELD_X);
      g_signal_connect((gpointer) entryYPosProperties, "activate",
                            G_CALLBACK (editor_react_properties_entries),
                            (gint*)FIELD_Y);
      g_signal_connect((gpointer) entryWidthProperties, "activate",
                            G_CALLBACK (editor_react_properties_entries),
                            (gint*)FIELD_W);
      g_signal_connect((gpointer) entryHeightProperties, "activate",
                            G_CALLBACK (editor_react_properties_entries),
                            (gint*)FIELD_H);
      g_signal_connect ((gpointer) buttonRazDimProperties, "clicked",
                    G_CALLBACK (editor_cancel_position_size_change),
                    NULL);		
  /* onglet propriétés : parties 'styles'    */			
      gpointer *data_styles=g_new(gpointer,12);
	  data_styles[0]=spinbuttonPropertiesRFColor;/* rouge remplissage */
	  data_styles[1]=spinbuttonPropertiesVFColor;
	  data_styles[2]=spinbuttonPropertiesBFColor;
	  data_styles[3]=spinbuttonPropertiesAFColor;/* alpha remplissage */
	  data_styles[4]=spinbuttonPropertiesRSColor;/* rouge contour */
	  data_styles[5]=spinbuttonPropertiesVSColor;
	  data_styles[6]=spinbuttonPropertiesBSColor;
	  data_styles[7]=spinbuttonPropertiesASColor;
	  data_styles[8]=drawingareaPropertiesFColor;/* preview remplissage */
	  data_styles[9]=drawingareaPropertiesSColor;/* preview contour */
	  data_styles[10]=spinbuttonPropertiesLineWidth;/* épaisseur ligne */
	  data_styles[11]=buttonPropertiesUndoStyleChange;
	  /* transférer le pointeur vers l'éditeur */
	  editor_store_pointer_to_styles(data_styles);
	  /* brancher les callbacks */
      g_signal_connect  ((gpointer) spinbuttonPropertiesRFColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                           data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesVFColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesBFColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesAFColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesRSColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesVSColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesBSColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);
      g_signal_connect  ((gpointer) spinbuttonPropertiesASColor, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                           data_styles);
      g_signal_connect  ((gpointer) drawingareaPropertiesFColor, "expose_event",
                          G_CALLBACK (editor_react_styles_area_fillcolor_expose_event),
                           data_styles);
     // g_signal_connect  ((gpointer) drawingareaPropertiesFColor, "button_press_event",
       //                     G_CALLBACK (on_drawingareaPropertiesFColor_button_press_event),
         //                   GTK_OBJECT (data_styles));
     g_signal_connect  ((gpointer) drawingareaPropertiesSColor, "expose_event",
                            G_CALLBACK (editor_react_styles_area_strokecolor_expose_event),
                           data_styles);
      //g_signal_connect  ((gpointer) drawingareaPropertiesSColor, "button_press_event",
                       //     G_CALLBACK (on_drawingareaPropertiesSColor_button_press_event),
                         //   GTK_OBJECT (data_styles));
      g_signal_connect  ((gpointer) spinbuttonPropertiesLineWidth, "value_changed",
                            G_CALLBACK (editor_react_properties_styles_entries),
                            data_styles);	  
	  g_signal_connect  ((gpointer) buttonPropertiesUndoStyleChange, "clicked",
                           G_CALLBACK (editor_undo_entries_styles),
                           data_styles);													
/* boutons onglet 'transitions' */
  g_signal_connect ((gpointer) checkbuttonInterpola, "toggled",
                    G_CALLBACK (on_checkbuttonInterpola_toggled),
                    NULL);
  g_signal_connect ((gpointer) checkbuttonAlphaT, "toggled",
                    G_CALLBACK (on_checkbuttonAlpha_toggled),
                    NULL);					
  g_signal_connect ((gpointer) checkbuttonZoom, "toggled",
                    G_CALLBACK (on_checkbuttonZoom_toggled),
                    NULL);
  g_signal_connect ((gpointer) checkbuttonRotation, "toggled",
                    G_CALLBACK (on_checkbuttonRotation_toggled),
                    NULL);					
  g_signal_connect ((gpointer) checkbuttonMorphing, "toggled",
                    G_CALLBACK (on_checkbuttonMorphing_toggled),
                    NULL); 
/* boutons de l'onglet bibliothèque */
      /* >> gérés dans le module 'biblio.c' */ 
  
/* boutons barre supérieure */
  g_signal_connect ((gpointer) buttonNew, "clicked",
                    G_CALLBACK (on_buttonNew_clicked),
                    window_ella);
  g_signal_connect ((gpointer) buttonOpen, "clicked",
                    G_CALLBACK (on_buttonOpen_clicked),
                    window_ella);
  g_signal_connect ((gpointer) buttonSave, "clicked",
                    G_CALLBACK (on_buttonSave_clicked),
                    window_ella );
  g_signal_connect ((gpointer) buttonCopy, "clicked",
                    G_CALLBACK (on_buttonCopy_clicked),
                    NULL);
  g_signal_connect ((gpointer) buttonPaste, "clicked",
                    G_CALLBACK (on_buttonPaste_clicked),
                    NULL);
  g_signal_connect ((gpointer) buttonCut, "clicked",
                    G_CALLBACK (on_buttonCut_clicked),
                    NULL);
  g_signal_connect ((gpointer) buttonUndo, "clicked",
                    G_CALLBACK (on_buttonUndo_clicked),
                    NULL);
  g_signal_connect ((gpointer) checkbuttonEditorAttractionMagnetic, "toggled",
                    G_CALLBACK (on_checkbuttonEditorAttractionMagnetic_toggled),
                    NULL);					
  /* partie lecteur */
  g_signal_connect ((gpointer)  buttonPlayerPlay, "clicked",
                    G_CALLBACK (on_buttonPlayerPlay_clicked),
                    data);
  g_signal_connect ((gpointer)  buttonPlayerPause, "clicked",
                    G_CALLBACK (on_buttonPlayerPause_clicked),
                    data);  					
  g_signal_connect ((gpointer)  buttonPlayerStop, "clicked",
                    G_CALLBACK (on_buttonPlayerStop_clicked),
                    data);
  gtk_window_add_accel_group (GTK_WINDOW (window_ella), accel_group);
  /* création du canevas, et obtention en retour du pointeur sur la "père" des items */
  rootEditorItems = init_canvas_editor(ScrolledWindowEditor,labelEditorRotation,labelEditorMousePosition,labelEditorHelp);/* variable exportée vers editeur.h */
  /* création d'un layer fictif */
  //rootGroup=init_items_layer(rootEditorItems);
  /* activer événements souris */
  gtk_widget_add_events (window_ella, GDK_BUTTON_PRESS_MASK |GDK_POINTER_MOTION_MASK
  							| GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_RELEASE_MASK 
  							| GDK_LEAVE_NOTIFY_MASK );
  /* attention, les callbacks ne peuvent être branchés qu'après la réalisation de la
   *  fenêtre */
  /* signaux liés à l'éditeur vectoriel */
  g_signal_connect (rootEditorItems, "button_press_event",
		             (GtkSignalFunc) cb_editeur_button_press, window_ella);
  g_signal_connect (rootEditorItems, "button_release_event",
		             (GtkSignalFunc) cb_editeur_button_release, window_ella);
  g_signal_connect (rootEditorItems, "motion_notify_event",
		             (GtkSignalFunc) cb_editor_mouse_move, window_ella); 
  g_signal_connect ((gpointer) window_ella, "key_press_event",
                    G_CALLBACK (on_editor_key_press_event ),
                    window_ella);
  g_signal_connect ((gpointer) window_ella, "key_release_event",
                    G_CALLBACK (on_editor_key_press_event ),
                    window_ella);
  /* signaux liés à la timeline */

  canvasTimeLine=init_ihm(ScrolledWindowTimeLine,rootEditorItems,ScrolledWindowEditor); 
  g_signal_connect(  (gpointer)spinbuttonMoviePosition,"value-changed",
                     G_CALLBACK(on_spin_button_position_changed),labelTimeLinePosition);                          
  g_signal_connect(  (gpointer)spinbuttonMovieFrequency,"value-changed",
                     G_CALLBACK(on_spin_button_frequence_changed),labelTimeLinePosition); 
				 
  /* créer le DnD pour la page */
  gtk_drag_dest_set(ScrolledWindowEditor, GTK_DEST_DEFAULT_ALL, targetentries,
                     4,
                      GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);
  /* créer la DnD pour la timeline */
  gtk_drag_dest_set(ScrolledWindowTimeLine, GTK_DEST_DEFAULT_ALL, targetentries,
                     4,
                      GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);					  
  /* init store biblio */
  pBiblio=biblio_init(scrolledwindowBiblio,
                      buttonBiblioRemove,buttonBiblioArchive,buttonBiblioOpenArchive);							  
  /* connection du signal DnD de réception 
   * il faut le signal drag_drop, 
   * */
  g_signal_connect(ScrolledWindowEditor, "drag_drop",
                     G_CALLBACK(view_onDragDataReceived), pListViewBiblio);	
  /* connection signal réception de la timeline */
  g_signal_connect(ScrolledWindowTimeLine, "drag_drop",
                     G_CALLBACK(view_onDragDataReceivedTimeline), pListViewBiblio);						 
  /* initalisation biblio */
 		 
  return window_ella;
}
/*******************************************
 * close splash screen after a timeout
 * request
 * ****************************************/
gboolean  splahScreenOff(void *data)
{ 
	if ( SplahScreenId )
		g_source_remove ( SplahScreenId );
    gtk_widget_destroy(GTK_WIDGET(data));  
    return FALSE;
}
/*****************************************
 * splash screen
 * **************************************/

GtkWidget*
create_window_SplashScreen (void)
{
  GtkWidget *window_SplashScreen;
  GtkWidget *vbox20;
  GtkWidget *image2138;
  GtkWidget *hseparatorSplashScreen;
  GtkWidget *labelSplashLoading;
  GdkPixbuf *ico;


  window_SplashScreen = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_window_set_position (GTK_WINDOW (window_SplashScreen), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (window_SplashScreen), 318, 200);
  gtk_window_set_resizable (GTK_WINDOW (window_SplashScreen), FALSE);
  gtk_window_set_decorated (GTK_WINDOW (window_SplashScreen), FALSE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window_SplashScreen), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (window_SplashScreen), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (window_SplashScreen), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);

  vbox20 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox20);
  gtk_container_add (GTK_CONTAINER (window_SplashScreen), vbox20);

  ico = gdk_pixbuf_new_from_xpm_data((const char **)splash_screen_xpm); 
  image2138 = gtk_image_new_from_pixbuf(ico);
  g_object_unref(ico);  
  
  gtk_widget_show (image2138);
  gtk_box_pack_start (GTK_BOX (vbox20), image2138, TRUE, TRUE, 0);

  hseparatorSplashScreen = gtk_hseparator_new ();
  gtk_widget_show (hseparatorSplashScreen);
  gtk_box_pack_start (GTK_BOX (vbox20), hseparatorSplashScreen, TRUE, TRUE, 0);

  labelSplashLoading = gtk_label_new (gettext("<b><small><i>Loading, please wait ...</i></small></b>"));
  gtk_widget_show (labelSplashLoading);
  gtk_box_pack_start (GTK_BOX (vbox20), labelSplashLoading, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (labelSplashLoading), TRUE); 
  return window_SplashScreen;
}
/**********************************
 * fonction principale
 * *******************************/
int main (int argc, char *argv[])
{
 //translations
  setlocale( LC_ALL, "" );
  bindtextdomain( "ella", LOCALEDIR );
  textdomain( "ella" );
  GtkWidget *window_ella;
  GdkPixbuf *icopb;
  gint erreur=0;
  gchar path_local[512],tmp_file[512];
  gchar *path_test;
  gboolean existe;
  FILE *f;
  gint width,height,marges,fps,footage;
  gint rouge, vert,bleu,f_archive,t_zoom;
  gint x_corner,y_corner,w_win,h_win;
  gchar buf[130];
  GtkWidget *w_splash;
  
  gtk_set_locale ();
  gtk_init (&argc, &argv);
  
  /* show the splash screen */
   w_splash=create_window_SplashScreen ();
   gtk_widget_show_all (w_splash);
   SplahScreenId = gtk_timeout_add(3000, splahScreenOff, (gpointer *)w_splash);  
  /* tester la présence du fichier de préférences */
  path_test=g_strdup_printf("%s/.ellaprefs",getenv("HOME"));
  existe= g_file_test(path_test,G_FILE_TEST_IS_REGULAR);
  if (!existe)
     {
      /* créer le fichier de préférences */
      printf("fichier prefs absent ! \n");
	  f=fopen(path_test,"w");
	  /* width height margins pagecolor fps footage */
	  fprintf(f,"550\n440\n80\n0xFFFF\n0xFFFF\n0xFFFF\n12\n512\n");
	  fclose(f);
	  /* poser valeurs par défaut ! */
	  rouge=0xFFFF;
	  vert=0xFFFF;
	  bleu=0xFFFF;
	  fps=12;
	  footage=512;
	  width=550;
	  height=440;
	  marges=80;
	 }
  else
    {
		/* lire prefs */
		f=fopen(path_test,"r");
		fgets(buf,128,f);
		width=atoi(buf);
		if ((width<=0)||(width>1024))
		   width=550;
		fgets(buf,128,f);
		height=atoi(buf);
		if ((height<=0)||(height>1024))
		   height=440;  
		fgets(buf,128,f);
		marges=atoi(buf);
		if ((marges<=0)||(marges>256))
		   marges=80;
		fgets(buf,128,f);
		rouge=atoi(buf);
		fgets(buf,128,f);
		vert=atoi(buf);
		fgets(buf,128,f);
		bleu=atoi(buf);
		fgets(buf,128,f);
		fps=atoi(buf);
		if ((fps<=0)||(fps>29))
		   fps=12;
		fgets(buf,128,f);
		footage=atoi(buf);
		if ((footage<=0)||(footage>MAX_FOOTAGE))
		   footage=512;   
		/* flag autoarchive */
		fgets(buf,128,f);
		f_archive=atoi(buf);
		if (f_archive==1)
		  set_autoarchive_biblio(TRUE);
		else
		   set_autoarchive_biblio(FALSE);   
		/* valeur zoom timeline */
		fgets(buf,128,f);
		t_zoom=atoi(buf);   
		timeline_set_default_movie_zoom(t_zoom);
		/* geometry */
		fgets(buf,128,f);
		x_fen=atoi(buf);
		fgets(buf,128,f);
		y_fen=atoi(buf);
		fgets(buf,128,f);
		w_fen=atoi(buf);
		fgets(buf,128,f);
		h_fen=atoi(buf);
		fclose(f);
	}
  /* et il faut transmettre les valeurs des prefs */

  cb_prefered_folder_init();
  editor_set_page_width((gdouble) width);
  editor_set_page_height((gdouble) height);
  editor_set_default_page_width((gdouble) width);
  editor_set_default_page_height((gdouble) height);
  /* pour les marges, perfectible ! */
  editor_set_default_margins((gdouble)marges);
  timeline_set_default_movie_freq(fps);
  timeline_set_movie_freq(fps, FALSE);
  /* zooming par défaut */
  
  /**** reste à faire footage ***/
  /* initialisation des dossiers cachés */
  /* récupérer le dossier Home */
   strcpy(path_local, getenv("HOME"));
  /* on teste la présence du répertoire caché */
  sprintf(tmp_file,"chdir %s/.ella",path_local);
  erreur = system(tmp_file);
  /* il n'existe pas encore, on va le créer */
  if (erreur!=0)
     {
        sprintf(tmp_file,"mkdir -m 777 %s/.ella",path_local);
        erreur= system ( tmp_file); // toutes les permissions pour tout le monde                   
     }
 
  icopb = gdk_pixbuf_new_from_xpm_data((const char **)logo_xpm); /* transtypage obligatoire */
  window_ella = create_window_ella ();
  /* window ella must be initialized before adding recent files menu */
    cb_reload_recent_files_list();
  /* setting saved geometry */
  gtk_window_resize(GTK_WINDOW(window_ella),w_fen,h_fen);
  gtk_window_set_icon(GTK_WINDOW(window_ella), icopb);
  gtk_widget_show_all (window_ella);/* show ALL sinon la barre d'outils perso n'est pas affichée ! */
  g_object_unref (icopb);
  /* attendre l'affichage ! */
  editor_set_default_page_color(rouge,vert,bleu);
  editor_set_page_color( rouge,vert,bleu);
  /* éditeur */
  editor_reset_item_counter();
  /* timeline */
  interconnect_adj();
  
  /* on ne peut brancher les callbacks qu'après initialisation */
  g_signal_connect ((gpointer) buttonLayerNew, "clicked",
                    G_CALLBACK (on_button_layer_new_clicked),
                    (gpointer *) pListViewLayers);
  g_signal_connect ((gpointer) buttonLayerDuplicate, "clicked",
                    G_CALLBACK (on_button_layer_duplicate_clicked),
                    G_OBJECT(pListViewLayers));
  g_signal_connect ((gpointer) buttonLayerUp, "clicked",
                    G_CALLBACK (on_button_layer_up_clicked),
                    G_OBJECT(pListViewLayers));
  g_signal_connect ((gpointer) buttonLayerDown, "clicked",
                    G_CALLBACK (on_button_layer_down_clicked),
                    G_OBJECT(pListViewLayers));
  g_signal_connect ((gpointer) buttonLayerRemove, "clicked",
                    G_CALLBACK (on_button_layer_delete_clicked),
                    G_OBJECT(pListViewLayers));
					
 /*** clipboard****/
  clipboard_init();			
  undo_init();
 /******/	
  
  /* now, we can close the splash screen */		
  //gtk_widget_destroy(w_splash);	
  gtk_main ();
  free_icones();
// printf("géométrie : x=%d y=%d w=%d h=%d \n",x_fen,y_fen,w_fen,h_fen);
  /* nettoyer le dossier caché */
  sprintf(tmp_file,"rm %s/.ella/*.*",path_local);
  erreur = system(tmp_file);  
  /* sauvegarde des prefs */
  /* il faut récupérer l'état actuel des variables */
  /* attention: ce doit être les variables par DEFAUT, pas les courantes !*/
  f=fopen(path_test,"w");
  fprintf(f,"%d\n",(gint)editor_get_default_page_width());
  fprintf(f,"%d\n",(gint) editor_get_default_page_height());
  fprintf(f,"%d\n",(gint)editor_get_default_margins());
  fprintf(f,"%d\n",(gint)(editor_get_default_page_color().red));
  fprintf(f,"%d\n",(gint)(editor_get_default_page_color().green));
  fprintf(f,"%d\n",(gint)(editor_get_default_page_color().blue));
  fprintf(f,"%d\n",timeline_get_default_movie_freq());
  fprintf(f,"%d\n",footage);	
  /* flag pour sauvegarde auto des archives de bibliothèque */
  if (get_autoarchive_biblio())
     fprintf(f,"%d\n",1);
  else
     fprintf(f,"%d\n",0);	
  fprintf(f,"%d\n",dlg_get_timeline_default_zoom());/* 0=100, 1=75, 2=50 */	 
  /* saving geometry */
  fprintf(f,"%d\n",x_fen);
  fprintf(f,"%d\n",y_fen);
  fprintf(f,"%d\n",w_fen);
  fprintf(f,"%d\n",h_fen);
  fclose(f);
  g_free(path_test);
  erreur=cb_save_recent_files_list(cb_get_recent_files_list());
  return 0;
}
