/****************************************
SVG export and utilities
****************************************/
/*
This file is part of Ella.

Ella is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Ella is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ella.  If not, see <http://www.gnu.org/licenses/>.
*/


/****************************
 * bibliothèque d'exportation
 * vers le format SWF
 * **************************/
//translations
#include <locale.h>
#include <libintl.h>

#include "math.h"
#include <gtk/gtk.h>
#include <goocanvas.h>
#include "ellasyms.h"
#include "pango/pangofc-font.h"
#include "pango/pangofc-fontmap.h"
#include "svgfuncs.h"

/* objects types */
enum 
  {   SVG_EMPTY,
	  SVG_RECT,
	  SVG_ELLIPSE,
	  SVG_LINE,
	  SVG_TEXT,
	  SVG_BITMAP,
	  SVG_OUTLINE,
	  SVG_PATH,
	  SVG_BLANK
  };
/* structure to store items on the stack */
typedef struct
      {
        gint x; /* position en pixels */
        gint y;
		gint x2;/* for lines */
		gint y2;
		gint cx;/* center of objects, for ellipses and rotations */
		gint cy;
		gint rx;
		gint ry;
		gint w;/* pour gérer le zooming */
		gint h;
		gint lw;/* stroke width */
        gdouble rotation;/* en degrés */    
        gchar *titre;/* nom de l'objet */  
		gboolean fDel;/* flag pour savoir s'il a subi un .del */  
		gint frame;/* frame position in order to code the time position for svg */
		guint effect;
		guint32 f_color;
		guint32 s_color;
		gdouble alpha;
		gint object_type;
		gint fps;//movie frequency ; I know, it's a repetition !
		gint fnt_size;
		gchar *fnt_family;
		gchar *texte;
		gchar *path;
      } s_datas;
/* structure to store all items of a layer on a special stack */
typedef struct
      {	  gchar *name;
		  gchar *long_name;
	  } s_name;
/* variables locales de stockages divers */
GSList *pile_singles=NULL;
GSList *pile_svg_items=NULL;
static gint marge_x,marge_y;
/***********************************
 * chercher un item dont le
 * titre est connu dans la pile
 * des images ; si trouvé, renvoit
 * l'adresse du pixbuf, sinon NULL
 * *********************************/
GdkPixbuf *svg_find_item_on_images_stack(GSList *pile,gchar *titre)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
   GdkPixbuf *pb=NULL;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return NULL;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
   if (titre==NULL)
	  return NULL;	
   /* début boucle */
	temp = g_slist_nth (pile, j);				  
	while ((temp!=NULL)&&(trouve==FALSE))
	  { 
		if (  strcmp(   ((stack_images *)(temp->data))->nom,titre)==0   )
          {          	
			trouve=TRUE;
			pb=((stack_images *)(temp->data))->pixb;
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */			  
   return pb;
}
/*******************************************
 * obtain size of a file
 * ****************************************/
gint fsize(const gchar * fname, long * ptr)
{
    /* Cette fonction retourne 0 en cas de succes, une valeur differente dans le cas contraire. */
    /* La taille du fichier, si elle a pu etre calculee, est retournee dans *ptr                */
    
    FILE * f;
    int ret = 0;
    
    f = fopen(fname, "rb");   
    if (f != NULL)
    {
        fseek(f, 0, SEEK_END); /* aller a la fin du fichier */
        *ptr = ftell(f); /* lire l'offset de la position courante par rapport au debut du fichier */
        fclose(f);
    }
    else
        ret = 1;
    
    return ret;
}
/*******************************************
 * a function to encode a PNG file to the
 * Base64 ASCII format
 * *****************************************/
gint svg_encode64(gchar *f_source, FILE *fDest)
{
	gint c_ligne,retour;
    FILE* fSrc;
    guchar *buffer;
    gint NbLus;
	gchar *ch64;
	long f_size;
			
	if ((fSrc = fopen(f_source, "rb")) == NULL)
      {
        return 1;
      }
        
	
	retour=fsize(f_source,&f_size); 
	buffer=g_malloc0(f_size*sizeof(guchar)+2);
    c_ligne=0;  
    while ((NbLus = fread(buffer, sizeof(guchar), f_size, fSrc)) != 0)// we're reading ONE byte at once
	   { 
		 /* the byte is encode to 64 gchar */
		 ch64=g_base64_encode ((const guchar *)buffer, f_size);// const guchar* mandatory, in order to avoid segfault 
         fprintf(fDest,"%s",ch64);
		 g_free (ch64);
	   }   
	g_free(buffer);    
    fclose(fSrc);    
    return 0;	  
}
/**********************************************
 * a function to force the use of 'point' as
 * decimal separator
 * ******************************************/
/**********************************
 * recoder les RC en escapes
 * *******************************/
gchar *str_replace_commas (const gchar *string)
{
   gchar *strip = NULL;

   if (string != NULL)
   {
      strip = malloc (2*sizeof (*strip) * (strlen (string) + 1));
      if (strip != NULL)
      {
         gint i, j;

         for (i = 0, j = 0; string[i]; i++)
         {
            if (string[i] == ',')
            {
           
                  strip[j] = '.';
				  j++;				  
            }
            else
            {
               strip[j] = string[i];
               j++;
            }
         }
        strip[j] = '\0';
      }
      else
      {
         fprintf (stderr, "Memoire insuffisante\n");
         exit (EXIT_FAILURE);
      }
   }
   return strip;
}
/****************************************
 * two functions to correct coordinates
 * by the x and y margins
 * *************************************/
gint svg_correct_x(gint v)
{
	return (gint)(v-marge_x);
}

gint svg_correct_y(gint v)
{
	return (gint)(v-marge_y);
}
/**********************************
 * function to return a time chain
 * from frame position and
 * movie frequency
 * ******************************/
gchar *svg_position_to_time(gint frame, gint freq)
{
	gchar *retour=NULL;
	gint reste;
	gfloat position;
	gfloat dixiemes=0.0;
	
	position=(gfloat)frame;
	reste = (gint)position % freq;
	dixiemes = reste/(gfloat)freq*10;	
	retour = g_strdup_printf("%d.%d",(gint)position/freq,(gint)dixiemes);
    return retour;	
}
/**********************************
 * tester si l'image courante est
 * une image-clé ; si OUI,
 * la fonction renvoit un ensemble
 * de bits allumés ou éteints
 * indiquant les effets activés
 * le paramètre est en coordonnées
 * PC.
 * GSList correspond à keyframes_list
 * ***********************************/
guint svg_get_transitions_keyframe(GSList *list,guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
    guint effets=0;
	GSList  *temp;
	effects_data_type *temp_data;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {          	
			  trouve=TRUE;
			  /* passer les bits : 0 rien,
			   * 1 mouvement, 2 zoom, 4 rotation, 8 luminance,
			   * 16 alpha */
			  if (((effects_data_type *)(temp->data))->effect_motion)
			    effets = effets | 1;
			  if (((effects_data_type *)(temp->data))->effect_zooming)
			    effets = effets | 2;
			  if (((effects_data_type *)(temp->data))->effect_rotation)
			    effets = effets | 4;
			  if (((effects_data_type *)(temp->data))->effect_lumi)
			    effets = effets | 8;
			  if (((effects_data_type *)(temp->data))->effect_alpha)
			    effets = effets | 16;				
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return effets;
}
/**********************
 RAZ de la pile
**********************/
void svg_raz_pile(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		//printf("j'efface :%s\n",((s_datas *)(node->data))->titre);  
		g_free(((s_datas *)(node->data))->titre); 
		if ( ((s_datas *)(node->data))->path!=NULL  )
		    g_free(((s_datas *)(node->data))->path);
		if (((s_datas *)(node->data))->texte!=NULL)
		    g_free(((s_datas *)(node->data))->texte);
		if (((s_datas *)(node->data))->fnt_family!=NULL)	 
		    g_free(((s_datas *)(node->data))->fnt_family);
       // g_free(((s_datas *)(node->data)));
      }/* for */
   g_slist_free (list);
}	  	 

/*********************************
 * add an item on virtual stack
 * ******************************/
GSList  *svg_add_pile(GSList *list, gchar *name, gint x, gint y, gint rx, gint ry,
                        gint w,gint h,gdouble rota)
{
 GSList *temp;
 s_datas *temp_data;
 
 temp_data = g_malloc(sizeof(s_datas));
 temp_data->fDel=FALSE;/* flag .del déjà ou pas */
 temp_data->x=x;
 temp_data->y=y;
 temp_data->w=w;
 temp_data->h=h;
 temp_data->rotation=rota;
 temp_data->titre = g_strdup(name); 
 temp = g_slist_append (list,temp_data);
 return temp;	
} 
/*********************************
 * ajouter un item sur la pile des
 * objets UNIQUES
 * ******************************/
GSList  *svg_add_pile_unique_item(GSList *list, gchar *nom, gchar *long_name)
{
 GSList *temp;
 s_name *temp_data;
 
 temp_data = g_malloc(sizeof(s_name));
 temp_data->name = g_strdup(nom); 
 temp_data->long_name= g_strdup(long_name);
 temp = g_slist_append (list,temp_data);
 return temp;	
} 
/**********************
 RAZ de la pile
**********************/
void svg_raz_pile_singles(GSList *list)
{
   gint    nIndex;
   GSList *node;

   if (list==NULL)
        return;
   //printf("nombre éléments:%d\n",g_slist_length(list));
   for (nIndex = 0; node = g_slist_nth (list, nIndex); nIndex++) 
      {
		//  printf("j'efface :%s\n",((s_name *)(node->data))->name);
		g_free(((s_name *)(node->data))->name); 
		g_free(((s_name *)(node->data))->long_name); 
        //g_free(((s_name *)(node->data)));
      }/* for */
   g_slist_free (list);
}	  	 
/****************************************
 * find if an item is on the 'unique'
 * items stack
 * *************************************/

/******************************
 * chercher un item dont le
 * titre est connu dans la pile
 * ****************************/
gboolean svg_find_unique_item_on_stack(GSList *pile,gchar *nom_item)
{
   gboolean	trouve;
   gint    nIndex;
   GSList *temp;
   gint j;
      
   trouve=FALSE;
   j=0;
   if (pile==NULL)
        return FALSE;/* pourquoi la pile peut-être NULL parfois alors qu'elle a un élément ? */
	if (nom_item==NULL)
	  return FALSE;
	  
   /* début boucle */
	temp = g_slist_nth (pile, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (  strcmp(   ((s_name *)(temp->data))->name,nom_item)==0   )
          {          	
			trouve=TRUE;		
          }	
		else
		  {	  
		    j++;
		    temp = g_slist_nth (pile, j);
		  }
	  }/* wend */				  
   return trouve;
}
/**********************************
 * tester si l'image courante est
 * une image-clé ; si OUI,
 * la fonction renvoit un pointeur
 * valide sur le groupe d'items,
 * sinon elle renvoit NULL ;
 * le paramètre est en coordonnées
 * PC
 * 
 * GSList correspond à keyframes_list
 * ***********************************/
GooCanvasItem *svg_get_valid_keyframe(GSList *list,guint kf)
{
    gint j=0;
    gboolean trouve=FALSE;  
	GooCanvasItem *item=NULL;
	GSList  *temp;
	effects_data_type *temp_data;
	
	/* récupérer un pointeur sur le groupe d'items de l'image-clé éventuelle */
    j=0;
	item=NULL;
	trouve=FALSE;
	/* début boucle */
	temp = g_slist_nth (list, j);
	while ((temp!=NULL)&&(trouve==FALSE))
	  {
		if (((effects_data_type *)(temp->data))->keyframe==kf)
            {          	
          	  item=((effects_data_type *)(temp->data))->rootKF; 
			  trouve=TRUE;
            }		  
		  j++;
		  temp = g_slist_nth (list, j);
	  }/* wend */	
	return item;
}
/*************************************
 * push on stack all 'uniques' items
 * of the current canvas
 * **********************************/
void svg_unique_item_push(GooCanvasItem *list, gint layer_type)
{
	gboolean trouve;
	gint j,nIndex;
	GSList  *temp,*node;
	effects_data_type *temp_data;
	GooCanvasItem *source, *dest, *item;
	gchararray title;
    gchar *path,*diese, *titre_court;	
	
	j=0;
	trouve =FALSE;
	temp=NULL;
	
	/* before all tests, is it a sound layer ? So, we don't care about the 'list' variable */
	if (layer_type==SOUND_LAYER)		   
		   return;
	if (list==NULL)
	    return; /* liste d'items vide, donc rien à faire ! */	
	/* lire tout le groupe d'items et agir selon la situation */	
	nIndex=0;
	while (nIndex<goo_canvas_item_get_n_children (list))		
	   {
	   	 item=goo_canvas_item_get_child(list,nIndex);
		 g_object_get(item,"title",&title,NULL);
	   	 if ((item!=NULL)&&(title!=NULL))
	   	   {			   			 
			 /* two cases, because path items have special titles ! */  
			 if ( (strncmp(title,"U",1)==0) || (strncmp(title,"P",1)==0) )
	             {					 
					diese= g_strrstr(title, "#");
	                if (diese!=NULL)
	                    {
							titre_court= (gchar *)file_str_sub_extension (title, 0, diese-title-1);
							if (!svg_find_unique_item_on_stack(pile_singles,titre_court))
				                 pile_singles=svg_add_pile_unique_item(pile_singles,titre_court,title);
							g_free(titre_court);	 
				        }
				 }
			 else
                {/* other objects */ 
                   if (!svg_find_unique_item_on_stack(pile_singles,title))
				       pile_singles=svg_add_pile_unique_item(pile_singles,title,"");
				}
		   }	
		 nIndex++;	
	   }/* wend */		
}
/*********************************************************
 * with the 'unique' items list for the current layer, and
 * for each 'unique' item, we check all the appearances,
 * and push cordinates and so on on the stack
 * *****************************************************/
GSList *svg_push_unique_appearances(GSList *list, GooCanvasItem *objets, gchar *nom_item, gint frame, guint effet, GSList *pile)
{
	gint i,j,max_elem;
	gboolean trouve=FALSE;
	gboolean transfo;
	GSList *temp;
	s_datas *temp_data;
	gchararray title,texte,fonte;
	GooCanvasItem *item;
	GooCanvasPoints *l_pts;
	gdouble x,y,width,height,xt,yt,xc,yc,rx,ry,valeur_rotation,echelle,valeur_alpha;
    guint32 fcolor,scolor;
    gdouble lw;
	PangoFontDescription *fnt;
	gchar *fnt_family;
	gint fnt_size;
	GooCanvasBounds area;
	gchar *image_file;
	GdkPixbuf *pb;
	gboolean erreur;	
    gchar *path,*diese,*titre_court;
	
    /* il faut chercher dans le groupe d'items la présence de l'item fourni */
    j=0;
    max_elem=goo_canvas_item_get_n_children(objets);/* combien l'origine des items a-t-elle d'objets ? */
    if (max_elem==0)
     {
	   temp=list;
	   //printf("j'ai trouvé une image clé vide à la position :%d\n",frame);
	   /* it's a blank keyframe */
	   temp_data = g_malloc(sizeof(s_datas));
       temp_data->titre = g_strdup_printf("%s",nom_item); /* thus we can know the 'object_type' of item with the first letter */      
       temp_data->object_type=SVG_BLANK;
       temp_data->frame=frame;
	   temp = g_slist_append (list,temp_data);
	   return temp;
     }
	/* ici ! */ 
    while ((!trouve)&&(j<max_elem))
    {
		item=goo_canvas_item_get_child(objets,j);
		g_object_get(item,"title",&title,NULL);
		/* two cases */
        if ( (strncmp(title,"U",1)==0) || (strncmp(title,"P",1)==0) )
	             {					 
					diese= g_strrstr(title, "#");
	                if (diese!=NULL)
	                    {
							titre_court= (gchar *)file_str_sub_extension (title, 0, diese-title-1);	
							if  (strcmp(nom_item,titre_court)==0)
		                       trouve=TRUE;					
							g_free(titre_court);   		 
				        }
				 }
		 else
           {/* other objects */             		
		     if  (strcmp(nom_item,title)==0)
		          trouve=TRUE;
		   }
		j++;
	}
   if (trouve==FALSE)
      return list;

printf("dans appera nom:%s<\n",nom_item);
   temp_data = g_malloc(sizeof(s_datas));
   temp_data->titre = g_strdup_printf("%s",nom_item); /* thus we can know the 'object_type' of item with the first letter */
   /* switch with the first letter */
   temp_data->object_type=SVG_EMPTY;// security
   temp_data->frame=frame;// for coding of time animation
   //temp_data->fps=fps;
   if (strncmp(temp_data->titre,"R",1)==0)
        {
     	  /* rectangle */
		  temp_data->object_type=SVG_RECT;
	    }
   if (strncmp(temp_data->titre,"L",1)==0)
        {
     	  /* single line */
		  temp_data->object_type=SVG_LINE;
	    }		
   if (strncmp(temp_data->titre,"C",1)==0)
        {
     	  /* ellipse */
		  temp_data->object_type=SVG_ELLIPSE;
	    }		
   if (strncmp(temp_data->titre,"T",1)==0)
        {
     	  /* text */
		  temp_data->object_type=SVG_TEXT;
	    }		
   if (strncmp(temp_data->titre,"B",1)==0)
        {
     	  /* bitmap */
		  temp_data->object_type=SVG_BITMAP;
	    }		
   if (strncmp(temp_data->titre,"U",1)==0)
        { /* outline */
		  temp_data->object_type=SVG_OUTLINE;
	    }		
   if (strncmp(temp_data->titre,"P",1)==0)
        { /* path */
		  temp_data->object_type=SVG_PATH;
	    }		
   temp_data->effect=effet;
   g_object_get(item, "fill_color_rgba" ,&fcolor,"stroke-color-rgba",&scolor,"line-width",&lw,NULL);
   transfo=goo_canvas_item_get_simple_transform(item,&xt,&yt,&echelle,&valeur_rotation);	
   if (temp_data->object_type!=SVG_OUTLINE)
         valeur_alpha=(fcolor & 0x000000000000FF)/256.0*100;  
    else
	   valeur_alpha=(scolor & 0x000000000000FF)/256.0*100;
   temp_data->rotation=valeur_rotation;
   /* colors */
   temp_data->alpha=valeur_alpha;
   temp_data->f_color=fcolor;
   temp_data->s_color=scolor;
   temp_data->texte=NULL;
   temp_data->fnt_family=NULL;
   temp_data->path=NULL;
   switch (temp_data->object_type)
     {
		 case SVG_RECT:
		   {
			   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
			   /* coordinates */
			   temp_data->x=x;
	           temp_data->y=y;
	           temp_data->w=width;
	           temp_data->h=height;
	           temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));	
			   temp_data->lw=lw;           
			   break;
		   }
		 case SVG_PATH:
		   {
			   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
			   /* coordinates */
			   temp_data->x=x;
	           temp_data->y=y;
	           temp_data->w=width;
	           temp_data->h=height;
	           temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));	
			   temp_data->lw=lw;    
			   /* path */
			   diese= g_strrstr(title, "#");       
			   if (diese!=NULL)
	              {
			         path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
					 temp_data->path=g_strdup_printf("%s",path);
					 g_free(path);
				  }
			   break;
		   }  
		 case SVG_OUTLINE:
		   {
			   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
			   /* coordinates */
			   temp_data->x=x;
	           temp_data->y=y;
	           temp_data->w=width;
	           temp_data->h=height;
	           temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));	
			   temp_data->lw=lw;    
			   /* path */
			   diese= g_strrstr(title, "#");       
			   if (diese!=NULL)
	              {
			         path=(gchar *)file_str_sub_extension (title, diese-title+1,strlen(title)-1);	
					 temp_data->path=g_strdup_printf("%s",path);
					 g_free(path);
				  }
			   break;
		   }  		   
		 case SVG_ELLIPSE:
		   {  
			   g_object_get(item,"center_x",&xc,"center_y",&yc,"radius_x",&rx,"radius_y",&ry,
				            "width",&width,"height",&height,NULL);	
			   temp_data->cx=xc;
	           temp_data->cy=yc;
			   temp_data->x=xc;
			   temp_data->y=yc;/* to avoid a bug */
	           temp_data->w=width;
	           temp_data->h=height;			
			   temp_data->rx=rx;
			   temp_data->ry=ry;
			   temp_data->lw=lw;
			   break;
		   }  
		 case SVG_LINE:
		   {  
			   l_pts= goo_canvas_points_new(2);/* lignes simples */
               g_object_get(item,"points",&l_pts,"width",&width,"height",&height,NULL);
			   temp_data->x=(gint)l_pts->coords[0];
			   temp_data->y=(gint)l_pts->coords[1];
			   temp_data->x2=(gint)l_pts->coords[2];
			   temp_data->y2=(gint)l_pts->coords[3];
			   temp_data->w=width;
	           temp_data->h=height;	
			   temp_data->lw=lw;
			   temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));
			   goo_canvas_points_unref(l_pts);
			   break;
		   }   
		 case SVG_TEXT:
		   {  
			   g_object_get(item,"x",&x,"y",&y,"text",&texte,"font",&fonte,"width",&width,"height",&height,NULL);
			   goo_canvas_item_get_bounds(item,&area);	 
			   temp_data->texte=g_strdup_printf("%s",texte);		 
			   fnt=pango_font_description_from_string(fonte);		
			   fnt_family=(gchar *)pango_font_description_get_family(fnt);
			   fnt_size=(gint)(pango_font_description_get_size(fnt)/1000); 
			   temp_data->fnt_size=fnt_size;
			   temp_data->fnt_family=g_strdup_printf("%s",fnt_family);			   
			   /* attention width =-1 by default, and so on for height, so we must compute Bbox for the text ! */
	           temp_data->w=(gint)(area.x2-area.x1);
	           temp_data->h=(gint)(area.y2-area.y1);
			   temp_data->x=x-(temp_data->w/2);
			   temp_data->y=y;
	           //temp_data->y=y-(temp_data->y/2);
	           temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));
			   //printf("fonte:%s: size=%d\n",fnt_family,(gint)(fonte,fnt_size/1000));// taille en points=fnt_size/1000		 
			   break;
		   }   
		 case SVG_BITMAP:
		   {  
			   g_object_get(item,"x",&x,"y",&y,"width",&width,"height",&height,NULL); 
			    /* coordinates  for animation, x y are used, not cx, cy*/
			   temp_data->x=x;
	           temp_data->y=y;
	           temp_data->w=width;
	           temp_data->h=height;
	           temp_data->cx=(gint)(x+(width/2));
	           temp_data->cy=(gint)(y+(height/2));		
			   image_file=g_strdup_printf("%s/.ella/%s%s",getenv("HOME"),nom_item,".png");
			   pb=svg_find_item_on_images_stack(pile,temp_data->titre);
			   erreur = gdk_pixbuf_save (pb, image_file, "png", NULL,NULL);
			   g_free(image_file); 		   			   
			   break;
		   }   
		 default:; 
	 }/* end switch */
   temp = g_slist_append (list,temp_data);
  
   return temp;	
}
/******************************
 * mouchard
 * ***************************/
void svg_mouchard(GSList *list)
{
	gint nIndex;
	GSList *temp;
	
	printf("voici la pile des uniques :\n");
	for (nIndex = 0; temp = g_slist_nth (list, nIndex); nIndex++) 
      {      	       
		 printf("item:%s<<",((s_name *)(temp->data))->name);
      }/* end */	
	  printf("-----\n");
}
/*******************************************
 * output filled paths to svg format
 * ***************************************/
void svg_path(FILE *f,gchar *path,gint lw,gint x,gint y,gint w, gint h, gint cx, gint cy, guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
    fprintf(f, "<path d=\"%s z\" \n",path);
	/* output the color */
	/* we translate the path object, because under svg, we don't have to use borders for scenery */	
	fprintf(f,"style=\"fill:%s;stroke:%s;stroke-width:%d\" opacity=\"%s\" visibility=\"hidden\" transform=\"translate(-%d,-%d)\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(scolor),lw,
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)),
					 marge_x,marge_y);	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");

	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));	
}
/*******************************************
 * output not filled paths to svg format
 * ***************************************/
void svg_outline(FILE *f,gchar *path,gint lw,gint x,gint y,gint w, gint h, gint cx, gint cy, guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
    fprintf(f, "<path d=\"%s z\" \n",path);
	/* output the color */
	/* we translate the path object, because under svg, we don't have to use borders for scenery */	
	fprintf(f,"style=\"fill:none;stroke:%s;stroke-width:%d\" opacity=\"%s\" visibility=\"hidden\" transform=\"translate(-%d,-%d)\"",
	                 svg_strgoocanvas8bits(scolor),lw,
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)),
					 marge_x,marge_y);	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");

	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));	
}
/******************************************
 * output boxes to svg format ;
 * very important : don't use any <space>
 * before 'fill' token in order to obtain
 * a correct preview under Nautilus !
 * 
 * nB : opacity=alpha/100
 * nb visibility="hidden" pour le cacher
 
 * *****************************************/
void svg_rect(FILE *f, gint x, gint y, gint w, gint h, gint cx, gint cy,gint lw, guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
	fprintf(f, "<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" ",svg_correct_x(x),svg_correct_y(y),(gint)w, (gint)h);
	/* output the color */
	fprintf(f,"style=\"fill:%s;stroke:%s;stroke-width:%d\" opacity=\"%s\" visibility=\"hidden\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(scolor),lw,
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)));	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");
	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));
   
}
/*************************************
 * output ellipses to svg format
 * **********************************/
void svg_ellipse(FILE *f,gint cx, gint cy, gint rx, gint ry,gint lw,guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
	fprintf(f,"<ellipse cx=\"%d\" cy=\"%d\" rx=\"%d\" ry=\"%d\"\n",svg_correct_x(cx),svg_correct_y(cy),(gint)rx, (gint)ry); 
	/* output the color */
	fprintf(f,"style=\"fill:%s;stroke:%s;stroke-width:%d\" opacity=\"%s\" visibility=\"hidden\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(scolor),lw,
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)));	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");
	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));	
} 
/**********************************
 * output single lines to SVG 
 * format
 * *******************************/
void svg_line(FILE *f,gint x1,gint y1,gint x2,gint y2,gint cx,gint cy,guint32 fcolor,gint lw, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
	fprintf(f,"<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\"\n",svg_correct_x(x1),svg_correct_y(y1),svg_correct_x(x2), svg_correct_y(y2));
	/* output the color */
	fprintf(f,"style=\"fill:%s;stroke:%s;stroke-width:%d\" opacity=\"%s\" visibility=\"hidden\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(scolor),lw,
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)));	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");
	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));	
} 
/************************************
 * output text objects to SVG
 * format
 * *********************************/
void svg_text(FILE *f,gchar *txt, gchar *ffamily, gint x, gint y, gint w, gint h, gint fsize, gint cx, gint cy, guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
	gchar *str,**contenu;
	gint i,nbtotal=0;
	gdouble pas;
		
	fprintf(f,"<text x=\"%d\" y=\"%d\" font-family=\"%s\" font-size=\"%d\"\n",svg_correct_x(x),svg_correct_y(y),ffamily,fsize);
	/* output the color */
	fprintf(f,"    style=\"fill:%s;stroke:%s\" opacity=\"%s\" visibility=\"hidden\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(fcolor),
					 str_replace_commas(g_strdup_printf("%.3f",alpha/100.0)));	
    if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
    fprintf(f," >\n");
	/* first, we must split the txt chain */
	contenu=g_strsplit_set(txt, "\n",-1);
	/* now we count all the lines */
	for(i=0;contenu[i];i++)
		   nbtotal++;
    //printf("nbre total valeurs:%d\n",nbtotal);
    pas=(gdouble)(h/nbtotal);
	for(i=0;contenu[i];i++)
	   {
		   fprintf(f,"    <tspan x=\"%d\" y=\"%d\">\%s</tspan>\n",svg_correct_x(x),svg_correct_y((gint)(y+(pas*i))),contenu[i]);
	   }     	   
	/* and freed the memory */
	g_strfreev(contenu);		
	fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));		
}
/**************************************
 * output bitmap (PNG) images to SVG
 * ***********************************/
void svg_image(FILE *f, gchar *path_to_image,gint x, gint y, gint w, gint h, gint cx, gint cy, guint32 fcolor, guint32 scolor,gdouble alpha, gdouble rota, gint frame, gint fps)
{
  gint erreur;
  	
  fprintf(f," <image x=\"%d\" y=\"%d\" width=\"%dpx\" height=\"%dpx\" ",svg_correct_x(x),svg_correct_y(y),w,h);
  fprintf(f,"style=\"fill:%s;stroke:%s\" opacity=\"%s\" visibility=\"hidden\"",
	                 svg_strgoocanvas8bits(fcolor),
					 svg_strgoocanvas8bits(scolor),
					 "1.0");	

  if (rota!=0)
		   fprintf(f," transform=\"rotate(%s,%d,%d)\"",str_replace_commas(g_strdup_printf("%.3f", rota)),
		                      svg_correct_x(cx),svg_correct_y(cy));					   
  //fprintf(f," xlink:href=\"%s\"\n",path_to_image);
  // mode 64 for included pics
      fprintf(f,"  xlink:href=\"data:image/png;base64,\n");
      erreur= svg_encode64(path_to_image, f);
      printf("erreur:%d\n",erreur);
      fprintf(f,"\"\n");// end of mode64
  /* standard */	  
  fprintf(f," >\n");
  fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
  fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n", svg_position_to_time(frame, fps));
}
/******************************
 * mouchard
 * ***************************/
void svg_mouchard_items_codes(GSList *list, FILE *f, gint fps)
{
	gint nIndex, i, objet, previous_frame, effet;
	gint xmem,ymem, hmem,wmem;
	gdouble alphamem,rotationmem;
	gboolean hidden=FALSE;/* flag used when an object is wiped by a blank keyframe */
	GSList *temp;
	gchar buf[32];

	
	i=0;
	previous_frame=0;
	xmem=0;
	ymem=0;
	hmem=0;
	wmem=0;
	effet=0;
	alphamem=0.0;
	rotationmem=0.0;
	for (nIndex = 0; temp = g_slist_nth (list, nIndex); nIndex++) 
      {      	       
		 effet=((s_datas *)(temp->data))->effect;
		 if (i==0)// 1st appearance, we must create object 
		    {
		       switch (((s_datas *)(temp->data))->object_type)
		          {
			        case SVG_EMPTY:break;
					case SVG_BLANK:break;
			        case SVG_RECT:
					       {
						     objet=((s_datas *)(temp->data))->object_type;  
					         svg_rect(f,((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,((s_datas *)(temp->data))->w,
						           ((s_datas *)(temp->data))->h,
								   ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
								   ((s_datas*)(temp->data))->lw,
								   ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								   ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								   ((s_datas *)(temp->data))->frame,fps);

					         break;
				           }
					case SVG_PATH:
					    {
							objet=((s_datas *)(temp->data))->object_type;  
							svg_path(f,((s_datas*)(temp->data))->path,
							           ((s_datas*)(temp->data))->lw,
									   ((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,
									   ((s_datas *)(temp->data))->w,((s_datas *)(temp->data))->h,
									   ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
									   ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								       ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								       ((s_datas *)(temp->data))->frame,fps);
							break;
						}	   
					case SVG_OUTLINE:
					    {
							objet=((s_datas *)(temp->data))->object_type;  
							svg_outline(f,((s_datas*)(temp->data))->path,
							           ((s_datas*)(temp->data))->lw,
									   ((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,
									   ((s_datas *)(temp->data))->w,((s_datas *)(temp->data))->h,
									   ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
									   ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								       ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								       ((s_datas *)(temp->data))->frame,fps);
							break;
						}	
					case SVG_ELLIPSE:
						{
							objet=((s_datas *)(temp->data))->object_type; 
							svg_ellipse(f,((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
							              ((s_datas *)(temp->data))->rx,((s_datas *)(temp->data))->ry,
										  ((s_datas*)(temp->data))->lw,
										  ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								          ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								          ((s_datas *)(temp->data))->frame,fps);
							break;
						}
					case SVG_LINE:
						{
							objet=((s_datas *)(temp->data))->object_type;
							svg_line(f,((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,
							           ((s_datas *)(temp->data))->x2,((s_datas *)(temp->data))->y2,
									   ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
									   ((s_datas *)(temp->data))->lw,
									   ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								       ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								       ((s_datas *)(temp->data))->frame,fps);
							break;
						}
					case SVG_BITMAP:
						{							 
							objet=((s_datas *)(temp->data))->object_type;
							
							svg_image(f,g_strdup_printf("%s/.ella/%s%s",getenv("HOME"),((s_datas *)(temp->data))->titre,".png"),
							       ((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,
								   ((s_datas *)(temp->data))->w,
						           ((s_datas *)(temp->data))->h,
								   ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
								   ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								   ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								   ((s_datas *)(temp->data))->frame,fps);
	   
							break;
						}
					case SVG_TEXT:
						{
							objet=((s_datas *)(temp->data))->object_type; 
							svg_text(f,((s_datas *)(temp->data))->texte,
							           ((s_datas *)(temp->data))->fnt_family,
							           ((s_datas *)(temp->data))->x,((s_datas *)(temp->data))->y,
									   ((s_datas *)(temp->data))->w,((s_datas *)(temp->data))->h,
									    ((s_datas *)(temp->data))->fnt_size,
								       ((s_datas *)(temp->data))->cx,((s_datas *)(temp->data))->cy,
								       ((s_datas *)(temp->data))->f_color,((s_datas *)(temp->data))->s_color,
								       ((s_datas *)(temp->data))->alpha,((s_datas *)(temp->data))->rotation,
								       ((s_datas *)(temp->data))->frame,fps);
							break;
						}
					default:;
				  }//end switch
		   }/* if first appearance */		  
		 else  /* others appearances, so we can use memorized values like xmem */
		   {
			  /* it's a blank keyframe ? Then we must hide the object */
			  if (((s_datas *)(temp->data))->object_type==SVG_BLANK) 
			     {
					 hidden=TRUE;
					 fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	                       fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"hidden\"/>\n",
						         svg_position_to_time(((s_datas *)(temp->data))->frame, fps));
				 }
			  else
			     {
					if (hidden)
					   {
						   hidden=FALSE;
						   /* the object must re-appear */
						   fprintf(f,"    <set attributeName=\"visibility\" attributeType=\"XML\"\n");
	                       fprintf(f,"      begin=\"%ss\" dur=\"0s\" to=\"visible\"/>\n",
						         svg_position_to_time(((s_datas *)(temp->data))->frame, fps));
					   }
					/* output only effects for the CURRENT object */
					/* there is an effect ? */
					if (effet!=0)
						{
							/* passer les bits : 0 rien, 1 mouvement, 2 zoom, 4 rotation, 8 luminance, 16 alpha */
							if ((effet & 1) == 1)
							   {
								 //fprintf(f,"     <animate attributeName=\"x\"  attributeType=\"XML\"\n");	 
								 //fprintf(f,"       begin=\"%ss\" dur=\"%ss\" from=\"%d\" to=\"%d\" fill=\"freeze\" />\n",
								   //                     svg_position_to_time(previous_frame,fps),
								     //                   svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps),
										//				svg_correct_x(xmem),
											//			svg_correct_x(((s_datas *)(temp->data))->x));
								 //fprintf(f,"     <animate attributeName=\"y\"  attributeType=\"XML\"\n");	 
								 //fprintf(f,"       begin=\"%ss\" dur=\"%ss\" from=\"%d\" to=\"%d\" fill=\"freeze\" />\n",
								   //                     svg_position_to_time(previous_frame,fps),
								     //                   svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps),
										//				svg_correct_y(ymem),
											//			svg_correct_y(((s_datas *)(temp->data))->y));						
								 fprintf(f,"     <animateMotion  begin=\"%ss\" dur=\"%ss\"  fill=\"freeze\"\n",svg_position_to_time(previous_frame,fps),
								                 svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps));
								 fprintf(f,"           path=\"m %d,%d  l %d,%d\"\n",0,0,
								                                        ((s_datas *)(temp->data))->x-xmem,
																		((s_datas *)(temp->data))->y-ymem);
								 fprintf(f,"     />\n");	
							   }/* endif effect interpolation */
							if ((effet & 2) == 2)
							   {   
								   fprintf(f,"     <animate attributeName=\"width\" attributeType=\"XML\"\n");								   
								   fprintf(f,"       begin=\"%ss\" dur=\"%ss\" from=\"%d\" to=\"%d\" fill=\"freeze\" />\n",
								                        svg_position_to_time(previous_frame,fps),
								                        svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps),
														wmem,
														((s_datas *)(temp->data))->w);
								   fprintf(f,"     <animate attributeName=\"height\" attributeType=\"XML\"\n");						
                                   fprintf(f,"       begin=\"%ss\" dur=\"%ss\" from=\"%d\" to=\"%d\" fill=\"freeze\" />\n",
								                        svg_position_to_time(previous_frame,fps),
								                        svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps),
														hmem,
														((s_datas *)(temp->data))->h);								    
							   }/* endif zooming */
							if ((effet & 4) == 4)
							   {
								 fprintf(f,"      <animateTransform attributeName=\"transform\" type=\"rotate\"\n");
								 fprintf(f,"          values=\"%s %d %d; %s %d %d\"\n",str_replace_commas(g_strdup_printf("%.3f",rotationmem)),
								                                              svg_correct_x(((s_datas *)(temp->data))->cx),
								                                              svg_correct_y(((s_datas *)(temp->data))->cy),
																			  str_replace_commas(g_strdup_printf("%.3f", ((s_datas *)(temp->data))->rotation)),
																			  svg_correct_x(((s_datas *)(temp->data))->cx),
																			  svg_correct_y(((s_datas *)(temp->data))->cy) );
								 fprintf(f,"        begin=\"%ss\" dur=\"%ss\" fill=\"freeze\" />\n",svg_position_to_time(previous_frame,fps),
								                        svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps));   
							   }/* endif rotation */   
							if ((effet & 16)==16)
							   {
								 fprintf(f,"      <animate  attributeName=\"opacity\" attributeType=\"XML\"\n");
								 fprintf(f,"        begin=\"%ss\" dur=\"%ss\" from=\"%s\" to=\"%s\" fill=\"freeze\"/>\n",
								                     svg_position_to_time(previous_frame,fps),
													 svg_position_to_time(((s_datas *)(temp->data))->frame-previous_frame,fps),
													 str_replace_commas(g_strdup_printf("%.3f",alphamem)),
													 str_replace_commas(g_strdup_printf("%.3f",((s_datas *)(temp->data))->alpha/100.0)));  
							   }/* endif alpha/opacity effect */   
						}						
					else
						{
						/* maybe a 'jump' request ? */
							if (((s_datas *)(temp->data))->x!=xmem)
								{
									fprintf(f,"  <set  attributeName=\"x\" attributeType=\"XML\" begin=\"%ss\" dur=\"0s\" to=\"%d\"/>\n",
									  svg_position_to_time(((s_datas *)(temp->data))->frame, fps),
									  svg_correct_x(((s_datas *)(temp->data))->x));
								}
							if (((s_datas *)(temp->data))->y!=ymem)
								{
									fprintf(f,"  <set  attributeName=\"y\" attributeType=\"XML\" begin=\"%ss\" dur=\"0s\" to=\"%d\"/>\n",
										svg_position_to_time(((s_datas *)(temp->data))->frame, fps),
										svg_correct_y(((s_datas *)(temp->data))->y));
								}	
						}/* jump*/
				}/* not a blank */
		   }/* endif */				  
		 previous_frame= ((s_datas *)(temp->data))->frame;// memory for animate tags about durations and animations' starts
		 xmem=((s_datas *)(temp->data))->x;
		 ymem=((s_datas *)(temp->data))->y;
		 wmem=((s_datas *)(temp->data))->w;
		 hmem=((s_datas *)(temp->data))->h;
		 alphamem=((s_datas *)(temp->data))->alpha/100.0;
		 rotationmem=((s_datas *)(temp->data))->rotation;
		 i++;
      }/* next */	
	/* trailer of the SVG object */
	switch (objet)
	  {
			   case SVG_EMPTY:break;
			   case SVG_RECT:
			     {
					 fprintf(f,"  </rect>\n"); 
					 break;
				 }
			   case SVG_PATH:
			     {
					 fprintf(f,"  </path>\n"); 
					 break;
				 }	 
			   case SVG_OUTLINE:
			     {
					 fprintf(f,"  </path>\n"); 
					 break;
				 }		 
			   case SVG_ELLIPSE:
			     {
					 fprintf(f,"  </ellipse>\n"); 
					 break;
				 }
			   case SVG_LINE:
			     {
					 fprintf(f,"  </line>\n"); 
					 break;
				 }
			   case SVG_BITMAP:
			     {
					 fprintf(f,"  </image>\n");	
					 break;
				 }
			   case SVG_TEXT:
			     {
					 fprintf(f,"  </text>\n");
					 break;
				 }
			   default:;		  
	  }/* end case */  
}
/******************************
 * conversion des couleurs
 * GOOCANVAS sur 32 bits vers 
 * chaînes couleurs
 * en 8 bits 
 * **************************/
gchar *svg_strgoocanvas8bits(guint32 couleur)
{
	gint16 r,v,b;
	gchar *temp=NULL;
	
	/* passage de 32 bits RGBA en 16 bits composantes */
    r=(couleur & 0xFF000000)>>24;
    v=(couleur & 0x00FF0000)>>16;
    b=(couleur & 0x0000FF00)>>8;
	/* passage de 16 bits en 8 bits par plan */
	temp=g_strdup_printf("#%.2x%.2x%.2x",r,v,b);  	
	return temp;	
}
/******************************
 * conversion des couleurs
 * Gdk 16 bits vers 
 * chaînes couleurs
 * en 8 bits 
 * **************************/
gchar *svg_strgdk8bits(GdkColor couleur)
{
	gint16 r,v,b;
	gchar *temp=NULL;
	
	/* passage de 32 bits RGBA en 16 bits composantes */
    r=(gint)((couleur.red/65535.0)*255);
    v=(gint)((couleur.green/65535.0)*255);
    b=(gint)((couleur.blue/65535.0)*255);
	/* passage de 16 bits en 8 bits par plan */
	temp=g_strdup_printf("#%.2x%.2x%.2x",r,v,b);  	
	return temp;	
}
/******************************************
 * open a file as SVG for output ;
 * le gchar parameter must a a complete
 * path to a target file, WITH extension
 * **************************************/
FILE *svg_open_file_to_export(gchar *nom_fichier)
{
	FILE *f;
	
	f=fopen(nom_fichier,"w");
	return f;	
}
/**********************************************
 * close an already opened SVG file for export
 * ********************************************/
void svg_close_file_to_export(FILE *f)
{
	 fprintf(f,"</svg>\n");
	 fclose(f);
}
/**************************************************
 * output the svg header
 * input : width, height of the project
 * ***********************************************/
void svg_put_header_to_file(FILE *f, gdouble width, gdouble height, gdouble margex,gdouble margey)
{
	/* store horizontal & vertcal margins */
	marge_x=(gint)margex;
	marge_y=(gint)margey;
	/* header of the svg file */
	fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
	fprintf(f,"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" ");
	fprintf(f,"\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
	fprintf(f,"<svg width=\"%d\" height=\"%d\" version=\"1.1\" ",(gint)width,(gint)height);
    fprintf(f, "\t\txmlns:xlink=\"http://www.w3.org/1999/xlink\"\n");
	fprintf(f,"\t\txmlns=\"http://www.w3.org/2000/svg\">\n");
	fprintf(f,"\t<title>Exported by Ella</title>\n");	
}

/****************************************************
 * output the background of the movie
 * *************************************************/
void svg_put_background(FILE *f, gdouble width, gdouble height, GdkColor couleur)
{
	fprintf(f, "<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" ",(gint)width, (gint)height);
	/* output the color */
	fprintf(f,"style=\"fill:%s\" />\n",svg_strgdk8bits(couleur));
}

/*****************************************
 * output the entire movie : a pointer
 * upon the GtkListStore as input
 * **************************************/
void svg_put_movie(FILE *f, GtkWidget *pListLayers, gint nb_layers, gint maxframes, gint fps, GSList *pile_img)
{
 GtkTreeModel *model;
 GtkTreePath *path,*path3;
 gchar *path1,*path2;
 GtkTreeIter pIter;
 gint count, j, ligne, type_layer,k;
 guint cell;
 GSList *keyframes_list,*node;
 GooCanvasItem *groupe;
 gboolean frame_posee;/* flag indiquant si le repère de keyframes a déjà été utilisé */
 guint effets=0;
 gchar *fichier_son;	

//effets= svg_encode64("/home/luc/Bureau/petite.png", "/home/luc/Bureau/result.txt");
  for (ligne=0;ligne<nb_layers;ligne++) 
   {
	 frame_posee=FALSE;
	 pile_singles=NULL;
	 pile_svg_items=NULL;
     for (j=0;j<maxframes;j++)
       {		
   	    /* pointer sur les données du calque */
  	    model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListLayers));  
		path1=g_strdup_printf ("%d:0",ligne);/* pointe sur la 1ère case du contenu du layer */
		path=gtk_tree_path_new_from_string(path1);      
		gtk_tree_model_get_iter(model, &pIter, path);				
		/* récupérer la liste des images_clés = keyframe_list */
		gtk_tree_model_get(model, &pIter,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list,LAYER_TYPE,&type_layer, -1);/*LAYER_MOVEMENT_INDEX== 5=rang du pointeur sur les images_clés */
		/* now, we're searching for all UNIQUE items for the current layer */
		/* faire une boucle sur toute la keyframe_list */		
		groupe=svg_get_valid_keyframe(keyframes_list,j);
		/* gérer les items si c'est une image-clé non vide*/
		if (groupe!=NULL)
		   {
			 /* on dispose d'un pointeur sur l'ensemble des items sur ce calque et pour CETTE image-clé */
			 /* récupérer contenu cellule pour savoir si mouvement */
			 path2=g_strdup_printf ("%d:%d",ligne,j);
			 path3=gtk_tree_path_new_from_string(path2);      
             gtk_tree_model_get_iter(model, &pIter, path3);
             gtk_tree_model_get (model, &pIter,j+N_COLUMN,&cell,  -1);          
             gtk_tree_path_free(path3);
			 g_free(path2);
			 /* maintenant, chercher et définir tous les items seulement si pas déjà définis */			
			 svg_unique_item_push(groupe,type_layer);			
		   }/* endif keyframe!=NULL */	
		g_free(path1);
		gtk_tree_path_free(path);	
	   }/* for frame j*/
	   /* maintenant, on fait une boucle sur l'ensemble des 'uniques' et l'on définit à chaque fois leurs caractéristiques */
	   for (j=0;node = g_slist_nth (pile_singles, j); j++) 
	     {
		     pile_svg_items=NULL;		 
			 if (node!=NULL)
			   {				   
				   for (k=0;k<maxframes;k++)
				     /* looking for all appearances of the current item */
				     {
						/* pointer sur les données du calque */
						model=gtk_tree_view_get_model(GTK_TREE_VIEW(pListLayers));  
						path1=g_strdup_printf ("%d:0",ligne);/* pointe sur la 1ère case du contenu du layer */
						path=gtk_tree_path_new_from_string(path1);      
						gtk_tree_model_get_iter(model, &pIter, path);				
						/* récupérer la liste des images_clés = keyframe_list */
						gtk_tree_model_get(model, &pIter,LAYER_MOVEMENT_INDEX,(gpointer)&keyframes_list,LAYER_TYPE,&type_layer, -1);/*LAYER_MOVEMENT_INDEX== 5=rang du pointeur sur les images_clés */
						/* now, we're searching for all UNIQUE items for the current layer */
						/* faire une boucle sur toute la keyframe_list */
						effets=svg_get_transitions_keyframe(keyframes_list,k);/* obtain flag on animation type */
						groupe=svg_get_valid_keyframe(keyframes_list,k);											 
						if (groupe!=NULL)
						   { 
								/* on dispose d'un pointeur sur l'ensemble des items sur ce calque et pour CETTE image-clé */
								/* récupérer contenu cellule pour savoir si mouvement */
								path2=g_strdup_printf ("%d:%d",ligne,k);
								path3=gtk_tree_path_new_from_string(path2);      
								gtk_tree_model_get_iter(model, &pIter, path3);
								gtk_tree_model_get (model, &pIter,k+N_COLUMN,&cell,  -1);          
								gtk_tree_path_free(path3);
								g_free(path2);							   
								pile_svg_items=svg_push_unique_appearances(pile_svg_items,
						              groupe,
								      ((s_name *)(node->data))->name, k,effets,pile_img);									  
						   }						
						g_free(path1);
						gtk_tree_path_free(path);									   
					 }/* for k */
				  /* à ce stade, on a un pointeur sur la liste des items pré-codés */				 
				 /* now we computing animations for the current item */	 
				
				 if (pile_svg_items!=NULL)
				    {
						svg_mouchard_items_codes(pile_svg_items,f, fps);	
					}			
				 svg_raz_pile(pile_svg_items);	 
				 				printf("juste apres raz \n");
			   }/* endif node */
		 }
	   /* erase stacks */	   
	   //svg_raz_pile(pile_svg_items);
	   svg_raz_pile_singles(pile_singles);		   
   }/* for ligne */
}
